-- "Set define off" turns off substitution variables
Set define off;

--
-- Drops
--
ALTER TABLE RHNCONFIGCHANNEL DROP CONSTRAINT RHN_CONFCHAN_OID_LABEL_NAME_UQ DROP INDEX;
DROP TRIGGER RHN_ERRATAREL_MOD_TRIG;
DROP TRIGGER RHN_ERELTMP_MOD_TRIG;
DROP TRIGGER RHN_CHANNELREL_MOD_TRIG;
DROP INDEX RHN_CONFCHAN_OID_LABEL_UQ;
DROP SEQUENCE RHN_ERRATATMP_ID_SEQ;

--
-- Create Sequences
--
CREATE SEQUENCE RHN_ACTIONKS_XENGUEST_ID_SEQ START WITH 1;
CREATE SEQUENCE RHN_CHANNELCOMPS_ID_SEQ START WITH 101;
CREATE SEQUENCE RHN_KVT_ID_SEQ START WITH 1;
CREATE SEQUENCE RHN_VIEL_ID_SEQ START WITH 1;
CREATE SEQUENCE RHN_VIET_ID_SEQ START WITH 1;
CREATE SEQUENCE RHN_VIIL_ID_SEQ START WITH 1;
CREATE SEQUENCE RHN_VII_ID_SEQ START WITH 1;
CREATE SEQUENCE RHN_VIRT_SL_SEQ START WITH 1;
CREATE SEQUENCE RHN_VIS_ID_SEQ START WITH 1;
CREATE SEQUENCE RHN_VIT_ID_SEQ START WITH 1;
CREATE SEQUENCE RHN_VI_ID_SEQ START WITH 1;

--
-- Create tables, indexes, triggers
--
create table
rhnActionVirtStart
(
    action_id   number
                constraint rhn_avstart_aid_nn not null,
    uuid        varchar(128)
                constraint rhn_avstart_uuid_nn not null,
    created     date default(sysdate)
                constraint rhn_avstart_creat_nn not null,
    modified    date default(sysdate)
                constraint rhn_avstart_mod_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create unique index rhn_avstart_aid_uq
    on rhnActionVirtStart( action_id )
    storage ( freelists 16 )
    initrans 32;

alter table rhnActionVirtStart add constraint rhn_avstart_aid_pk
    primary key ( action_id );

create or replace trigger
rhn_avstart_mod_trig
before insert or update on rhnActionVirtStart
for each row
begin
    :new.modified := sysdate;
end;
/
show errors;

create table rhnErrataSeverity (
    id number
        constraint rhn_errata_sev_id_nn not null
        constraint rhn_errata_sev_id_pk primary key,
    rank number
        constraint rhn_errata_sev_rank_nn not null,
    label varchar2(40)
        constraint rhn_errata_sev_label_nn not null
);

create table
rhnKickstartVirtualizationType
(
	id			number
				constraint rhn_kvt_id_nn not null
				constraint rhn_kvt_id_pk primary key,
	name			varchar2(128)
				constraint rhn_kvt_name_nn not null,
	label			varchar2(128)
				constraint rhn_kvt_label_nn not null,
	created			date default (sysdate)
				constraint rhn_kvt_created_nn not null,
	modified		date default (sysdate)
				constraint rhn_kvt_modified_nn not null
)
	storage ( freelists 16 )
	initrans 32;

create table rhnVirtSubLevel (
    id      number
            constraint rhn_vsl_id_pk primary key,
    label   varchar2(32)
            constraint rhn_vsl_label_nn not null,
    name    varchar2(128)
            constraint rhn_vsl_name_nn not null,
    created date default sysdate
            constraint rhn_vsl_created_nn not null,
    modified date default sysdate
             constraint rhn_vsl_modified_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create index rhn_vsl_label_id_name_idx
    on rhnVirtSubLevel(label, id, name)
    storage ( freelists 16 )
    initrans 32;

create table
rhnVirtualInstanceEventType
(
	id			number
				constraint rhn_viet_id_nn not null
				constraint rhn_viet_id_pk primary key,
	name			varchar2(128)
				constraint rhn_viet_name_nn not null,
	label			varchar2(128)
				constraint rhn_viet_label_nn not null,
	created			date default (sysdate)
				constraint rhn_viet_created_nn not null,
	modified		date default (sysdate)
				constraint rhn_viet_modified_nn not null
)
	storage ( freelists 16 )
	initrans 32;

create table
rhnVirtualInstanceState
(
	id			number
				constraint rhn_vis_id_nn not null
				constraint rhn_vis_id_pk primary key,
	name			varchar2(128)
				constraint rhn_vis_name_nn not null,
	label			varchar2(128)
				constraint rhn_vis_label_nn not null,
	created			date default (sysdate)
				constraint rhn_vis_created_nn not null,
	modified		date default (sysdate)
				constraint rhn_vis_modified_nn not null
)
	storage ( freelists 16 )
	initrans 32;

create table
rhnVirtualInstanceType
(
	id			number
				constraint rhn_vit_id_nn not null
				constraint rhn_vit_id_pk primary key,
	name			varchar2(128)
				constraint rhn_vit_name_nn not null,
	label			varchar2(128)
				constraint rhn_vit_label_nn not null,
	created			date default (sysdate)
				constraint rhn_vit_created_nn not null,
	modified		date default (sysdate)
				constraint rhn_vit_modified_nn not null
)
	storage ( freelists 16 )
	initrans 32;

create table
rhnChannelCloned
(
	original_id		number
				constraint rhn_channelclone_fcid_nn not null
				constraint rhn_channelclone_fcid_fk
					references rhnChannel(id)
					on delete cascade,
	id		        number
				constraint rhn_channelclone_tcid_nn not null
				constraint rhn_channelclone_tcid_fk
					references rhnChannel(id)
					on delete cascade,
	created			date default(sysdate)
				constraint rhn_channelclone_created_nn not null,
	modified		date default(sysdate)
				constraint rhn_channelclone_modified_nn not null
)
	storage ( freelists 16 )
	initrans 32;

create index rhn_channelclone_fcid_tcid_idx
	on rhnChannelCloned ( original_id, id )
	storage ( freelists 16 )
	initrans 32;
alter table rhnChannelCloned add constraint rhn_channelclone_fcid_tcid_uq
	unique ( original_id, id );
alter table rhnChannelCloned add constraint rhn_channelclone_id_pk
        primary key ( id );

create index rhn_channelclone_tcid_fcid_idx
	on rhnChannelCloned ( id, original_id )
	storage ( freelists 16 )
	initrans 32;

create or replace trigger
rhn_channelclone_mod_trig
before insert or update on rhnChannelCloned
for each row
begin
	:new.modified := sysdate;
end;
/
show errors;

create table
rhnChannelComps
(
   id       number
        constraint rhn_channelcomps_id_nn not null
        constraint rhn_channelcomps_id_pk primary key,
   channel_id      number
        constraint rhn_channelcomps_cid_nn not null
        constraint rhn_channelcomps_cid_fk
            references rhnChannel(id)
            on delete cascade,
    relative_filename   varchar2(256)
        constraint rhn_channelcomps_rfn_nn not null,
    last_modified   date default(sysdate)
        constraint rhn_channelcomps_lastmod_nn not null,
    created     date default(sysdate)
        constraint rhn_channelcomps_created_nn not null,
    modified    date default(sysdate)
        constraint rhn_channelcomps_modified_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create or replace trigger
rhn_channelcomps_mod_trig
before insert or update on rhnChannelComps
for each row
begin
    :new.modified := sysdate;
    -- allow us to manually set last_modified if we wish
    if :new.last_modified = :old.last_modified
    then
        :new.last_modified := sysdate;
        end if;
end rhn_channelcomps_mod_trig;
/
show errors;

create table rhnChannelFamilyVirtSubLevel (
    channel_family_id number
                      constraint rhn_cfvsl_cfid_nn not null
                      constraint rhn_cfvsl_cfid_fk
                        references rhnChannelFamily(id),
    virt_sub_level_id number
                      constraint rhn_cfvsl_vslid_nn not null
                      constraint rhn_cfvsl_vslid_fk
                        references rhnVirtSubLevel(id),
    created           date
                      default sysdate
                      constraint rhn_cfvsl_created_nn not null,
    modified          date
                      default sysdate
                      constraint rhn_cfvsl_modified_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create index rhn_cfvsl_cfid_vslid_idx
    on rhnChannelFamilyVirtSubLevel(channel_family_id, virt_sub_level_id)
    storage ( freelists 16 )
    initrans 32;

create index rhn_cfvsl_vslid_cfid_idx
    on rhnChannelFamilyVirtSubLevel(virt_sub_level_id, channel_family_id)
    storage ( freelists 16 )
    initrans 32;

create table
rhnErrataCloned
(
	original_id		number
				constraint rhn_errataclone_feid_nn not null
				constraint rhn_errataclone_feid_fk
					references rhnErrata(id)
					on delete cascade,
	id		        number
				constraint rhn_errataclone_teid_nn not null
				constraint rhn_errataclone_teid_fk
					references rhnErrata(id)
					on delete cascade,
	created			date default(sysdate)
				constraint rhn_errataclone_created_nn not null,
	modified		date default(sysdate)
				constraint rhn_errataclone_modified_nn not null
)
	storage ( freelists 16 )
	initrans 32;

create index rhn_errataclone_feid_teid_idx
	on rhnErrataCloned ( original_id, id )
	storage ( freelists 16 )
	initrans 32;
alter table rhnErrataCloned add constraint rhn_errataclone_feid_teid_uq
	unique ( original_id, id );
alter table rhnErrataCloned add constraint rhn_errataclone_id_pk
        primary key ( id );

create index rhn_errataclone_teid_feid_idx
	on rhnErrataCloned ( id, original_id )
	storage ( freelists 16 )
	initrans 32;

create or replace trigger
rhn_errataclone_mod_trig
before insert or update on rhnErrataCloned
for each row
begin
	:new.modified := sysdate;
end;
/
show errors;

create table rhnSGTypeVirtSubLevel (
    server_group_type_id    number
                            constraint rhn_sgtvsl_sgtid_nn not null
                            constraint rhn_sgtvsl_sgtid_fk
                                references rhnServerGroupType(id),
    virt_sub_level_id number
                      constraint rhn_sgtvsl_vslid_nn not null
                      constraint rhn_sgtvsl_vslid_fk
                        references rhnVirtSubLevel(id),
    created           date
                      default sysdate
                      constraint rhn_sgtvsl_created_nn not null,
    modified          date
                      default sysdate
                      constraint rhn_sgtvsl_modified_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create index rhn_sgtvsl_sgtid_vslid
    on rhnSGTypeVirtSubLevel(server_group_type_id, virt_sub_level_id)
    storage ( freelists 16 )
    initrans 32;

create index rhn_sgtvsl_vslid_sgtid
    on rhnSGTypeVirtSubLevel(virt_sub_level_id, server_group_type_id)
    storage ( freelists 16 )
    initrans 32;

create table
rhnActionVirtDestroy
(
    action_id   number
                constraint rhn_avd_aid_nn not null
                constraint rhn_avd_aid_fk
                    references rhnAction(id)
                    on delete cascade,
    uuid        varchar(128)
                constraint rhn_avd_uuid_nn not null,
    created     date default(sysdate)
                constraint rhn_avd_creat_nn not null,
    modified    date default(sysdate)
                constraint rhn_avd_mod_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create unique index rhn_avd_aid_uq
    on rhnActionVirtDestroy( action_id )
    storage ( freelists 16 )
    initrans 32;

alter table rhnActionVirtDestroy add constraint rhn_avd_aid_pk
    primary key ( action_id );

create or replace trigger
rhn_avd_mod_trig
before insert or update on rhnActionVirtDestroy
for each row
begin
    :new.modified := sysdate;
end;
/
show errors;

create table
rhnActionVirtReboot
(
    action_id   number
                constraint rhn_avreboot_aid_nn not null
                constraint rhn_avreboot_aid_fk
                    references rhnAction(id)
                    on delete cascade,
    uuid        varchar(128)
                constraint rhn_avreboot_uuid_nn not null,
    created     date default(sysdate)
                constraint rhn_avreboot_creat_nn not null,
    modified    date default(sysdate)
                constraint rhn_avreboot_mod_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create unique index rhn_avreboot_aid_uq
    on rhnActionVirtReboot( action_id )
    storage ( freelists 16 )
    initrans 32;

alter table rhnActionVirtReboot add constraint rhn_avreboot_aid_pk
    primary key ( action_id );

create or replace trigger
rhn_avreboot_mod_trig
before insert or update on rhnActionVirtReboot
for each row
begin
    :new.modified := sysdate;
end;
/
show errors;

create table
rhnActionVirtRefresh
(
    action_id   number
                constraint rhn_avrefresh_aid_nn not null
                constraint rhn_avrefresh_aid_fk
                    references rhnAction(id)
                    on delete cascade,
    created     date default(sysdate)
                constraint rhn_avrefresh_creat_nn not null,
    modified    date default(sysdate)
                constraint rhn_avrefresh_mod_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create unique index rhn_avrefresh_aid_uq
    on rhnActionVirtRefresh( action_id )
    initrans 32;

alter table rhnActionVirtRefresh add constraint rhn_avrefresh_aid_pk
    primary key ( action_id );

create or replace trigger
rhn_avrefresh_mod_trig
before insert or update on rhnActionVirtRefresh
for each row
begin
    :new.modified := sysdate;
end;
/
show errors;

create table
rhnActionVirtResume
(
    action_id   number
                constraint rhn_avresume_aid_nn not null
                constraint rhn_avresume_aid_fk
                    references rhnAction( id )
                    on delete cascade,
    uuid        varchar(128)
                constraint rhn_avresume_uuid_nn not null,
    created     date default(sysdate)
                constraint rhn_avresume_creat_nn not null,
    modified    date default(sysdate)
                constraint rhn_avresume_mod_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create unique index rhn_avresume_aid_uq
    on rhnActionVirtResume( action_id )
    storage ( freelists 16 )
    initrans 32;

alter table rhnActionVirtResume add constraint rhn_avresume_aid_pk
    primary key ( action_id );

create or replace trigger
rhn_avresume_mod_trig
before insert or update on rhnActionVirtResume
for each row
begin
    :new.modified := sysdate;
end;
/
show errors;

create table
rhnActionVirtSchedulePoller
(
    action_id   number
                constraint rhn_avsp_aid_nn not null
                constraint rhn_avsp_aid_fk
                    references rhnAction(id)
                    on delete cascade,
    minute      number,
    hour        number,
    dom         number,
    month       number,
    dow         number,
    created     date default(sysdate)
                constraint rhn_avsp_creat_nn not null,
    modified    date default(sysdate)
                constraint rhn_avsp_mod_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create unique index rhn_avsp_aid_uq
    on rhnActionVirtSchedulePoller( action_id )
    storage ( freelists 16 )
    initrans 32;

alter table rhnActionVirtSchedulePoller add constraint rhn_avsp_aid_pk
    primary key ( action_id );

create or replace trigger
rhn_avsp_mod_trig
before insert or update on rhnActionVirtSchedulePoller
for each row
begin
    :new.modified := sysdate;
end;
/
show errors;

create table
rhnActionVirtSetMemory
(
    action_id   number
                constraint rhn_avsm_aid_nn not null
                constraint rhn_avsm_aid_fk
                    references rhnAction(id)
                    on delete cascade,
    uuid        varchar(128)
                constraint rhn_avsm_uuid_nn not null,
    memory      number
                constraint rhn_avsm_mem_nn not null,
    created     date default(sysdate)
                constraint rhn_avsm_creat_nn not null,
    modified    date default(sysdate)
                constraint rhn_avsm_mod_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create unique index rhn_avsm_aid_uq
    on rhnActionVirtSetMemory( action_id )
    storage ( freelists 16 )
    initrans 32;

alter table rhnActionVirtSetMemory add constraint rhn_avsm_aid_pk
    primary key ( action_id );

create or replace trigger
rhn_avsm_mod_trig
before insert or update on rhnActionVirtSetMemory
for each row
begin
    :new.modified := sysdate;
end;
/
show errors;

create table
rhnActionVirtShutdown
(
    action_id   number
                constraint rhn_avshutdown_aid_nn not null
                constraint rhn_avshutdown_aid_fk
                    references rhnAction(id)
                    on delete cascade,
    uuid        varchar(128)
                constraint rhn_avshutdown_uuid_nn not null,
    created     date default(sysdate)
                constraint rhn_avshutdown_creat_nn not null,
    modified    date default(sysdate)
                constraint rhn_avshutdown_mod_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create unique index rhn_avshutdown_aid_uq
    on rhnActionVirtShutdown( action_id )
    storage ( freelists 16 )
    initrans 32;

alter table rhnActionVirtShutdown add constraint rhn_avshutdown_aid_pk
    primary key ( action_id );

create or replace trigger
rhn_avshutdown_mod_trig
before insert or update on rhnActionVirtShutdown
for each row
begin
    :new.modified := sysdate;
end;
/
show errors;

create table
rhnActionVirtVcpu
(
    action_id   number
                constraint rhn_avcpu_aid_nn not null
                constraint rhn_avcpu_aid_fk
                    references rhnAction(id)
                    on delete cascade,
    uuid        varchar(128)
                constraint rhn_avcpu_uuid_nn not null,
    vcpu      number
                constraint rhn_avcpu_vcpus_nn not null,
    created     date default(sysdate)
                constraint rhn_avcpu_creat_nn not null,
    modified    date default(sysdate)
                constraint rhn_avcpu_mod_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create unique index rhn_avcpu_aid_uq
    on rhnActionVirtVcpu( action_id )
    storage ( freelists 16 )
    initrans 32;

alter table rhnActionVirtVcpu add constraint rhn_avcpu_aid_pk
    primary key ( action_id );

create or replace trigger
rhn_avcpu_mod_trig
before insert or update on rhnActionVirtVcpu
for each row
begin
    :new.modified := sysdate;
end;
/
show errors;

create table
rhnErrataClonedTmp
(
	original_id		number
				constraint rhn_eclonedtmp_feid_nn not null
				constraint rhn_eclonedtmp_feid_fk
					references rhnErrata(id)
					on delete cascade,
	id		                                                                number
				constraint rhn_eclonedtmp_teid_nn not null
				constraint rhn_eclonedtmp_teid_fk
					references rhnErrataTmp(id)
					on delete cascade,
	created			date default(sysdate)
				constraint rhn_eclonedtmp_created_nn not null,
	modified		date default(sysdate)
				constraint rhn_eclonedtmp_modified_nn not null
)
	storage ( freelists 16 )
	initrans 32;

create index rhn_eclonedtmp_feid_teid_idx
	on rhnErrataClonedTmp ( original_id, id )
	storage ( freelists 16 )
	initrans 32;
alter table rhnErrataClonedTmp add constraint rhn_eclonedtmp_feid_teid_uq
	unique ( original_id, id );
alter table rhnErrataClonedTmp add constraint rhn_eclonedtmp_id_pk
        primary key ( id );

create index rhn_eclonedtmp_teid_feid_idx
	on rhnErrataClonedTmp ( id, original_id )
	storage ( freelists 16 )
	initrans 32;

create or replace trigger
rhn_eclonedtmp_mod_trig
before insert or update on rhnErrataClonedTmp
for each row
begin
	:new.modified := sysdate;
end;
/
show errors;

create table
rhnKickstartChildChannel
(
        channel_id   number
                         constraint rhn_ks_cc_cid_nn not null
                         constraint rhn_ks_cc_cid_fk
                                references rhnChannel(id),
        ksdata_id number
                        constraint rhn_ks_cc_ksd_nn not null
                        constraint rhn_ks_cc_ksd_fk
                                references rhnKSData(id)
                                on delete cascade,
        created         date default(sysdate)
                        constraint rhn_ks_cc_cre_nn not null,
        modified        date default(sysdate)
                        constraint rhn_ks_cc_mod_nn not null
)
        storage( freelists 16 )
        initrans 32;

create unique index rhn_ks_cc_uq
        on rhnKickstartChildChannel(channel_id, ksdata_id)
        storage( freelists 16 )
        initrans 32;

create or replace trigger
rhn_ks_cc_mod_trig
before insert or update on rhnKickstartChildChannel
for each row
begin
        :new.modified := sysdate;
end;
/
show errors;

create table
rhnVirtualInstance
(
	id			number
				constraint rhn_vi_id_nn not null
				constraint rhn_vi_id_pk primary key,
	host_system_id		number
				constraint rhn_vi_hsi_fk
					references rhnServer(id),
	virtual_system_id	number
				constraint rhn_vi_vsi_fk
					references rhnServer(id),
	uuid			varchar2(128),
        confirmed               number(1) default 1
                                constraint rhn_vi_c_nn not null,
	created			date default (sysdate)
				constraint rhn_vi_created_nn not null,
	modified		date default (sysdate)
				constraint rhn_vi_modified_nn not null
)
	storage ( freelists 16 )
	initrans 32;

create index rhn_vi_hsid_vsid_idx
	on rhnVirtualInstance(host_system_id, virtual_system_id)
	storage ( freelists 16 )
	initrans 32;

create index rhn_vi_vsid_hsid_idx
	on rhnVirtualInstance(virtual_system_id, host_system_id)
	storage ( freelists 16 )
	initrans 32;

create or replace trigger
rhn_virtinst_mod_trig
before insert or update on rhnVirtualInstance
for each row
begin
        :new.modified := sysdate;
end;
/
show errors;

create table
rhnVirtualInstanceEventLog
(
	id			number
				constraint rhn_viel_id_nn not null
				constraint rhn_viel_id_pk primary key,
	virtual_instance_id	number
				constraint rhn_viel_vii_fk
					references rhnVirtualInstance(id)
					on delete cascade,
	event_type		number
				constraint rhn_viel_et_nn not null
				constraint rhn_viel_et_fk
					references
					rhnVirtualInstanceEventType(id),
-- Either this:
	event_metadata		varchar2(4000),
-- Or these 10 (10, so far!  Yuck!) columns:
	old_state		number
				constraint rhn_viel_old_state_nn not null
				constraint rhn_viel_old_state_fk
					   references rhnVirtualInstanceState(id),
	new_state               number
				constraint rhn_viel_new_state_nn not null
				constraint rhn_viel_new_state_fk
					   references rhnVirtualInstanceState(id),
	old_memory_size_k	number,
	new_memory_size_k	number,
	old_vcpus		number,
	new_vcpus		number,
	old_host_system_id	number,
	new_host_system_id	number,
	old_host_system_name	varchar2(128),
	new_host_system_name	varchar2(128),
--
	local_timestamp		date
				constraint rhn_viel_lt_nn not null,
	created			date default (sysdate)
				constraint rhn_viel_created_nn not null,
	modified		date default (sysdate)
				constraint rhn_viel_modified_nn not null
)
	storage ( freelists 16 )
	initrans 32;

create index rhn_viel_vii_idx
	on rhnVirtualInstanceEventLog(virtual_instance_id)
	storage ( freelists 16 )
	initrans 32;

create table
rhnVirtualInstanceInstallLog
(
    id                 number
                           constraint rhn_viil_id_nn not null
                           constraint rhn_viil_id_pk primary key,
    log_message        varchar2(4000)
                           constraint rhn_viil_lm_nn not null,
    ks_session_id      number
                           constraint rhn_viil_ks_sid_fk
                               references rhnKickstartSession(id)
                               on delete cascade,
    created            date default (sysdate)
                           constraint rhn_viil_created_nn not null,
    modified           date default (sysdate)
                           constraint rhn_viil_modified_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create table
rhnActionKickstartGuest
(
	id			number
				constraint rhn_actionks_xenguest_id_nn not null,
	action_id		number
				constraint rhn_actionks_xenguest_aid_nn not null
				constraint rhn_actionks_xenguest_aid_fk
					references rhnAction(id)
					on delete cascade,
	append_string		varchar2(1024),
        ks_session_id           number
                                constraint rhn_actionks_xenguest_ksid_fk
                                        references rhnKickstartSession(id)
                                        on delete cascade,
	kstree_id		number
				constraint rhn_actionks_xenguest_kstid_nn not null
				constraint rhn_actionks_xenguest_kstid_fk
					references rhnKickstartableTree(id)
					on delete cascade,
	guest_name		varchar2(256),
	mem_kb			number,
	vcpus			number,
	disk_gb			number,
	created			date default(sysdate)
				constraint rhn_actionks_xenguest_creat_nn not null,
	modified		date default(sysdate)
				constraint rhn_actionks_xenguest_mod_nn not null
)
	storage ( freelists 16 )
	initrans 32;

create unique index rhn_actionks_xenguest_aid_uq
	on rhnActionKickstartGuest( action_id )
	storage ( freelists 16 )
	initrans 32;

create index rhn_actionks_xenguest_id_idx
	on rhnActionKickstartGuest( id )
	storage ( freelists 16 )
	initrans 32;

alter table rhnActionKickstartGuest add constraint rhn_actionks_xenguest_id_pk
	primary key ( id );

create or replace trigger
rhn_actionks_xenguest_mod_trig
before insert or update on rhnActionKickstartGuest
for each row
begin
	:new.modified := sysdate;
end;
/
show errors;

create table
rhnActionVirtSuspend
(
    action_id   number
                constraint rhn_avsuspend_aid_nn not null
                constraint rhn_avsuspend_aid_fk
                    references rhnAction(id)
                    on delete cascade,
    uuid        varchar(128)
                constraint rhn_avsuspend_uuid_nn not null,
    created     date default(sysdate)
                constraint rhn_avsuspend_creat_nn not null,
    modified    date default(sysdate)
                constraint rhn_avsuspend_mod_nn not null
)
    storage ( freelists 16 )
    initrans 32;

create unique index rhn_avsuspend_aid_uq
    on rhnActionVirtSuspend( action_id )
    storage ( freelists 16 )
    initrans 32;

alter table rhnActionVirtSuspend add constraint rhn_avsuspend_aid_pk
    primary key ( action_id );

create or replace trigger
rhn_avsuspend_mod_trig
before insert or update on rhnActionVirtSuspend
for each row
begin
    :new.modified := sysdate;
end;
/
show errors;

create table
rhnVirtualInstanceInfo
(
	name                    varchar(128),
	instance_id		number
				constraint rhn_vii_viid_nn not null
				constraint rhn_vii_viid_fk
					references rhnVirtualInstance(id)
					on delete cascade,
	instance_type		number
				constraint rhn_vii_it_nn not null
				constraint rhn_vii_it_fk
					references rhnVirtualInstanceType(id),
	memory_size_k		number,
	vcpus			number,
	state			number
				constraint rhn_vii_state_nn not null
				constraint rhn_vii_state_fk
					   references rhnVirtualInstanceState(id),
	created			date default (sysdate)
				constraint rhn_vii_created_nn not null,
	modified		date default (sysdate)
				constraint rhn_vii_modified_nn not null
)
	storage ( freelists 16 )
	initrans 32;

create unique index rhn_vii_viid_uq
	on rhnVirtualInstanceInfo(instance_id)
	storage ( freelists 16 )
	initrans 32;

CREATE UNIQUE INDEX RHN_CONFCHAN_OID_LABEL_TYPE_UQ ON RHNCONFIGCHANNEL
(ORG_ID, LABEL, CONFCHAN_TYPE_ID)
LOGGING
NOPARALLEL;

--
-- Alter tables
--
ALTER TABLE RHNERRATA ADD (SEVERITY_ID  NUMBER);
ALTER TABLE RHNERRATAFILE MODIFY(FILENAME VARCHAR2(1024 BYTE));
ALTER TABLE RHNKICKSTARTSESSION ADD (HOST_SERVER_ID  NUMBER);
ALTER TABLE RHNKICKSTARTSESSION ADD (CLIENT_IP  VARCHAR2(15 BYTE));
ALTER TABLE RHNCONFIGREVISION ADD (CHANGED_BY_ID  NUMBER DEFAULT null);
ALTER TABLE RHNERRATA ADD CONSTRAINT RHN_ERRATA_SEVID_FK FOREIGN KEY (SEVERITY_ID)
    REFERENCES RHNERRATASEVERITY (ID);
ALTER TABLE RHNKICKSTARTSESSION ADD CONSTRAINT RHN_KS_SESSION_HSID_FK FOREIGN KEY (HOST_SERVER_ID)
    REFERENCES RHNSERVER (ID)
    ON DELETE CASCADE;
ALTER TABLE RHNCONFIGREVISION ADD CONSTRAINT RHN_CONFREVISION_CID_FK FOREIGN KEY (CHANGED_BY_ID)
    REFERENCES WEB_CONTACT (ID);
CREATE INDEX RHN_KS_SESSION_HSID_IDX ON RHNKICKSTARTSESSION
(HOST_SERVER_ID)
LOGGING
NOPARALLEL;
ALTER TABLE RHNCPU MODIFY(FLAGS VARCHAR2(2048 BYTE));

--
-- virtualization_type
--
ALTER TABLE RHNKICKSTARTSESSION ADD (VIRTUALIZATION_TYPE  NUMBER);
ALTER TABLE RHNKICKSTARTDEFAULTS ADD (VIRTUALIZATION_TYPE  NUMBER);


ALTER TABLE RHNKICKSTARTDEFAULTS ADD CONSTRAINT RHN_KSD_KVT_FK FOREIGN KEY (VIRTUALIZATION_TYPE)
    REFERENCES RHNKICKSTARTVIRTUALIZATIONTYPE (ID)
    ON DELETE SET NULL;

ALTER TABLE RHNKICKSTARTSESSION ADD CONSTRAINT RHN_KSS_KVT_FK FOREIGN KEY (VIRTUALIZATION_TYPE)
    REFERENCES RHNKICKSTARTVIRTUALIZATIONTYPE (ID)
    ON DELETE SET NULL;

alter table rhnSatelliteInfo add (new_cert blob);
declare
    cursor certs is
    select server_id, cert
    from rhnSatelliteInfo;
    blob_cert BLOB;
begin
    for cert in certs loop
        blob_cert := utl_raw.cast_to_raw(cert.cert);
        update rhnSatelliteInfo
        set new_cert = blob_cert
        where server_id = cert.server_id;
    end loop;
end;
/
commit;
alter table rhnSatelliteInfo drop (cert);
alter table rhnSatelliteInfo rename column new_cert to cert;
alter table rhnSatelliteInfo modify
(cert constraint rhn_satellite_info_cert_nn not null);

--
-- Views
--
CREATE OR REPLACE FORCE VIEW RHNSERVERENTITLEMENTPHYSICAL
(SERVER_ID, SERVER_GROUP_ID, SERVER_GROUP_TYPE_ID, LABEL, PERMANENT,
 IS_BASE, MODIFIED)
AS
select
   distinct
   sgm.server_id,
   sg.id,
   sgt.id,
   sgt.label,
   sgt.permanent,
   sgt.is_base,
   sgm.modified
from
   rhnServerGroupType sgt,
   rhnServerGroup sg,
   rhnServerGroupMembers sgm
where
   sg.id = sgm.server_group_id
   and sg.group_type = sgt.id
   and not exists (
        select 1
        from
            rhnServerGroup sg2,
            rhnServerGroupMembers sgm2,
            rhnVirtualInstance vi
        where
            vi.virtual_system_id = sgm.server_id
            and vi.host_system_id = sgm2.server_id
            and sgm2.server_group_id = sg2.id
            and sg2.group_type = sg.group_type
            and exists (
                select 1
                from
                    rhnServerGroupType sgt3,
                    rhnServerGroup sg3,
                    rhnServerGroupMembers sgm3
                where
                    sgm3.server_id = sgm2.server_id
                    and sgm3.server_group_id = sg3.id
                    and sg3.group_type = sgt3.id
                    and sgt3.label in ('virtualization_host',
                                       'virtualization_host_platform')
                )
        );
show errors;

CREATE OR REPLACE FORCE VIEW RHNSERVERENTITLEMENTVIRTUAL
(SERVER_ID, SERVER_GROUP_ID, SERVER_GROUP_TYPE_ID, LABEL, PERMANENT,
 IS_BASE)
AS
select
   distinct
   sgm.server_id,
   sg.id,
   sgt.id,
   sgt.label,
   sgt.permanent,
   sgt.is_base
from
   rhnServerGroupType sgt,
   rhnServerGroup sg,
   rhnServerGroupMembers sgm
where
   sg.id = sgm.server_group_id
   and sg.group_type = sgt.id
   and exists (
        select 1
        from
            rhnServerGroup sg2,
            rhnServerGroupMembers sgm2,
            rhnVirtualInstance vi
        where
            vi.virtual_system_id = sgm.server_id
            and vi.host_system_id = sgm2.server_id
            and sgm2.server_group_id = sg2.id
            and sg2.group_type = sg.group_type
            and exists (
                select 1
                from
                    rhnServerGroupType sgt3,
                    rhnServerGroup sg3,
                    rhnServerGroupMembers sgm3
                where
                    sgm3.server_id = sgm2.server_id
                    and sgm3.server_group_id = sg3.id
                    and sg3.group_type = sgt3.id
                    and sgt3.label in ('virtualization_host',
                                       'virtualization_host_platform')
                )
        );
show errors;

CREATE OR REPLACE FORCE VIEW RHNCHANNELFAMILYSERVERPHYSICAL
(CUSTOMER_ID, CHANNEL_FAMILY_ID, SERVER_ID, CREATED, MODIFIED)
AS
select	rs.org_id			customer_id,
		rcfm.channel_family_id		channel_family_id,
		rsc.server_id			server_id,
		rsc.created			created,
		rsc.modified			modified
	from	rhnChannelFamilyMembers		rcfm,
		rhnChannelFamily		rcf,
		rhnServerChannel		rsc,
		rhnServer			rs
	where
		rcfm.channel_id = rsc.channel_id
		and rcfm.channel_family_id = rcf.id
		and rsc.server_id = rs.id
        and not exists (
                select 1
                from
                    rhnChannelFamilyVirtSubLevel cfvsl,
                    rhnSGTypeVirtSubLevel sgtvsl,
                    rhnServerEntitlementView sev,
                    rhnVirtualInstance vi
                where
                    vi.virtual_system_id = rs.id
                    and vi.host_system_id = sev.server_id
                    and sev.label in ('virtualization_host',
                                      'virtualization_host_platform')
                    and sev.server_group_type_id = sgtvsl.server_group_type_id
                    and sgtvsl.virt_sub_level_id = cfvsl.virt_sub_level_id
                    and cfvsl.channel_family_id = rcf.id
                );
show errors;

CREATE OR REPLACE FORCE VIEW RHNCHANNELFAMILYSERVERVIRTUAL
(CUSTOMER_ID, CHANNEL_FAMILY_ID, SERVER_ID, CREATED, MODIFIED)
AS
select	rs.org_id			customer_id,
		rcfm.channel_family_id		channel_family_id,
		rsc.server_id			server_id,
		rsc.created			created,
		rsc.modified			modified
	from	rhnChannelFamilyMembers		rcfm,
		rhnChannelFamily		rcf,
		rhnServerChannel		rsc,
		rhnServer			rs
	where
		rcfm.channel_id = rsc.channel_id
		and rcfm.channel_family_id = rcf.id
		and rsc.server_id = rs.id
        and exists (
                select 1
                from
                    rhnChannelFamilyVirtSubLevel cfvsl,
                    rhnSGTypeVirtSubLevel sgtvsl,
                    rhnServerEntitlementView sev,
                    rhnVirtualInstance vi
                where
                    vi.virtual_system_id = rs.id
                    and vi.host_system_id = sev.server_id
                    and sev.label in ('virtualization_host',
                                      'virtualization_host_platform')
                    and sev.server_group_type_id = sgtvsl.server_group_type_id
                    and sgtvsl.virt_sub_level_id = cfvsl.virt_sub_level_id
                    and cfvsl.channel_family_id = rcf.id
                );
show errors;

--
-- PL/SQL
--
CREATE OR REPLACE package body
rhn_cache
is
	body_version varchar2(100) := '$Id: rhn_cache.pkb 105067 2006-10-30 21:00:46Z jslagle $';
	procedure update_perms_for_server(
		server_id_in in number
	) is
	begin
		delete from rhnUserServerPerms where server_id = server_id_in;
		insert into rhnUserServerPerms(user_id, server_id) (
				select	distinct user_id, server_id_in
				from	rhnUserServerPermsDupes
				where	server_id = server_id_in
			);
	end update_perms_for_server;
	procedure update_perms_for_user(
		user_id_in in number
	) is
	begin
        delete from rhnUserServerPerms
        where user_id = user_id_in
            and server_id in
            (select server_id
             from rhnUserServerPerms
             where user_id = user_id_in
             minus
             select server_id
             from rhnUserServerPermsDupes uspd
             where uspd.user_id = user_id_in);
        insert into rhnUserServerPerms (user_id, server_id)
        select distinct user_id_in, server_id
        from rhnUserServerPermsDupes uspd
        where uspd.user_id = user_id_in
            and not exists
            (select 1
             from rhnUserServerPerms usp
             where usp.user_id = user_id_in
                 and usp.server_id = uspd.server_id);
	end update_perms_for_user;
	procedure update_perms_for_server_group(
		server_group_id_in in number
	) is
		cursor users is
			select	usgp.user_id id
			from	rhnUserServerGroupPerms usgp
			where	usgp.server_group_id = server_group_id_in
				and not exists (
					select	1
					from	rhnUserGroup ug,
							rhnUserGroupMembers ugm,
							rhnServerGroup sg,
							rhnUserGroupType ugt
					where	ugt.label = 'org_admin'
						and sg.id = server_group_id_in
						and ugm.user_id = usgp.user_id
						and ug.org_id = sg.org_id
						and ugm.user_group_id = ug.id
					);
	begin
		for u in users loop
			update_perms_for_user(u.id);
		end loop;
	end update_perms_for_server_group;
end rhn_cache;
/
SHOW ERRORS;


CREATE OR REPLACE
PACKAGE rhn_channel
IS
	version varchar2(100) := '$Id: rhn_channel.pks 111885 2007-02-25 19:38:13Z jslagle $';

    CURSOR server_base_subscriptions(server_id_in NUMBER) IS
	   SELECT C.id
	     FROM rhnChannel C, rhnServerChannel SC
	    WHERE C.id = SC.channel_id
	      AND SC.server_id = server_id_in
	      AND C.parent_channel IS NULL;

    CURSOR check_server_subscription(server_id_in NUMBER, channel_id_in NUMBER) IS
           SELECT channel_id
	     FROM rhnServerChannel
	    WHERE server_id = server_id_in
	      AND channel_id = channel_id_in;

    CURSOR check_server_parent_membership(server_id_in NUMBER, channel_id_in NUMBER) IS
	   SELECT C.id
	     FROM rhnChannel C, rhnServerChannel SC
	    WHERE C.parent_channel = channel_id_in
	      AND C.id = SC.channel_id
	      AND SC.server_id = server_id_in;

    CURSOR channel_family_perm_cursor(channel_family_id_in NUMBER, org_id_in NUMBER) IS
           SELECT *
	     FROM rhnOrgChannelFamilyPermissions
	    WHERE channel_family_id = channel_family_id_in
	      AND org_id = org_id_in;

    PROCEDURE license_consent(channel_id_in IN NUMBER, user_id_in IN NUMBER, server_id_in IN NUMBER);
    FUNCTION get_license_path(channel_id_in IN NUMBER) RETURN VARCHAR2;

    PROCEDURE unsubscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, unsubscribe_children_in number := 0,
                                 deleting_server in number := 0 );
    PROCEDURE subscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, user_id_in number := null);

    function can_server_consume_virt_channl(
        server_id_in IN NUMBER,
        family_id_in in number)
    return number;

    FUNCTION guess_server_base(server_id_in IN NUMBER) RETURN NUMBER;

    FUNCTION base_channel_for_release_arch(release_in in varchar2,
	server_arch_in in varchar2, org_id_in in number := -1,
	user_id_in in number := null) RETURN number;

    FUNCTION base_channel_rel_archid(release_in in varchar2,
	server_arch_id_in in number, org_id_in in number := -1,
	user_id_in in number := null) RETURN number;

    FUNCTION channel_priority(channel_id_in in number) RETURN number;

    PROCEDURE bulk_subscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER);
    PROCEDURE bulk_unsubscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER);

    PROCEDURE bulk_server_base_change(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER);
    procedure bulk_server_basechange_from(
	set_label_in in varchar2,
	set_uid_in in number,
	old_channel_id_in in number,
	new_channel_id_in in number);

    procedure bulk_guess_server_base(
	set_label_in in varchar2,
	set_uid_in in number);

    procedure bulk_guess_server_base_from(
	set_label_in in varchar2,
	set_uid_in in number,
	channel_id_in in number);

    PROCEDURE clear_subscriptions(server_id_in IN NUMBER, deleting_server in number := 0 );

    FUNCTION available_family_subscriptions(channel_family_id_in IN NUMBER, org_id_in IN NUMBER) RETURN NUMBER;

    function channel_family_current_members(channel_family_id_in IN NUMBER,
                                            org_id_in IN NUMBER)
    return number;

    PROCEDURE update_family_counts(channel_family_id_in IN NUMBER, org_id_in IN NUMBER);
    FUNCTION family_for_channel(channel_id_in IN NUMBER) RETURN NUMBER;

    FUNCTION available_chan_subscriptions(channel_id_in IN NUMBER, org_id_in IN NUMBER) RETURN NUMBER;

    procedure entitle_customer(customer_id_in in number, channel_family_id_in in number, quantity_in in number);
    procedure set_family_maxmembers(customer_id_in in number, channel_family_id_in in number, quantity_in in number);
    procedure unsubscribe_server_from_family(server_id_in in number, channel_family_id_in in number);

    procedure delete_server_channels(server_id_in in number);
    procedure refresh_newest_package(channel_id_in in number, caller_in in varchar2 := '(unknown)');

    function get_org_id(channel_id_in in number) return number;
    PRAGMA RESTRICT_REFERENCES(get_org_id, WNDS, RNPS, WNPS);

    function get_org_access(channel_id_in in number, org_id_in in number) return number;
    PRAGMA RESTRICT_REFERENCES(get_org_access, WNDS, RNPS, WNPS);

    function get_cfam_org_access(cfam_id_in in number, org_id_in in number) return number;

    function user_role_check_debug(channel_id_in in number, user_id_in in number, role_in in varchar2, reason_out out varchar2)
	RETURN NUMBER;
    PRAGMA RESTRICT_REFERENCES(user_role_check_debug, WNDS, RNPS, WNPS);

    function user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
	RETURN NUMBER;
    PRAGMA RESTRICT_REFERENCES(user_role_check, WNDS, RNPS, WNPS);

    function loose_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
	RETURN NUMBER;
    PRAGMA RESTRICT_REFERENCES(loose_user_role_check, WNDS, RNPS, WNPS);

    function direct_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
	RETURN NUMBER;
    PRAGMA RESTRICT_REFERENCES(direct_user_role_check, WNDS, RNPS, WNPS);

    function org_channel_setting(channel_id_in in number, org_id_in in number, setting_in in varchar2)
	RETURN NUMBER;

    PROCEDURE update_channel ( channel_id_in in number, invalidate_ss in number := 0,
                               date_to_use in date := sysdate );

    PROCEDURE  update_channels_by_package ( package_id_in in number, date_to_use in date := sysdate );

     PROCEDURE update_channels_by_errata ( errata_id_in number, date_to_use in date := sysdate );


    PRAGMA RESTRICT_REFERENCES(org_channel_setting, WNDS, RNPS, WNPS);

END rhn_channel;
/
SHOW ERRORS;

CREATE OR REPLACE PACKAGE BODY rhn_channel
IS
	body_version varchar2(100) := '$Id: rhn_channel.pkb 115306 2007-04-23 21:18:00Z jslagle $';
	cursor	base_channel_cursor(
		release_in in varchar2,
		server_arch_id_in in number,
		org_id_in in number
	) return rhnChannel%ROWTYPE is
		select distinct c.*
		from	rhnDistChannelMap			dcm,
				rhnServerChannelArchCompat	scac,
				rhnChannel					c,
				rhnChannelPermissions		cp
		where	cp.org_id = org_id_in
			and cp.channel_id = c.id
			and c.parent_channel is null
			and c.id = dcm.channel_id
			and c.channel_arch_id = dcm.channel_arch_id
			and dcm.release = release_in
			and scac.server_arch_id = server_arch_id_in
			and scac.channel_arch_id = c.channel_arch_id;
    FUNCTION get_license_path(channel_id_in IN NUMBER)
    RETURN VARCHAR2
    IS
	license_val VARCHAR2(1000);
    BEGIN
	SELECT CFL.license_path INTO license_val
	  FROM rhnChannelFamilyLicense CFL, rhnChannelFamilyMembers CFM
	 WHERE CFM.channel_id = channel_id_in
	   AND CFM.channel_family_id = CFL.channel_family_id;
	RETURN license_val;
    EXCEPTION
	WHEN NO_DATA_FOUND
	    THEN
	    RETURN NULL;
    END get_license_path;
    PROCEDURE license_consent(channel_id_in IN NUMBER, user_id_in IN NUMBER, server_id_in IN NUMBER)
    IS
	channel_family_id_val NUMBER;
    BEGIN
	channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
	IF channel_family_id_val IS NULL
	THEN
	    rhn_exception.raise_exception('channel_subscribe_no_family');
	END IF;
	IF rhn_channel.get_license_path(channel_id_in) IS NULL
	THEN
	    rhn_exception.raise_exception('channel_consent_no_license');
	END IF;
	INSERT INTO rhnChannelFamilyLicenseConsent (channel_family_id, user_id, server_id)
	VALUES (channel_family_id_val, user_id_in, server_id_in);
    END license_consent;
    PROCEDURE subscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, user_id_in in number := null)
    IS
        channel_parent_val      rhnChannel.parent_channel%TYPE;
        parent_subscribed       BOOLEAN;
        server_has_base_chan    BOOLEAN;
	server_already_in_chan  BOOLEAN;
	channel_family_id_val   NUMBER;
	server_org_id_val       NUMBER;
	available_subscriptions NUMBER;
	consenting_user         NUMBER;
	allowed			number := 0;
    BEGIN
	if user_id_in is not null then
		allowed := rhn_channel.user_role_check(channel_id_in, user_id_in, 'subscribe');
	else
		allowed := 1;
	end if;
	if allowed = 0 then
		rhn_exception.raise_exception('no_subscribe_permissions');
	end if;
        SELECT parent_channel INTO channel_parent_val FROM rhnChannel WHERE id = channel_id_in;
        IF channel_parent_val IS NOT NULL
        THEN
            parent_subscribed := FALSE;
            FOR check_subscription IN check_server_subscription(server_id_in, channel_parent_val)
            LOOP
                parent_subscribed := TRUE;
            END LOOP check_subscription;
            IF NOT parent_subscribed
            THEN
		RETURN;
            END IF;
        ELSE
            server_has_base_chan := FALSE;
            FOR base IN server_base_subscriptions(server_id_in)
            LOOP
                server_has_base_chan := TRUE;
            END LOOP base;
            IF server_has_base_chan
            THEN
                rhn_exception.raise_exception('channel_server_one_base');
            END IF;
        END IF;
        FOR check_subscription IN check_server_subscription(server_id_in, channel_id_in)
        LOOP
            server_already_in_chan := TRUE;
        END LOOP check_subscription;
        IF server_already_in_chan
        THEN
	    RETURN;
        END IF;
	channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
	IF channel_family_id_val IS NULL
	THEN
	    rhn_exception.raise_exception('channel_subscribe_no_family');
	END IF;
	SELECT org_id INTO server_org_id_val
	  FROM rhnServer
	 WHERE id = server_id_in;
	available_subscriptions := rhn_channel.available_family_subscriptions(channel_family_id_val, server_org_id_val);
	IF available_subscriptions IS NULL OR
       available_subscriptions > 0 or
       can_server_consume_virt_channl(server_id_in, channel_family_id_val) = 1
	THEN
	    IF rhn_channel.get_license_path(channel_id_in) IS NOT NULL
	    THEN
		BEGIN
		SELECT user_id INTO consenting_user
		  FROM rhnChannelFamilyLicenseConsent
		 WHERE channel_family_id = channel_family_id_val
		   AND server_id = server_id_in;
		EXCEPTION
		    WHEN NO_DATA_FOUND
			THEN
			    rhn_exception.raise_exception('channel_subscribe_no_consent');
		END;
	    END IF;
	    insert into rhnServerHistory (id,server_id,summary,details) (
		select	rhn_event_id_seq.nextval,
			server_id_in,
			'subscribed to channel ' || SUBSTR(c.label, 0, 106),
			c.label
		from	rhnChannel c
		where	c.id = channel_id_in
	    );
	    UPDATE rhnServer SET channels_changed = sysdate WHERE id = server_id_in;
            INSERT INTO rhnServerChannel (server_id, channel_id) VALUES (server_id_in, channel_id_in);
	    rhn_channel.update_family_counts(channel_family_id_val, server_org_id_val);
	    queue_server(server_id_in, immediate_in);
	ELSE
	    rhn_exception.raise_exception('channel_family_no_subscriptions');
	END IF;
    END subscribe_server;
    function can_server_consume_virt_channl(
        server_id_in in number,
        family_id_in in number )
    return number
    is
        cursor server_virt_families is
            select vi.virtual_system_id, cfvsl.channel_family_id
            from
                rhnChannelFamilyVirtSubLevel cfvsl,
                rhnSGTypeVirtSubLevel sgtvsl,
                rhnVirtualInstance vi
            where
                vi.virtual_system_id = server_id_in
                and sgtvsl.virt_sub_level_id = cfvsl.virt_sub_level_id
                and cfvsl.channel_family_id = family_id_in
                and exists (
                    select 1
                    from rhnServerEntitlementView sev
                    where vi.host_system_id = sev.server_id
                    and sev.server_group_type_id = sgtvsl.server_group_type_id );
    begin
        for server_virt_family in server_virt_families loop
            return 1;
        end loop;
        return 0;
    end;
    PROCEDURE bulk_subscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
            rhn_channel.subscribe_server(server.element, channel_id_in, 0, set_uid_in);
        END LOOP server;
    END bulk_subscribe_server;
    PROCEDURE bulk_server_base_change(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
	    IF rhn_server.can_change_base_channel(server.element) = 1
	    THEN
                rhn_channel.clear_subscriptions(TO_NUMBER(server.element));
                rhn_channel.subscribe_server(server.element, channel_id_in, 0, set_uid_in);
            END IF;
        END LOOP server;
    END bulk_server_base_change;
    procedure bulk_server_basechange_from(
        set_label_in in varchar2,
        set_uid_in in number,
        old_channel_id_in in number,
        new_channel_id_in in number
    ) is
    cursor servers is
        select  sc.server_id id
        from    rhnChannel nc,
                rhnServerChannelArchCompat scac,
                rhnServer s,
                rhnChannel oc,
                rhnServerChannel sc,
                rhnSet st
        where   1=1
            and st.label = set_label_in
            and st.user_id = set_uid_in
            and st.element = sc.server_id
            and sc.channel_id = old_channel_id_in
            and sc.channel_id = oc.id
            and oc.parent_channel is null
            and nc.id = new_channel_id_in
            and nc.parent_channel is null
            and sc.server_id = s.id
            and s.server_arch_id = scac.server_arch_id
            and scac.channel_arch_id = nc.channel_arch_id;
    begin
        for s in servers loop
            insert into rhnSet (
                    user_id, label, element
                ) values (
                    set_uid_in,
                    set_label_in || 'basechange',
                    s.id
                );
        end loop channel;
        bulk_server_base_change(new_channel_id_in,
                                set_label_in || 'basechange',
                                set_uid_in);
        delete from rhnSet
            where   label = set_label_in||'basechange'
                and user_id = set_uid_in;
    end bulk_server_basechange_from;
    procedure bulk_guess_server_base(
	set_label_in in varchar2,
	set_uid_in in number
    ) is
	channel_id number;
    begin
	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    begin
		if rhn_server.can_change_base_channel(server.element) = 1
		then
	            channel_id := guess_server_base(TO_NUMBER(server.element));
		    rhn_channel.clear_subscriptions(TO_NUMBER(server.element));
		    rhn_channel.subscribe_server(TO_NUMBER(server.element), channel_id, 0, set_uid_in);
		end if;
	    exception when others then
		null;
	    end;
	end loop server;
    end;
    function guess_server_base(
	server_id_in in number
    ) RETURN number is
	cursor server_cursor is
	    select s.server_arch_id, s.release, s.org_id
	      from rhnServer s
	     where s.id = server_id_in;
    begin
	for s in server_cursor loop
	    for channel in base_channel_cursor(s.release,
		s.server_arch_id, s.org_id)
	    loop
		return channel.id;
	    end loop base_channel_cursor;
        end loop server_cursor;
        return null;
    end;
    function normalize_server_arch(server_arch_in in varchar2)
    return varchar2
    is
	suffix VARCHAR2(128) := '-redhat-linux';
	suffix_len NUMBER := length(suffix);
    begin
	if server_arch_in is NULL then
	    return NULL;
	end if;
        if instr(server_arch_in, '-') > 0
	then
	    return server_arch_in;
	end if;
	return server_arch_in || suffix;
    end normalize_server_arch;
    function base_channel_for_release_arch(
	release_in in varchar2,
	server_arch_in in varchar2,
	org_id_in in number := -1,
	user_id_in in number := null
    ) return number is
	server_arch varchar2(256) := normalize_server_arch(server_arch_in);
	server_arch_id number;
    begin
	begin
	    select id
	      into server_arch_id
	      from rhnServerArch
	     where label = server_arch;
	exception
	    when no_data_found then
		rhn_exception.raise_exception('server_arch_not_found');
	end;
	return base_channel_rel_archid(release_in, server_arch_id,
	    org_id_in, user_id_in);
    end base_channel_for_release_arch;
    function base_channel_rel_archid(
	release_in in varchar2,
	server_arch_id_in in number,
	org_id_in in number := -1,
	user_id_in in number := null
    ) return number is
	denied_channel_id number := null;
	valid_org_id number := org_id_in;
	valid_user_id number := user_id_in;
	channel_subscribable number;
    begin
	if org_id_in = -1 and user_id_in is not null then
	    begin
		select org_id
		  into valid_org_id
		  from web_contact
		 where id = user_id_in;
	    exception
		when no_data_found then
		    valid_user_id := null;
		    valid_org_id := -1;
	    end;
	end if;
	for c in base_channel_cursor(release_in, server_arch_id_in, valid_org_id)
	loop
	    if valid_user_id is null then
		return c.id;
	    end if;
	    select loose_user_role_check(c.id, user_id_in, 'subscribe')
	      into channel_subscribable
	      from dual;
	    if channel_subscribable = 1 then
		return c.id;
	    end if;
	    denied_channel_id := c.id;
	end loop base_channel_fetch;
	if denied_channel_id is not null then
	    rhn_exception.raise_exception('no_subscribe_permissions');
	end if;
	return NULL;
    end base_channel_rel_archid;
    procedure bulk_guess_server_base_from(
	set_label_in in varchar2,
	set_uid_in in number,
	channel_id_in in number
    ) is
	cursor channels(server_id_in in number) is
	    select	rsc.channel_id
	    from	rhnServerChannel rsc,
			rhnChannel rc
	    where	server_id_in = rsc.server_id
			and rsc.channel_id = rc.id
			and rc.parent_channel is null;
    begin
	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    for channel in channels(server.element)
	    loop
		if channel.channel_id = channel_id_in
		then
		    insert into rhnSet (user_id, label, element) values (set_uid_in, set_label_in || 'baseguess', server.element);
		end if;
	    end loop channel;
	end loop server;
	bulk_guess_server_base(set_label_in||'baseguess',set_uid_in);
	delete from rhnSet where label = set_label_in||'baseguess' and user_id = set_uid_in;
    end;
    PROCEDURE clear_subscriptions(server_id_in IN NUMBER, deleting_server IN NUMBER := 0 )
    IS
	cursor server_channels(server_id_in in number) is
		select	s.org_id, sc.channel_id, cfm.channel_family_id
		from	rhnServer s,
			rhnServerChannel sc,
			rhnChannelFamilyMembers cfm
		where	s.id = server_id_in
			and s.id = sc.server_id
			and sc.channel_id = cfm.channel_id;
    BEGIN
	for channel in server_channels(server_id_in)
	loop
		unsubscribe_server(server_id_in, channel.channel_id, 1, 1, deleting_server);
		rhn_channel.update_family_counts(channel.channel_family_id, channel.org_id);
	end loop channel;
    END clear_subscriptions;
    PROCEDURE unsubscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, unsubscribe_children_in number := 0,
                                 deleting_server IN NUMBER := 0 )
    IS
	channel_family_id_val   NUMBER;
	server_org_id_val       NUMBER;
	available_subscriptions NUMBER;
	server_already_in_chan  BOOLEAN;
	cursor	channel_family_is_proxy(channel_family_id_in in number) is
		select	1
		from	rhnChannelFamily
		where	id = channel_family_id_in
		    and label = 'rhn-proxy';
	cursor	channel_family_is_satellite(channel_family_id_in in number) is
		select	1
		from	rhnChannelFamily
		where	id = channel_family_id_in
		    and label = 'rhn-satellite';
	cursor local_chk_server_parent_memb (
			server_id_in number,
			channel_id_in number ) is
		select	c.id
		from	rhnChannel			c,
				rhnServerChannel	sc
		where	1=1
			and c.parent_channel = channel_id_in
			and c.id = sc.channel_id
			and sc.server_id = server_id_in;
    BEGIN
        FOR child IN local_chk_server_parent_memb(server_id_in, channel_id_in)
        LOOP
	    if unsubscribe_children_in = 1 then
		unsubscribe_server(server_id_in => server_id_in,
								channel_id_in => child.id,
								immediate_in => immediate_in,
								unsubscribe_children_in => unsubscribe_children_in,
                        deleting_server => deleting_server);
	    else
		rhn_exception.raise_exception('channel_unsubscribe_child_exists');
	    end if;
        END LOOP child;
	server_already_in_chan := FALSE;
        FOR check_subscription IN check_server_subscription(server_id_in, channel_id_in)
        LOOP
            server_already_in_chan := TRUE;
        END LOOP check_subscription;
        IF NOT server_already_in_chan
        THEN
	    RETURN;
        END IF;
   if deleting_server = 0 then
      insert into rhnServerHistory (id,server_id,summary,details) (
          select  rhn_event_id_seq.nextval,
                server_id_in,
             'unsubscribed from channel ' || SUBSTR(c.label, 0, 106),
             c.label
          from    rhnChannel c
          where   c.id = channel_id_in
      );
	UPDATE rhnServer SET channels_changed = sysdate WHERE id = server_id_in;
   end if;
   DELETE FROM rhnServerChannel WHERE server_id = server_id_in AND channel_id = channel_id_in;
   if deleting_server = 0 then
	queue_server(server_id_in, immediate_in);
   end if;
	channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
	IF channel_family_id_val IS NULL
	THEN
	    rhn_exception.raise_exception('channel_unsubscribe_no_family');
	END IF;
	for ignore in channel_family_is_satellite(channel_family_id_val) loop
		delete from rhnSatelliteInfo where server_id = server_id_in;
		delete from rhnSatelliteChannelFamily where server_id = server_id_in;
	end loop;
	for ignore in channel_family_is_proxy(channel_family_id_val) loop
		delete from rhnProxyInfo where server_id = server_id_in;
	end loop;
	DELETE FROM rhnChannelFamilyLicenseConsent
	 WHERE channel_family_id = channel_family_id_val
	   AND server_id = server_id_in;
	SELECT org_id INTO server_org_id_val
	  FROM rhnServer
	 WHERE id = server_id_in;
	rhn_channel.update_family_counts(channel_family_id_val, server_org_id_val);
    END unsubscribe_server;
    PROCEDURE bulk_unsubscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
            rhn_channel.unsubscribe_server(server.element, channel_id_in, 0);
        END LOOP server;
    END bulk_unsubscribe_server;
    FUNCTION family_for_channel(channel_id_in IN NUMBER)
    RETURN NUMBER
    IS
	channel_family_id_val NUMBER;
    BEGIN
	SELECT channel_family_id INTO channel_family_id_val
	  FROM rhnChannelFamilyMembers
         WHERE channel_id = channel_id_in;
	RETURN channel_family_id_val;
    EXCEPTION
        WHEN NO_DATA_FOUND
	THEN
	    RETURN NULL;
    END family_for_channel;
    FUNCTION available_family_subscriptions(channel_family_id_in IN NUMBER, org_id_in IN NUMBER)
    RETURN NUMBER
    IS
	cfp channel_family_perm_cursor%ROWTYPE;
	current_members_val NUMBER;
	max_members_val     NUMBER;
	found               NUMBER;
    BEGIN
	IF NOT channel_family_perm_cursor%ISOPEN
	THEN
	    OPEN channel_family_perm_cursor(channel_family_id_in, org_id_in);
	END IF;
	FETCH channel_family_perm_cursor INTO cfp;
	WHILE channel_family_perm_cursor%FOUND
	LOOP
	    found := 1;
	    current_members_val := cfp.current_members;
	    max_members_val := cfp.max_members;
	    FETCH channel_family_perm_cursor INTO cfp;
	END LOOP;
	IF channel_family_perm_cursor%ISOPEN
	THEN
	    CLOSE channel_family_perm_cursor;
	END IF;
	IF found IS NULL
	THEN
	    RETURN 0;
	END IF;
	IF max_members_val IS NULL
	THEN
	    RETURN NULL;
	END IF;
	RETURN max_members_val - current_members_val;
    END available_family_subscriptions;
    function channel_family_current_members(channel_family_id_in IN NUMBER,
                                            org_id_in IN NUMBER)
    return number
    is
        current_members_count number := 0;
    begin
        select	count(sc.server_id)
        into    current_members_count
        from	rhnChannelFamilyMembers cfm,
                rhnServerChannel sc,
                rhnServer s
        where	s.org_id = org_id_in
            and s.id = sc.server_id
            and cfm.channel_family_id = channel_family_id_in
            and cfm.channel_id = sc.channel_id
            and exists (
                select 1
                from rhnChannelFamilyServerPhysical cfsp
                where cfsp.CHANNEL_FAMILY_ID = channel_family_id_in
                    and cfsp.server_id = s.id
                );
        return current_members_count;
    end;
    PROCEDURE update_family_counts(channel_family_id_in IN NUMBER,
                                   org_id_in IN NUMBER)
    IS
    BEGIN
		update rhnPrivateChannelFamily
		set current_members = (
                channel_family_current_members(channel_family_id_in, org_id_in)
		)
			where org_id = org_id_in
				and channel_family_id = channel_family_id_in;
    END update_family_counts;
    FUNCTION available_chan_subscriptions(channel_id_in IN NUMBER,
                                          org_id_in IN NUMBER)
    RETURN NUMBER
    IS
	    channel_family_id_val NUMBER;
    BEGIN
	SELECT channel_family_id INTO channel_family_id_val
	    FROM rhnChannelFamilyMembers
	    WHERE channel_id = channel_id_in;
	    RETURN rhn_channel.available_family_subscriptions(
                           channel_family_id_val, org_id_in);
    END available_chan_subscriptions;
    procedure entitle_customer(customer_id_in in number,
                               channel_family_id_in in number,
                               quantity_in in number)
    is
		cursor permissions is
			select	1
			from	rhnPrivateChannelFamily pcf
			where	pcf.org_id = customer_id_in
				and	pcf.channel_family_id = channel_family_id_in;
    begin
		for perm in permissions loop
			set_family_maxmembers(
				customer_id_in,
				channel_family_id_in,
				quantity_in
			);
			rhn_channel.update_family_counts(
				channel_family_id_in,
				customer_id_in
			);
			return;
		end loop;
		insert into rhnPrivateChannelFamily pcf (
				channel_family_id, org_id, max_members, current_members
			) values (
				channel_family_id_in, customer_id_in, quantity_in, 0
			);
    end;
    procedure set_family_maxmembers(customer_id_in in number,
                                    channel_family_id_in in number,
                                    quantity_in in number)
    is
        cursor servers is
            select  server_id from (
            select	rownum row_number, server_id, modified from (
                select  rcfsp.server_id,
                        rcfsp.modified
                from    rhnChannelFamilyServerPhysical rcfsp
                where   rcfsp.customer_id = customer_id_in
                    and rcfsp.channel_family_id = channel_family_id_in
                order by modified
            )
            where rownum > quantity_in
            );
    begin
        for server in servers loop
            rhn_channel.unsubscribe_server_from_family(server.server_id,
                                                       channel_family_id_in);
        end loop;
        update	rhnPrivateChannelFamily pcf
        set	pcf.max_members = quantity_in
        where	pcf.org_id = customer_id_in
            and pcf.channel_family_id = channel_family_id_in;
    end;
    procedure unsubscribe_server_from_family(server_id_in in number,
                                             channel_family_id_in in number)
    is
    begin
        delete
        from	rhnServerChannel rsc
        where	rsc.server_id = server_id_in
            and channel_id in (
                select	rcfm.channel_id
                from	rhnChannelFamilyMembers rcfm
                where	rcfm.channel_family_id = channel_family_id_in);
    end;
    function get_org_id(channel_id_in in number)
    return number
    is
	org_id_out number;
    begin
	select org_id into org_id_out
	    from rhnChannel
	    where id = channel_id_in;
	    return org_id_out;
    end get_org_id;
    function get_cfam_org_access(cfam_id_in in number, org_id_in in number)
    return number
    is
	cursor	families is
			select	1
			from	rhnOrgChannelFamilyPermissions cfp
			where	cfp.org_id = org_id_in;
    begin
		for family in families loop
		return 1;
		end loop;
		return 0;
    end;
    function get_org_access(channel_id_in in number, org_id_in in number)
    return number
    is
	throwaway number;
    begin
        select distinct 1 into throwaway
          from rhnChannelFamilyMembers CFM,
               rhnOrgChannelFamilyPermissions CFP
         where cfp.org_id = org_id_in
           and CFM.channel_family_id = CFP.channel_family_id
           and CFM.channel_id = channel_id_in;
        return 1;
        exception
            when no_data_found
            then
            return 0;
    end;
    function user_role_check_debug(channel_id_in in number,
                                   user_id_in in number,
                                   role_in in varchar2,
                                   reason_out out varchar2)
    return number
    is
	org_id number;
    begin
	org_id := rhn_user.get_org_id(user_id_in);
	if role_in = 'manage' and
           NVL(rhn_channel.get_org_id(channel_id_in), -1) <> org_id then
	        reason_out := 'channel_not_owned';
	       return 0;
	    end if;
	if role_in = 'subscribe' and
           rhn_channel.get_org_access(channel_id_in, org_id) = 0 then
	        reason_out := 'channel_not_available';
	        return 0;
	    end if;
	if rhn_user.check_role_implied(user_id_in, 'channel_admin') = 1 then
            reason_out := 'channel_admin';
            return 1;
	    end if;
	if role_in = 'subscribe'
	then
	    if rhn_channel.org_channel_setting(channel_id_in,
                       org_id,
                       'not_globally_subscribable') = 0 then
		reason_out := 'globally_subscribable';
		    return 1;
	    end if;
	end if;
	reason_out := 'direct_permission';
	return rhn_channel.direct_user_role_check(channel_id_in,
                                              user_id_in, role_in);
    end;
    function user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
	throwaway varchar2(256);
    begin
	return rhn_channel.user_role_check_debug(channel_id_in, user_id_in, role_in, throwaway);
    end;
    function loose_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
    begin
        if user_id_in is null then
            return 1;
        end if;
        return user_role_check(channel_id_in, user_id_in, role_in);
    end loose_user_role_check;
    function direct_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
	throwaway number;
    begin
	select 1 into throwaway
	  from rhnChannelPermissionRole CPR,
	       rhnChannelPermission CP
	 where CP.user_id = user_id_in
	   and CP.channel_id = channel_id_in
	   and CPR.label = role_in
	   and CP.role_id = CPR.id;
	return 1;
    exception
	when no_data_found
	    then
	    return 0;
    end;
    function org_channel_setting(channel_id_in in number, org_id_in in number, setting_in in varchar2)
    return number
    is
	throwaway number;
    begin
	select 1 into throwaway
	  from rhnOrgChannelSettingsType OCST,
	       rhnOrgChannelSettings OCS
	 where OCS.org_id = org_id_in
	   and OCS.channel_id = channel_id_in
	   and OCST.label = setting_in
	   and OCS.setting_id = OCST.id;
	return 1;
    exception
	when no_data_found
	    then
	    return 0;
    end;
    FUNCTION channel_priority(channel_id_in IN number)
    RETURN number
    IS
         channel_name varchar2(64);
         priority number;
         end_of_life_val date;
         org_id_val number;
    BEGIN
        select name, end_of_life, org_id
        into channel_name, end_of_life_val, org_id_val
        from rhnChannel
        where id = channel_id_in;
        if end_of_life_val is not null then
          return -400;
        end if;
        if channel_name like 'Red Hat Enterprise Linux%' or channel_name like 'RHEL%' then
          priority := 1000;
          if channel_name not like '%Beta%' then
            priority := priority + 1000;
          end if;
          priority := priority +
            case
              when channel_name like '%v. 5%' then 600
              when channel_name like '%v. 4%' then 500
              when channel_name like '%v. 3%' then 400
              when channel_name like '%v. 2%' then 300
              when channel_name like '%v. 1%' then 200
              else 0
            end;
          priority := priority +
            case
              when channel_name like 'Red Hat Enterprise Linux (v. 5%' then 60
              when (channel_name like '%AS%' and channel_name not like '%Extras%') then 50
              when (channel_name like '%ES%' and channel_name not like '%Extras%') then 40
              when (channel_name like '%WS%' and channel_name not like '%Extras%') then 30
              when (channel_name like '%Desktop%' and channel_name not like '%Extras%') then 20
              when channel_name like '%Extras%' then 10
              else 0
            end;
          priority := priority +
            case
              when channel_name like '%)' then 5
              else 0
            end;
          priority := priority +
            case
              when channel_name like '%32-bit x86%' then 4
              when channel_name like '%64-bit Intel Itanium%' then 3
              when channel_name like '%64-bit AMD64/Intel EM64T%' then 2
              else 0
            end;
        elsif channel_name like 'Red Hat Desktop%' then
            priority := 900;
            if channel_name not like '%Beta%' then
               priority := priority + 50;
            end if;
          priority := priority +
            case
              when channel_name like '%v. 4%' then 40
              when channel_name like '%v. 3%' then 30
              when channel_name like '%v. 2%' then 20
              when channel_name like '%v. 1%' then 10
              else 0
            end;
          priority := priority +
            case
              when channel_name like '%32-bit x86%' then 4
              when channel_name like '%64-bit Intel Itanium%' then 3
              when channel_name like '%64-bit AMD64/Intel EM64T%' then 2
              else 0
            end;
        elsif org_id_val is not null then
          priority := 600;
        else
          priority := 500;
        end if;
      return -priority;
    end channel_priority;
    procedure delete_server_channels(server_id_in in number)
    is
    begin
	update	rhnPrivateChannelFamily
	set	current_members = current_members -1
	where	org_id in (
			select	org_id
			from	rhnServer
			where	id = server_id_in
		)
		and channel_family_id in (
			select	rcfm.channel_family_id
			from	rhnChannelFamilyMembers rcfm,
				rhnServerChannel rsc
			where	rsc.server_id = server_id_in
				and rsc.channel_id = rcfm.channel_id
                and not exists (
                    select 1
                    from
                        rhnChannelFamilyVirtSubLevel cfvsl,
                        rhnSGTypeVirtSubLevel sgtvsl,
                        rhnServerEntitlementView sev,
                        rhnVirtualInstance vi
                    where
                        vi.virtual_system_id = server_id_in
                        and vi.host_system_id = sev.server_id
                        and sev.label in ('virtualization_host',
                                          'virtualization_host_platform')
                        and sev.server_group_type_id =
                            sgtvsl.server_group_type_id
                        and sgtvsl.virt_sub_level_id = cfvsl.virt_sub_level_id
                        and cfvsl.channel_family_id = rcfm.channel_family_id
                    )
		);
    end;
	procedure refresh_newest_package(channel_id_in in number, caller_in in varchar2 := '(unknown)')
	is
	begin
		delete from rhnChannelNewestPackage where channel_id = channel_id_in;
		insert into rhnChannelNewestPackage
			( channel_id, name_id, evr_id, package_id, package_arch_id )
			(	select	channel_id,
						name_id, evr_id,
						package_id, package_arch_id
				from	rhnChannelNewestPackageView
				where	channel_id = channel_id_in
			);
                insert into rhnChannelNewestPackageAudit (channel_id, caller)
                    values (channel_id_in, caller_in);
                update rhnChannel
                    set last_modified = greatest(sysdate, last_modified + 1/86400)
                    where id = channel_id_in;
	end;
   procedure update_channel ( channel_id_in in number, invalidate_ss in number := 0,
                              date_to_use in date := sysdate )
   is
   channel_last_modified date;
   last_modified_value date;
   cursor snapshots is
   select  snapshot_id id
   from    rhnSnapshotChannel
   where   channel_id = channel_id_in;
   begin
      select last_modified
      into channel_last_modified
      from rhnChannel
      where id = channel_id_in;
      last_modified_value := date_to_use;
      if last_modified_value <= channel_last_modified then
          last_modified_value := last_modified_value + 1/86400;
      end if;
      update rhnChannel set last_modified = last_modified_value
      where id = channel_id_in;
      if invalidate_ss = 1 then
        for snapshot in snapshots loop
            update rhnSnapshot
            set invalid = lookup_snapshot_invalid_reason('channel_modified')
            where id = snapshot.id;
        end loop;
      end if;
   end update_channel;
   procedure update_channels_by_package ( package_id_in in number, date_to_use in date := sysdate )
   is
   cursor channels is
   select channel_id
   from rhnChannelPackage
   where package_id = package_id_in
   order by channel_id;
   begin
      for channel in channels loop
         rhn_channel.update_channel ( channel.channel_id, 1, date_to_use );
      end loop;
   end update_channels_by_package;
   procedure update_channels_by_errata ( errata_id_in number, date_to_use in date := sysdate )
   is
   cursor channels is
   select channel_id
   from rhnChannelErrata
   where errata_id = errata_id_in
   order by channel_id;
   begin
      for channel in channels loop
         rhn_channel.update_channel ( channel.channel_id, 0, date_to_use );
      end loop;
   end update_channels_by_errata;
END rhn_channel;
/
SHOW ERRORS;

CREATE OR REPLACE PACKAGE BODY rhn_config_channel
IS
    FUNCTION action_diff_revision_status(action_config_revision_id_in IN NUMBER)
    RETURN VARCHAR2
    IS
	failure_reason VARCHAR2(4000);
	result_is_null NUMBER;
	result_exists NUMBER;
    BEGIN
	select extant, is_null, reason
	into   result_exists, result_is_null, failure_reason
	from   (
		SELECT ACRR.action_config_revision_id extant,
		       1 is_null, -- NVL2(ACRR.result, 0, 1),
		       CFF.name reason
		  FROM rhnConfigFileFailure CFF,
		       rhnActionConfigRevisionResult ACRR,
		       rhnActionConfigRevision ACR
		 WHERE ACR.id = action_config_revision_id_in
		   AND ACR.id = ACRR.action_config_revision_id (+)
		   and acrr.result is null
		   AND ACR.failure_id = CFF.id (+)
		union all
		SELECT ACRR.action_config_revision_id extant,
		       0 is_null, -- NVL2(ACRR.result, 0, 1),
		       CFF.name reason
		  FROM rhnConfigFileFailure CFF,
		       rhnActionConfigRevisionResult ACRR,
		       rhnActionConfigRevision ACR
		 WHERE ACR.id = action_config_revision_id_in
		   AND ACR.id = ACRR.action_config_revision_id (+)
		   and acrr.result is not null
		   AND ACR.failure_id = CFF.id (+)
		);
	IF failure_reason IS NOT NULL
	THEN
	    RETURN failure_reason;
	END IF;
	IF result_exists IS NOT NULL
	THEN
	    IF result_is_null = 1
	    THEN
		RETURN 'No differences';
	    END IF;
	    RETURN 'Differences exist';
	END IF;
	RETURN NULL;
    END action_diff_revision_status;
    FUNCTION get_user_chan_access(config_channel_id_in IN NUMBER, user_id_in IN NUMBER)
    RETURN NUMBER
    IS
	server_id NUMBER;
	org_matches NUMBER;
	global_channel VARCHAR2(30);
	any_visible_servers_subscribed NUMBER;
    BEGIN
	org_matches := 0;
	BEGIN
	  SELECT 1 INTO org_matches
	    FROM rhnConfigChannel CC,
	         web_contact WC
	   WHERE WC.id = user_id_in
	     AND CC.id = config_channel_id_in
	     AND WC.org_id = CC.org_id;
	EXCEPTION
	  WHEN NO_DATA_FOUND
	    THEN RETURN 0;
	END;
	global_channel := 'unknown';
	SELECT CCT.label INTO global_channel
	  FROM rhnConfigChannel CC,
	       rhnConfigChannelType CCT
	 WHERE CC.id = config_channel_id_in
	   AND CCT.id = CC.confchan_type_id;
	IF (rhn_user.check_role_implied(user_id_in, 'config_admin') = 1) AND (global_channel = 'normal')
	THEN
	    RETURN 1;
	END IF;
	any_visible_servers_subscribed := 0;
	BEGIN
	  SELECT 1 INTO any_visible_servers_subscribed
	    FROM DUAL
           WHERE EXISTS (
	     SELECT SCC.server_id
	       FROM rhnServerConfigChannel SCC,
	            rhnUserServerPermsDupes USPD
	      WHERE USPD.user_id = user_id_in
	        AND USPD.server_id = SCC.server_id
	        AND SCC.config_channel_id = config_channel_id_in
	   );
	EXCEPTION
	  WHEN NO_DATA_FOUND
	    THEN RETURN 0;
	END;
	RETURN any_visible_servers_subscribed;
    END get_user_chan_access;
    FUNCTION get_user_revision_access(config_revision_id_in IN NUMBER, user_id_in IN NUMBER)
    RETURN NUMBER
    IS
	config_channel_id NUMBER;
    BEGIN
	SELECT CF.config_channel_id INTO config_channel_id
	  FROM rhnConfigFile CF,
	       rhnConfigRevision CR
	 WHERE CF.id = CR.config_file_id
	   AND CR.id = config_revision_id_in;
	RETURN rhn_config_channel.get_user_chan_access(config_channel_id, user_id_in);
    END get_user_revision_access;
    FUNCTION get_user_file_access(config_file_id_in IN NUMBER, user_id_in IN NUMBER)
    RETURN NUMBER
    IS
	config_channel_id NUMBER;
    BEGIN
	SELECT CF.config_channel_id INTO config_channel_id
	  FROM rhnConfigFile CF
	 WHERE CF.id = config_file_id_in;
	RETURN rhn_config_channel.get_user_chan_access(config_channel_id, user_id_in);
    END get_user_file_access;
END rhn_config_channel;
/
SHOW ERRORS;

create or replace
package rhn_entitlements
is
	body_version varchar2(100) := '$Id: rhn_entitlements.pks 112265 2007-02-28 15:33:48Z jslagle $';

   type ents_array is varray(10) of rhnServerGroupType.label%TYPE;


    function entitlement_grants_service (
	    entitlement_in in varchar2,
		service_level_in in varchar2
	) return number;

	function lookup_entitlement_group (
		org_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) return number;

	function create_entitlement_group (
		org_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) return number;

   function can_entitle_server (
      server_id_in   in number,
      type_label_in  in varchar2
   )
   return number;

   function can_switch_base (
      server_id_in   in    integer,
      type_label_in  in    varchar2
   )
   return number;

   function find_compatible_sg (
      server_id_in in number,
      type_label_in in varchar2,
      sgid_out out number
   )
   return boolean;

	procedure entitle_server (
		server_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	);

	procedure remove_server_entitlement (
		server_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled',
        repoll_virt_guests in number := 1
	);

	procedure unentitle_server (
		server_id_in in number
	);

    procedure repoll_virt_guest_entitlements(
        server_id_in in number
    );

	function get_server_entitlement (
		server_id_in in number
	) return ents_array;

	procedure modify_org_service (
		org_id_in in number,
		service_label_in in varchar2,
		enable_in in char
	);

    procedure set_customer_enterprise (
		customer_id_in in number
	);

	procedure set_customer_provisioning (
		customer_id_in in number
	);

	procedure set_customer_nonlinux (
		customer_id_in in number
	);

    procedure unset_customer_enterprise (
		customer_id_in in number
	);

	procedure unset_customer_provisioning (
		customer_id_in in number
	);

	procedure unset_customer_nonlinux (
		customer_id_in in number
	);

    procedure set_group_count (
		customer_id_in in number,	-- customer_id
		type_in in char,			-- 'U' or 'S'
		group_type_in in number,	-- rhn[User|Server]GroupType.id
		quantity_in in number		-- quantity
    );

    procedure set_family_count (
		customer_id_in in number,		-- customer_id
		channel_family_id_in in number,	-- 246
		quantity_in in number			-- 3
    );

    -- this makes NO checks that the quantity is within max,
    -- so we should NEVER run this unless we KNOW that we won't be
    -- violating the max
    procedure entitle_last_modified_servers (
		customer_id_in in number,	-- customer_id
		type_label_in in varchar2,	-- 'enterprise_entitled'
		quantity_in in number		-- 3
    );

	procedure prune_everything (
		customer_id_in in number
	);

	procedure subscribe_newest_servers (
		customer_id_in in number
	);
end rhn_entitlements;
/
show errors;

CREATE OR REPLACE package body rhn_entitlements
is
	body_version varchar2(100) := '$Id: rhn_entitlements.pkb 113460 2007-03-20 14:20:18Z jslagle $';
	function entitlement_grants_service (
	    entitlement_in in varchar2,
	    service_level_in in varchar2
	) return number	is
	begin
		if service_level_in = 'provisioning' then
			if entitlement_in = 'provisioning_entitled' then
				return 1;
			else
				return 0;
			end if;
		elsif service_level_in = 'management' then
			if entitlement_in = 'enterprise_entitled' then
				return 1;
			else
				return 0;
			end if;
		elsif service_level_in = 'monitoring' then
			if entitlement_in = 'monitoring_entitled' then
				return 1;
			end if;
		elsif service_level_in = 'updates' then
			return 1;
		else
			return 0;
		end if;
	end entitlement_grants_service;
	function lookup_entitlement_group (
		org_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) return number is
		cursor server_groups is
			select	sg.id				server_group_id
			from	rhnServerGroup		sg,
					rhnServerGroupType	sgt
			where	sgt.label = type_label_in
				and sgt.id = sg.group_type
				and sg.org_id = org_id_in;
	begin
		for sg in server_groups loop
			return sg.server_group_id;
		end loop;
		return rhn_entitlements.create_entitlement_group(
				org_id_in,
				type_label_in
			);
	end lookup_entitlement_group;
	function create_entitlement_group (
		org_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) return number is
		sg_id_val number;
	begin
		select	rhn_server_group_id_seq.nextval
		into	sg_id_val
		from	dual;
		insert into rhnServerGroup (
				id, name, description, max_members, current_members,
				group_type, org_id
			) (
				select	sg_id_val, sgt.label, sgt.label,
						0, 0, sgt.id, org_id_in
				from	rhnServerGroupType sgt
				where	sgt.label = type_label_in
			);
		return sg_id_val;
	end create_entitlement_group;
   function can_entitle_server (
        server_id_in in number,
        type_label_in in varchar2 )
   return number is
      cursor addon_servergroups (base_label_in in varchar2,
                                 addon_label_in in varchar2) is
         select
            addon_id
         from
            rhnSGTypeBaseAddonCompat
         where base_id = lookup_sg_type (base_label_in)
           and addon_id = lookup_sg_type (addon_label_in);
      previous_ent        rhn_entitlements.ents_array;
      is_base_in          char   := 'N';
      is_base_current     char   := 'N';
      i                   number := 0;
      sgid                number := 0;
   begin
      previous_ent := rhn_entitlements.ents_array();
      previous_ent := rhn_entitlements.get_server_entitlement(server_id_in);
      select distinct is_base
      into is_base_in
      from rhnServerGroupType
      where label = type_label_in;
      if previous_ent.count = 0 then
         if (is_base_in = 'Y' and rhn_entitlements.find_compatible_sg (server_id_in, type_label_in, sgid)) then
            return 1;
         else
            return 0;
         end if;
      elsif is_base_in = 'Y' then
         return 0;
      else
         is_base_current := 'N';
         i := 0;
         while is_base_current = 'N' and i <= previous_ent.count
         loop
            i := i + 1;
            select is_base
            into is_base_current
            from rhnServerGroupType
            where label = previous_ent(i);
         end loop;
         if is_base_current  = 'N' then
            return 0;
         end if;
         for addon_servergroup in addon_servergroups  (previous_ent(i), type_label_in) loop
            if rhn_entitlements.find_compatible_sg (server_id_in, type_label_in, sgid) then
               return 1;
            else
               return 0;
            end if;
         end loop;
      end if;
      return 0;
   end can_entitle_server;
   function can_switch_base (
      server_id_in   in    integer,
      type_label_in  in    varchar2
   ) return number is
      type_label_in_is_base   char(1);
      sgid                    number;
   begin
      begin
         select is_base into type_label_in_is_base
         from rhnServerGroupType
         where label = type_label_in;
      exception
         when no_data_found then
            rhn_exception.raise_exception ( 'invalid_entitlement' );
      end;
      if type_label_in_is_base = 'N' then
         rhn_exception.raise_exception ( 'invalid_entitlement' );
      elsif rhn_entitlements.find_compatible_sg ( server_id_in,
                                                  type_label_in, sgid ) then
         return 1;
      else
         return 0;
      end if;
   end can_switch_base;
   function find_compatible_sg (
      server_id_in    in   number,
      type_label_in   in   varchar2,
      sgid_out        out  number
   ) return boolean is
      cursor servergroups is
         select sg.id            id
           from rhnServerGroupType             sgt,
                rhnServerGroup                 sg,
                rhnServer                     s,
                rhnServerServerGroupArchCompat ssgac
          where s.id = server_id_in
            and s.org_id = sg.org_id
            and sgt.label = type_label_in
            and sg.group_type = sgt.id
            and ssgac.server_group_type = sgt.id
            and ssgac.server_arch_id = s.server_arch_id
            and not exists (
                     select 1
                      from rhnServerGroupMembers sgm
                     where sgm.server_group_id = sg.id
                       and sgm.server_id = s.id);
   begin
      for servergroup in servergroups loop
         sgid_out := servergroup.id;
         return true;
      end loop;
      sgid_out := 0;
      return false;
   end find_compatible_sg;
	procedure entitle_server (
		server_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) is
      sgid  number := 0;
      is_virt number := 0;
	begin
          begin
          select 1 into is_virt
            from rhnServerEntitlementView
           where server_id = server_id_in
             and label in ('virtualization_host', 'virtualization_host_platform');
	  exception
            when no_data_found then
              is_virt := 0;
          end;
      if is_virt = 0 and (type_label_in = 'virtualization_host' or
                          type_label_in = 'virtualization_host_platform') then
        is_virt := 1;
      end if;
      if rhn_entitlements.can_entitle_server(server_id_in,
                                             type_label_in) = 1 then
         if rhn_entitlements.find_compatible_sg (server_id_in,
                                                 type_label_in, sgid) then
            insert into rhnServerHistory ( id, server_id, summary, details )
            values ( rhn_event_id_seq.nextval, server_id_in,
                     'added system entitlement ',
                      case type_label_in
                       when 'enterprise_entitled' then 'Management'
                       when 'sw_mgr_entitled' then 'Update'
                       when 'provisioning_entitled' then 'Provisioning'
                       when 'monitoring_entitled' then 'Monitoring'
                       when 'virtualization_host' then 'Virtualization'
                       when 'virtualization_host_platform' then
                            'Virtualization Platform' end  );
            rhn_server.insert_into_servergroup (server_id_in, sgid);
            if is_virt = 1 then
              rhn_entitlements.repoll_virt_guest_entitlements(server_id_in);
            end if;
         else
            rhn_exception.raise_exception ('no_available_server_group');
         end if;
      else
         rhn_exception.raise_exception ('invalid_entitlement');
      end if;
   end entitle_server;
	procedure remove_server_entitlement (
		server_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled',
        repoll_virt_guests in number := 1
	) is
		group_id number;
      type_is_base char;
      is_virt number := 0;
	begin
      begin
        begin
          select 1 into is_virt
            from rhnServerEntitlementView
           where server_id = server_id_in
             and label in ('virtualization_host', 'virtualization_host_platform');
        exception
          when no_data_found then
            is_virt := 0;
        end;
		select	sg.id, sgt.is_base
		into	group_id, type_is_base
		from	rhnServerGroupType sgt,
			rhnServerGroup sg,
				rhnServerGroupMembers sgm,
				rhnServer s
		where	s.id = server_id_in
			and s.id = sgm.server_id
			and sgm.server_group_id = sg.id
			and sg.org_id = s.org_id
			and sgt.label = type_label_in
			and sgt.id = sg.group_type;
      if ( type_is_base = 'Y' ) then
         unentitle_server ( server_id_in );
      else
         insert into rhnServerHistory ( id, server_id, summary, details )
         values ( rhn_event_id_seq.nextval, server_id_in,
                  'removed system entitlement ',
                   case type_label_in
                    when 'enterprise_entitled' then 'Management'
                    when 'sw_mgr_entitled' then 'Update'
                    when 'provisioning_entitled' then 'Provisioning'
                    when 'monitoring_entitled' then 'Monitoring'
                    when 'virtualization_host' then 'Virtualization'
                    when 'virtualization_host_platform' then
                         'Virtualization Platforrm' end  );
         rhn_server.delete_from_servergroup(server_id_in, group_id);
         if type_label_in = 'monitoring_entitled' then
           DELETE
             FROM state_change
            WHERE o_id IN (SELECT probe_id
                             FROM rhn_check_probe
                            WHERE host_id = server_id_in);
           DELETE
             FROM time_series
            WHERE SUBSTR(o_id, INSTR(o_id, '-') + 1,
                        (INSTR(o_id, '-', INSTR(o_id, '-') + 1) - INSTR(o_id, '-')) - 1)
              IN (SELECT probe_id
                    FROM rhn_check_probe
                   WHERE host_id = server_id_in);
           DELETE
             FROM rhn_probe
            WHERE recid IN (SELECT probe_id
                              FROM rhn_check_probe
                             WHERE host_id = server_id_in);
         end if;
         if is_virt = 1 and repoll_virt_guests = 1 then
           rhn_entitlements.repoll_virt_guest_entitlements(server_id_in);
         end if;
      end if;
		exception
		when no_data_found then
				rhn_exception.raise_exception('invalid_server_group_member');
      end;
	end remove_server_entitlement;
   procedure unentitle_server (server_id_in in number) is
      cursor servergroups is
         select distinct sgt.label, sg.id server_group_id
         from  rhnServerGroupType sgt,
               rhnServerGroup sg,
               rhnServer s,
               rhnServerGroupMembers sgm
         where s.id = server_id_in
            and s.org_id = sg.org_id
            and sg.group_type = sgt.id
            and sgm.server_group_id = sg.id
            and sgm.server_id = s.id;
     is_virt number := 0;
   begin
      begin
        select 1 into is_virt
          from rhnServerEntitlementView
         where server_id = server_id_in
           and label in ('virtualization_host', 'virtualization_host_platform');
      exception
        when no_data_found then
          is_virt := 0;
      end;
      for servergroup in servergroups loop
         insert into rhnServerHistory ( id, server_id, summary, details )
         values ( rhn_event_id_seq.nextval, server_id_in,
                  'removed system entitlement ',
                   case servergroup.label
                    when 'enterprise_entitled' then 'Management'
                    when 'sw_mgr_entitled' then 'Update'
                    when 'provisioning_entitled' then 'Provisioning'
                    when 'monitoring_entitled' then 'Monitoring'
                    when 'virtualization_host' then 'Virtualization'
                    when 'virtualization_host_platform' then
                         'Virtualization Platform' end  );
         rhn_server.delete_from_servergroup(server_id_in,
                                            servergroup.server_group_id );
      end loop;
      if is_virt = 1 then
        rhn_entitlements.repoll_virt_guest_entitlements(server_id_in);
      end if;
   end unentitle_server;
    procedure repoll_virt_guest_entitlements(server_id_in in number)
    is
        cursor families is
            select distinct cfs.channel_family_id
            from
                rhnChannelFamilyServers cfs,
                rhnVirtualInstance vi
            where
                vi.host_system_id = server_id_in
                and vi.virtual_system_id = cfs.server_id;
        cursor group_types is
            select distinct sg.group_type, sgt.label
            from
                rhnServerGroupType sgt,
                rhnServerGroup sg,
                rhnServerGroupMembers sgm,
                rhnVirtualInstance vi
            where
                vi.host_system_id = server_id_in
                and vi.virtual_system_id = sgm.server_id
                and sgm.server_group_id = sg.id
                and sg.group_type = sgt.id;
        cursor virt_servers_cfam(family_id_in in number, quantity_in in number)  is
            select virtual_system_id
            from (
                select rownum, vi.virtual_system_id
                from
                    rhnChannelFamilyMembers cfm,
                    rhnServerChannel sc,
                    rhnVirtualInstance vi
                where
                    vi.host_system_id = server_id_in
                    and vi.virtual_system_id = sc.server_id
                    and sc.channel_id = cfm.channel_id
                    and cfm.channel_family_id = family_id_in
                order by sc.modified desc
                )
            where rownum <= quantity_in;
        cursor virt_servers_sgt(group_type_in in number, quantity_in in number)  is
            select virtual_system_id
            from (
                select rownum, vi.virtual_system_id
                from
                    rhnServerGroup sg,
                    rhnServerGroupMembers sgm,
                    rhnVirtualInstance vi
                where
                    vi.host_system_id = server_id_in
                    and vi.virtual_system_id = sgm.server_id
                    and sgm.server_group_id = sg.id
                    and sg.group_type = group_type_in
                order by sgm.modified desc
                )
            where rownum <= quantity_in;
        org_id_val number;
        max_members_val number;
        current_members_calc number;
        sg_id number;
    begin
        select org_id
        into org_id_val
        from rhnServer
        where id = server_id_in;
        for family in families loop
            current_members_calc :=
                rhn_channel.channel_family_current_members(family.channel_family_id,
                                                           org_id_val); -- fixed transposed args
            select max_members
            into max_members_val
            from rhnPrivateChannelFamily
            where channel_family_id = family.channel_family_id
            and org_id = org_id_val;
            if current_members_calc > max_members_val then
                for virt_server in virt_servers_cfam(family.channel_family_id,
                                current_members_calc - max_members_val) loop
                    rhn_channel.unsubscribe_server_from_family(
                                virt_server.virtual_system_id,
                                family.channel_family_id);
                end loop;
                if current_members_calc > max_members_val then
                    set_family_count(org_id_val,
                                     family.channel_family_id,
                                     max_members_val);
                end if;
           end if;
            rhn_channel.update_family_counts(family.channel_family_id,
                                             org_id_val);
        end loop;
        for a_group_type in group_types loop
          select max_members, id
            into max_members_val, sg_id
            from rhnServerGroup
            where group_type = a_group_type.group_type
            and org_id = org_id_val;
	  select count(sep.server_id) into current_members_calc
            from rhnServerEntitlementPhysical sep
           where sep.server_group_id = sg_id
             and sep.server_group_type_id = a_group_type.group_type;
          if current_members_calc > max_members_val then
            for virt_server in virt_servers_sgt(a_group_type.group_type,
                                                current_members_calc - max_members_val) loop
              rhn_entitlements.remove_server_entitlement(virt_server.virtual_system_id, a_group_type.label);
              current_members_calc := current_members_calc - 1;
            end loop;
          end if;
          update rhnServerGroup set current_members = current_members_calc
           where org_id = org_id_val
             and group_type = a_group_type.group_type;
        end loop;
    end repoll_virt_guest_entitlements;
	function get_server_entitlement (
		server_id_in in number
	) return ents_array is
		cursor server_groups is
			select	sgt.label
			from	rhnServerGroupType		sgt,
					rhnServerGroup			sg,
					rhnServerGroupMembers	sgm
			where	1=1
				and sgm.server_id = server_id_in
				and sg.id = sgm.server_group_id
				and sgt.id = sg.group_type
				and sgt.label in (
					'sw_mgr_entitled','enterprise_entitled',
					'provisioning_entitled', 'nonlinux_entitled',
					'monitoring_entitled', 'virtualization_host',
                                        'virtualization_host_platform'
					);
         ent_array ents_array;
	begin
      ent_array := ents_array();
		for sg in server_groups loop
         ent_array.extend;
         ent_array(ent_array.count) := sg.label;
		end loop;
		return ent_array;
	end get_server_entitlement;
	procedure modify_org_service (
		org_id_in in number,
		service_label_in in varchar2,
		enable_in in char
	) is
		type roles_v is varray(10) of rhnUserGroupType.label%TYPE;
		roles_to_process roles_v;
		cursor roles(role_label_in in varchar2) is
			select	label, id
			from	rhnUserGroupType
			where	label = role_label_in;
		cursor org_roles(role_label_in in varchar2) is
			select	1
			from	rhnUserGroup ug,
					rhnUserGroupType ugt
			where	ugt.label = role_label_in
				and ug.org_id = org_id_in
				and ugt.id = ug.group_type;
		type ents_v is varray(10) of rhnOrgEntitlementType.label%TYPE;
		ents_to_process ents_v;
		cursor ents(ent_label_in in varchar2) is
			select	label, id
			from	rhnOrgEntitlementType
			where	label = ent_label_in;
		cursor org_ents(ent_label_in in varchar2) is
			select	1
			from	rhnOrgEntitlements oe,
					rhnOrgEntitlementType oet
			where	oet.label = ent_label_in
				and oe.org_id = org_id_in
				and oet.id = oe.entitlement_id;
		create_row char(1);
	begin
		ents_to_process := ents_v();
		roles_to_process := roles_v();
		if service_label_in = 'enterprise' or
           service_label_in = 'management' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'sw_mgr_enterprise';
			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'org_admin';
			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'system_group_admin';
			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'activation_key_admin';
			if is_satellite() = 1 then
				roles_to_process.extend;
				roles_to_process(roles_to_process.count) := 'org_applicant';
			end if;
		elsif service_label_in = 'provisioning' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'rhn_provisioning';
			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'system_group_admin';
			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'activation_key_admin';
			if is_satellite() = 1 then
				roles_to_process.extend;
				roles_to_process(roles_to_process.count) := 'config_admin';
			end if;
			if enable_in = 'Y' then
				ents_to_process.extend;
				ents_to_process(ents_to_process.count) := 'sw_mgr_enterprise';
			end if;
		elsif service_label_in = 'monitoring' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'rhn_monitor';
			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'monitoring_admin';
		elsif service_label_in = 'virtualization' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'rhn_virtualization';
			if is_satellite() = 1 then
				roles_to_process.extend;
				roles_to_process(roles_to_process.count) := 'config_admin';
			end if;
        elsif service_label_in = 'virtualization_platform' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'rhn_virtualization_platform';
			if is_satellite() = 1 then
				roles_to_process.extend;
				roles_to_process(roles_to_process.count) := 'config_admin';
			end if;
	elsif service_label_in = 'nonlinux' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'rhn_nonlinux';
			if is_satellite() = 1 then
				roles_to_process.extend;
				roles_to_process(roles_to_process.count) := 'config_admin';
			end if;
		end if;
		if enable_in = 'Y' then
			for i in 1..ents_to_process.count loop
				for ent in ents(ents_to_process(i)) loop
					create_row := 'Y';
					for oe in org_ents(ent.label) loop
						create_row := 'N';
					end loop;
					if create_row = 'Y' then
						insert into rhnOrgEntitlements(org_id, entitlement_id)
							values (org_id_in, ent.id);
					end if;
				end loop;
			end loop;
			for i in 1..roles_to_process.count loop
				for role in roles(roles_to_process(i)) loop
					create_row := 'Y';
					for o_r in org_roles(role.label) loop
						create_row := 'N';
					end loop;
					if create_row = 'Y' then
						insert into rhnUserGroup(
								id, name, description, current_members,
								group_type, org_id
							) (
								select	rhn_user_group_id_seq.nextval,
										ugt.name || 's',
										ugt.name || 's for Org ' ||
											o.name || ' ('|| o.id ||')',
										0, ugt.id, o.id
								from	rhnUserGroupType ugt,
										web_customer o
								where	o.id = org_id_in
									and ugt.id = role.id
							);
					end if;
				end loop;
			end loop;
		else
			for i in 1..ents_to_process.count loop
				for ent in ents(ents_to_process(i)) loop
					delete from rhnOrgEntitlements
					 where org_id = org_id_in
					   and entitlement_id = ent.id;
				end loop;
			end loop;
		end if;
	end modify_org_service;
	procedure set_customer_enterprise (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'enterprise', 'Y');
	end set_customer_enterprise;
	procedure set_customer_provisioning (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'provisioning', 'Y');
	end set_customer_provisioning;
	procedure set_customer_nonlinux (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'nonlinux', 'Y');
	end set_customer_nonlinux;
	procedure unset_customer_enterprise (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'enterprise', 'N');
	end unset_customer_enterprise;
	procedure unset_customer_provisioning (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'provisioning', 'N');
	end unset_customer_provisioning;
	procedure unset_customer_nonlinux (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'nonlinux', 'N');
	end unset_customer_nonlinux;
	procedure prune_group (
		group_id_in in number,
		type_in in char,
		quantity_in in number
	) is
		cursor usergroups is
			select	user_id, user_group_id, ugt.label
			from	rhnUserGroupType	ugt,
					rhnUserGroup		ug,
					rhnUserGroupMembers	ugm
			where	1=1
				and ugm.user_group_id = group_id_in
				and ugm.user_id in (
					select	user_id
					from	(
						select	rownum row_number,
								user_id,
								time
						from	(
							select	user_id,
									modified time
							from	rhnUserGroupMembers
							where	user_group_id = group_id_in
							order by time asc
						)
					)
					where	row_number > quantity_in
				)
				and ugm.user_group_id = ug.id
				and ug.group_type = ugt.id;
        cursor servergroups is
           select  server_id, server_group_id, sgt.id group_type_id, sgt.label
            from    rhnServerGroupType              sgt,
                            rhnServerGroup                  sg,
                            rhnServerGroupMembers   sgm
            where   1=1
                    and sgm.server_group_id = group_id_in
                    and sgm.server_id in (
                            select  server_id
                            from    (
                                    select  rownum row_number,
                                                    server_id,
                                                    time
                                    from    (
                                            select  sep.server_id,
                                                    sep.modified time
                                            from
                                                rhnServerEntitlementPhysical sep
                                            where
                                                sep.server_group_id = group_id_in
                                            order by time asc
                                    )
                            )
                            where   row_number > quantity_in
                    )
                    and sgm.server_group_id = sg.id
                    and sg.group_type = sgt.id;
      type_is_base char;
	begin
		if type_in = 'U' then
			update		rhnUserGroup
				set		max_members = quantity_in
				where	id = group_id_in;
			for ug in usergroups loop
				rhn_user.remove_from_usergroup(ug.user_id, ug.user_group_id);
			end loop;
		elsif type_in = 'S' then
			update		rhnServerGroup
				set		max_members = quantity_in
				where	id = group_id_in;
			for sg in servergroups loop
				remove_server_entitlement(sg.server_id, sg.label);
            select is_base
            into type_is_base
            from rhnServerGroupType sgt
            where sgt.id = sg.group_type_id;
            if ( type_is_base = 'Y' ) then
				   rhn_channel.clear_subscriptions(sg.server_id);
            end if;
			end loop;
		end if;
	end prune_group;
	procedure set_group_count (
		customer_id_in in number,
		type_in in char,
		group_type_in in number,
		quantity_in in number
	) is
		group_id number;
		quantity number;
	begin
		quantity := quantity_in;
		if quantity is not null and quantity < 0 then
			quantity := 0;
		end if;
		if type_in = 'U' then
			select	rug.id
			into	group_id
			from	rhnUserGroup rug
			where	1=1
				and rug.org_id = customer_id_in
				and rug.group_type = group_type_in;
		elsif type_in = 'S' then
			select	rsg.id
			into	group_id
			from	rhnServerGroup rsg
			where	1=1
				and rsg.org_id = customer_id_in
				and rsg.group_type = group_type_in;
		end if;
		rhn_entitlements.prune_group(
			group_id,
			type_in,
			quantity
		);
	exception
		when no_data_found then
			if type_in = 'U' then
				insert into rhnUserGroup (
						id, name, description, max_members, current_members,
						group_type, org_id, created, modified
					) (
						select	rhn_user_group_id_seq.nextval, name, name,
								quantity, 0, id, customer_id_in,
								sysdate, sysdate
						from	rhnUserGroupType
						where	id = group_type_in
				);
			elsif type_in = 'S' then
				insert into rhnServerGroup (
						id, name, description, max_members, current_members,
						group_type, org_id, created, modified
					) (
						select	rhn_server_group_id_seq.nextval, name, name,
								quantity, 0, id, customer_id_in,
								sysdate, sysdate
						from	rhnServerGroupType
						where	id = group_type_in
				);
			end if;
	end set_group_count;
	procedure prune_family (
		customer_id_in in number,
		channel_family_id_in in number,
		quantity_in in number
	) is
		cursor serverchannels is
			select	sc.server_id,
					sc.channel_id
			from	rhnServerChannel sc,
					rhnChannelFamilyMembers cfm
			where	1=1
				and cfm.channel_family_id = channel_family_id_in
				and cfm.channel_id = sc.channel_id
				and server_id in (
					select	server_id
					from	(
						select	server_id,
								time,
								rownum row_number
						from	(
							select	rs.id					server_id,
									rcfm.modified			time
							from
									rhnServerChannel		rsc,
									rhnChannelFamilyMembers	rcfm,
                                    rhnServer				rs
							where	1=1
								and rs.org_id = customer_id_in
								and rs.id = rsc.server_id
								and rsc.channel_id = rcfm.channel_id
								and rcfm.channel_family_id =
									channel_family_id_in
                                and exists (
                                    select 1
                                    from rhnChannelFamilyServerPhysical cfsp
                                    where cfsp.server_id = rs.id
                                    and cfsp.channel_family_id =
                                        channel_family_id_in
                                    )
							order by time asc
						)
					)
					where row_number > quantity_in
				);
	begin
		if customer_id_in is null then
			return;
		end if;
		update		rhnPrivateChannelFamily
			set		max_members = quantity_in
			where	1=1
				and org_id = customer_id_in
				and channel_family_id = channel_family_id_in;
		for sc in serverchannels loop
			rhn_channel.unsubscribe_server(sc.server_id, sc.channel_id, 1, 1);
		end loop;
	end prune_family;
	procedure set_family_count (
		customer_id_in in number,
		channel_family_id_in in number,
		quantity_in in number
	) is
		cursor privperms is
			select	1
			from	rhnPrivateChannelFamily
			where	org_id = customer_id_in
				and channel_family_id = channel_family_id_in;
		cursor pubperms is
			select	o.id org_id
			from	web_customer o,
					rhnPublicChannelFamily pcf
			where	pcf.channel_family_id = channel_family_id_in;
		quantity number;
		done number := 0;
	begin
		quantity := quantity_in;
		if quantity is not null and quantity < 0 then
			quantity := 0;
		end if;
		if customer_id_in is not null then
			for perm in privperms loop
				rhn_entitlements.prune_family(
					customer_id_in,
					channel_family_id_in,
					quantity
				);
				update rhnPrivateChannelFamily
					set max_members = quantity
					where org_id = customer_id_in
						and channel_family_id = channel_family_id_in;
				return;
			end loop;
			insert into rhnPrivateChannelFamily (
					channel_family_id, org_id, max_members, current_members
				) values (
					channel_family_id_in, customer_id_in, quantity, 0
				);
			return;
		end if;
		for perm in pubperms loop
			if quantity = 0 then
				rhn_entitlements.prune_family(
					perm.org_id,
					channel_family_id_in,
					quantity
				);
				if done = 0 then
					delete from rhnPublicChannelFamily
						where channel_family_id = channel_family_id_in;
				end if;
			end if;
			done := 1;
		end loop;
		if done != 1 then
			insert into rhnPublicChannelFamily (
					channel_family_id
				) values (
					channel_family_id_in
				);
		end if;
	end set_family_count;
	procedure entitle_last_modified_servers (
		customer_id_in in number,
		type_label_in in varchar2,
		quantity_in in number
	) is
		cursor servers(cid_in in number, quant_in in number) is
			select	server_id
			from	(
						select	rownum row_number,
								server_id
						from	(
									select	rs.id server_iD
									from	rhnServer rs
									where	1=1
										and rs.org_id = cid_in
										and not exists (
											select	1
											from	rhnServerGroup sg,
													rhnServerGroupMembers rsgm
											where	rsgm.server_id = rs.id
												and rsgm.server_group_id = sg.id
												and sg.group_type is not null
										)
                                        and not exists (
                                            select 1
                                            from rhnVirtualInstance vi
                                            where vi.virtual_system_id =
                                                  rs.id
                                        )                                                                           order by modified desc
								)
					)
			where	row_number <= quant_in;
	begin
		for server in servers(customer_id_in, quantity_in) loop
			rhn_entitlements.entitle_server(server.server_id, type_label_in);
		end loop;
	end entitle_last_modified_servers;
	procedure prune_everything (
		customer_id_in in number
	) is
		cursor everything is
			select	sg.id					id,
					'S'						type,
					sg.max_members			quantity
			from	rhnServerGroup			sg
			where	sg.org_id = customer_id_in
			union
			select	ug.id					id,
					'U'						type,
					ug.max_members 			quantity
			from	rhnUserGroup			ug
			where	ug.org_id = customer_id_in
			union (
			select	cfp.channel_family_id	id,
					'C'						type,
					cfp.max_members			quantity
			from	rhnOrgChannelFamilyPermissions cfp
			where	cfp.org_id = customer_id_in
			union
			select	cfm.channel_family_id	id,
					'C'						type,
					0						quantity
			from	rhnChannelFamily		cf,
					rhnChannelFamilyMembers	cfm,
					rhnServerChannel		sc,
					rhnServer				s
			where	s.org_id = customer_id_in
				and s.id = sc.server_id
				and sc.channel_id = cfm.channel_id
				and cfm.channel_family_id = cf.id
				and cf.org_id is not null
				and cf.org_id != customer_id_in
				and not exists (
					select	1
					from	rhnOrgChannelFamilyPermissions cfp
					where	cfp.org_id = customer_id_in
						and cfp.channel_family_id = cfm.channel_family_id
					)
			);
	begin
		for one in everything loop
			if one.type in ('U','S') then
				prune_group(one.id, one.type, one.quantity);
			else
				prune_family(customer_id_in, one.id, one.quantity);
			end if;
		end loop;
	end prune_everything;
	procedure subscribe_newest_servers (
		customer_id_in in number
	) is
		cursor servers(cid_in in number) is
			select	s.id
			from	rhnServer			s
			where	1=1
				and s.org_id = cid_in
				and not exists (
						select 1
						from	rhnChannel			c,
								rhnServerChannel	sc
						where	sc.server_id = s.id
							and sc.channel_id = c.id
							and c.parent_channel is null
					)
                and not exists (
                        select 1
                        from rhnVirtualInstance vi
                        where vi.virtual_system_id = s.id
                    )
			order by s.modified desc;
		channel_id number;
	begin
		for server in servers(customer_id_in) loop
			channel_id := rhn_channel.guess_server_base(server.id);
			if channel_id is not null then
				begin
					rhn_channel.subscribe_server(server.id, channel_id);
					commit;
				exception
					when others then
						null;
				end;
			end if;
		end loop;
	end subscribe_newest_servers;
end rhn_entitlements;
/
SHOW ERRORS;

create or replace
package rhn_server
is

    -- i.e., "can this box do management stuff?" and yes if provisioning box
    function system_service_level(
	server_id_in in number,
	service_level_in in varchar2
    ) return number;

    function can_change_base_channel(
	server_id_in in number
    ) return number;

    procedure set_custom_value(
	server_id_in in number,
	user_id_in in number,
	key_label_in varchar2,
	value_in in varchar2
    );

    procedure bulk_set_custom_value(
	key_label_in in varchar2,
	value_in in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    );

    procedure snapshot_server(
	server_id_in in number,
	reason_in in varchar2
    );

    procedure bulk_snapshot(
	reason_in in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    );

    procedure tag_delete(
	server_id_in in number,
	tag_id_in in number
    );

    procedure tag_snapshot(
	snapshot_id_in in number,
	org_id_in in number,
	tagname_in in varchar2
    );

    procedure bulk_snapshot_tag(
	org_id_in in number,
	tagname_in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    );

    procedure remove_action(
	server_id_in in number,
	action_id_in in number
    );

    function check_user_access(server_id_in in number, user_id_in in number) return number;


    function can_server_consume_virt_slot(server_id_in in number,
                                              group_type_in in
                                              rhnServerGroupType.label%TYPE)
    return number;

    procedure insert_into_servergroup (
	server_id_in in number,
	server_group_id_in in number
    );

    function insert_into_servergroup_maybe (
	server_id_in in number,
	server_group_id_in in number
    ) return number;

	procedure insert_set_into_servergroup (
	server_group_id_in in number,
	user_id_in in number,
	set_label_in in varchar2
	);

    procedure delete_from_servergroup (
	server_id_in in number,
	server_group_id_in in number
    );

	procedure delete_set_from_servergroup (
	server_group_id_in in number,
	user_id_in in number,
	set_label_in in varchar2
	);

	procedure clear_servergroup (
	server_group_id_in in number
	);

	procedure delete_from_org_servergroups (
	server_id_in in number
	);

	function get_ip_address (
		server_id_in in number
	) return varchar2;
end rhn_server;
/
SHOW ERRORS;

CREATE OR REPLACE package body rhn_server
is
    function system_service_level(
	server_id_in in number,
	service_level_in in varchar2
    ) return number is
    cursor ents is
      select label from rhnServerEntitlementView
      where server_id = server_id_in;
    retval number := 0;
    begin
         for ent in ents loop
            retval := rhn_entitlements.entitlement_grants_service (ent.label, service_level_in);
            if retval = 1 then
               return retval;
            end if;
         end loop;
         return retval;
    end system_service_level;
    function can_change_base_channel(server_id_in IN NUMBER)
    return number
    is
	throwaway number;
    begin
	select 1 into throwaway
	  from rhnServer S
	 where S.id = server_id_in
	   and not exists (select 1 from rhnSatelliteInfo SI where SI.server_id = S.id)
	   and not exists (select 1 from rhnProxyInfo PI where PI.server_id = S.id);
	return 1;
    exception
	when no_data_found
	    then
	    return 0;
    end can_change_base_channel;
    procedure set_custom_value(
	server_id_in in number,
	user_id_in in number,
	key_label_in in varchar2,
	value_in in varchar2
    ) is
	key_id_val number;
    begin
	select CDK.id into key_id_val
	  from rhnCustomDataKey CDK,
	       rhnServer S
	 where S.id = server_id_in
	   and S.org_id = CDK.org_id
	   and CDK.label = key_label_in;
	begin
	    insert into rhnServerCustomDataValue (server_id, key_id, value, created_by, last_modified_by)
	    values (server_id_in, key_id_val, value_in, user_id_in, user_id_in);
	exception
	    when DUP_VAL_ON_INDEX
		then
		update rhnServerCustomDataValue
		   set value = value_in,
		       last_modified_by = user_id_in
		 where server_id = server_id_in
		   and key_id = key_id_val;
	end;
    end set_custom_value;
    procedure bulk_set_custom_value(
	key_label_in in varchar2,
	value_in in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    ) is
    begin
	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    if rhn_server.system_service_level(server.element, 'provisioning') = 1 then
		rhn_server.set_custom_value(server.element, set_uid_in, key_label_in, value_in);
	    end if;
	end loop server;
    end bulk_set_custom_value;
    procedure bulk_snapshot_tag(
	org_id_in in number,
        tagname_in in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    ) is
	snapshot_id number;
    begin
	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    if rhn_server.system_service_level(server.element, 'provisioning') = 1 then
		begin
		    select max(id) into snapshot_id
		    from rhnSnapshot
		    where server_id = server.element;
		exception
		    when NO_DATA_FOUND then
			rhn_server.snapshot_server(server.element, 'tagging system:  ' || tagname_in);
			select max(id) into snapshot_id
			from rhnSnapshot
			where server_id = server.element;
		end;
		begin
		    rhn_server.tag_snapshot(snapshot_id, org_id_in, tagname_in);
		exception
		    when DUP_VAL_ON_INDEX
			then
			null;
		end;
	    end if;
	end loop server;
    end bulk_snapshot_tag;
    procedure tag_delete(
	server_id_in in number,
	tag_id_in in number
    ) is
	cursor snapshots is
		select	snapshot_id
		from	rhnSnapshotTag
		where	tag_id = tag_id_in;
	tag_id_tmp number;
    begin
	select	id into tag_id_tmp
	from	rhnTag
	where	id = tag_id_in
	for update;
	delete
		from	rhnSnapshotTag
		where	server_id = server_id_in
			and tag_id = tag_id_in;
	for snapshot in snapshots loop
		return;
	end loop;
	delete
		from rhnTag
		where id = tag_id_in;
    end tag_delete;
    procedure tag_snapshot(
        snapshot_id_in in number,
	org_id_in in number,
	tagname_in in varchar2
    ) is
    begin
	insert into rhnSnapshotTag (snapshot_id, server_id, tag_id)
	select snapshot_id_in, server_id, lookup_tag(org_id_in, tagname_in)
	from rhnSnapshot
	where id = snapshot_id_in;
    end tag_snapshot;
    procedure bulk_snapshot(
	reason_in in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    ) is
    begin
	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    if rhn_server.system_service_level(server.element, 'provisioning') = 1 then
		rhn_server.snapshot_server(server.element, reason_in);
	    end if;
	end loop server;
    end bulk_snapshot;
    procedure snapshot_server(
	server_id_in in number,
	reason_in in varchar2
    ) is
	snapshot_id number;
	cursor revisions is
		select distinct
			cr.id
		from	rhnConfigRevision	cr,
			rhnConfigFileName	cfn,
			rhnConfigFile		cf,
			rhnConfigChannel	cc,
			rhnServerConfigChannel	scc
		where	1=1
			and scc.server_id = server_id_in
			and scc.config_channel_id = cc.id
			and cc.id = cf.config_channel_id
			and cf.id = cr.config_file_id
			and cr.id = cf.latest_config_revision_id
			and cf.config_file_name_id = cfn.id
			and cf.id = lookup_first_matching_cf(scc.server_id, cfn.path);
	locked integer;
    begin
	select rhn_snapshot_id_seq.nextval into snapshot_id from dual;
	insert into rhnSnapshot (id, org_id, server_id, reason) (
		select	snapshot_id,
			s.org_id,
			server_id_in,
			reason_in
		from	rhnServer s
		where	s.id = server_id_in
	);
	insert into rhnSnapshotChannel (snapshot_id, channel_id) (
		select	snapshot_id, sc.channel_id
		from	rhnServerChannel sc
		where	sc.server_id = server_id_in
	);
	insert into rhnSnapshotServerGroup (snapshot_id, server_group_id) (
		select	snapshot_id, sgm.server_group_id
		from	rhnServerGroupMembers sgm
		where	sgm.server_id = server_id_in
	);
        locked := 0;
        while true loop
            begin
                insert into rhnPackageNEVRA (id, name_id, evr_id, package_arch_id)
                select rhn_pkgnevra_id_seq.nextval, sp.name_id, sp.evr_id, sp.package_arch_id
                from rhnServerPackage sp
                where sp.server_id = server_id_in
                        and not exists
                        (select 1
                                from rhnPackageNEVRA nevra
                                where nevra.name_id = sp.name_id
                                        and nevra.evr_id = sp.evr_id
                                        and (nevra.package_arch_id = sp.package_arch_id
                                            or (nevra.package_arch_id is null
                                                and sp.package_arch_id is null)));
                exit;
            exception when dup_val_on_index then
                if locked = 1 then
                    raise;
                else
                    lock table rhnPackageNEVRA in exclusive mode;
                    locked := 1;
                end if;
            end;
        end loop;
	insert into rhnSnapshotPackage (snapshot_id, nevra_id) (
                select distinct snapshot_id, nevra.id
                from    rhnServerPackage sp, rhnPackageNEVRA nevra
                where   sp.server_id = server_id_in
                        and nevra.name_id = sp.name_id
                        and nevra.evr_id = sp.evr_id
                        and (nevra.package_arch_id = sp.package_arch_id
                            or (nevra.package_arch_id is null
                                and sp.package_arch_id is null))
	);
	insert into rhnSnapshotConfigChannel ( snapshot_id, config_channel_id ) (
		select	snapshot_id, scc.config_channel_id
		from	rhnServerConfigChannel scc
		where	server_id = server_id_in
	);
	for revision in revisions loop
		insert into rhnSnapshotConfigRevision (
				snapshot_id, config_revision_id
			) values (
				snapshot_id, revision.id
			);
	end loop;
    end snapshot_server;
    procedure remove_action(
	server_id_in in number,
	action_id_in in number
    ) is
	cursor chained_actions is
		select	id, prerequisite
		from	rhnAction
		start with id = action_id_in
		connect by prior id = prerequisite
		order by prerequisite desc;
	cursor sessions is
		select	s.id
		from	rhnKickstartSession s
		where	server_id_in in (s.old_server_id, s.new_server_id)
			and s.action_id = action_id_in
			and not exists (
				select	1
				from	rhnKickstartSessionState ss
				where	ss.id = s.state_id
					and ss.label in ('failed','complete')
			);
	type chain_end_type is table of number index by binary_integer;
	chain_ends chain_end_type;
	i number;
	prereq number := 1;
    begin
	select	prerequisite
	into	prereq
	from	rhnAction
	where	id = action_id_in;
	if prereq is not null then
		rhn_exception.raise_exception('action_is_child');
	end if;
	i := 0;
	for action in chained_actions loop
		if action.prerequisite is null then
			chain_ends(i) := action.id;
			i := i + 1;
		else
			delete from rhnServerAction
				where server_id = server_id_in
				and action_id = action.id;
		end if;
	end loop;
	i := chain_ends.first;
	while i is not null loop
		delete from rhnServerAction
			where server_id = server_id_in
			and action_id = chain_ends(i);
		i := chain_ends.next(i);
	end loop;
	for s in sessions loop
		update rhnKickstartSession
			set 	state_id = (
					select	id
					from	rhnKickstartSessionState
					where	label = 'failed'
				),
				action_id = null
			where	id = s.id;
		set_ks_session_history_message(s.id, 'failed', 'Kickstart cancelled due to action removal');
	end loop;
    end remove_action;
    function check_user_access(server_id_in in number, user_id_in in number)
    return number
    is
	has_access number;
    begin
        select 1 into has_access
	  from rhnServer S,
	       web_contact wc
	 where wc.org_id = s.org_id
	   and s.id = server_id_in
	   and wc.id = user_id_in;
	if rhn_user.check_role(user_id_in, 'org_admin') = 1
	then
	    return 1;
	end if;
	select 1 into has_access
	  from rhnServerGroupMembers SGM,
	       rhnUserServerGroupPerms USG
	 where SGM.server_group_id = USG.server_group_id
	   and SGM.server_id = server_id_in
	   and USG.user_id = user_id_in
	   and rownum = 1;
	return 1;
    exception
	when no_data_found
	    then
	    return 0;
    end check_user_access;
    function can_server_consume_virt_slot(server_id_in in number,
                                           group_type_in in
                                           rhnServerGroupType.label%TYPE)
    return number
    is
        cursor server_virt_slots is
            select vi.VIRTUAL_SYSTEM_ID
            from
                rhnVirtualInstance vi
            where
                vi.VIRTUAL_SYSTEM_ID = server_id_in
                and exists ( select 1
                     from rhnServerEntitlementView sev
                 where vi.HOST_SYSTEM_ID = sev.server_id
                 and sev.label in ('virtualization_host',
                                   'virtualization_host_platform') )
                and exists ( select 1
                     from rhnServerEntitlementView sev2
                 where vi.HOST_SYSTEM_ID = sev2.server_id
                 and sev2.label = group_type_in );
    begin
        for server_virt_slot in server_virt_slots loop
            return 1;
        end loop;
        return 0;
    end can_server_consume_virt_slot;
    procedure insert_into_servergroup (
		server_id_in in number,
		server_group_id_in in number
    ) is
		cursor sg_users is
			select	user_id id
			from	rhnUserServerGroupPerms
			where	server_group_id = server_group_id_in;
		used_slots number;
		max_slots number;
		org_id number;
		mgmt_available number;
		mgmt_upgrade number;
		mgmt_sgid number;
		prov_available number;
		prov_upgrade number;
		prov_sgid number;
		group_label rhnServerGroupType.label%TYPE;
		group_type number;
	begin
		select	sg.group_type, sg.org_id, sg.current_members, sg.max_members
		into	group_type, org_id, used_slots, max_slots
		from	rhnServerGroup sg
		where	sg.id = server_group_id_in
		for update of sg.current_members;
		if group_type is null then
			if used_slots >= max_slots then
				rhn_exception.raise_exception('servergroup_max_members');
			end if;
			insert into rhnServerGroupMembers(
					server_id, server_group_id
				) values (
					server_id_in, server_group_id_in
				);
			update rhnServerGroup
				set current_members = current_members + 1
				where id = server_group_id_in;
			for u in sg_users loop
				rhn_cache.update_perms_for_user(u.id);
			end loop;
			return;
		end if;
		select	label
		into	group_label
		from	rhnServerGroupType	sgt
		where	sgt.id = group_type;
		if group_label in ('sw_mgr_entitled',
                           'enterprise_entitled',
                           'monitoring_entitled',
                           'provisioning_entitled',
                           'virtualization_host',
                           'virtualization_host_platform') then
			if used_slots >= max_slots and
               (can_server_consume_virt_slot(server_id_in, group_label) != 1)
               then
				rhn_exception.raise_exception('servergroup_max_members');
			end if;
			insert into rhnServerGroupMembers(
					server_id, server_group_id
				) values (
					server_id_in, server_group_id_in
				);
            if can_server_consume_virt_slot(server_id_in, group_label) = 0 then
                update rhnServerGroup
                set current_members = current_members + 1
                where id = server_group_id_in;
            end if;
			return;
		end if;
	end;
	function insert_into_servergroup_maybe (
		server_id_in in number,
		server_group_id_in in number
	) return number is
		retval number := 0;
		cursor servergroups is
			select	s.id	server_id,
					sg.id	server_group_id
			from	rhnServerGroup	sg,
					rhnServer		s
			where	s.id = server_id_in
				and sg.id = server_group_id_in
				and s.org_id = sg.org_id
				and not exists (
					select	1
					from	rhnServerGroupMembers sgm
					where	sgm.server_id = s.id
						and sgm.server_group_id = sg.id
				);
	begin
		for sgm in servergroups loop
			rhn_server.insert_into_servergroup(sgm.server_id, sgm.server_group_id);
			retval := retval + 1;
		end loop;
		return retval;
	end insert_into_servergroup_maybe;
	procedure insert_set_into_servergroup (
		server_group_id_in in number,
		user_id_in in number,
		set_label_in in varchar2
	) is
		cursor servers is
			select	st.element	id
			from	rhnSet		st
			where	st.user_id = user_id_in
				and st.label = set_label_in
				and exists (
					select	1
					from	rhnUserManagedServerGroups umsg
					where	umsg.server_group_id = server_group_id_in
						and umsg.user_id = user_id_in
					)
				and not exists (
					select	1
					from	rhnServerGroupMembers sgm
					where	sgm.server_id = st.element
						and sgm.server_group_id = server_group_id_in
				);
	begin
		for s in servers loop
			rhn_server.insert_into_servergroup(s.id, server_group_id_in);
		end loop;
	end insert_set_into_servergroup;
    procedure delete_from_servergroup (
	server_id_in in number,
		server_group_id_in in number
    ) is
		cursor sg_users is
			select	user_id id
			from	rhnUserServerGroupPerms
			where	server_group_id = server_group_id_in;
        cursor server_virt_groups is
            select 1
            from rhnServerEntitlementVirtual sev
            where sev.server_id = server_id_in
            and sev.server_group_id = server_group_id_in;
		oid number;
		mgmt_sgid number;
		label rhnServerGroupType.label%TYPE;
		group_type number;
	begin
		begin
			select	sg.group_type, sg.org_id
			into	group_type,	oid
			from	rhnServerGroupMembers	sgm,
					rhnServerGroup			sg
			where	sg.id = server_group_id_in
				and sg.id = sgm.server_group_id
				and sgm.server_id = server_id_in
			for update of sg.current_members;
		exception
			when no_data_found then
				rhn_exception.raise_exception('server_not_in_group');
		end;
		if group_type is null then
			delete from rhnServerGroupMembers
				where server_group_id = server_group_id_in
				and	server_id = server_id_in;
			update rhnServerGroup
				set current_members = current_members - 1
				where id = server_group_id_in;
			for u in sg_users loop
				rhn_cache.update_perms_for_user(u.id);
			end loop;
			return;
		end if;
		select	sgt.label
		into	label
		from	rhnServerGroupType sgt
		where	sgt.id = group_type;
		if label in ('sw_mgr_entitled',
                     'enterprise_entitled',
                     'provisioning_entitled',
                     'monitoring_entitled',
                     'virtualization_host',
                     'virtualization_host_platform') then
            for server_virt_group in server_virt_groups loop
                delete from rhnServerGroupMembers
                where server_group_id = server_group_id_in
                and	server_id = server_id_in;
                return;
            end loop;
            delete from rhnServerGroupMembers
            where server_group_id = server_group_id_in
            and	server_id = server_id_in;
            update rhnServerGroup
            set current_members = current_members - 1
            where id = server_group_id_in;
		end if;
	end;
	procedure delete_set_from_servergroup (
		server_group_id_in in number,
		user_id_in in number,
		set_label_in in varchar2
	) is
		cursor servergroups is
			select	sgm.server_id, sgm.server_group_id
			from	rhnSet st,
					rhnServerGroupMembers sgm
			where	sgm.server_group_id = server_group_id_in
				and st.user_id = user_id_in
				and st.label = set_label_in
				and sgm.server_id = st.element
				and exists (
					select	1
					from	rhnUserManagedServerGroups usgp
					where	usgp.server_group_id = server_group_id_in
						and usgp.user_id = user_id_in
				);
	begin
		for sgm in servergroups loop
			rhn_server.delete_from_servergroup(sgm.server_id, server_group_id_in);
		end loop;
	end delete_set_from_servergroup;
	procedure clear_servergroup (
		server_group_id_in in number
	) is
		cursor servers is
			select	sgm.server_id	id
			from	rhnServerGroupMembers sgm
			where	sgm.server_group_id = server_group_id_in;
	begin
		for s in servers loop
			rhn_server.delete_from_servergroup(s.id, server_group_id_in);
		end loop;
	end clear_servergroup;
	procedure delete_from_org_servergroups (
		server_id_in in number
	) is
		cursor servergroups is
			select	sgm.server_group_id id
			from	rhnServerGroup sg,
					rhnServerGroupMembers sgm
			where	sgm.server_id = server_id_in
				and sgm.server_group_id = sg.id
				and sg.group_type is null;
	begin
		for sg in servergroups loop
			rhn_server.delete_from_servergroup(server_id_in, sg.id);
		end loop;
	end delete_from_org_servergroups;
	function get_ip_address (
		server_id_in in number
	) return varchar2 is
		cursor interfaces is
			select	name, ip_addr
			from	rhnServerNetInterface
			where	server_id = server_id_in
				and ip_addr != '127.0.0.1';
		cursor addresses is
			select	ipaddr ip_addr
			from	rhnServerNetwork
			where	server_id = server_id_in
				and ipaddr != '127.0.0.1';
	begin
		for addr in addresses loop
			return addr.ip_addr;
		end loop;
		for iface in interfaces loop
			return iface.ip_addr;
		end loop;
		return NULL;
	end get_ip_address;
end rhn_server;
/
SHOW ERRORS;

CREATE OR REPLACE function
lookup_virt_sub_level(label_in in varchar2)
return number
deterministic
is
	virt_sub_level_id number;
begin
	select	vsl.id
	into	virt_sub_level_id
	from	rhnVirtSubLevel vsl
	where	vsl.label = label_in;
	return virt_sub_level_id;
exception
        when no_data_found then
            rhn_exception.raise_exception('invalid_virt_sub_level');
end;
/
SHOW ERRORS;

create or replace
procedure delete_server (
	server_id_in in number
) is
	cursor servergroups is
		select	server_id, server_group_id
		from	rhnServerGroupMembers sgm
		where	sgm.server_id = server_id_in;
	cursor configchannels is
		select	cc.id
		from	rhnConfigChannel cc,
			rhnConfigChannelType cct,
			rhnServerConfigChannel scc
		where	1=1
			and scc.server_id = server_id_in
			and scc.config_channel_id = cc.id
			-- these config channel types are reserved
			-- for use by a single server, so we don't
			-- need to check for other servers subscribed
			and cct.label in
				('local_override','server_import')
			and cct.id = cc.confchan_type_id;
	cursor filelists is
		select	spfl.file_list_id id
		from	rhnServerPreserveFileList spfl
		where	spfl.server_id = server_id_in
			and not exists (
				select	1
				from	rhnServerPreserveFileList
				where	file_list_id = spfl.file_list_id
					and server_id != server_id_in
				union
				select	1
				from	rhnKickstartPreserveFileList
				where	file_list_id = spfl.file_list_id
			);
	cluster_id number;
    is_virt number := 0;
begin
	rhn_channel.delete_server_channels(server_id_in);
	-- rhn_channel.clear_subscriptions(server_id_in);

	for filelist in filelists loop
		delete from rhnFileList where id = filelist.id;
	end loop;

	for configchannel in configchannels loop
		rhn_config.delete_channel(configchannel.id);
	end loop;

    begin
      select 1 into is_virt
        from rhnServerEntitlementView
       where server_id = server_id_in
         and label in ('virtualization_host', 'virtualization_host_platform');
    exception
      when no_data_found then
        is_virt := 0;
    end;

	for sgm in servergroups loop
		rhn_server.delete_from_servergroup(
			sgm.server_id, sgm.server_group_id);
	end loop;

    if is_virt = 1 then
        rhn_entitlements.repoll_virt_guest_entitlements(server_id_in);
    end if;

	-- we're handling this instead of letting an "on delete
	-- set null" do it so that we don't run the risk
	-- of setting off the triggers and killing us with a
	-- mutating table

	-- we cna't use a cursor to do this because we need "case"
	-- in a select to do that, and 8.1.7.3.0 doesn't support it.
	update rhnKickstartSession
		set old_server_id = null
		where old_server_id = server_id_in;
	update rhnKickstartSession
		set new_server_id = null
		where new_server_id = server_id_in;

	rhn_channel.clear_subscriptions(server_id_in,1);

	-- A little complicated here, but the goal is to
	-- delete records from rhnVirtualInstace only if we don't
	-- care about them anymore.  We don't care about records
	-- in rhnVirtualInstance if we are deleting the host
	-- system and the virtual system is already null, or
	-- vice-versa.  We *do* care about them if either the
	-- host or virtual system is still registered because we
	-- still want them to show up in the UI.

        delete from rhnVirtualInstance
	      where host_system_id = server_id_in
		and virtual_system_id is null;
        delete from rhnVirtualInstance
	      where virtual_system_id = server_id_in
	        and host_system_id is null;

    -- If there's a newer row in rhnVirtualInstance with the same
    -- uuid, this guest must have been re-registered, so we can clean
    -- this data up.
    delete from rhnVirtualInstance vi
    where vi.virtual_system_id = server_id_in
    and vi.modified < (select max(vi2.modified)
                    from rhnVirtualInstance vi2
                    where vi2.uuid = vi.uuid);

        update rhnVirtualInstance
	   set host_system_id = null
	 where host_system_id = server_id_in;

	update rhnVirtualInstance
	   set virtual_system_id = null
	 where virtual_system_id = server_id_in;

	update rhnVirtualInstanceEventLog
	   set old_host_system_id = null
         where old_host_system_id = server_id_in;

	update rhnVirtualInstanceEventLog
	   set new_host_system_id = null
         where new_host_system_id = server_id_in;

	-- We're deleting everything with a foreign key to rhnServer
	-- here, now.  I'm hoping this will help aleviate our deadlock
	-- problem.

	delete from rhnActionConfigChannel where server_id = server_id_in;
	delete from rhnActionConfigRevision where server_id = server_id_in;
	delete from rhnActionPackageRemovalFailure where server_id = server_id_in;
	delete from rhnChannelFamilyLicenseConsent where server_id = server_id_in;
	delete from rhnClientCapability where server_id = server_id_in;
	delete from rhnCpu where server_id = server_id_in;
	-- there's still a cascade here, because the constraint keeps the
	-- table locked for too long to rebuild it.  Ugh...
	delete from rhnDevice where server_id = server_id_in;
	delete from rhnProxyInfo where server_id = server_id_in;
	delete from rhnRam where server_id = server_id_in;
	delete from rhnRegToken where server_id = server_id_in;
	delete from rhnSNPServerQueue where server_id = server_id_in;
	delete from rhnSatelliteChannelFamily where server_id = server_id_in;
	delete from rhnSatelliteInfo where server_id = server_id_in;
	-- this cascades to rhnActionConfigChannel and rhnActionConfigFileName
	delete from rhnServerAction where server_id = server_id_in;
	delete from rhnServerActionPackageResult where server_id = server_id_in;
	delete from rhnServerActionScriptResult where server_id = server_id_in;
	delete from rhnServerActionVerifyResult where server_id = server_id_in;
	delete from rhnServerActionVerifyMissing where server_id = server_id_in;
	-- counts are handled above.  this should be a delete_ function.
	delete from rhnServerChannel where server_id = server_id_in;
	delete from rhnServerConfigChannel where server_id = server_id_in;
	delete from rhnServerCustomDataValue where server_id = server_id_in;
	delete from rhnServerDMI where server_id = server_id_in;
	delete from rhnServerMessage where server_id = server_id_in;
	-- this gets rhnServerMessage (only) on cascade; it's handled just above
	delete from rhnServerEvent where server_id = server_id_in;
	delete from rhnServerHistory where server_id = server_id_in;
	delete from rhnServerInfo where server_id = server_id_in;
	delete from rhnServerInstallInfo where server_id = server_id_in;
	delete from rhnServerLocation where server_id = server_id_in;
	delete from rhnServerLock where server_id = server_id_in;
	delete from rhnServerNeededPackageCache where server_id = server_id_in;
	delete from rhnServerNeededErrataCache where server_id = server_id_in;
	delete from rhnServerNetwork where server_id = server_id_in;
	delete from rhnServerNotes where server_id = server_id_in;
	-- I'm not removing the foreign key from rhnServerPackage; that'll
	-- take forever.  Do the delete anyway.
	delete from rhnServerPackage where server_id = server_id_in;
	delete from rhnServerTokenRegs where server_id = server_id_in;
	delete from rhnSnapshotTag where server_id = server_id_in;
	-- this cascades to:
	--   rhnSnapshotChannel, rhnSnapshotConfigChannel, rhnSnapshotPackage,
	--   rhnSnapshotConfigRevision, rhnSnapshotServerGroup,
	--   rhnSnapshotTag.
	-- We may want to consider delete_snapshot() at some point, but
	--   I don't think we need to yet.
	delete from rhnSnapshot where server_id = server_id_in;
	delete from rhnTransaction where server_id = server_id_in;
	delete from rhnUserServerPrefs where server_id = server_id_in;
	-- hrm, this one's interesting... we _probably_ should delete
	-- everything for the parent server_id when we delete the proxy,
	-- but we don't currently.
	delete from rhnServerPath where server_id_in in (server_id, proxy_server_id);
	delete from rhnUserServerPerms where server_id = server_id_in;

	delete from rhn_interface_monitoring where server_id = server_id_in;
	delete from rhnServerNetInterface where server_id = server_id_in;
	delete from rhn_server_monitoring_info where recid = server_id_in;

	delete from rhnAppInstallSession where server_id = server_id_in;
	delete from rhnServerUuid where server_id = server_id_in;
    -- first we delete all the probes running directly against this
    -- system
    DELETE FROM rhn_probe_state PS WHERE PS.probe_id in
    (SELECT CP.probe_id
       FROM rhn_check_probe CP
      WHERE CP.host_id = server_id_in
    );

    DELETE FROM rhn_probe P  WHERE P.recid in
    (SELECT CP.probe_id
       FROM rhn_check_probe CP
      WHERE CP.host_id = server_id_in
    );
    -- Now we delete any probes that were using this Server
    -- as a Proxy Scout.
    DELETE FROM rhn_probe_state PS
      WHERE PS.probe_id in
      (SELECT CP.probe_id
       FROM rhn_check_probe CP
      WHERE CP.sat_cluster_id in
    (SELECT SN.sat_cluster_id
       FROM rhn_sat_node SN
      WHERE SN.server_id = server_id_in));

    DELETE FROM rhn_probe P
       WHERE P.recid in
      (SELECT CP.probe_id
         FROM rhn_check_probe CP
         WHERE CP.sat_cluster_id in
      (SELECT SN.sat_cluster_id
         FROM rhn_sat_node SN
         WHERE SN.server_id = server_id_in));

	delete from rhn_check_probe where host_id = server_id_in;
	delete from rhn_host_probe where host_id = server_id_in;

    delete from rhn_sat_cluster where recid in
      ( select sat_cluster_id from rhn_sat_node where server_id = server_id_in );

	delete from rhn_sat_node where server_id = server_id_in;

	-- now get rhnServer itself.
	delete
	from	rhnServer
		where id = server_id_in;

	delete
	from	rhnSet
	where	1=1
		and user_id in (
			select	wc.id
			from	rhnServer rs,
				web_contact wc
			where	rs.id = server_id_in
				and rs.org_id = wc.org_id
		)
		and label = 'system_list'
		and element = server_id_in;
end delete_server;
/
show errors;

create or replace
procedure delete_server_bulk (
	user_id_in in number
) is
	cursor systems is
		select	s.element id
		from	rhnSet s
		where	s.user_id = user_id_in
			and s.label = 'system_list';
	cursor servergroups is
		select	sgm.server_group_id, sgm.server_id
		from	rhnServerGroupMembers sgm,
			rhnSet s
		where	s.user_id = user_id_in
			and s.label = 'system_list'
			and s.element = sgm.server_id;
	cursor configchannels is
		select	cc.id
		from	rhnConfigChannel cc,
			rhnConfigChannelType cct,
			rhnServerConfigChannel scc,
			rhnSet s
		where	1=1
			and s.user_id = user_id_in
			and s.label = 'system_list'
			and s.element = scc.server_id
			and scc.config_channel_id = cc.id
			-- these config channel types are reserved
			-- for use by a single server, so we don't
			-- need to check for other servers subscribed
			and cct.label in
				('local_override','server_import')
			and cct.id = cc.confchan_type_id;
	cursor filelists is
		select	spfl.file_list_id id
		from	rhnServerPreserveFileList spfl,
			rhnSet s
		where	s.user_id = user_id_in
			and s.label = 'system_list'
			and s.element = spfl.server_id
			and not exists (
				select	1
				from	rhnKickstartPreserveFileList
				where	file_list_id = spfl.file_list_id
			)
			and not exists (
				-- this has the org_id just to make the set -much- smaller.
				select	spfl.server_id
				from	rhnServerPreserveFileList spfl,
					rhnServer s,
					web_contact u
				where	u.id = user_id_in
					and u.org_id = s.org_id
					and s.id = spfl.server_id
					and spfl.file_list_id = spfl.file_list_id
				minus
				select	element server_id
				from	rhnSet
				where	s.user_id = user_id_in
					and s.label = 'system_list'
			);
	cluster_id number;
    is_virt number := 0;
begin
	for filelist in filelists loop
		delete from rhnFileList where id = filelist.id;
	end loop;

	for cc in configchannels loop
		rhn_config.delete_channel(cc.id);
	end loop;

	for sgm in servergroups loop
        begin
          select 1 into is_virt
            from rhnServerEntitlementView
           where server_id = sgm.server_id
             and label in ('virtualization_host', 'virtualization_host_platform');
        exception
          when no_data_found then
            is_virt := 0;
        end;

		rhn_server.delete_from_servergroup(
			sgm.server_id, sgm.server_group_id);

        if is_virt = 1 then
            rhn_entitlements.repoll_virt_guest_entitlements(sgm.server_id);
        end if;
	end loop;

	-- it's too bad you can't use "current of" with deletes
	for s in systems loop
		-- we can't use a cursor like planned to for delete_server
		-- to fix rhnKickstartSession entries, because
		-- we miss entries where old_server_id and new_server_id
		-- are both in the set.
		update rhnKickstartSession
			set old_server_id = null
			where old_server_id = s.id;
		update rhnKickstartSession
			set new_server_id = null
			where new_server_id = s.id;
		rhn_channel.clear_subscriptions(s.id,1);

                -- A little complicated here, but the goal is to
		-- delete records from rhnVirtualInstace only if we don't
		-- care about them anymore.  We don't care about records
		-- in rhnVirtualInstance if we are deleting the host
		-- system and the virtual system is already null, or
		-- vice-versa.  We *do* care about them if either the
		-- host or virtual system is still registered because we
		-- still want them to show up in the UI.

                delete from rhnVirtualInstance
		      where host_system_id = s.id
		        and virtual_system_id is null;
                delete from rhnVirtualInstance
		      where virtual_system_id = s.id
		        and host_system_id is null;

		update rhnVirtualInstance
			set host_system_id = null
			where host_system_id = s.id;
		update rhnVirtualInstance
			set virtual_system_id = null
			where virtual_system_id = s.id;

		update rhnVirtualInstanceEventLog
		   set old_host_system_id = null
	         where old_host_system_id = s.id;

		update rhnVirtualInstanceEventLog
		   set new_host_system_id = null
	         where new_host_system_id = s.id;

		-- We're deleting everything with a foreign key to rhnServer
		-- here, now.  I'm hoping this will help aleviate our deadlock
		-- problem.

		delete from rhnActionConfigChannel where server_id = s.id;
		delete from rhnActionConfigRevision where server_id = s.id;
		delete from rhnActionPackageRemovalFailure where server_id = s.id;
		delete from rhnChannelFamilyLicenseConsent where server_id = s.id;
		delete from rhnClientCapability where server_id = s.id;
		delete from rhnCpu where server_id = s.id;
		-- there's still a cascade here, because the constraint keeps the
		-- table locked for too long to rebuild it.  Ugh...
		delete from rhnDevice where server_id = s.id;
		delete from rhnProxyInfo where server_id = s.id;
		delete from rhnRam where server_id = s.id;
		delete from rhnRegToken where server_id = s.id;
		delete from rhnSNPServerQueue where server_id = s.id;
		delete from rhnSatelliteChannelFamily where server_id = s.id;
		delete from rhnSatelliteInfo where server_id = s.id;
		-- this cascades to rhnActionConfigChannel and rhnActionConfigFileName
		delete from rhnServerAction where server_id = s.id;
		delete from rhnServerActionPackageResult where server_id = s.id;
		delete from rhnServerActionScriptResult where server_id = s.id;
		delete from rhnServerActionVerifyResult where server_id = s.id;
		delete from rhnServerActionVerifyMissing where server_id = s.id;
		-- counts are handled above.  this should be a delete_ function.
		delete from rhnServerChannel where server_id = s.id;
		delete from rhnServerConfigChannel where server_id = s.id;
		delete from rhnServerCustomDataValue where server_id = s.id;
		delete from rhnServerDMI where server_id = s.id;
		delete from rhnServerMessage where server_id = s.id;
		-- this gets rhnServerMessage (only) on cascade; it's handled just above
		delete from rhnServerEvent where server_id = s.id;
		delete from rhnServerHistory where server_id = s.id;
		delete from rhnServerInfo where server_id = s.id;
		delete from rhnServerInstallInfo where server_id = s.id;
		delete from rhnServerLocation where server_id = s.id;
		delete from rhnServerLock where server_id = s.id;
		delete from rhnServerNeededPackageCache where server_id = s.id;
		delete from rhnServerNeededErrataCache where server_id = s.id;
		delete from rhnServerNetwork where server_id = s.id;
		delete from rhnServerNotes where server_id = s.id;
		-- I'm not removing the foreign key from rhnServerPackage; that'll
		-- take forever.  Do the delete anyway.
		delete from rhnServerPackage where server_id = s.id;
		delete from rhnServerTokenRegs where server_id = s.id;
		delete from rhnSnapshotTag where server_id = s.id;
		-- this cascades to:
		--   rhnSnapshotChannel, rhnSnapshotConfigChannel, rhnSnapshotPackage,
		--   rhnSnapshotConfigRevision, rhnSnapshotServerGroup,
		--   rhnSnapshotTag.
		-- We may want to consider delete_snapshot() at some point, but
		--   I don't think we need to yet.
		delete from rhnSnapshot where server_id = s.id;
		delete from rhnTransaction where server_id = s.id;
		delete from rhnUserServerPrefs where server_id = s.id;
		-- hrm, this one's interesting... we _probably_ should delete
		-- everything for the parent server_id when we delete the proxy,
		-- but we don't currently.
		delete from rhnServerPath where s.id in (server_id, proxy_server_id);
		delete from rhnUserServerPerms where server_id = s.id;

		delete from rhn_interface_monitoring where server_id = s.id;
		delete from rhnServerNetInterface where server_id = s.id;
		delete from rhn_server_monitoring_info where recid = s.id;

		delete from rhnAppInstallSession where server_id = s.id;
		delete from rhnServerUuid where server_id = s.id;

        DELETE FROM rhn_probe_state PS WHERE PS.probe_id in
        (SELECT CP.probe_id
           FROM rhn_check_probe CP
          WHERE CP.host_id = s.id
        );

        DELETE FROM rhn_probe P  WHERE P.recid in
        (SELECT CP.probe_id
           FROM rhn_check_probe CP
          WHERE CP.host_id = s.id
        );

        -- Now we delete any probes that were using this Server
        -- as a Proxy Scout.
        DELETE
          FROM rhn_probe_state PS
         WHERE PS.probe_id in
        (SELECT CP.probe_id
           FROM rhn_check_probe CP
          WHERE CP.sat_cluster_id in
        (SELECT SN.sat_cluster_id
           FROM rhn_sat_node SN
          WHERE SN.server_id = s.id));

        DELETE FROM rhn_probe P
         WHERE P.recid in
        (SELECT CP.probe_id
          FROM rhn_check_probe CP
           WHERE CP.sat_cluster_id in
        (SELECT SN.sat_cluster_id
           FROM rhn_sat_node SN
           WHERE SN.server_id = s.id));

        delete from rhn_check_probe where host_id = s.id;
        delete from rhn_host_probe where host_id = s.id;

        delete from rhn_sat_cluster where recid in
         ( select sat_cluster_id from rhn_sat_node where server_id = s.id );
	delete from rhn_sat_node where server_id = s.id;

		delete from rhnServer
			where id = s.id;
		delete from rhnSet
			where user_id in (
				select	wc.id
				from	rhnServer rs,
					web_contact wc
				where	rs.id = s.id
					and rs.org_id = wc.org_id
			)
			and label = 'system_list'
			and element = s.id;
	end loop;
end delete_server_bulk;
/
show errors;

--
-- Table data changes
--
insert into rhnActionType values (35, 'kickstart_guest.initiate', 'Initiate a kickstart for a Xen guest.','N','Y');
insert into rhnActionType values (36, 'virt.shutdown', 'Shuts down a Xen domain.', 'N', 'N');
insert into rhnActionType values (37, 'virt.start', 'Starts up a Xen domain.', 'N', 'N');
insert into rhnActionType values (38, 'virt.suspend', 'Suspends a Xen domain.', 'N', 'N');
insert into rhnActionType values (39, 'virt.resume', 'Resumes a Xen domain.', 'N', 'N');
insert into rhnActionType values (40, 'virt.reboot', 'Reboots a Xen domain.', 'N', 'N');
insert into rhnActionType values (41, 'virt.destroy', 'Destroys a Xen Domain.', 'N', 'N');
insert into rhnActionType values (42, 'virt.setMemory', 'Sets the maximum memory usage for a Xen domain.', 'N', 'N');
insert into rhnActionType values (43, 'virt.schedulePoller', 'Sets when the poller should run.', 'N', 'N');
insert into rhnActionType values (44, 'kickstart_host.schedule_virt_host_pkg_install', 'Schedule a package install of host specific functionality.', 'N', 'N');
insert into rhnActionType values (45, 'kickstart_guest.schedule_virt_guest_pkg_install', 'Schedule a package install of guest specific functionality.', 'N', 'N');
insert into rhnActionType values (46, 'kickstart_host.add_tools_channel', 'Subscribes a server to the RHN Tools channel associated with its base channel.', 'N', 'N');
insert into rhnActionType values (47, 'kickstart_guest.add_tools_channel', 'Subscribes a virtualization guest to the RHN Tools channel associated with its base channel.', 'N', 'N');
insert into rhnActionType values (48, 'virt.setVCPUs', 'Sets the Vcpu  usage for a Xen domain.', 'N', 'N');
insert into rhnException
values (-20289,
        'invalid_virt_sub_level',
        'The specified virtualization subscription level does not exist.');
insert into rhnFeature (id, label, name)
values (rhn_feature_seq.nextval, 'ftr_xen_provision_domain', 'Provision new Xen domains');
insert into rhnFeature (id, label, name)
values (rhn_feature_seq.nextval, 'ftr_xen_manage_domains', 'Manage existing Xen domains');
insert into RHNINFOPANE(ID,LABEL,ACL) values (rhn_info_pane_id_seq.nextval, 'recently-registered-systems', null);
insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order, required)
values (rhn_kscommandname_id_seq.nextval, 'repo', 'Y', 33, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order, required)
values (rhn_kscommandname_id_seq.nextval, 'key', 'Y', 34, 'N');

--
-- Update existing RHEL 5 kickstarts (246300)
--

insert into rhnkickstartcommand
	( id, kickstart_id, ks_command_name_id, arguments, created )
select rhn_kscommand_id_seq.nextval,
rhnkickstartdefaults.kickstart_id,key_id.id, '--skip', sysdate
from rhnkickstartdefaults, (
	select rhnkickstartcommandname.id
	from rhnkickstartcommandname
	where rhnkickstartcommandname.name = 'key'
	) key_id, rhnkickstartabletree, rhnksinstalltype
where rhnkickstartdefaults.kstree_id = rhnkickstartabletree.id
	and rhnkickstartabletree.install_type = rhnksinstalltype.id
	and rhnksinstalltype.label = 'rhel_5'
	and not exists (
		select 1
		from rhnkickstartcommand
		where rhnkickstartdefaults.kickstart_id = rhnkickstartcommand.kickstart_id
			and rhnkickstartcommand.ks_command_name_id = key_id.id
	);
insert into rhnkickstartcommand
	( id, kickstart_id, ks_command_name_id, arguments, created )
select rhn_kscommand_id_seq.nextval, rhnkickstartdefaults.kickstart_id,
	repo_id.id,
	'--name=' || names.name || ' --baseurl='
		|| rtrim(replace(replace(rhnkickstartcommand.arguments,
			'--url ', ''), 'rhn/kickstart', 'kickstart/dist'), '/')
		|| '/' || names.name,
	sysdate
from rhnkickstartdefaults, (
	select rhnkickstartcommandname.id
	from rhnkickstartcommandname
	where rhnkickstartcommandname.name = 'repo'
	) repo_id, rhnkickstartabletree, rhnksinstalltype,
	(
	select rhnkickstartcommandname.id
	from rhnkickstartcommandname
	where rhnkickstartcommandname.name = 'url'
	) url_id, rhnkickstartcommand,
	(
	select 'Cluster' name from dual
	union all
	select 'ClusterStorage' name from dual
	union all
	select 'Workstation' name from dual
	union all
	select 'VT' name from dual
	) names
where rhnkickstartdefaults.kstree_id = rhnkickstartabletree.id
	and rhnkickstartabletree.install_type = rhnksinstalltype.id
	and rhnksinstalltype.label = 'rhel_5'
	and not exists (
		select 1
		from rhnkickstartcommand x
		where rhnkickstartdefaults.kickstart_id = x.kickstart_id
			and x.ks_command_name_id = repo_id.id
	)
	and rhnkickstartdefaults.kickstart_id = rhnkickstartcommand.kickstart_id
	and rhnkickstartcommand.ks_command_name_id = url_id.id
	and rhnkickstartcommand.arguments like '--url http%';

insert into rhnKickstartTimezone (id, label, name, install_type)
        values (rhn_ks_timezone_id_seq.nextval,
           'America/Toronto',
           'America/Toronto',
               (SELECT IT.id FROM rhnKSInstallType IT WHERE IT.label = 'rhel_2.1')
        );
insert into rhnKickstartTimezone (id, label, name, install_type)
        values (rhn_ks_timezone_id_seq.nextval,
           'America/Toronto',
           'America/Toronto',
               (SELECT IT.id FROM rhnKSInstallType IT WHERE IT.label = 'rhel_3')
        );

insert into rhnKickstartTimezone (id, label, name, install_type)
        values (rhn_ks_timezone_id_seq.nextval,
           'America/Toronto',
           'America/Toronto',
               (SELECT IT.id FROM rhnKSInstallType IT WHERE IT.label = 'rhel_4')
        );

insert into rhnKickstartTimezone (id, label, name, install_type)
        values (rhn_ks_timezone_id_seq.nextval,
           'America/Toronto',
           'America/Toronto',
               (SELECT IT.id FROM rhnKSInstallType IT WHERE IT.label = 'rhel_5')
        );

insert into rhnOrgEntitlementType (id, label, name)
        values (rhn_org_entitlement_type_seq.nextval,
                'rhn_virtualization', 'Red Hat Network Virtualization'
        );

insert into rhnOrgEntitlementType (id, label, name)
        values (rhn_org_entitlement_type_seq.nextval,
                'rhn_virtualization_platform', 'Red Hat Network Virtualization Platform'
        );


insert into rhnServerGroupType ( id, label, name, permanent, is_base)
   values ( rhn_servergroup_type_seq.nextval,
      'virtualization_host', 'Virtualization Host Entitled Servers',
      'N', 'N'
   );

insert into rhnServerGroupType ( id, label, name, permanent, is_base)
   values ( rhn_servergroup_type_seq.nextval,
      'virtualization_host_platform',
      'Virtualization Host Platform Entitled Servers',
      'N', 'N'
   );
insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('i386-redhat-linux'),
            lookup_sg_type('virtualization_host'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('i486-redhat-linux'),
            lookup_sg_type('virtualization_host'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('i586-redhat-linux'),
            lookup_sg_type('virtualization_host'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('i686-redhat-linux'),
            lookup_sg_type('virtualization_host'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('athlon-redhat-linux'),
            lookup_sg_type('virtualization_host'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('amd64-redhat-linux'),
            lookup_sg_type('virtualization_host'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('ia64-redhat-linux'),
            lookup_sg_type('virtualization_host'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('ia32e-redhat-linux'),
            lookup_sg_type('virtualization_host'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('s390-redhat-linux'),
            lookup_sg_type('virtualization_host'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('s390x-redhat-linux'),
            lookup_sg_type('virtualization_host'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('ppc-redhat-linux'),
            lookup_sg_type('virtualization_host'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('ppc64-redhat-linux'),
            lookup_sg_type('virtualization_host'));
insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('x86_64-redhat-linux'),
            lookup_sg_type('virtualization_host'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('i386-redhat-linux'),
            lookup_sg_type('virtualization_host_platform'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('i486-redhat-linux'),
            lookup_sg_type('virtualization_host_platform'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('i586-redhat-linux'),
            lookup_sg_type('virtualization_host_platform'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('i686-redhat-linux'),
            lookup_sg_type('virtualization_host_platform'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('athlon-redhat-linux'),
            lookup_sg_type('virtualization_host_platform'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('amd64-redhat-linux'),
            lookup_sg_type('virtualization_host_platform'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('ia64-redhat-linux'),
            lookup_sg_type('virtualization_host_platform'));

insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('ia32e-redhat-linux'),
            lookup_sg_type('virtualization_host_platform'));
insert into rhnServerServerGroupArchCompat ( server_arch_id, server_group_type)
    values (lookup_server_arch('x86_64-redhat-linux'),
            lookup_sg_type('virtualization_host_platform'));
insert into rhnSGTypeBaseAddonCompat (base_id, addon_id)
values (lookup_sg_type('enterprise_entitled'),
        lookup_sg_type('virtualization_host'));

insert into rhnSGTypeBaseAddonCompat (base_id, addon_id)
values (lookup_sg_type('enterprise_entitled'),
        lookup_sg_type('virtualization_host_platform'));

insert into rhnSGTypeBaseAddonCompat (base_id, addon_id)
values (lookup_sg_type('sw_mgr_entitled'),
        lookup_sg_type('virtualization_host'));

insert into rhnSGTypeBaseAddonCompat (base_id, addon_id)
values (lookup_sg_type('sw_mgr_entitled'),
        lookup_sg_type('virtualization_host_platform'));

--
-- New table data
--
INSERT INTO RHNErrataSeverity VALUES (0, 0, 'errata.sev.label.critical');
INSERT INTO RHNErrataSeverity VALUES (1, 1, 'errata.sev.label.important');
INSERT INTO RHNErrataSeverity VALUES (2, 2, 'errata.sev.label.moderate');
INSERT INTO RHNErrataSeverity VALUES (3, 3, 'errata.sev.label.low');
insert into rhnKickstartVirtualizationType (id, name, label)
     values (rhn_kvt_id_seq.nextval, 'Para-Virtualized Host', 'para_host');
insert into rhnKickstartVirtualizationType (id, name, label)
     values (rhn_kvt_id_seq.nextval, 'Para-Virtualized Guest', 'para_guest');
insert into rhnKickstartVirtualizationType (id, name, label)
     values (rhn_kvt_id_seq.nextval, 'none', 'none');
insert into rhnVirtSubLevel(id, label, name, created, modified)
values(rhn_virt_sl_seq.nextval, 'virtualization_free',
       'Virtualization free content group',
       sysdate, sysdate);
insert into rhnVirtSubLevel(id, label, name, created, modified)
values(rhn_virt_sl_seq.nextval, 'virtualization_platform_free',
       'Virtualization Platform free content group',
       sysdate, sysdate);
insert into rhnSGTypeVirtSubLevel
values(lookup_sg_type('virtualization_host'),
       lookup_virt_sub_level('virtualization_free'),
       sysdate, sysdate);
insert into rhnSGTypeVirtSubLevel
values(lookup_sg_type('virtualization_host_platform'),
       lookup_virt_sub_level('virtualization_platform_free'),
       sysdate, sysdate);
insert into rhnVirtualInstanceEventType (id, name, label) values (rhn_viet_id_seq.nextval, 'Create', 'create');
insert into rhnVirtualInstanceEventType (id, name, label) values (rhn_viet_id_seq.nextval, 'Destroy', 'destroy');
insert into rhnVirtualInstanceEventType (id, name, label) values (rhn_viet_id_seq.nextval, 'Shutdown', 'shutdown');
insert into rhnVirtualInstanceEventType (id, name, label) values (rhn_viet_id_seq.nextval, 'Pause', 'pause');
insert into rhnVirtualInstanceEventType (id, name, label) values (rhn_viet_id_seq.nextval, 'Unpause', 'unpause');
insert into rhnVirtualInstanceEventType (id, name, label) values (rhn_viet_id_seq.nextval, 'Migrate', 'migrate');
insert into rhnVirtualInstanceEventType (id, name, label) values (rhn_viet_id_seq.nextval, 'Vcpu-Set', 'vcpu-set');
insert into rhnVirtualInstanceEventType (id, name, label) values (rhn_viet_id_seq.nextval, 'Mem-Set', 'mem-set');
insert into rhnVirtualInstanceState (id, name, label) values (rhn_vis_id_seq.nextval, 'Unknown', 'unknown');
insert into rhnVirtualInstanceState (id, name, label) values (rhn_vis_id_seq.nextval, 'Running', 'running');
insert into rhnVirtualInstanceState (id, name, label) values (rhn_vis_id_seq.nextval, 'Stopped', 'stopped');
insert into rhnVirtualInstanceState (id, name, label) values (rhn_vis_id_seq.nextval, 'Crashed', 'crashed');
insert into rhnVirtualInstanceState (id, name, label) values (rhn_vis_id_seq.nextval, 'Paused', 'paused');
insert into rhnVirtualInstanceType (id, name, label)
     values (rhn_vit_id_seq.nextval, 'Fully Virtualized', 'fully_virtualized');
insert into rhnVirtualInstanceType (id, name, label)
     values (rhn_vit_id_seq.nextval, 'Para-Virtualized', 'para_virtualized');


update rhnkickstartdefaults set virtualization_type = (
	select id from rhnKickstartVirtualizationType where label = 'none'
	)
where virtualization_type is null;
alter table rhnkickstartdefaults modify virtualization_type constraint rhn_ksd_kvt_nn not null ;
update rhnkickstartsession set virtualization_type = (
	select id from rhnKickstartVirtualizationType where label = 'none'
	)
where virtualization_type is null;
alter table rhnkickstartsession modify virtualization_type constraint rhn_kss_kvt_nn not null ;

-- keep cloned channels
insert into rhnchannelcloned select from_channel_id as original_id,
to_channel_id as id, created, modified from rhnchannelrelationship;

-- keep cloned errata
insert into rhnerratacloned select from_errata_id as original_id, to_errata_id
as id, created, modified from rhnerratarelationship;

-- keep unpublished cloned errata
insert into rhnerrataclonedtmp select from_errata_id as original_id, to_errata_id
as id, created, modified from rhnerratarelationshiptmp;

-- more Drops
-- as we need to have these around to copy the data
-- before dropping them.
DROP TABLE RHNERRATARELATIONSHIPTMP CASCADE CONSTRAINTS;
DROP TABLE RHNERRATARELATIONSHIP CASCADE CONSTRAINTS;
DROP TABLE RHNCHANNELRELATIONSHIP CASCADE CONSTRAINTS;

--
-- Compile
--
declare
   schema_name varchar2(30);
begin
   select owner
   into schema_name
   from all_tables
   where table_name = 'RHNSERVER';

   dbms_utility.compile_schema (schema_name);
end;
/
show errors;

alter type evr_t compile body;
