set echo on;
whenever sqlerror exit failure;

spool satellite-3.6-to-3.7.log

variable evr_id number;
variable epoch varchar2(16);
variable version varchar2(64);
variable release varchar2(64);

variable message varchar2(80);

declare
   cursor evrs is
      select   e.id, e.epoch, e.version, e.release, e.evr
      from  rhnPackageEVR e,
         rhnVersionInfo rvi
      where rvi.label = 'schema'
         and rvi.name_id =
            lookup_package_name('rhn-satellite-schema')
         and rvi.evr_id = e.id;
   cursor valid_evrs is
      select   1
      from  dual
      where :evr_id in (
         lookup_evr('','3.6','216')
         );
begin
   :evr_id := null;
   :message := 'XXX Invalid satellite schema version.';
   for evr in evrs loop
      :evr_id := evr.id;
      :epoch := evr.epoch;
      :version := evr.version;
      :release := evr.release;
      :message :=  '*** Schema version is currently ' ||
         evr.evr.as_vre_simple() ||
         ', and will NOT be upgraded';
      for vevr in valid_evrs loop
         :message :=  '*** Schema version is currently ' ||
            evr.evr.as_vre_simple() ||
            ', and will be upgraded';
      end loop;
      return;
   end loop;
end;
/


select :message from dual;

declare
   invalid_schema_version exception;
   cursor valid_evrs is
      select   1
      from  dual
      where :evr_id in (
         lookup_evr('','3.6','216')
         );
begin
   for vevr in valid_evrs loop
      return;
   end loop;
   raise invalid_schema_version;
end;
/
show errors

--------------------------------------------------------------------------
-- "Set define off" turns off substitution variables
Set define off;
--
-- 1. RHN_CMETH_TZNMS_SCHED_ZONE_ID  (Constraint) 
--
ALTER TABLE RHN_CONTACT_METHODS DROP CONSTRAINT RHN_CMETH_TZNMS_SCHED_ZONE_ID;

--
-- 2. RHN_OSTATE_MOD_TRIG  (Trigger) 
--
DROP TRIGGER RHN_OSTATE_MOD_TRIG;

--
-- 3. RHN_OASTATES_MOD_TRIG  (Trigger) 
--
DROP TRIGGER RHN_OASTATES_MOD_TRIG;

--
-- 4. RHNPAIDERRATAMAIL  (View) 
-- 5. RHNFREEERRATAMAIL  (View) 
--
DROP VIEW RHNPAIDERRATAMAIL;
DROP VIEW RHNFREEERRATAMAIL;

--
-- RHNUSERSURVEYSTATUSTYPE (Table)
--
DROP TABLE RHNUSERSURVEYSTATUSTYPE CASCADE CONSTRAINTS;

--
-- 28. RHNUSERSURVEYSTATUS  (Table) 
--
DROP TABLE RHNUSERSURVEYSTATUS CASCADE CONSTRAINTS;

--
-- 29. RHNSURVEYQUESTIONRESPONSE  (Table) 
--
DROP TABLE RHNSURVEYQUESTIONRESPONSE CASCADE CONSTRAINTS;

--
-- 30. RHNSURVEYQUESTION  (Table) 
--
DROP TABLE RHNSURVEYQUESTION CASCADE CONSTRAINTS;

--
-- 31. RHNSURVEY  (Table) 
--
DROP TABLE RHNSURVEY CASCADE CONSTRAINTS;

--
-- 32. RHNPAYLOADERCONVERSION  (Table) 
--
DROP TABLE RHNPAYLOADERCONVERSION CASCADE CONSTRAINTS;

--
-- 33. RHNORGSTATETRANSITIONS  (Table) 
--
DROP TABLE RHNORGSTATETRANSITIONS CASCADE CONSTRAINTS;

--
-- 34. RHNORGSTATELOG  (Table) 
--
DROP TABLE RHNORGSTATELOG CASCADE CONSTRAINTS;

--
-- 35. RHNORGSTATE  (Table) 
--
DROP TABLE RHNORGSTATE CASCADE CONSTRAINTS;

--
-- 36. RHNORGAVAILABLESTATES  (Table) 
--
DROP TABLE RHNORGAVAILABLESTATES CASCADE CONSTRAINTS;

--
-- 37. RHN_USURVEYSTYPE_ID_SEQ  (Sequence) 
--
DROP SEQUENCE RHN_USURVEYSTYPE_ID_SEQ;

--
-- 38. RHN_SURVEY_ID_SEQ  (Sequence) 
--
DROP SEQUENCE RHN_SURVEY_ID_SEQ;

--
-- 39. RHN_SQUESTION_ID_SEQ  (Sequence) 
--
DROP SEQUENCE RHN_SQUESTION_ID_SEQ;

--
-- 40. RHN_OASTATES_ID_SEQ  (Sequence) 
--
DROP SEQUENCE RHN_OASTATES_ID_SEQ;

--
-- 41. RHNCONFIGFILETYPE  (Table) 
--
create table rhnConfigFileType
(
        id                      number
                                constraint rhn_conffiletype_id_nn not null
                                constraint rhn_conffiletype_id_pk primary key,
--                                        using index tablespace [[2m_tbs]],
   label    varchar2(64)
            constraint rhn_conffiletype_label_nn not null,
   name     varchar2(256)
            constraint rhn_conffiletype_name_nn not null,
        created                 date default(sysdate)
                                constraint rhn_conffiletype_creat_nn not null,
        modified                date default(sysdate)
                                constraint rhn_conffiletype_mod_nn not null
)
        storage ( freelists 16 )
        initrans 32;

create or replace trigger
rhn_conffiletype_mod_trig
before insert or update on rhnConfigFile
for each row
begin
        :new.modified := sysdate;
end;
/

--
-- Need the data for rhnConfigFileType
--
insert into rhnConfigFileType values (1, 'file', 'File', sysdate, sysdate);
insert into rhnConfigFileType values (2, 'directory', 'Directory', sysdate, sysdate);



ALTER TABLE RHNCONFIGREVISION ADD (CONFIG_FILE_TYPE_ID  NUMBER 
					DEFAULT 1 
					CONSTRAINT RHN_CONF_REV_CFTI_NN NOT NULL
					CONSTRAINT RHN_CONF_REV_CFTI_FK references rhnConfigFileType(id));


--
-- RHN_BEL  (Package Body) 
--
CREATE OR REPLACE package body rhn_bel
is
   function is_org_paid (
      org_id_in in number
   ) return number is
      cursor paids is
         select   1
         from  rhnPaidOrgs
         where org_id = org_id_in;
   begin
      for paid in paids loop return 1; end loop;
      return 0;
   end;
        function lookup_email_state (
                state_in in varchar2
        ) return number is
                retval number;
        begin
                select id into retval from rhnEmailAddressState where label = state_in;
                return retval;
        exception
                        when no_data_found then
                                rhn_exception.raise_exception('invalid_state');
        end lookup_email_state;
end;
/

SHOW ERRORS;

--
-- 44. RHN_CHANNEL  (Package Body) 
--
CREATE OR REPLACE PACKAGE BODY rhn_channel
IS
	body_version varchar2(100) := '$Id: satellite-3.6-to-3.7.sql 64427 2005-09-09 14:08:21Z jslagle $';
	cursor	base_channel_cursor(
		release_in in varchar2,
		server_arch_id_in in number,
		org_id_in in number
	) return rhnChannel%ROWTYPE is
		select distinct c.*
		from	rhnDistChannelMap			dcm,
				rhnServerChannelArchCompat	scac,
				rhnChannel					c,
				rhnChannelPermissions		cp
		where	cp.org_id = org_id_in
			and cp.channel_id = c.id
			and c.parent_channel is null
			and c.id = dcm.channel_id
			and c.channel_arch_id = dcm.channel_arch_id
			and dcm.release = release_in
			and scac.server_arch_id = server_arch_id_in
			and scac.channel_arch_id = c.channel_arch_id;
    FUNCTION get_license_path(channel_id_in IN NUMBER)
    RETURN VARCHAR2
    IS
    	license_val VARCHAR2(1000);
    BEGIN
    	SELECT CFL.license_path INTO license_val
	  FROM rhnChannelFamilyLicense CFL, rhnChannelFamilyMembers CFM
	 WHERE CFM.channel_id = channel_id_in
	   AND CFM.channel_family_id = CFL.channel_family_id;
    	RETURN license_val;
    EXCEPTION
    	WHEN NO_DATA_FOUND
	    THEN
	    RETURN NULL;
    END get_license_path;
    PROCEDURE license_consent(channel_id_in IN NUMBER, user_id_in IN NUMBER, server_id_in IN NUMBER)
    IS
    	channel_family_id_val NUMBER;
    BEGIN
    	channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
	IF channel_family_id_val IS NULL
	THEN
	    rhn_exception.raise_exception('channel_subscribe_no_family');
	END IF;
	IF rhn_channel.get_license_path(channel_id_in) IS NULL
	THEN
	    rhn_exception.raise_exception('channel_consent_no_license');
	END IF;
	INSERT INTO rhnChannelFamilyLicenseConsent (channel_family_id, user_id, server_id)
	VALUES (channel_family_id_val, user_id_in, server_id_in);
    END license_consent;
    PROCEDURE subscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, user_id_in in number := null)
    IS
        channel_parent_val      rhnChannel.parent_channel%TYPE;
        parent_subscribed       BOOLEAN;
        server_has_base_chan    BOOLEAN;
	server_already_in_chan  BOOLEAN;
	channel_family_id_val   NUMBER;
	server_org_id_val       NUMBER;
	available_subscriptions NUMBER;
	consenting_user         NUMBER;
	allowed			number := 0;
    BEGIN
    	if user_id_in is not null then
		allowed := rhn_channel.user_role_check(channel_id_in, user_id_in, 'subscribe');
	else
		allowed := 1;
	end if;
	if allowed = 0 then
		rhn_exception.raise_exception('no_subscribe_permissions');
	end if;
        SELECT parent_channel INTO channel_parent_val FROM rhnChannel WHERE id = channel_id_in;
        IF channel_parent_val IS NOT NULL
        THEN
            parent_subscribed := FALSE;
            FOR check_subscription IN check_server_subscription(server_id_in, channel_parent_val)
            LOOP
                parent_subscribed := TRUE;
            END LOOP check_subscription;
            IF NOT parent_subscribed
            THEN
    	    	RETURN;
            END IF;
        ELSE
            server_has_base_chan := FALSE;
            FOR base IN server_base_subscriptions(server_id_in)
            LOOP
                server_has_base_chan := TRUE;
            END LOOP base;
            IF server_has_base_chan
            THEN
                rhn_exception.raise_exception('channel_server_one_base');
            END IF;
        END IF;
        FOR check_subscription IN check_server_subscription(server_id_in, channel_id_in)
        LOOP
            server_already_in_chan := TRUE;
        END LOOP check_subscription;
        IF server_already_in_chan
        THEN
    	    RETURN;
        END IF;
	channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
	IF channel_family_id_val IS NULL
	THEN
	    rhn_exception.raise_exception('channel_subscribe_no_family');
	END IF;
	SELECT org_id INTO server_org_id_val
	  FROM rhnServer
	 WHERE id = server_id_in;
	available_subscriptions := rhn_channel.available_family_subscriptions(channel_family_id_val, server_org_id_val);
	IF available_subscriptions IS NULL OR available_subscriptions > 0
	THEN
	    IF rhn_channel.get_license_path(channel_id_in) IS NOT NULL
	    THEN
	    	BEGIN
	    	SELECT user_id INTO consenting_user
		  FROM rhnChannelFamilyLicenseConsent
		 WHERE channel_family_id = channel_family_id_val
		   AND server_id = server_id_in;
		EXCEPTION
    	    	    WHEN NO_DATA_FOUND
		    	THEN
			    rhn_exception.raise_exception('channel_subscribe_no_consent');
		END;
	    END IF;
	    insert into rhnServerHistory (id,server_id,summary,details) (
	    	select	rhn_event_id_seq.nextval,
			server_id_in,
			'subscribed to channel ' || SUBSTR(c.label, 0, 106),
			c.label
		from	rhnChannel c
		where	c.id = channel_id_in
	    );
	    UPDATE rhnServer SET channels_changed = sysdate WHERE id = server_id_in;
            INSERT INTO rhnServerChannel (server_id, channel_id) VALUES (server_id_in, channel_id_in);
    	    rhn_channel.update_family_counts(channel_family_id_val, server_org_id_val);
    	    queue_server(server_id_in, immediate_in);
	ELSE
	    rhn_exception.raise_exception('channel_family_no_subscriptions');
	END IF;
    END subscribe_server;
    PROCEDURE bulk_subscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
            rhn_channel.subscribe_server(server.element, channel_id_in, 0, set_uid_in);
        END LOOP server;
    END bulk_subscribe_server;
    PROCEDURE bulk_server_base_change(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
	    IF rhn_server.can_change_base_channel(server.element) = 1
	    THEN
                rhn_channel.clear_subscriptions(TO_NUMBER(server.element));
                rhn_channel.subscribe_server(server.element, channel_id_in, 0, set_uid_in);
            END IF;
        END LOOP server;
    END bulk_server_base_change;
    procedure bulk_server_basechange_from(
        set_label_in in varchar2,
        set_uid_in in number,
        old_channel_id_in in number,
        new_channel_id_in in number
    ) is
    cursor servers is
        select  sc.server_id id
        from    rhnChannel nc,
                rhnServerChannelArchCompat scac,
                rhnServer s,
                rhnChannel oc,
                rhnServerChannel sc,
                rhnSet st
        where   1=1
            and st.label = set_label_in
            and st.user_id = set_uid_in
            and st.element = sc.server_id
            and sc.channel_id = old_channel_id_in
            and sc.channel_id = oc.id
            and oc.parent_channel is null
            and nc.id = new_channel_id_in
            and nc.parent_channel is null
            and sc.server_id = s.id
            and s.server_arch_id = scac.server_arch_id
            and scac.channel_arch_id = nc.channel_arch_id;
    begin
        for s in servers loop
            insert into rhnSet (
                    user_id, label, element
                ) values (
                    set_uid_in,
                    set_label_in || 'basechange',
                    s.id
                );
        end loop channel;
        bulk_server_base_change(new_channel_id_in,
                                set_label_in || 'basechange',
                                set_uid_in);
        delete from rhnSet
            where   label = set_label_in||'basechange'
                and user_id = set_uid_in;
    end bulk_server_basechange_from;
    procedure bulk_guess_server_base(
	set_label_in in varchar2,
	set_uid_in in number
    ) is
	channel_id number;
    begin
	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    begin
	    	if rhn_server.can_change_base_channel(server.element) = 1
		then
    	            channel_id := guess_server_base(TO_NUMBER(server.element));
	    	    rhn_channel.clear_subscriptions(TO_NUMBER(server.element));
	    	    rhn_channel.subscribe_server(TO_NUMBER(server.element), channel_id, 0, set_uid_in);
		end if;
	    exception when others then
	    	null;
	    end;
	end loop server;
    end;
    function guess_server_base(
    	server_id_in in number
    ) RETURN number is
	cursor server_cursor is
	    select s.server_arch_id, s.release, s.org_id
	      from rhnServer s
	     where s.id = server_id_in;
    begin
	for s in server_cursor loop
	    for channel in base_channel_cursor(s.release,
		s.server_arch_id, s.org_id)
	    loop
		return channel.id;
	    end loop base_channel_cursor;
        end loop server_cursor;
        return null;
    end;
    function normalize_server_arch(server_arch_in in varchar2)
    return varchar2
    is
	suffix VARCHAR2(128) := '-redhat-linux';
	suffix_len NUMBER := length(suffix);
    begin
	if server_arch_in is NULL then
	    return NULL;
	end if;
        if instr(server_arch_in, '-') > 0
	then
	    return server_arch_in;
	end if;
	return server_arch_in || suffix;
    end normalize_server_arch;
    function base_channel_for_release_arch(
	release_in in varchar2,
	server_arch_in in varchar2,
	org_id_in in number := -1,
	user_id_in in number := null
    ) return number is
	server_arch varchar2(256) := normalize_server_arch(server_arch_in);
	server_arch_id number;
    begin
	begin
	    select id
	      into server_arch_id
	      from rhnServerArch
	     where label = server_arch;
	exception
	    when no_data_found then
		rhn_exception.raise_exception('server_arch_not_found');
	end;
	return base_channel_rel_archid(release_in, server_arch_id,
	    org_id_in, user_id_in);
    end base_channel_for_release_arch;
    function base_channel_rel_archid(
	release_in in varchar2,
	server_arch_id_in in number,
	org_id_in in number := -1,
	user_id_in in number := null
    ) return number is
	denied_channel_id number := null;
	valid_org_id number := org_id_in;
	valid_user_id number := user_id_in;
	channel_subscribable number;
    begin
	if org_id_in = -1 and user_id_in is not null then
	    begin
		select org_id
		  into valid_org_id
		  from web_contact
		 where id = user_id_in;
	    exception
		when no_data_found then
		    valid_user_id := null;
		    valid_org_id := -1;
	    end;
	end if;
	for c in base_channel_cursor(release_in, server_arch_id_in, valid_org_id)
	loop
	    if valid_user_id is null then
		return c.id;
	    end if;
	    select loose_user_role_check(c.id, user_id_in, 'subscribe')
	      into channel_subscribable
	      from dual;
	    if channel_subscribable = 1 then
		return c.id;
	    end if;
	    denied_channel_id := c.id;
	end loop base_channel_fetch;
	if denied_channel_id is not null then
	    rhn_exception.raise_exception('no_subscribe_permissions');
	end if;
	return NULL;
    end base_channel_rel_archid;
    procedure bulk_guess_server_base_from(
	set_label_in in varchar2,
	set_uid_in in number,
	channel_id_in in number
    ) is
	cursor channels(server_id_in in number) is
	    select	rsc.channel_id
	    from	rhnServerChannel rsc,
			rhnChannel rc
	    where	server_id_in = rsc.server_id
			and rsc.channel_id = rc.id
			and rc.parent_channel is null;
    begin
	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    for channel in channels(server.element)
	    loop
		if channel.channel_id = channel_id_in
		then
		    insert into rhnSet (user_id, label, element) values (set_uid_in, set_label_in || 'baseguess', server.element);
		end if;
	    end loop channel;
	end loop server;
	bulk_guess_server_base(set_label_in||'baseguess',set_uid_in);
	delete from rhnSet where label = set_label_in||'baseguess' and user_id = set_uid_in;
    end;
    PROCEDURE clear_subscriptions(server_id_in IN NUMBER)
    IS
    	cursor server_channels(server_id_in in number) is
		select	s.org_id, sc.channel_id, cfm.channel_family_id
		from	rhnServer s,
			rhnServerChannel sc,
			rhnChannelFamilyMembers cfm
		where	s.id = server_id_in
			and s.id = sc.server_id
			and sc.channel_id = cfm.channel_id;
    BEGIN
    	for channel in server_channels(server_id_in)
	loop
		unsubscribe_server(server_id_in, channel.channel_id, 1, 1);
		rhn_channel.update_family_counts(channel.channel_family_id, channel.org_id);
	end loop channel;
    END clear_subscriptions;
    PROCEDURE unsubscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, unsubscribe_children_in number := 0)
    IS
	channel_family_id_val   NUMBER;
	server_org_id_val       NUMBER;
	available_subscriptions NUMBER;
	server_already_in_chan  BOOLEAN;
	cursor	channel_family_is_proxy(channel_family_id_in in number) is
		select	1
		from	rhnChannelFamily
		where	id = channel_family_id_in
		    and label = 'rhn-proxy';
	cursor	channel_family_is_satellite(channel_family_id_in in number) is
		select	1
		from	rhnChannelFamily
		where	id = channel_family_id_in
		    and label = 'rhn-satellite';
	cursor local_chk_server_parent_memb (
			server_id_in number,
			channel_id_in number ) is
		select	c.id
		from	rhnChannel			c,
				rhnServerChannel	sc
		where	1=1
			and c.parent_channel = channel_id_in
			and c.id = sc.channel_id
			and sc.server_id = server_id_in;
    BEGIN
        FOR child IN local_chk_server_parent_memb(server_id_in, channel_id_in)
        LOOP
	    if unsubscribe_children_in = 1 then
	    	unsubscribe_server(server_id_in => server_id_in,
								channel_id_in => child.id,
								immediate_in => immediate_in,
								unsubscribe_children_in => unsubscribe_children_in);
	    else
	    	rhn_exception.raise_exception('channel_unsubscribe_child_exists');
	    end if;
        END LOOP child;
	server_already_in_chan := FALSE;
        FOR check_subscription IN check_server_subscription(server_id_in, channel_id_in)
        LOOP
            server_already_in_chan := TRUE;
        END LOOP check_subscription;
        IF NOT server_already_in_chan
        THEN
    	    RETURN;
        END IF;
	insert into rhnServerHistory (id,server_id,summary,details) (
	    select  rhn_event_id_seq.nextval,
	    	    server_id_in,
		    'unsubscribed from channel ' || SUBSTR(c.label, 0, 106),
		    c.label
	    from    rhnChannel c
	    where   c.id = channel_id_in
	);
	UPDATE rhnServer SET channels_changed = sysdate WHERE id = server_id_in;
        DELETE FROM rhnServerChannel WHERE server_id = server_id_in AND channel_id = channel_id_in;
    	queue_server(server_id_in, immediate_in);
	channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
	IF channel_family_id_val IS NULL
	THEN
	    rhn_exception.raise_exception('channel_unsubscribe_no_family');
	END IF;
	for ignore in channel_family_is_satellite(channel_family_id_val) loop
		delete from rhnSatelliteInfo where server_id = server_id_in;
		delete from rhnSatelliteChannelFamily where server_id = server_id_in;
	end loop;
	for ignore in channel_family_is_proxy(channel_family_id_val) loop
		delete from rhnProxyInfo where server_id = server_id_in;
	end loop;
	DELETE FROM rhnChannelFamilyLicenseConsent
	 WHERE channel_family_id = channel_family_id_val
	   AND server_id = server_id_in;
	SELECT org_id INTO server_org_id_val
	  FROM rhnServer
	 WHERE id = server_id_in;
	rhn_channel.update_family_counts(channel_family_id_val, server_org_id_val);
    END unsubscribe_server;
    PROCEDURE bulk_unsubscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
            rhn_channel.unsubscribe_server(server.element, channel_id_in, 0);
        END LOOP server;
    END bulk_unsubscribe_server;
    FUNCTION family_for_channel(channel_id_in IN NUMBER)
    RETURN NUMBER
    IS
    	channel_family_id_val NUMBER;
    BEGIN
    	SELECT channel_family_id INTO channel_family_id_val
	  FROM rhnChannelFamilyMembers
         WHERE channel_id = channel_id_in;
	RETURN channel_family_id_val;
    EXCEPTION
        WHEN NO_DATA_FOUND
	THEN
	    RETURN NULL;
    END family_for_channel;
    FUNCTION available_family_subscriptions(channel_family_id_in IN NUMBER, org_id_in IN NUMBER)
    RETURN NUMBER
    IS
    	cfp channel_family_perm_cursor%ROWTYPE;
    	current_members_val NUMBER;
	max_members_val     NUMBER;
	found               NUMBER;
    BEGIN
	IF NOT channel_family_perm_cursor%ISOPEN
	THEN
	    OPEN channel_family_perm_cursor(channel_family_id_in, org_id_in);
	END IF;
	FETCH channel_family_perm_cursor INTO cfp;
	WHILE channel_family_perm_cursor%FOUND
    	LOOP
	    found := 1;
	    current_members_val := cfp.current_members;
	    max_members_val := cfp.max_members;
	    FETCH channel_family_perm_cursor INTO cfp;
	END LOOP;
	IF channel_family_perm_cursor%ISOPEN
	THEN
	    CLOSE channel_family_perm_cursor;
	END IF;
	IF found IS NULL
	THEN
	    RETURN 0;
	END IF;
	IF max_members_val IS NULL
	THEN
	    RETURN NULL;
	END IF;
	RETURN max_members_val - current_members_val;
    END available_family_subscriptions;
    PROCEDURE update_family_counts(channel_family_id_in IN NUMBER, org_id_in IN NUMBER)
    IS
    BEGIN
		update rhnPrivateChannelFamily
		set current_members = (
				select	count(sc.server_id)
				from	rhnServerChannel sc,
						rhnChannelFamilyMembers cfm,
						rhnServer s
				where	s.org_id = org_id_in
					and s.id = sc.server_id
					and cfm.channel_family_id = channel_family_id_in
					and cfm.channel_id = sc.channel_id
				)
			where org_id = org_id_in
				and channel_family_id = channel_family_id_in;
    END update_family_counts;
    FUNCTION available_chan_subscriptions(channel_id_in IN NUMBER, org_id_in IN NUMBER)
    RETURN NUMBER
    IS
	channel_family_id_val NUMBER;
    BEGIN
    	SELECT channel_family_id INTO channel_family_id_val
	  FROM rhnChannelFamilyMembers
	 WHERE channel_id = channel_id_in;
	 RETURN rhn_channel.available_family_subscriptions(channel_family_id_val, org_id_in);
    END available_chan_subscriptions;
    procedure entitle_customer(customer_id_in in number, channel_family_id_in in number, quantity_in in number)
    is
		cursor permissions is
			select	1
			from	rhnPrivateChannelFamily pcf
			where	pcf.org_id = customer_id_in
				and	pcf.channel_family_id = channel_family_id_in;
    begin
		for perm in permissions loop
			set_family_maxmembers(
				customer_id_in,
				channel_family_id_in,
				quantity_in
			);
			rhn_channel.update_family_counts(
				channel_family_id_in,
				customer_id_in
			);
			return;
		end loop;
		insert into rhnPrivateChannelFamily pcf (
				channel_family_id, org_id, max_members, current_members
			) values (
				channel_family_id_in, customer_id_in, quantity_in, 0
			);
    end;
    procedure set_family_maxmembers(customer_id_in in number, channel_family_id_in in number, quantity_in in number)
    is
	cursor servers is
	    select  server_id from (
		select	rownum row_number, server_id, modified from (
		    select  rcfs.server_id,
			    rcfs.modified
		    from    rhnChannelFamilyServers rcfs
		    where   rcfs.customer_id = customer_id_in
			    and rcfs.channel_family_id = channel_family_id_in
		    order by modified
		)
		where rownum > quantity_in
	    );
    begin
	for server in servers
	loop
	    rhn_channel.unsubscribe_server_from_family(server.server_id, channel_family_id_in);
	end loop;
	update	rhnPrivateChannelFamily pcf
	set	pcf.max_members = quantity_in
	where	pcf.org_id = customer_id_in
		and pcf.channel_family_id = channel_family_id_in;
    end;
    procedure unsubscribe_server_from_family(server_id_in in number, channel_family_id_in in number)
    is
    begin
	delete
	from	rhnServerChannel rsc
	where	rsc.server_id = server_id_in
		and channel_id in (
			select	rcfm.channel_id
			from	rhnChannelFamilyMembers rcfm
			where	rcfm.channel_family_id = channel_family_id_in);
    end;
    function get_org_id(channel_id_in in number)
    return number
    is
    	org_id_out number;
    begin
    	select org_id into org_id_out
	  from rhnChannel
	 where id = channel_id_in;
	return org_id_out;
    end get_org_id;
    function get_cfam_org_access(cfam_id_in in number, org_id_in in number)
    return number
    is
    	cursor	families is
			select	1
			from	rhnOrgChannelFamilyPermissions cfp
			where	cfp.org_id = org_id_in;
    begin
		for family in families loop
	    	return 1;
		end loop;
		return 0;
    end;
    function get_org_access(channel_id_in in number, org_id_in in number)
    return number
    is
    	throwaway number;
    begin
	select distinct 1 into throwaway
	  from rhnChannelFamilyMembers CFM,
	       rhnOrgChannelFamilyPermissions CFP
	 where cfp.org_id = org_id_in
	   and CFM.channel_family_id = CFP.channel_family_id
	   and CFM.channel_id = channel_id_in;
	return 1;
    exception
    	when no_data_found
	    then
	    return 0;
    end;
    function user_role_check_debug(channel_id_in in number, user_id_in in number, role_in in varchar2, reason_out out varchar2)
    return number
    is
    	org_id number;
    begin
    	org_id := rhn_user.get_org_id(user_id_in);
    	if role_in = 'manage' and NVL(rhn_channel.get_org_id(channel_id_in), -1) <> org_id
	then
	    reason_out := 'channel_not_owned';
	    return 0;
	end if;
    	if role_in = 'subscribe' and rhn_channel.get_org_access(channel_id_in, org_id) = 0
	then
	    reason_out := 'channel_not_available';
	    return 0;
	end if;
    	if rhn_user.check_role_implied(user_id_in, 'channel_admin') = 1
	then
	    reason_out := 'channel_admin';
	    return 1;
	end if;
	if role_in = 'subscribe'
	then
	    if rhn_channel.org_channel_setting(channel_id_in, org_id, 'not_globally_subscribable') = 0
	    then
	    	reason_out := 'globally_subscribable';
		return 1;
	    end if;
	end if;
	reason_out := 'direct_permission';
	return rhn_channel.direct_user_role_check(channel_id_in, user_id_in, role_in);
    end;
    function user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
  	throwaway varchar2(256);
    begin
    	return rhn_channel.user_role_check_debug(channel_id_in, user_id_in, role_in, throwaway);
    end;
    function loose_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
    begin
        if user_id_in is null then
            return 1;
        end if;
        return user_role_check(channel_id_in, user_id_in, role_in);
    end loose_user_role_check;
    function direct_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
    	throwaway number;
    begin
	select 1 into throwaway
	  from rhnChannelPermissionRole CPR,
	       rhnChannelPermission CP
	 where CP.user_id = user_id_in
	   and CP.channel_id = channel_id_in
	   and CPR.label = role_in
	   and CP.role_id = CPR.id;
	return 1;
    exception
    	when no_data_found
	    then
	    return 0;
    end;
    function org_channel_setting(channel_id_in in number, org_id_in in number, setting_in in varchar2)
    return number
    is
    	throwaway number;
    begin
	select 1 into throwaway
	  from rhnOrgChannelSettingsType OCST,
	       rhnOrgChannelSettings OCS
	 where OCS.org_id = org_id_in
	   and OCS.channel_id = channel_id_in
	   and OCST.label = setting_in
	   and OCS.setting_id = OCST.id;
	return 1;
    exception
    	when no_data_found
	    then
	    return 0;
    end;
    FUNCTION channel_priority(channel_id_in IN number)
    RETURN number
    IS
         channel_name varchar2(64);
         priority number;
         end_of_life_val date;
         org_id_val number;
    BEGIN
        select name, end_of_life, org_id
        into channel_name, end_of_life_val, org_id_val
        from rhnChannel
        where id = channel_id_in;
        if end_of_life_val is not null then
          return -400;
        end if;
        if channel_name like 'Red Hat Enterprise Linux%' then
          priority := 1000;
          if channel_name not like '%Beta%' then
            priority := priority + 1000;
          end if;
          priority := priority +
            case
              when channel_name like '%v. 4%' then 500
              when channel_name like '%v. 3%' then 400
              when channel_name like '%v. 2%' then 300
              when channel_name like '%v. 1%' then 200
            end;
          priority := priority +
            case
              when channel_name like '%AS%' then 50
              when channel_name like '%ES%' then 40
              when channel_name like '%WS%' then 30
              when channel_name like '%Desktop%' then 20
            end;
          priority := priority +
            case
              when channel_name like '%)' then 5
              else 0
            end;
          priority := priority +
            case
              when channel_name like '%32-bit x86%' then 4
              when channel_name like '%64-bit Intel Itanium%' then 3
              when channel_name like '%64-bit AMD64/Intel EM64T%' then 2
              else 0
            end;
        elsif channel_name like 'Red Hat Desktop%' then
            priority := 900;
            if channel_name not like '%Beta%' then
               priority := priority + 50;
            end if;
          priority := priority +
            case
              when channel_name like '%v. 4%' then 40
              when channel_name like '%v. 3%' then 30
              when channel_name like '%v. 2%' then 20
              when channel_name like '%v. 1%' then 10
            end;
          priority := priority +
            case
              when channel_name like '%32-bit x86%' then 4
              when channel_name like '%64-bit Intel Itanium%' then 3
              when channel_name like '%64-bit AMD64/Intel EM64T%' then 2
              else 0
            end;
        elsif org_id_val is not null then
          priority := 600;
        else
          priority := 500;
        end if;
      return -priority;
    end channel_priority;
    procedure delete_server_channels(server_id_in in number)
    is
    begin
	update	rhnPrivateChannelFamily
	set	current_members = current_members -1
	where	org_id in (
			select	org_id
			from	rhnServer
			where	id = server_id_in
		)
		and channel_family_id in (
			select	rcfm.channel_family_id
			from	rhnChannelFamilyMembers rcfm,
				rhnServerChannel rsc
			where	rsc.server_id = server_id_in
				and rsc.channel_id = rcfm.channel_id
		);
    end;
	procedure refresh_newest_package(channel_id_in in number, caller_in in varchar2 := '(unknown)')
	is
	begin
		delete from rhnChannelNewestPackage where channel_id = channel_id_in;
		insert into rhnChannelNewestPackage
			( channel_id, name_id, evr_id, package_id, package_arch_id )
			(	select	channel_id,
						name_id, evr_id,
						package_id, package_arch_id
				from	rhnChannelNewestPackageView
				where	channel_id = channel_id_in
			);
                insert into rhnChannelNewestPackageAudit (channel_id, caller)
                    values (channel_id_in, caller_in);
                update rhnChannel
                    set last_modified = greatest(sysdate, last_modified + 1/86400)
                    where id = channel_id_in;
	end;
END rhn_channel;
/

SHOW ERRORS;

--
-- 45. RHN_CONFIG  (Package Body) 
--
CREATE OR REPLACE package body
rhn_config
is
	procedure prune_org_configs (
		org_id_in in number,
		total_in in number
	) is
	begin
		null;
	end prune_org_configs;
	function insert_revision (
		revision_in in number,
		config_file_id_in in number,
		config_content_id_in in number,
		config_info_id_in in number,
		delim_start_in in varchar2 := '{@',
		delim_end_in in varchar2 := '@}',
      config_file_type_id_in in number := 1
	) return number is
		retval number;
		cursor affected_orgs is
			select	cc.org_id id
			from	rhnConfigChannel cc,
					rhnConfigFile cf
			where	cf.id = config_file_id_in
				and cf.config_channel_id = cc.id;
	begin
		insert into rhnConfigRevision(id, revision, config_file_id,
				config_content_id, config_info_id, delim_start, delim_end,
            config_file_type_id
			) values (
				rhn_confrevision_id_seq.nextval, revision_in, config_file_id_in,
				config_content_id_in, config_info_id_in, delim_start_in,
				delim_end_in, config_file_type_id_in
			)
			returning id into retval;
		for org in affected_orgs loop
			rhn_quota.update_org_quota(org.id);
		end loop;
		return retval;
	end insert_revision;
	procedure delete_revision (
		config_revision_id_in in number,
		org_id_in in number := -1
	) is
		cfid number;
		ccid number;
		oid number;
		latest_crid number;
		others number := 0;
		cursor snapshots is
			select	scr.snapshot_id id
			from	rhnSnapshot s,
					rhnSnapshotConfigRevision scr
			where	scr.config_revision_id = config_revision_id_in
					and scr.snapshot_id = s.id
					and s.invalid is null;
		cursor other_revisions(config_content_id_in in number) is
			select	1
			from	rhnConfigRevision
			where	config_content_id = config_content_id_in;
	begin
		for snapshot in snapshots loop
			update		rhnSnapshot s
				set		s.invalid =
							lookup_snapshot_invalid_reason('cr_removed')
				where	s.id = snapshot.id;
		end loop;
		if org_id_in < 0 then
			select	cr.config_content_id, cc.org_id
			into	ccid, oid
			from	rhnConfigChannel cc,
					rhnConfigFile cf,
					rhnConfigRevision cr
			where	cr.id = config_revision_id_in
				and cr.config_file_id = cf.id
				and cf.config_channel_id = cc.id;
		else
			select	cr.config_content_id, org_id_in
			into	ccid, oid
			from	rhnConfigRevision cr
			where	cr.id = config_revision_id_in;
		end if;
		delete from rhnConfigRevision where id = config_revision_id_in;
		for other_revision in other_revisions(ccid) loop
			others := 1;
			exit;
		end loop;
		if others = 0 then
			delete from rhnConfigContent where id = ccid;
		end if;
		if org_id_in < 0 then
			select	cf.latest_config_revision_id,
					cf.id
			into	latest_crid,
					cfid
			from	rhnConfigFile cf,
					rhnConfigRevision cr
			where	cr.id = config_revision_id_in
				and cr.config_file_id = cf.id;
			if latest_crid = config_revision_id_in then
				latest_crid := rhn_config.get_latest_revision(cfid);
				if latest_crid is not null then
					update rhnConfigFile set latest_config_revision_id = latest_crid
						where id = cfid;
				else
					delete from rhnConfigFile where id = cfid;
				end if;
			end if;
		end if;
		rhn_quota.update_org_quota(oid);
	end delete_revision;
	function get_latest_revision (
		config_file_id_in in number
	) return number is
		cursor revisions is
			select	cr.id
			from	rhnConfigRevision cr
			where	cr.config_file_id = config_file_id_in
			order by revision desc;
	begin
		for revision in revisions loop
			return revision.id;
		end loop;
		return null;
	end get_latest_revision;
	function insert_file (
		config_channel_id_in in number,
		name_in in varchar2
	) return number is
		retval number;
	begin
		select	rhn_conffile_id_seq.nextval
		into	retval
		from	dual;
		insert into rhnConfigFile(id, config_channel_id, config_file_name_id,
				state_id
			) (
				select	retval,
						config_channel_id_in,
						lookup_config_filename(name_in),
						id
				from	rhnConfigFileState
				where	label = 'alive'
			);
		return retval;
	end insert_file;
	procedure delete_file (
		config_file_id_in in number
	) is
		cursor revisions is
			select	cr.id, cc.org_id
			from	rhnConfigChannel cc,
					rhnConfigRevision cr,
					rhnConfigFile cf
			where	cf.id = config_file_id_in
				and cf.config_channel_id = cc.id
				and cr.config_file_id = cf.id;
		org_id number;
	begin
		for revision in revisions loop
			rhn_config.delete_revision(revision.id, revision.org_id);
			org_id := revision.org_id;
		end loop;
		rhn_quota.update_org_quota(org_id);
		delete from rhnConfigFile where id = config_file_id_in;
	end delete_file;
	function insert_channel (
		org_id_in in number,
		type_in in varchar2,
		name_in in varchar2,
		label_in in varchar2,
		description_in in varchar2
	) return number is
		retval number;
	begin
		select	rhn_confchan_id_seq.nextval
		into	retval
		from	dual;
		insert into rhnConfigChannel(id, org_id, confchan_type_id,
				name, label, description
			) (
				select	retval,
						org_id_in,
						cct.id,
						name_in,
						label_in,
						description_in
				from	rhnConfigChannelType cct
				where	label = type_in
			);
		return retval;
	end insert_channel;
	procedure delete_channel (
		config_channel_id_in in number
	) is
		cursor config_files is
			select	id
			from	rhnConfigFile
			where	config_channel_id = config_channel_id_in;
	begin
		for config_file in config_files loop
			rhn_config.delete_file(config_file.id);
		end loop;
		delete from rhnConfigChannel where id = config_channel_id_in;
	end delete_channel;
end rhn_config;
/

SHOW ERRORS;

--
-- 46. RHN_CUSTOMER_MONITORING  (View) 
--
CREATE OR REPLACE FORCE VIEW RHN_CUSTOMER_MONITORING
(RECID, DESCRIPTION, SCHEDULE_ID, DEF_ACK_WAIT, DEF_STRATEGY, 
 PREFERRED_TIME_ZONE, AUTO_UPDATE)
AS 
select	org.id			recid,
	org.name		description,
	1			schedule_id,	--24 x 7
	0			def_ack_wait,
	1			def_strategy,	--Broadcast, No Ack
	'GMT'			preferred_time_zone,
	0			auto_update	--Windows only
from
	web_customer org
where	1=1;

--
-- 47. PRODUCT  (Column) 
--
ALTER TABLE RHNERRATATMP
MODIFY(PRODUCT VARCHAR2(64 BYTE));


--
-- 48. CREATE_FIRST_ORG  (Procedure) 
--
CREATE OR REPLACE procedure
create_first_org
(
	name_in in varchar2,
	password_in in varchar2
) is
	ug_type			number;
	group_val		number;
begin
	insert into web_customer (
		id, name, password,
		oracle_customer_id, oracle_customer_number,
		customer_type
	) values (
		1, name_in, password_in,
		1, 1, 'B'
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'org_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Organization Administrators',
		'Organization Administrators for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'org_applicant';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) VALues (
		group_val, 'Organization Applicants',
		'Organization Applicants for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'system_group_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'System Group Administrators',
		'System Group Administrators for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'activation_key_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Activation Key Administrators',
		'Activation Key Administrators for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'channel_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Channel Administrators',
		'Channel Administrators for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	insert into rhnOrgQuota(
		org_id, total
	) values (
		1, 1024*1024*1024*16
	);
        insert into rhnServerGroup
		( id, name, description, max_members, group_type, org_id )
		select rhn_server_group_id_seq.nextval, sgt.name, sgt.name,
			0, sgt.id, 1
		from rhnServerGroupType sgt
		where sgt.label = 'sw_mgr_entitled';
end create_first_org;
/

SHOW ERRORS;

--
-- 49. RELEASE_NOTES_URL  (Column) 
--
ALTER TABLE RHNDOWNLOADS ADD (RELEASE_NOTES_URL  VARCHAR2(512 BYTE));

--
-- 50. RELEASE_NOTES  (Column) 
--
ALTER TABLE RHNDOWNLOADS DROP COLUMN RELEASE_NOTES;

--
-- 51. RHNSERVERCACHEINFO  (Table) 
--
create table
rhnServerCacheInfo
(
  server_id       number
                  constraint rhn_server_cache_info_sid_nn not null
                  constraint rhn_server_cache_info_sid_fk
                     references rhnServer(id),
   update_time    date
)
        storage ( pctincrease 1 freelists 16 )
        initrans 32;

create unique index rhn_server_cache_info_sid_idx
        on rhnServerCacheInfo(server_id)
        storage( pctincrease 1 freelists 16 )
        initrans 32;

--
-- 52. AGREED_TO_WS_TERMS  (Column) 
--
ALTER TABLE RHNUSERINFO ADD (AGREED_TO_WS_TERMS  CHAR(1 BYTE));

--
-- 53. AGREED_TO_ES_TERMS  (Column) 
--
ALTER TABLE RHNUSERINFO ADD (AGREED_TO_ES_TERMS  CHAR(1 BYTE));

--
-- 54. SCHEDULE_ZONE_ID  (Column) 
--
ALTER TABLE RHN_CONTACT_METHODS DROP COLUMN SCHEDULE_ZONE_ID;

--
-- 55. RHN_ACTSCRIPT_AID_UQ_IDX  (Index) 
--
drop index rhn_actscript_aid_idx;

CREATE UNIQUE INDEX RHN_ACTSCRIPT_AID_UQ_IDX ON RHNACTIONSCRIPT(ACTION_ID)
INITRANS   32
STORAGE    ( FREELISTS 16   );


--
-- 57. RHN_BEL  (Package) 
--
CREATE OR REPLACE package rhn_bel
is
   function is_org_paid (
      org_id_in in number
   ) return number;
        function lookup_email_state (
                state_in in varchar2
        ) return number;
end;
/

SHOW ERRORS;

--
-- 58. RHNSERVERNEEDEDPACKAGEVIEW  (View) 
--
CREATE OR REPLACE FORCE VIEW RHNSERVERNEEDEDPACKAGEVIEW
(ORG_ID, SERVER_ID, ERRATA_ID, PACKAGE_ID, PACKAGE_NAME_ID)
AS 
SELECT   S.org_id,
         S.id,
	  (SELECT EP.errata_id
	     FROM rhnErrataPackage EP,
	          rhnChannelErrata CE,
		  rhnServerChannel SC
	    WHERE SC.server_id = S.id
	      AND SC.channel_id = CE.channel_id
	      AND CE.errata_id = EP.errata_id
	      AND EP.package_id = P.id
	      AND ROWNUM = 1),
	 P.id,
	 P.name_id
FROM
	 rhnPackage P,
	 rhnServerPackageArchCompat SPAC,
	 rhnPackageEVR P_EVR,
	 rhnPackageEVR SP_EVR,
	 rhnServerPackage SP,
	 rhnChannelPackage CP,
	 rhnServerChannel SC,
         rhnServer S
WHERE
    	 SC.server_id = S.id
  AND  	 SC.channel_id = CP.channel_id
  AND    CP.package_id = P.id
  AND    p.package_arch_id = spac.package_arch_id
  AND    spac.server_arch_id = s.server_arch_id
  AND    SP_EVR.id = SP.evr_id
  AND    P_EVR.id = P.evr_id
  AND    SP.server_id = S.id
  AND    SP.name_id = P.name_id
  AND    SP.evr_id != P.evr_id
  AND    SP_EVR.evr < P_EVR.evr
  AND    SP_EVR.evr = (SELECT MAX(PE.evr) FROM rhnServerPackage SP2, rhnPackageEvr PE WHERE PE.id = SP2.evr_id AND SP2.server_id = SP.server_id AND SP2.name_id = SP.name_id);

--
-- 59. RHNERRATAFILECHANNELTMP  (Table) 
--
create table
rhnErrataFileChannelTmp
(
        channel_id      number
                        constraint rhn_efilectmp_cid_nn not null
                        constraint rhn_efilectmp_cid_fk
                                references rhnChannel(id)
                                on delete cascade,
        errata_file_id  number
                        constraint rhn_efilectmp_eid_nn not null
                        constraint rhn_efilectmp_eid_fk
                                references rhnErrataFileTmp(id)
                                on delete cascade,
        created         date default (sysdate)
                        constraint rhn_efilectmp_created_nn not null,
        modified        date default (sysdate)
                        constraint rhn_efilectmp_modified_nn not null
)
        storage ( freelists 16 )
        initrans 32;

create or replace trigger
rhn_efilectmp_mod_trig
before insert or update on rhnErrataFileChannelTmp
for each row
begin
        :new.modified := sysdate;
end rhn_efilectmp_mod_trig;
/
show errors

create index rhn_efilectmp_efid_cid_idx
        on rhnErrataFileChannelTmp(errata_file_id, channel_id)
        storage ( freelists 16 )
        initrans 32;
alter table rhnErrataFileChannelTmp add constraint rhn_efilectmp_efid_cid_uq
        unique ( errata_file_id, channel_id );

create index rhn_efilectmp_cid_efid_idx
        on rhnErrataFileChannelTmp(channel_id, errata_file_id)
        storage ( freelists 16 )
        initrans 32;

--
-- 60. RHNERRATAFILEPACKAGETMP  (Table) 
--
create table
rhnErrataFilePackageTmp
(
        package_id      number
                        constraint rhn_efileptmp_pid_nn not null
                        constraint rhn_efileptmp_pid_fk
                                references rhnPackage(id)
                                on delete cascade,
        errata_file_id  number
                        constraint rhn_efileptmp_fileid_nn not null
                        constraint rhn_efileptmp_fileid_fk
                                references rhnErrataFileTmp(id)
                                on delete cascade,
        created         date default (sysdate)
                        constraint rhn_efileptmp_created_nn not null,
        modified        date default (sysdate)
                        constraint rhn_efileptmp_modified_nn not null
)
        storage ( freelists 16 )
        initrans 32;

create or replace trigger
rhn_efileptmp_mod_trig
before insert or update on rhnErrataFilePackageTmp
for each row
begin
        :new.modified := sysdate;
end rhn_efilep_mod_trig;
/
show errors

create index rhn_efileptmp_efid_pid_idx
        on rhnErrataFilePackageTmp( errata_file_id, package_id )
        storage ( freelists 16 )
        initrans 32;
alter table rhnErrataFilePackageTmp add constraint rhn_efileptmp_efid_uq
        unique ( errata_file_id );

create index rhn_efileptmp_pid_idx
        on rhnErrataFilePackageTmp ( package_id )
        storage ( freelists 16 )
        initrans 32;


--
-- 65. RHN_CONTACT_MONITORING  (View) 
--
CREATE OR REPLACE FORCE VIEW RHN_CONTACT_MONITORING
(RECID, CUSTOMER_ID, CONTACT_LAST_NAME, CONTACT_FIRST_NAME, EMAIL_ADDRESS, 
 USERNAME, PASSWORD, SCHEDULE_ID, PREFERRED_TIME_ZONE)
AS 
select	u.id			recid,
	u.org_id		customer_id,
	wupi.last_name		contact_last_name,
	wupi.first_names	contact_first_name,
	rhn_user.find_mailable_address(u.id)
				email_address,
	u.login			username,
	u.password		password,
	1			schedule_id,
	'GMT'			preferred_time_zone
from
	web_user_personal_info wupi,
	web_contact u
where	1=1
	and u.id = wupi.web_user_id;


--
-- 69. RHN_CONFIG  (Package) 
--
CREATE OR REPLACE package
rhn_config
is
	procedure prune_org_configs (
		org_id_in in number,
		total_in in number
	);
	function insert_revision (
		revision_in in number,
		config_file_id_in in number,
		config_content_id_in in number,
		config_info_id_in in number,
		delim_start_in in varchar2 := '{@',
		delim_end_in in varchar2 := '@}',
      config_file_type_id_in number := 1
	) return number;
	procedure delete_revision (
		config_revision_id_in in number,
		org_id_in in number := -1
	);
	function get_latest_revision (
		config_file_id_in in number
	) return number;
	function insert_file (
		config_channel_id_in in number,
		name_in in varchar2
	) return number;
	procedure delete_file (
		config_file_id_in in number
	);
	function insert_channel (
		org_id_in in number,
		type_in in varchar2,
		name_in in varchar2,
		label_in in varchar2,
		description_in in varchar2
	) return number;
	procedure delete_channel (
		config_channel_id_in in number
	);
end rhn_config;
/

SHOW ERRORS;

--
-- 71. RHN_USER_INFO_ES_CK  (Constraint) 
--
ALTER TABLE RHNUSERINFO ADD CONSTRAINT RHN_USER_INFO_ES_CK CHECK (agreed_to_es_terms is null or agreed_to_es_terms in ('Y','N'));

--
-- 72. RHN_USER_INFO_WS_CK  (Constraint) 
--
ALTER TABLE RHNUSERINFO ADD CONSTRAINT RHN_USER_INFO_WS_CK CHECK (agreed_to_ws_terms is null or agreed_to_ws_terms in ('Y','N'));


update RHN_COMMAND_PARAMETER
set DESCRIPTION = 'Warn if number of blocked processes exceeds'
where COMMAND_ID = 231
and param_name = 'nblocked_warn';

update RHN_COMMAND_PARAMETER
set DESCRIPTION = 'Critical if number of blocked processes exceeds'
where COMMAND_ID = 231
and param_name = 'nblocked_critical';

update RHN_COMMAND_PARAMETER
set DESCRIPTION = 'Warn if number of children for any process exceeds'
where COMMAND_ID = 231
and param_name = 'nchildren_warn';

update RHN_COMMAND_PARAMETER
set DESCRIPTION = 'Critical if number of children for any process exceeds'
where COMMAND_ID = 231
and param_name = 'nchildren_critical';

update RHN_COMMAND_PARAMETER
set DESCRIPTION = 'Warn if number of defunct processes exceeds'
where COMMAND_ID = 231
and param_name = 'ndefunct_warn';

update RHN_COMMAND_PARAMETER
set DESCRIPTION = 'Critical if number of defunct processes exceeds'
where COMMAND_ID = 231
and param_name = 'ndefunct_critical';

update RHN_COMMAND_PARAMETER
set DESCRIPTION = 'Warn if number of stopped processes exceeds'
where COMMAND_ID = 231
and param_name = 'nstopped_warn';

update RHN_COMMAND_PARAMETER
set DESCRIPTION = 'Critical if number of stopped processes exceeds'
where COMMAND_ID = 231
and param_name = 'nstopped_critical';

update RHN_COMMAND_PARAMETER
set DESCRIPTION = 'Warn if number of swapped processes exceeds'
where COMMAND_ID = 231
and param_name = 'nswapped_warn';
update RHN_COMMAND_PARAMETER
set DESCRIPTION = 'Critical if number of swapped processes exceeds'
where COMMAND_ID = 231
and param_name = 'nswapped_critical';
UPDATE rhn_command_parameter
SET default_value = '0.1'
WHERE param_name = 'warnbelow'
AND command_id = '96';

UPDATE rhn_command_parameter
SET default_value = '0.05'
WHERE param_name = 'criticalbelow'
AND command_id = '96';
UPDATE rhn_command_parameter
SET default_value = '/opt/oracle'
WHERE param_name = 'ORACLE_HOME'
AND command_id in (23, 85, 86, 87, 88, 89, 90, 91, 92, 278, 279, 280, 283, 284);

UPDATE rhn_probe_param_value
SET value = '/opt/oracle'
WHERE param_name = 'ORACLE_HOME';
UPDATE rhn_notification_formats
SET body_format = 'This is RHN Monitoring Satellite notification ^[alert id].\n\nTime:      ^[timestamp:"%a %b %d, %H:%M:%S %Z"]\nState:     ^[probe state]\nHost:      ^[hostname] (^[host IP])\nCheck:     ^[probe description]\nMessage:   ^[probe output]\nRun from:  ^[satellite description]'
where recid = 4;

commit;

alter trigger rhn_confrevision_acct_trig compile;
alter trigger rhn_confrevision_mod_trig compile;
alter trigger rhn_download_mod_trig compile;
alter trigger rhn_erratatmp_mod_trig compile;
alter trigger rhn_user_info_mod_trig compile;
alter trigger rhn_confrevision_del_trig compile;
alter trigger rhn_cmeth_val_trig compile;

alter function create_new_user compile;

alter procedure delete_errata compile;
alter procedure new_user_postop compile;
alter procedure queue_server compile;

alter package rhn_bel compile package;
alter package rhn_config_channel compile package;
alter package rhn_config compile package;
alter package rhn_entitlements compile package;
alter package rhn_org compile package;

alter package rhn_server compile package;
alter package rhn_server compile body;

alter package rhn_quota compile package;
alter package rhn_quota compile body;

alter package rhn_channel compile package;
alter package rhn_channel compile body;

alter procedure delete_server compile;

alter package rhn_config compile;
alter package rhn_org compile;
alter package rhn_entitlements compile;
alter package rhn_quota compile body;
alter package rhn_server compile body;

alter procedure delete_server_bulk compile;

commit;

update rhnVersionInfo set evr_id = lookup_evr(null, '3.7', '116')
   where label = 'schema'
   and name_id = lookup_package_name('rhn-satellite-schema');
commit;

select   '*** Schema version is now ' || e.evr.as_vre_simple()
from  rhnPackageEVR e, rhnVersionInfo vi
where vi.evr_id = e.id
  and   vi.label = 'schema';

quit;


