set echo on;
whenever sqlerror exit failure;

spool satellite-3.4-to-3.6.log

variable evr_id number;
variable epoch varchar2(16);
variable version varchar2(64);
variable release varchar2(64);

variable message varchar2(80);

declare
	cursor evrs is
		select	e.id, e.epoch, e.version, e.release, e.evr
		from	rhnPackageEVR e,
			rhnVersionInfo	rvi
		where	rvi.label = 'schema'
			and rvi.name_id =
				lookup_package_name('rhn-satellite-schema')
			and rvi.evr_id = e.id;
	cursor valid_evrs is
		select	1
		from	dual
		where	:evr_id in (
			lookup_evr('','3.4','48')
			);
begin
	:evr_id := null;
	:message := 'XXX Invalid satellite schema version.';
	for evr in evrs loop
		:evr_id := evr.id;
		:epoch := evr.epoch;
		:version := evr.version;
		:release := evr.release;
		:message :=  '*** Schema version is currently ' || 
			evr.evr.as_vre_simple() ||
			', and will NOT be upgraded';
		for vevr in valid_evrs loop
			:message :=  '*** Schema version is currently ' || 
				evr.evr.as_vre_simple() ||
				', and will be upgraded';
		end loop;
		return;
	end loop;
end;
/

select :message from dual;

declare
	invalid_schema_version exception;
	cursor valid_evrs is
		select	1
		from	dual
		where	:evr_id in (
			lookup_evr('','3.4','48')
			);
begin
	for vevr in valid_evrs loop
		return;
	end loop;
	raise invalid_schema_version;
end;
/
show errors

--------------------------------------------------------------------------
-- Play this script in RHNSAT@SAT36 to make it look like CHIP01@CHIP01
--                                                                      --
-- Please review the script before using it to make sure it won't       --
-- cause any unacceptable data loss.                                    --
--                                                                      --
-- RHNSAT@SAT36 Schema Extracted by User RHNSAT 
-- CHIP01@CHIP01 Schema Extracted by User CHIP01 
--------------------------------------------------------------------------
-- "Set define off" turns off substitution variables
Set define off;
--
-- 1. RHN_CDATAKEY_LABEL_UQ  (Constraint) 
--
ALTER TABLE RHNCUSTOMDATAKEY DROP CONSTRAINT RHN_CDATAKEY_LABEL_UQ;

--
-- 2. RHN_ERRATAFILE_MOD_TRIG  (Trigger) 
--
DROP TRIGGER RHN_ERRATAFILE_MOD_TRIG;

--
-- 3. RHNUSERSERVERPERMS  (View) 
--
DROP VIEW RHNUSERSERVERPERMS;

--
-- 4. ADLER32  (Function) 
--
DROP FUNCTION ADLER32;

--
-- 5. PROCESS_ERRATA_CACHE_QUEUE  (Procedure) 
--
DROP PROCEDURE PROCESS_ERRATA_CACHE_QUEUE;

--
-- 6. RHN_SNPC_SID_EID_PID_IDX  (Index) 
--
DROP INDEX RHN_SNPC_SID_EID_PID_IDX;

--
-- 7. RHN_SNPC_EID_SID_IDX  (Index) 
--
DROP INDEX RHN_SNPC_EID_SID_IDX;

--
-- 8. RHN_KSTREE_LABEL_UQ  (Index) 
--
DROP INDEX RHN_KSTREE_LABEL_UQ;

--
-- 9. RHN_CDATAKEY_LABEL_UQ  (Index) 
--
-- rhn: not necessary; step 1. dropped the constraint for us
-- DROP INDEX RHN_CDATAKEY_LABEL_UQ;

--
-- 10. EVR_T  (Type) 
--
-- rhn: removed this step, toad was confused

--
-- 11. RHN_APPINST_INSTANCE_ID_SEQ  (Sequence) 
--
--
-- RHN_APPINST_INSTANCE_ID_SEQ  (Sequence) 
--
CREATE SEQUENCE RHN_APPINST_INSTANCE_ID_SEQ
  START WITH 1
  MAXVALUE 1E27
  MINVALUE 1
  NOCYCLE
  CACHE 20
  NOORDER;

--
-- 12. RHN_APPINST_SDATA_ID_SEQ  (Sequence) 
--
--
-- RHN_APPINST_SDATA_ID_SEQ  (Sequence) 
--
CREATE SEQUENCE RHN_APPINST_SDATA_ID_SEQ
  START WITH 1
  MAXVALUE 1E27
  MINVALUE 1
  NOCYCLE
  CACHE 20
  NOORDER;

--
-- 13. RHN_APPINST_SESSION_ID_SEQ  (Sequence) 
--
--
-- RHN_APPINST_SESSION_ID_SEQ  (Sequence) 
--
CREATE SEQUENCE RHN_APPINST_SESSION_ID_SEQ
  START WITH 1
  MAXVALUE 1E27
  MINVALUE 1
  NOCYCLE
  CACHE 20
  NOORDER;

--
-- 14. RHN_KSSCRIPT_ID_SEQ  (Sequence) 
--
--
-- RHN_KSSCRIPT_ID_SEQ  (Sequence) 
--
CREATE SEQUENCE RHN_KSSCRIPT_ID_SEQ
  START WITH 1
  MAXVALUE 1E27
  MINVALUE 1
  NOCYCLE
  CACHE 20
  NOORDER;

--
-- 15. RHN_PCLIENT_ID_SEQ  (Sequence) 
--
--
-- RHN_PCLIENT_ID_SEQ  (Sequence) 
--
CREATE SEQUENCE RHN_PCLIENT_ID_SEQ
  START WITH 1
  MAXVALUE 1E27
  MINVALUE 1
  NOCYCLE
  CACHE 20
  NOORDER;

--
-- 16. RHN_PCLIENT_STATE_ID_SEQ  (Sequence) 
--
--
-- RHN_PCLIENT_STATE_ID_SEQ  (Sequence) 
--
CREATE SEQUENCE RHN_PCLIENT_STATE_ID_SEQ
  START WITH 21
  MAXVALUE 1E27
  MINVALUE 1
  NOCYCLE
  CACHE 20
  NOORDER;

--
-- 17. RHN_PUSHDISPATCH_ID_SEQ  (Sequence) 
--
--
-- RHN_PUSHDISPATCH_ID_SEQ  (Sequence) 
--
CREATE SEQUENCE RHN_PUSHDISPATCH_ID_SEQ
  START WITH 1
  MAXVALUE 1E27
  MINVALUE 1
  NOCYCLE
  CACHE 20
  NOORDER;

--
-- 18. RHN_TIMEZONE_ID_SEQ  (Sequence) 
--
--
-- RHN_TIMEZONE_ID_SEQ  (Sequence) 
--
CREATE SEQUENCE RHN_TIMEZONE_ID_SEQ
  START WITH 7040
  MAXVALUE 1E27
  MINVALUE 1
  NOCYCLE
  CACHE 20
  ORDER;

--
-- 19. RHNAPPINSTALLINSTANCE  (Table) 
--
--
-- RHNAPPINSTALLINSTANCE  (Table) 
--
CREATE TABLE RHNAPPINSTALLINSTANCE
(
  ID        NUMBER CONSTRAINT RHN_APPINST_INSTANCE_ID_NN NOT NULL,
  NAME      VARCHAR2(128 BYTE) CONSTRAINT RHN_APPINST_INSTANCE_NAME_NN NOT NULL,
  LABEL     VARCHAR2(128 BYTE) CONSTRAINT RHN_APPINST_INSTANCE_LABEL_NN NOT NULL,
  VERSION   VARCHAR2(32 BYTE) CONSTRAINT RHN_APPINST_INSTANCE_VERS_NN NOT NULL,
  CREATED   DATE                                DEFAULT (sysdate) CONSTRAINT RHN_APPINST_INSTANCE_CREAT_NN NOT NULL,
  MODIFIED  DATE                                DEFAULT (sysdate) CONSTRAINT RHN_APPINST_INSTANCE_MOD_NN NOT NULL
)
LOGGING 
NOCACHE
NOPARALLEL;

create table
rhnPushClientState
(
	id		number
			constraint rhn_pclient_state_id_nn not null
			constraint rhn_pclient_state_id_pk primary key,
	label		varchar2(64)
			constraint rhn_pclient_state_label_nn not null,
	name		varchar2(256)
			constraint rhn_pclient_state_name_nn not null,
	created		date default sysdate
			constraint rhn_pclient_state_created_nn not null,
	modified	date default sysdate
			constraint rhn_pclient_state_modified_nn not null
)
	storage ( freelists 16 )
	initrans 32;

create unique index rhn_pclient_state_label_uq
	on rhnPushClientState( label )
	storage ( freelists 16 )
	initrans 32;

create unique index rhn_pclient_state_name_uq
	on rhnPushClientState( name )
	storage ( freelists 16 )
	initrans 32;

create or replace trigger
rhn_pclient_state_mod_trig
before insert or update on rhnPushClientState
for each row
begin
	:new.modified := sysdate;
end rhn_pclient_state_mod_trig;
/
show errors

-- 20a. add in static data

insert into rhnPushClientState (id, label, name)
    values (rhn_pclient_state_id_seq.nextval, 'online', 'online');
insert into rhnPushClientState (id, label, name)
    values (rhn_pclient_state_id_seq.nextval, 'offline', 'offline');
 
commit;

--
-- 21. RHNPUSHDISPATCHER  (Table) 
--
--
-- RHNPUSHDISPATCHER  (Table) 
--
CREATE TABLE RHNPUSHDISPATCHER
(
  ID            NUMBER CONSTRAINT RHN_PUSHDISPATCH_ID_NN NOT NULL,
  JABBER_ID     VARCHAR2(128 BYTE) CONSTRAINT RHN_PUSHDISPATCH_JID_NN NOT NULL,
  LAST_CHECKIN  DATE                            DEFAULT sysdate CONSTRAINT RHN_PUSHDISPATCH_LC_NN NOT NULL,
  HOSTNAME      VARCHAR2(256 BYTE) CONSTRAINT RHN_PUSHDISPATCH_HN_NN NOT NULL,
  PORT          NUMBER CONSTRAINT RHN_PUSHDISPATCH_PORT_NN NOT NULL,
  CREATED       DATE                            DEFAULT sysdate CONSTRAINT RHN_PUSHDISPATCH_CREAT_NN NOT NULL,
  MODIFIED      DATE                            DEFAULT sysdate CONSTRAINT RHN_PUSHDISPATCH_MOD_NN NOT NULL
)
LOGGING 
NOCACHE
NOPARALLEL;

--
-- 22. CERT  (Column) 
--
ALTER TABLE RHNSATELLITECERT DROP CONSTRAINT RHN_SATCERT_CERT_NN;
ALTER TABLE RHNSATELLITECERT MODIFY(CERT CONSTRAINT RHN_SATCERT_CERT_NN NOT NULL);


--
-- 23. RHNSATELLITECERT  (Table) 
--
-- Difference : Column Order (No action taken) 

--
-- 24. RHNTIMEZONE  (Table) 
--
--
-- RHNTIMEZONE  (Table) 
--
-- rhn: copied from original schema
create table 
rhnTimezone
(
        id              number
			constraint rhn_timezone_id_nn not null,
        olson_name      varchar2(128)
	    	    	constraint rhn_timezone_olson_nn not null,
	display_name    varchar2(128)
	    	    	constraint rhn_timezone_display_nn not null
)
	storage ( freelists 16 )
	initrans 32;
create index rhn_timezone_id_idx
	on rhnTimezone( id )
	storage ( freelists 16 )
	initrans 32;
alter table rhnTimezone add constraint rhn_timezone_id_pk
	primary key ( id );

create unique index rhn_timezone_olson_uq
	on rhnTimezone(olson_name);

create unique index rhn_timezone_display_uq
	on rhnTimezone(display_name);

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Australia/Sydney', 'Australia (Eastern)');


insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Asia/Tokyo', 'Japan');


insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Asia/Hong_Kong', 'Australia (Western)');


insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Europe/Moscow', 'Russia (Moscow)');


insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Europe/Bucharest', 'Europe (Eastern)');


insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Europe/Paris', 'Europe (Central)');


insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'GMT', 'Greenwich Mean Time');


insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'America/St_Johns', 'Canada (Newfoundland)');


insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'America/Halifax', 'Canada (Atlantic)');


insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'America/New_York', 'United States (Eastern)');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'America/Indianapolis', 'United States (Indiana)');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'America/Chicago', 'United States (Central)');


insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'America/Denver', 'United States (Mountain)');


insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'America/Phoenix', 'United States (Arizona)');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'America/Los_Angeles', 'United States (Pacific)');


insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'America/Anchorage', 'United States (Alaska)');


insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Pacific/Honolulu', 'United States (Hawaii)');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Pacific/Enderbury', 'Tonga');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Pacific/Wallis', 'New Zealand');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Pacific/Norfolk', 'Norfolk Island');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Pacific/Guadalcanal', 'Micronesia');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Australia/Darwin', 'Australia (Central/Darwin)');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Asia/Jakarta', 'Thailand');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Indian/Cocos', 'Burma');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Indian/Chagos', 'Bangladesh');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Asia/Katmandu', 'Nepal');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Asia/Calcutta', 'India (Calcutta)');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Indian/Maldives', 'Pakistan');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Asia/Kabul', 'Afghanistan');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Asia/Tehran', 'Iran');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Indian/Mauritius', 'Russia (Samara)');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Atlantic/Azores', 'Azores');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Atlantic/South_Georgia', 'Sandwich Islands');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'Pacific/Midway', 'Midway');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'America/Sao_Paulo', 'Brazil');

commit;

--
-- 25. STATE_CHANGE  (Table) 
--
--
-- STATE_CHANGE  (Table) 
--
CREATE TABLE STATE_CHANGE
(
  O_ID        VARCHAR2(64 BYTE) CONSTRAINT STATE_CHANGE_O_ID_NN NOT NULL,
  ENTRY_TIME  NUMBER CONSTRAINT STATE_CHANGE_ETIME_NN NOT NULL,
  DATA        VARCHAR2(4000 BYTE)
)
LOGGING 
NOCACHE
NOPARALLEL;

--
-- 26. TIME_SERIES  (Table) 
--
--
-- TIME_SERIES  (Table) 
--
CREATE TABLE TIME_SERIES
(
  O_ID        VARCHAR2(64 BYTE) CONSTRAINT TIME_SERIES_O_ID_NN NOT NULL,
  ENTRY_TIME  NUMBER CONSTRAINT TIME_SERIES_ETIME_NN NOT NULL,
  DATA        VARCHAR2(1024 BYTE)
)
LOGGING 
NOCACHE
NOPARALLEL;

--
-- 27. RHN_APPINST_INSTANCE_ID_IDX  (Index) 
--
--
-- RHN_APPINST_INSTANCE_ID_IDX  (Index) 
--
--  Dependencies: 
--   RHNAPPINSTALLINSTANCE (Table)
--
CREATE INDEX RHN_APPINST_INSTANCE_ID_IDX ON RHNAPPINSTALLINSTANCE
(ID)
LOGGING
NOPARALLEL;

--
-- 28. RHN_APPINST_INSTANCE_LV_ID_IDX  (Index) 
--
--
-- RHN_APPINST_INSTANCE_LV_ID_IDX  (Index) 
--
--  Dependencies: 
--   RHNAPPINSTALLINSTANCE (Table)
--
CREATE INDEX RHN_APPINST_INSTANCE_LV_ID_IDX ON RHNAPPINSTALLINSTANCE
(LABEL, VERSION, ID)
LOGGING
NOPARALLEL;

--
-- 32. RHN_PUSHDISPATCH_ID_PK  (Index) 
--
--
-- RHN_PUSHDISPATCH_ID_PK  (Index) 
--
--  Dependencies: 
--   RHNPUSHDISPATCHER (Table)
--
CREATE UNIQUE INDEX RHN_PUSHDISPATCH_ID_PK ON RHNPUSHDISPATCHER
(ID)
LOGGING
NOPARALLEL;

--
-- 33. RHN_PUSHDISPATCH_JID_ID_IDX  (Index) 
--
--
-- RHN_PUSHDISPATCH_JID_ID_IDX  (Index) 
--
--  Dependencies: 
--   RHNPUSHDISPATCHER (Table)
--
CREATE INDEX RHN_PUSHDISPATCH_JID_ID_IDX ON RHNPUSHDISPATCHER
(JABBER_ID, ID)
LOGGING
NOPARALLEL;

-- 34. through 36. removed in favor of previous rhnTimezone create stmts

--
-- 37. STATE_CHANGE_OID_ENTRY_IDX  (Index) 
--
--
-- STATE_CHANGE_OID_ENTRY_IDX  (Index) 
--
--  Dependencies: 
--   STATE_CHANGE (Table)
--
CREATE INDEX STATE_CHANGE_OID_ENTRY_IDX ON STATE_CHANGE
(O_ID, ENTRY_TIME)
LOGGING
NOPARALLEL;

--
-- 38. TIME_SERIES_OID_ENTRY_IDX  (Index) 
--
--
-- TIME_SERIES_OID_ENTRY_IDX  (Index) 
--
--  Dependencies: 
--   TIME_SERIES (Table)
--
CREATE INDEX TIME_SERIES_OID_ENTRY_IDX ON TIME_SERIES
(O_ID, ENTRY_TIME)
LOGGING
NOPARALLEL;

--
-- 39. RHN_CACHE  (Package Body) 
--
--
-- RHN_CACHE  (Package Body) 
--
CREATE OR REPLACE package body
rhn_cache
is
	body_version varchar2(100) := '$Id: satellite-3.4-to-3.6.sql 64427 2005-09-09 14:08:21Z jslagle $';
	procedure update_perms_for_server(
		server_id_in in number
	) is
	begin
		delete from rhnUserServerPerms where server_id = server_id_in;
		insert into rhnUserServerPerms(user_id, server_id) (
				select	distinct user_id, server_id_in
				from	rhnUserServerPermsDupes
				where	server_id = server_id_in
			);
	end update_perms_for_server;
	procedure update_perms_for_user(
		user_id_in in number
	) is
	begin
		delete from rhnUserServerPerms where user_id = user_id_in;
		insert into rhnUserServerPerms(user_id, server_id) (
				select	distinct user_id_in, server_id
				from	rhnUserServerPermsDupes
				where	user_id = user_id_in
			);
	end update_perms_for_user;
	procedure update_perms_for_server_group(
		server_group_id_in in number
	) is
		cursor users is
			select	usgp.user_id id
			from	rhnUserServerGroupPerms usgp
			where	usgp.server_group_id = server_group_id_in
				and not exists (
					select	1
					from	rhnUserGroup ug,
							rhnUserGroupMembers ugm,
							rhnServerGroup sg,
							rhnUserGroupType ugt
					where	ugt.label = 'org_admin'
						and sg.id = server_group_id_in
						and ugm.user_id = usgp.user_id
						and ug.org_id = sg.org_id
						and ugm.user_group_id = ug.id
					);
	begin
		for u in users loop
			update_perms_for_user(u.id);
		end loop;
	end update_perms_for_server_group;
end rhn_cache;
/

SHOW ERRORS;

--
-- 40. RHN_CHANNEL  (Package Body) 
--
--
-- RHN_CHANNEL  (Package Body) 
--
CREATE OR REPLACE PACKAGE BODY rhn_channel
IS
	body_version varchar2(100) := '$Id: satellite-3.4-to-3.6.sql 64427 2005-09-09 14:08:21Z jslagle $';
	cursor	base_channel_cursor(
		release_in in varchar2,
		server_arch_id_in in number,
		org_id_in in number
	) return rhnChannel%ROWTYPE is
		select distinct c.*
		from	rhnDistChannelMap			dcm,
				rhnServerChannelArchCompat	scac,
				rhnChannel					c,
				rhnChannelPermissions		cp
		where	cp.org_id = org_id_in
			and cp.channel_id = c.id
			and c.parent_channel is null
			and c.id = dcm.channel_id
			and c.channel_arch_id = dcm.channel_arch_id
			and dcm.release = release_in
			and scac.server_arch_id = server_arch_id_in
			and scac.channel_arch_id = c.channel_arch_id;
    FUNCTION get_license_path(channel_id_in IN NUMBER)
    RETURN VARCHAR2
    IS
    	license_val VARCHAR2(1000);
    BEGIN
    	SELECT CFL.license_path INTO license_val
	  FROM rhnChannelFamilyLicense CFL, rhnChannelFamilyMembers CFM
	 WHERE CFM.channel_id = channel_id_in
	   AND CFM.channel_family_id = CFL.channel_family_id;
    	RETURN license_val;
    EXCEPTION
    	WHEN NO_DATA_FOUND
	    THEN
	    RETURN NULL;
    END get_license_path;
    PROCEDURE license_consent(channel_id_in IN NUMBER, user_id_in IN NUMBER, server_id_in IN NUMBER)
    IS
    	channel_family_id_val NUMBER;
    BEGIN
    	channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
	IF channel_family_id_val IS NULL
	THEN
	    rhn_exception.raise_exception('channel_subscribe_no_family');
	END IF;
	IF rhn_channel.get_license_path(channel_id_in) IS NULL
	THEN
	    rhn_exception.raise_exception('channel_consent_no_license');
	END IF;
	INSERT INTO rhnChannelFamilyLicenseConsent (channel_family_id, user_id, server_id)
	VALUES (channel_family_id_val, user_id_in, server_id_in);
    END license_consent;
    PROCEDURE subscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, user_id_in in number := null)
    IS
        channel_parent_val      rhnChannel.parent_channel%TYPE;
        parent_subscribed       BOOLEAN;
        server_has_base_chan    BOOLEAN;
	server_already_in_chan  BOOLEAN;
	channel_family_id_val   NUMBER;
	server_org_id_val       NUMBER;
	available_subscriptions NUMBER;
	consenting_user         NUMBER;
	allowed			number := 0;
    BEGIN
    	if user_id_in is not null then
		allowed := rhn_channel.user_role_check(channel_id_in, user_id_in, 'subscribe');
	else
		allowed := 1;
	end if;
	if allowed = 0 then
		rhn_exception.raise_exception('no_subscribe_permissions');
	end if;
        SELECT parent_channel INTO channel_parent_val FROM rhnChannel WHERE id = channel_id_in;
        IF channel_parent_val IS NOT NULL
        THEN
            parent_subscribed := FALSE;
            FOR check_subscription IN check_server_subscription(server_id_in, channel_parent_val)
            LOOP
                parent_subscribed := TRUE;
            END LOOP check_subscription;
            IF NOT parent_subscribed
            THEN
    	    	RETURN;
            END IF;
        ELSE
            server_has_base_chan := FALSE;
            FOR base IN server_base_subscriptions(server_id_in)
            LOOP
                server_has_base_chan := TRUE;
            END LOOP base;
            IF server_has_base_chan
            THEN
                rhn_exception.raise_exception('channel_server_one_base');
            END IF;
        END IF;
        FOR check_subscription IN check_server_subscription(server_id_in, channel_id_in)
        LOOP
            server_already_in_chan := TRUE;
        END LOOP check_subscription;
        IF server_already_in_chan
        THEN
    	    RETURN;
        END IF;
	channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
	IF channel_family_id_val IS NULL
	THEN
	    rhn_exception.raise_exception('channel_subscribe_no_family');
	END IF;
	SELECT org_id INTO server_org_id_val
	  FROM rhnServer
	 WHERE id = server_id_in;
	available_subscriptions := rhn_channel.available_family_subscriptions(channel_family_id_val, server_org_id_val);
	IF available_subscriptions IS NULL OR available_subscriptions > 0
	THEN
	    IF rhn_channel.get_license_path(channel_id_in) IS NOT NULL
	    THEN
	    	BEGIN
	    	SELECT user_id INTO consenting_user
		  FROM rhnChannelFamilyLicenseConsent
		 WHERE channel_family_id = channel_family_id_val
		   AND server_id = server_id_in;
		EXCEPTION
    	    	    WHEN NO_DATA_FOUND
		    	THEN
			    rhn_exception.raise_exception('channel_subscribe_no_consent');
		END;
	    END IF;
	    insert into rhnServerHistory (id,server_id,summary,details) (
	    	select	rhn_event_id_seq.nextval,
			server_id_in,
			'subscribed to channel ' || SUBSTR(c.label, 0, 106),
			c.label
		from	rhnChannel c
		where	c.id = channel_id_in
	    );
	    UPDATE rhnServer SET channels_changed = sysdate WHERE id = server_id_in;
            INSERT INTO rhnServerChannel (server_id, channel_id) VALUES (server_id_in, channel_id_in);
    	    rhn_channel.update_family_counts(channel_family_id_val, server_org_id_val);
    	    queue_server(server_id_in, immediate_in);
	ELSE
	    rhn_exception.raise_exception('channel_family_no_subscriptions');
	END IF;
    END subscribe_server;
    PROCEDURE bulk_subscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
            rhn_channel.subscribe_server(server.element, channel_id_in, 0, set_uid_in);
        END LOOP server;
    END bulk_subscribe_server;
    PROCEDURE bulk_server_base_change(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
	    IF rhn_server.can_change_base_channel(server.element) = 1
	    THEN
                rhn_channel.clear_subscriptions(TO_NUMBER(server.element));
                rhn_channel.subscribe_server(server.element, channel_id_in, 0, set_uid_in);
            END IF;
        END LOOP server;
    END bulk_server_base_change;
    procedure bulk_server_basechange_from(
        set_label_in in varchar2,
        set_uid_in in number,
        old_channel_id_in in number,
        new_channel_id_in in number
    ) is
    cursor servers is
        select  sc.server_id id
        from    rhnChannel nc,
                rhnServerChannelArchCompat scac,
                rhnServer s,
                rhnChannel oc,
                rhnServerChannel sc,
                rhnSet st
        where   1=1
            and st.label = set_label_in
            and st.user_id = set_uid_in
            and st.element = sc.server_id
            and sc.channel_id = old_channel_id_in
            and sc.channel_id = oc.id
            and oc.parent_channel is null
            and nc.id = new_channel_id_in
            and nc.parent_channel is null
            and sc.server_id = s.id
            and s.server_arch_id = scac.server_arch_id
            and scac.channel_arch_id = nc.channel_arch_id;
    begin
        for s in servers loop
            insert into rhnSet (
                    user_id, label, element
                ) values (
                    set_uid_in,
                    set_label_in || 'basechange',
                    s.id
                );
        end loop channel;
        bulk_server_base_change(new_channel_id_in,
                                set_label_in || 'basechange',
                                set_uid_in);
        delete from rhnSet
            where   label = set_label_in||'basechange'
                and user_id = set_uid_in;
    end bulk_server_basechange_from;
    procedure bulk_guess_server_base(
	set_label_in in varchar2,
	set_uid_in in number
    ) is
	channel_id number;
    begin
	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    begin
	    	if rhn_server.can_change_base_channel(server.element) = 1
		then
    	            channel_id := guess_server_base(TO_NUMBER(server.element));
	    	    rhn_channel.clear_subscriptions(TO_NUMBER(server.element));
	    	    rhn_channel.subscribe_server(TO_NUMBER(server.element), channel_id, 0, set_uid_in);
		end if;
	    exception when others then
	    	null;
	    end;
	end loop server;
    end;
    function guess_server_base(
    	server_id_in in number
    ) RETURN number is
	cursor server_cursor is
	    select s.server_arch_id, s.release, s.org_id
	      from rhnServer s
	     where s.id = server_id_in;
    begin
	for s in server_cursor loop
	    for channel in base_channel_cursor(s.release,
		s.server_arch_id, s.org_id)
	    loop
		return channel.id;
	    end loop base_channel_cursor;
        end loop server_cursor;
        return null;
    end;
    function normalize_server_arch(server_arch_in in varchar2)
    return varchar2
    is
	suffix VARCHAR2(128) := '-redhat-linux';
	suffix_len NUMBER := length(suffix);
    begin
	if server_arch_in is NULL then
	    return NULL;
	end if;
        if instr(server_arch_in, '-') > 0
	then
	    return server_arch_in;
	end if;
	return server_arch_in || suffix;
    end normalize_server_arch;
    function base_channel_for_release_arch(
	release_in in varchar2,
	server_arch_in in varchar2,
	org_id_in in number := -1,
	user_id_in in number := null
    ) return number is
	server_arch varchar2(256) := normalize_server_arch(server_arch_in);
	server_arch_id number;
    begin
	begin
	    select id
	      into server_arch_id
	      from rhnServerArch
	     where label = server_arch;
	exception
	    when no_data_found then
		rhn_exception.raise_exception('server_arch_not_found');
	end;
	return base_channel_rel_archid(release_in, server_arch_id,
	    org_id_in, user_id_in);
    end base_channel_for_release_arch;
    function base_channel_rel_archid(
	release_in in varchar2,
	server_arch_id_in in number,
	org_id_in in number := -1,
	user_id_in in number := null
    ) return number is
	denied_channel_id number := null;
	valid_org_id number := org_id_in;
	valid_user_id number := user_id_in;
	channel_subscribable number;
    begin
	if org_id_in = -1 and user_id_in is not null then
	    begin
		select org_id
		  into valid_org_id
		  from web_contact
		 where id = user_id_in;
	    exception
		when no_data_found then
		    valid_user_id := null;
		    valid_org_id := -1;
	    end;
	end if;
	for c in base_channel_cursor(release_in, server_arch_id_in, valid_org_id)
	loop
	    if valid_user_id is null then
		return c.id;
	    end if;
	    select loose_user_role_check(c.id, user_id_in, 'subscribe')
	      into channel_subscribable
	      from dual;
	    if channel_subscribable = 1 then
		return c.id;
	    end if;
	    denied_channel_id := c.id;
	end loop base_channel_fetch;
	if denied_channel_id is not null then
	    rhn_exception.raise_exception('no_subscribe_permissions');
	end if;
	return NULL;
    end base_channel_rel_archid;
    procedure bulk_guess_server_base_from(
	set_label_in in varchar2,
	set_uid_in in number,
	channel_id_in in number
    ) is
	cursor channels(server_id_in in number) is
	    select	rsc.channel_id
	    from	rhnServerChannel rsc,
			rhnChannel rc
	    where	server_id_in = rsc.server_id
			and rsc.channel_id = rc.id
			and rc.parent_channel is null;
    begin
	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    for channel in channels(server.element)
	    loop
		if channel.channel_id = channel_id_in
		then
		    insert into rhnSet (user_id, label, element) values (set_uid_in, set_label_in || 'baseguess', server.element);
		end if;
	    end loop channel;
	end loop server;
	bulk_guess_server_base(set_label_in||'baseguess',set_uid_in);
	delete from rhnSet where label = set_label_in||'baseguess' and user_id = set_uid_in;
    end;
    PROCEDURE clear_subscriptions(server_id_in IN NUMBER)
    IS
    	cursor server_channels(server_id_in in number) is
		select	s.org_id, sc.channel_id, cfm.channel_family_id
		from	rhnServer s,
			rhnServerChannel sc,
			rhnChannelFamilyMembers cfm
		where	s.id = server_id_in
			and s.id = sc.server_id
			and sc.channel_id = cfm.channel_id;
    BEGIN
    	for channel in server_channels(server_id_in)
	loop
		unsubscribe_server(server_id_in, channel.channel_id, 1, 1);
		rhn_channel.update_family_counts(channel.channel_family_id, channel.org_id);
	end loop channel;
    END clear_subscriptions;
    PROCEDURE unsubscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, unsubscribe_children_in number := 0)
    IS
	channel_family_id_val   NUMBER;
	server_org_id_val       NUMBER;
	available_subscriptions NUMBER;
	server_already_in_chan  BOOLEAN;
	cursor	channel_family_is_proxy(channel_family_id_in in number) is
		select	1
		from	rhnChannelFamily
		where	id = channel_family_id_in
		    and label = 'rhn-proxy';
	cursor	channel_family_is_satellite(channel_family_id_in in number) is
		select	1
		from	rhnChannelFamily
		where	id = channel_family_id_in
		    and label = 'rhn-satellite';
	cursor local_chk_server_parent_memb (
			server_id_in number,
			channel_id_in number ) is
		select	c.id
		from	rhnChannel			c,
				rhnServerChannel	sc
		where	1=1
			and c.parent_channel = channel_id_in
			and c.id = sc.channel_id
			and sc.server_id = server_id_in;
    BEGIN
        FOR child IN local_chk_server_parent_memb(server_id_in, channel_id_in)
        LOOP
	    if unsubscribe_children_in = 1 then
	    	unsubscribe_server(server_id_in => server_id_in,
								channel_id_in => child.id,
								immediate_in => immediate_in,
								unsubscribe_children_in => unsubscribe_children_in);
	    else
	    	rhn_exception.raise_exception('channel_unsubscribe_child_exists');
	    end if;
        END LOOP child;
	server_already_in_chan := FALSE;
        FOR check_subscription IN check_server_subscription(server_id_in, channel_id_in)
        LOOP
            server_already_in_chan := TRUE;
        END LOOP check_subscription;
        IF NOT server_already_in_chan
        THEN
    	    RETURN;
        END IF;
	insert into rhnServerHistory (id,server_id,summary,details) (
	    select  rhn_event_id_seq.nextval,
	    	    server_id_in,
		    'unsubscribed from channel ' || SUBSTR(c.label, 0, 106),
		    c.label
	    from    rhnChannel c
	    where   c.id = channel_id_in
	);
	UPDATE rhnServer SET channels_changed = sysdate WHERE id = server_id_in;
        DELETE FROM rhnServerChannel WHERE server_id = server_id_in AND channel_id = channel_id_in;
    	queue_server(server_id_in, immediate_in);
	channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
	IF channel_family_id_val IS NULL
	THEN
	    rhn_exception.raise_exception('channel_unsubscribe_no_family');
	END IF;
	for ignore in channel_family_is_satellite(channel_family_id_val) loop
		delete from rhnSatelliteInfo where server_id = server_id_in;
		delete from rhnSatelliteChannelFamily where server_id = server_id_in;
	end loop;
	for ignore in channel_family_is_proxy(channel_family_id_val) loop
		delete from rhnProxyInfo where server_id = server_id_in;
	end loop;
	DELETE FROM rhnChannelFamilyLicenseConsent
	 WHERE channel_family_id = channel_family_id_val
	   AND server_id = server_id_in;
	SELECT org_id INTO server_org_id_val
	  FROM rhnServer
	 WHERE id = server_id_in;
	rhn_channel.update_family_counts(channel_family_id_val, server_org_id_val);
    END unsubscribe_server;
    PROCEDURE bulk_unsubscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
            rhn_channel.unsubscribe_server(server.element, channel_id_in, 0);
        END LOOP server;
    END bulk_unsubscribe_server;
    FUNCTION family_for_channel(channel_id_in IN NUMBER)
    RETURN NUMBER
    IS
    	channel_family_id_val NUMBER;
    BEGIN
    	SELECT channel_family_id INTO channel_family_id_val
	  FROM rhnChannelFamilyMembers
         WHERE channel_id = channel_id_in;
	RETURN channel_family_id_val;
    EXCEPTION
        WHEN NO_DATA_FOUND
	THEN
	    RETURN NULL;
    END family_for_channel;
    FUNCTION available_family_subscriptions(channel_family_id_in IN NUMBER, org_id_in IN NUMBER)
    RETURN NUMBER
    IS
    	cfp channel_family_perm_cursor%ROWTYPE;
    	current_members_val NUMBER;
	max_members_val     NUMBER;
	found               NUMBER;
    BEGIN
	IF NOT channel_family_perm_cursor%ISOPEN
	THEN
	    OPEN channel_family_perm_cursor(channel_family_id_in, org_id_in);
	END IF;
	FETCH channel_family_perm_cursor INTO cfp;
	WHILE channel_family_perm_cursor%FOUND
    	LOOP
	    found := 1;
	    current_members_val := cfp.current_members;
	    max_members_val := cfp.max_members;
	    FETCH channel_family_perm_cursor INTO cfp;
	END LOOP;
	IF channel_family_perm_cursor%ISOPEN
	THEN
	    CLOSE channel_family_perm_cursor;
	END IF;
	IF found IS NULL
	THEN
	    RETURN 0;
	END IF;
	IF max_members_val IS NULL
	THEN
	    RETURN NULL;
	END IF;
	RETURN max_members_val - current_members_val;
    END available_family_subscriptions;
    PROCEDURE update_family_counts(channel_family_id_in IN NUMBER, org_id_in IN NUMBER)
    IS
    BEGIN
		update rhnPrivateChannelFamily
		set current_members = (
				select	count(sc.server_id)
				from	rhnServerChannel sc,
						rhnChannelFamilyMembers cfm,
						rhnServer s
				where	s.org_id = org_id_in
					and s.id = sc.server_id
					and cfm.channel_family_id = channel_family_id_in
					and cfm.channel_id = sc.channel_id
				)
			where org_id = org_id_in
				and channel_family_id = channel_family_id_in;
    END update_family_counts;
    FUNCTION available_chan_subscriptions(channel_id_in IN NUMBER, org_id_in IN NUMBER)
    RETURN NUMBER
    IS
	channel_family_id_val NUMBER;
    BEGIN
    	SELECT channel_family_id INTO channel_family_id_val
	  FROM rhnChannelFamilyMembers
	 WHERE channel_id = channel_id_in;
	 RETURN rhn_channel.available_family_subscriptions(channel_family_id_val, org_id_in);
    END available_chan_subscriptions;
    procedure entitle_customer(customer_id_in in number, channel_family_id_in in number, quantity_in in number)
    is
		cursor permissions is
			select	1
			from	rhnPrivateChannelFamily pcf
			where	pcf.org_id = customer_id_in
				and	pcf.channel_family_id = channel_family_id_in;
    begin
		for perm in permissions loop
			set_family_maxmembers(
				customer_id_in,
				channel_family_id_in,
				quantity_in
			);
			rhn_channel.update_family_counts(
				channel_family_id_in,
				customer_id_in
			);
			return;
		end loop;
		insert into rhnPrivateChannelFamily pcf (
				channel_family_id, org_id, max_members, current_members
			) values (
				channel_family_id_in, customer_id_in, quantity_in, 0
			);
    end;
    procedure set_family_maxmembers(customer_id_in in number, channel_family_id_in in number, quantity_in in number)
    is
	cursor servers is
	    select  server_id from (
		select	rownum row_number, server_id, modified from (
		    select  rcfs.server_id,
			    rcfs.modified
		    from    rhnChannelFamilyServers rcfs
		    where   rcfs.customer_id = customer_id_in
			    and rcfs.channel_family_id = channel_family_id_in
		    order by modified
		)
		where rownum > quantity_in
	    );
    begin
	for server in servers
	loop
	    rhn_channel.unsubscribe_server_from_family(server.server_id, channel_family_id_in);
	end loop;
	update	rhnPrivateChannelFamily pcf
	set	pcf.max_members = quantity_in
	where	pcf.org_id = customer_id_in
		and pcf.channel_family_id = channel_family_id_in;
    end;
    procedure unsubscribe_server_from_family(server_id_in in number, channel_family_id_in in number)
    is
    begin
	delete
	from	rhnServerChannel rsc
	where	rsc.server_id = server_id_in
		and channel_id in (
			select	rcfm.channel_id
			from	rhnChannelFamilyMembers rcfm
			where	rcfm.channel_family_id = channel_family_id_in);
    end;
    function get_org_id(channel_id_in in number)
    return number
    is
    	org_id_out number;
    begin
    	select org_id into org_id_out
	  from rhnChannel
	 where id = channel_id_in;
	return org_id_out;
    end get_org_id;
    function get_cfam_org_access(cfam_id_in in number, org_id_in in number)
    return number
    is
    	cursor	families is
			select	1
			from	rhnOrgChannelFamilyPermissions cfp
			where	cfp.org_id = org_id_in;
    begin
		for family in families loop
	    	return 1;
		end loop;
		return 0;
    end;
    function get_org_access(channel_id_in in number, org_id_in in number)
    return number
    is
    	throwaway number;
    begin
	select distinct 1 into throwaway
	  from rhnChannelFamilyMembers CFM,
	       rhnOrgChannelFamilyPermissions CFP
	 where cfp.org_id = org_id_in
	   and CFM.channel_family_id = CFP.channel_family_id
	   and CFM.channel_id = channel_id_in;
	return 1;
    exception
    	when no_data_found
	    then
	    return 0;
    end;
    function user_role_check_debug(channel_id_in in number, user_id_in in number, role_in in varchar2, reason_out out varchar2)
    return number
    is
    	org_id number;
    begin
    	org_id := rhn_user.get_org_id(user_id_in);
    	if role_in = 'manage' and NVL(rhn_channel.get_org_id(channel_id_in), -1) <> org_id
	then
	    reason_out := 'channel_not_owned';
	    return 0;
	end if;
    	if role_in = 'subscribe' and rhn_channel.get_org_access(channel_id_in, org_id) = 0
	then
	    reason_out := 'channel_not_available';
	    return 0;
	end if;
    	if rhn_user.check_role_implied(user_id_in, 'channel_admin') = 1
	then
	    reason_out := 'channel_admin';
	    return 1;
	end if;
	if role_in = 'subscribe'
	then
	    if rhn_channel.org_channel_setting(channel_id_in, org_id, 'not_globally_subscribable') = 0
	    then
	    	reason_out := 'globally_subscribable';
		return 1;
	    end if;
	end if;
	reason_out := 'direct_permission';
	return rhn_channel.direct_user_role_check(channel_id_in, user_id_in, role_in);
    end;
    function user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
  	throwaway varchar2(256);
    begin
    	return rhn_channel.user_role_check_debug(channel_id_in, user_id_in, role_in, throwaway);
    end;
    function loose_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
    begin
        if user_id_in is null then
            return 1;
        end if;
        return user_role_check(channel_id_in, user_id_in, role_in);
    end loose_user_role_check;
    function direct_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
    	throwaway number;
    begin
	select 1 into throwaway
	  from rhnChannelPermissionRole CPR,
	       rhnChannelPermission CP
	 where CP.user_id = user_id_in
	   and CP.channel_id = channel_id_in
	   and CPR.label = role_in
	   and CP.role_id = CPR.id;
	return 1;
    exception
    	when no_data_found
	    then
	    return 0;
    end;
    function org_channel_setting(channel_id_in in number, org_id_in in number, setting_in in varchar2)
    return number
    is
    	throwaway number;
    begin
	select 1 into throwaway
	  from rhnOrgChannelSettingsType OCST,
	       rhnOrgChannelSettings OCS
	 where OCS.org_id = org_id_in
	   and OCS.channel_id = channel_id_in
	   and OCST.label = setting_in
	   and OCS.setting_id = OCST.id;
	return 1;
    exception
    	when no_data_found
	    then
	    return 0;
    end;
    FUNCTION channel_priority(channel_id_in IN number)
    RETURN number
    IS
        cursor channels is
                select  2 priority
                from    rhnChannel
                where   id = channel_id_in
                        and org_id is not null
                union all
                select  4 priority
                from    rhnChannel
                where   id = channel_id_in
                        and end_of_life is not null
                union all
                select  1 priority
                from    rhnChannel
                where   id = channel_id_in
                        and name like '%Enterprise%'
                union all
                select  3 priority
                from    dual;
    BEGIN
        for channel in channels loop
                return channel.priority;
        end loop;
    end channel_priority;
    procedure delete_server_channels(server_id_in in number)
    is
    begin
	update	rhnPrivateChannelFamily
	set	current_members = current_members -1
	where	org_id in (
			select	org_id
			from	rhnServer
			where	id = server_id_in
		)
		and channel_family_id in (
			select	rcfm.channel_family_id
			from	rhnChannelFamilyMembers rcfm,
				rhnServerChannel rsc
			where	rsc.server_id = server_id_in
				and rsc.channel_id = rcfm.channel_id
		);
    end;
	procedure refresh_newest_package(channel_id_in in number, caller_in in varchar2 := '(unknown)')
	is
	begin
		delete from rhnChannelNewestPackage where channel_id = channel_id_in;
		insert into rhnChannelNewestPackage
			( channel_id, name_id, evr_id, package_id, package_arch_id )
			(	select	channel_id,
						name_id, evr_id,
						package_id, package_arch_id
				from	rhnChannelNewestPackageView
				where	channel_id = channel_id_in
			);
                insert into rhnChannelNewestPackageAudit (channel_id, caller)
                    values (channel_id_in, caller_in);
                update rhnChannel
                    set last_modified = greatest(sysdate, last_modified + 1/86400)
                    where id = channel_id_in;
	end;
END rhn_channel;
/

SHOW ERRORS;

--
-- 41. RHN_CONFIG  (Package Body) 
--
--
-- RHN_CONFIG  (Package Body) 
--
CREATE OR REPLACE package body
rhn_config
is
	procedure prune_org_configs (
		org_id_in in number,
		total_in in number
	) is
	begin
		null;
	end prune_org_configs;
	function insert_revision (
		revision_in in number,
		config_file_id_in in number,
		config_content_id_in in number,
		config_info_id_in in number,
		delim_start_in in varchar2 := '{@',
		delim_end_in in varchar2 := '@}'
	) return number is
		retval number;
		cursor affected_orgs is
			select	cc.org_id id
			from	rhnConfigChannel cc,
					rhnConfigFile cf
			where	cf.id = config_file_id_in
				and cf.config_channel_id = cc.id;
	begin
		insert into rhnConfigRevision(id, revision, config_file_id,
				config_content_id, config_info_id, delim_start, delim_end
			) values (
				rhn_confrevision_id_seq.nextval, revision_in, config_file_id_in,
				config_content_id_in, config_info_id_in, delim_start_in,
				delim_end_in
			)
			returning id into retval;
		for org in affected_orgs loop
			rhn_quota.update_org_quota(org.id);
		end loop;
		return retval;
	end insert_revision;
	procedure delete_revision (
		config_revision_id_in in number,
		org_id_in in number := -1
	) is
		cfid number;
		ccid number;
		oid number;
		latest_crid number;
		others number := 0;
		cursor snapshots is
			select	scr.snapshot_id id
			from	rhnSnapshot s,
					rhnSnapshotConfigRevision scr
			where	scr.config_revision_id = config_revision_id_in
					and scr.snapshot_id = s.id
					and s.invalid is null;
		cursor other_revisions(config_content_id_in in number) is
			select	1
			from	rhnConfigRevision
			where	config_content_id = config_content_id_in;
	begin
		for snapshot in snapshots loop
			update		rhnSnapshot s
				set		s.invalid =
							lookup_snapshot_invalid_reason('cr_removed')
				where	s.id = snapshot.id;
		end loop;
		if org_id_in < 0 then
			select	cr.config_content_id, cc.org_id
			into	ccid, oid
			from	rhnConfigChannel cc,
					rhnConfigFile cf,
					rhnConfigRevision cr
			where	cr.id = config_revision_id_in
				and cr.config_file_id = cf.id
				and cf.config_channel_id = cc.id;
		else
			select	cr.config_content_id, org_id_in
			into	ccid, oid
			from	rhnConfigRevision cr
			where	cr.id = config_revision_id_in;
		end if;
		delete from rhnConfigRevision where id = config_revision_id_in;
		for other_revision in other_revisions(ccid) loop
			others := 1;
			exit;
		end loop;
		if others = 0 then
			delete from rhnConfigContent where id = ccid;
		end if;
		if org_id_in < 0 then
			select	cf.latest_config_revision_id,
					cf.id
			into	latest_crid,
					cfid
			from	rhnConfigFile cf,
					rhnConfigRevision cr
			where	cr.id = config_revision_id_in
				and cr.config_file_id = cf.id;
			if latest_crid = config_revision_id_in then
				latest_crid := rhn_config.get_latest_revision(cfid);
				if latest_crid is not null then
					update rhnConfigFile set latest_config_revision_id = latest_crid
						where id = cfid;
				else
					delete from rhnConfigFile where id = cfid;
				end if;
			end if;
		end if;
		rhn_quota.update_org_quota(oid);
	end delete_revision;
	function get_latest_revision (
		config_file_id_in in number
	) return number is
		cursor revisions is
			select	cr.id
			from	rhnConfigRevision cr
			where	cr.config_file_id = config_file_id_in
			order by revision desc;
	begin
		for revision in revisions loop
			return revision.id;
		end loop;
		return null;
	end get_latest_revision;
	function insert_file (
		config_channel_id_in in number,
		name_in in varchar2
	) return number is
		retval number;
	begin
		select	rhn_conffile_id_seq.nextval
		into	retval
		from	dual;
		insert into rhnConfigFile(id, config_channel_id, config_file_name_id,
				state_id
			) (
				select	retval,
						config_channel_id_in,
						lookup_config_filename(name_in),
						id
				from	rhnConfigFileState
				where	label = 'alive'
			);
		return retval;
	end insert_file;
	procedure delete_file (
		config_file_id_in in number
	) is
		cursor revisions is
			select	cr.id, cc.org_id
			from	rhnConfigChannel cc,
					rhnConfigRevision cr,
					rhnConfigFile cf
			where	cf.id = config_file_id_in
				and cf.config_channel_id = cc.id
				and cr.config_file_id = cf.id;
		org_id number;
	begin
		for revision in revisions loop
			rhn_config.delete_revision(revision.id, revision.org_id);
			org_id := revision.org_id;
		end loop;
		rhn_quota.update_org_quota(org_id);
		delete from rhnConfigFile where id = config_file_id_in;
	end delete_file;
	function insert_channel (
		org_id_in in number,
		type_in in varchar2,
		name_in in varchar2,
		label_in in varchar2,
		description_in in varchar2
	) return number is
		retval number;
	begin
		select	rhn_confchan_id_seq.nextval
		into	retval
		from	dual;
		insert into rhnConfigChannel(id, org_id, confchan_type_id,
				name, label, description
			) (
				select	retval,
						org_id_in,
						cct.id,
						name_in,
						label_in,
						description_in
				from	rhnConfigChannelType cct
				where	label = type_in
			);
		return retval;
	end insert_channel;
	procedure delete_channel (
		config_channel_id_in in number
	) is
		cursor config_files is
			select	id
			from	rhnConfigFile
			where	config_channel_id = config_channel_id_in;
	begin
		for config_file in config_files loop
			rhn_config.delete_file(config_file.id);
		end loop;
		delete from rhnConfigChannel where id = config_channel_id_in;
	end delete_channel;
end rhn_config;
/

SHOW ERRORS;

--
-- 42. RHN_ENTITLEMENTS  (Package Body) 
--
--
-- RHN_ENTITLEMENTS  (Package Body) 
--
CREATE OR REPLACE package body rhn_entitlements
is
	body_version varchar2(100) := '$Id: satellite-3.4-to-3.6.sql 64427 2005-09-09 14:08:21Z jslagle $';
	function entitlement_grants_service (
	    entitlement_in in varchar2,
	    service_level_in in varchar2
	) return number	is
	begin
		if service_level_in = 'provisioning' then
			if entitlement_in = 'provisioning_entitled' then
				return 1;
			else
				return 0;
			end if;
		elsif service_level_in = 'management' then
			if entitlement_in = 'provisioning_entitled' then
				return 1;
			elsif entitlement_in = 'enterprise_entitled' then
				return 1;
			elsif entitlement_in = 'nonlinux_entitled' then
				return 1;
			else
				return 0;
			end if;
		elsif service_level_in = 'monitoring' then
			if entitlement_in = 'monitoring_entitled' then
				return 1;
			end if;
		elsif service_level_in = 'updates' then
			return 1;
		else
			return 0;
		end if;
	end entitlement_grants_service;
	function lookup_entitlement_group (
		org_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) return number is
		cursor server_groups is
			select	sg.id				server_group_id
			from	rhnServerGroup		sg,
					rhnServerGroupType	sgt
			where	sgt.label = type_label_in
				and sgt.id = sg.group_type
				and sg.org_id = org_id_in;
	begin
		for sg in server_groups loop
			return sg.server_group_id;
		end loop;
		return rhn_entitlements.create_entitlement_group(
				org_id_in,
				type_label_in
			);
	end lookup_entitlement_group;
	function create_entitlement_group (
		org_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) return number is
		sg_id_val number;
	begin
		select	rhn_server_group_id_seq.nextval
		into	sg_id_val
		from	dual;
		insert into rhnServerGroup (
				id, name, description, max_members, current_members,
				group_type, org_id
			) (
				select	sg_id_val, sgt.label, sgt.label,
						0, 0, sgt.id, org_id_in
				from	rhnServerGroupType sgt
				where	sgt.label = type_label_in
			);
		return sg_id_val;
	end create_entitlement_group;
	procedure entitle_server (
		server_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) is
		cursor servergroups is
			select	sg.id id,
					s.server_arch_id server_arch_id,
					sgt.id server_group_type
			from	rhnServerGroupType sgt,
					rhnServerGroup sg,
					rhnServer s
			where	s.id = server_id_in
				and s.org_id = sg.org_id
				and sgt.label = type_label_in
				and sg.group_type = sgt.id;
		cursor sg_arches(sgt_id_in in number, sa_id_in in number) is
			select	1
			from	rhnServerServerGroupArchCompat
			where	server_group_type = sgt_id_in
				and server_arch_id = sa_id_in;
		previous_ent rhnServerGroupType.label%TYPE;
	begin
		previous_ent := rhn_entitlements.get_server_entitlement(server_id_in);
		if previous_ent != 'none' then
			rhn_exception.raise_exception(
								'no_server_multiple_swmgr_entitlements');
		end if;
		for sg in servergroups loop
			for a in sg_arches(sg.server_group_type, sg.server_arch_id) loop
				rhn_server.insert_into_servergroup(server_id_in, sg.id);
				return;
			end loop;
			rhn_exception.raise_exception('mismatching_entitlement');
		end loop;
		rhn_exception.raise_exception('servergroup_max_members');
	end entitle_server;
	procedure unentitle_server (
		server_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) is
		group_id number;
	begin
		select	sg.id
		into	group_id
		from	rhnServerGroupType sgt,
				rhnServerGroup sg,
				rhnServerGroupMembers sgm,
				rhnServer s
		where	s.id = server_id_in
			and s.id = sgm.server_id
			and sgm.server_group_id = sg.id
			and sg.org_id = s.org_id
			and sgt.label = type_label_in
			and sgt.id = sg.group_type;
		rhn_server.delete_from_servergroup(server_id_in, group_id);
		exception
		when no_data_found then
					null;
	end unentitle_server;
	procedure upgrade_server (
		server_id_in in number
	) is
		old_sgid number;
		new_sgid number;
		new_sg_max number;
		new_sg_used number;
	begin
		begin
			select	sg0.id, sg1.id,
					sg1.max_members, sg1.current_members
			into	old_sgid, new_sgid,
					new_sg_max, new_sg_used
			from	rhnServerGroup sg1,
					rhnServerGroupType sgt1,
					rhnServerGroupMembers sgm,
					rhnServerGroup sg0,
					rhnServerGroupType sgt0,
					rhnServer s
			where	s.id = server_id_in
				and s.org_id = sg0.org_id
				and sgt0.label = 'enterprise_entitled'
				and sgt0.id = sg0.group_type
				and s.id = sgm.server_id
				and sgm.server_group_id = sg0.id
				and s.org_id = sg1.org_id
				and sgt1.label = 'provisioning_entitled'
				and sgt1.id = sg1.group_type;
		exception
			when no_data_found then
				rhn_exception.raise_exception('invalid_upgrade');
		end;
		if new_sg_used >= new_sg_max then
			rhn_exception.raise_exception('servergroup_max_members');
		end if;
		rhn_server.delete_from_servergroup(server_id_in, old_sgid);
		rhn_server.insert_into_servergroup(server_id_in, new_sgid);
	end upgrade_server;
	function get_server_entitlement (
		server_id_in in number
	) return varchar2 is
		cursor server_groups is
			select	sgt.label
			from	rhnServerGroupType		sgt,
					rhnServerGroup			sg,
					rhnServerGroupMembers	sgm
			where	1=1
				and sgm.server_id = server_id_in
				and sg.id = sgm.server_group_id
				and sgt.id = sg.group_type
				and sgt.label in (
					'sw_mgr_entitled','enterprise_entitled',
					'provisioning_entitled', 'nonlinux_entitled',
					'monitoring_entitled'
					);
	begin
		for sg in server_groups loop
			return sg.label;
		end loop;
		return 'none';
	end get_server_entitlement;
	procedure modify_org_service (
		org_id_in in number,
		service_label_in in varchar2,
		enable_in in char
	) is
		type roles_v is varray(10) of rhnUserGroupType.label%TYPE;
		roles_to_process roles_v;
		cursor roles(role_label_in in varchar2) is
			select	label, id
			from	rhnUserGroupType
			where	label = role_label_in;
		cursor org_roles(role_label_in in varchar2) is
			select	1
			from	rhnUserGroup ug,
					rhnUserGroupType ugt
			where	ugt.label = role_label_in
				and ug.org_id = org_id_in
				and ugt.id = ug.group_type;
		type ents_v is varray(10) of rhnOrgEntitlementType.label%TYPE;
		ents_to_process ents_v;
		cursor ents(ent_label_in in varchar2) is
			select	label, id
			from	rhnOrgEntitlementType
			where	label = ent_label_in;
		cursor org_ents(ent_label_in in varchar2) is
			select	1
			from	rhnOrgEntitlements oe,
					rhnOrgEntitlementType oet
			where	oet.label = ent_label_in
				and oe.org_id = org_id_in
				and oet.id = oe.entitlement_id;
		create_row char(1);
	begin
		ents_to_process := ents_v();
		roles_to_process := roles_v();
		if service_label_in = 'enterprise' or service_label_in = 'management' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'sw_mgr_enterprise';
			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'org_admin';
			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'system_group_admin';
			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'activation_key_admin';
			if is_satellite() = 1 then
				roles_to_process.extend;
				roles_to_process(roles_to_process.count) := 'org_applicant';
			end if;
		elsif service_label_in = 'provisioning' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'rhn_provisioning';
			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'system_group_admin';
			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'activation_key_admin';
			if is_satellite() = 1 then
				roles_to_process.extend;
				roles_to_process(roles_to_process.count) := 'config_admin';
			end if;
			if enable_in = 'Y' then
				ents_to_process.extend;
				ents_to_process(ents_to_process.count) := 'sw_mgr_enterprise';
			end if;
		elsif service_label_in = 'monitoring' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'rhn_monitor';
			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'monitoring_admin';
		elsif service_label_in = 'nonlinux' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'rhn_nonlinux';
			if is_satellite() = 1 then
				roles_to_process.extend;
				roles_to_process(roles_to_process.count) := 'config_admin';
			end if;
		end if;
		if enable_in = 'Y' then
			for i in 1..ents_to_process.count loop
				for ent in ents(ents_to_process(i)) loop
					create_row := 'Y';
					for oe in org_ents(ent.label) loop
						create_row := 'N';
					end loop;
					if create_row = 'Y' then
						insert into rhnOrgEntitlements(org_id, entitlement_id)
							values (org_id_in, ent.id);
					end if;
				end loop;
			end loop;
			for i in 1..roles_to_process.count loop
				for role in roles(roles_to_process(i)) loop
					create_row := 'Y';
					for o_r in org_roles(role.label) loop
						create_row := 'N';
					end loop;
					if create_row = 'Y' then
						insert into rhnUserGroup(
								id, name, description, current_members,
								group_type, org_id
							) (
								select	rhn_user_group_id_seq.nextval,
										ugt.name || 's',
										ugt.name || 's for Org ' ||
											o.name || ' ('|| o.id ||')',
										0, ugt.id, o.id
								from	rhnUserGroupType ugt,
										web_customer o
								where	o.id = org_id_in
									and ugt.id = role.id
							);
					end if;
				end loop;
			end loop;
		else
			for i in 1..ents_to_process.count loop
				for ent in ents(ents_to_process(i)) loop
					delete from rhnOrgEntitlements
					 where org_id = org_id_in
					   and entitlement_id = ent.id;
				end loop;
			end loop;
		end if;
	end modify_org_service;
	procedure set_customer_enterprise (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'enterprise', 'Y');
	end set_customer_enterprise;
	procedure set_customer_provisioning (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'provisioning', 'Y');
	end set_customer_provisioning;
	procedure set_customer_nonlinux (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'nonlinux', 'Y');
	end set_customer_nonlinux;
	procedure unset_customer_enterprise (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'enterprise', 'N');
	end unset_customer_enterprise;
	procedure unset_customer_provisioning (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'provisioning', 'N');
	end unset_customer_provisioning;
	procedure unset_customer_nonlinux (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'nonlinux', 'N');
	end unset_customer_nonlinux;
	procedure prune_group (
		group_id_in in number,
		type_in in char,
		quantity_in in number
	) is
		cursor usergroups is
			select	user_id, user_group_id, ugt.label
			from	rhnUserGroupType	ugt,
					rhnUserGroup		ug,
					rhnUserGroupMembers	ugm
			where	1=1
				and ugm.user_group_id = group_id_in
				and ugm.user_id in (
					select	user_id
					from	(
						select	rownum row_number,
								user_id,
								time
						from	(
							select	user_id,
									modified time
							from	rhnUserGroupMembers
							where	user_group_id = group_id_in
							order by time asc
						)
					)
					where	row_number > quantity_in
				)
				and ugm.user_group_id = ug.id
				and ug.group_type = ugt.id;
		cursor servergroups is
			select	server_id, server_group_id, sgt.label
			from	rhnServerGroupType		sgt,
					rhnServerGroup			sg,
					rhnServerGroupMembers	sgm
			where	1=1
				and sgm.server_group_id = group_id_in
				and sgm.server_id in (
					select	server_id
					from	(
						select	rownum row_number,
								server_id,
								time
						from	(
							select	server_id,
									modified time
							from	rhnServerGroupMembers
							where	server_group_id = group_id_in
							order by time asc
						)
					)
					where	row_number > quantity_in
				)
				and sgm.server_group_id = sg.id
				and sg.group_type = sgt.id;
	begin
		if type_in = 'U' then
			update		rhnUserGroup
				set		max_members = quantity_in
				where	id = group_id_in;
			for ug in usergroups loop
				rhn_user.remove_from_usergroup(ug.user_id, ug.user_group_id);
			end loop;
		elsif type_in = 'S' then
			update		rhnServerGroup
				set		max_members = quantity_in
				where	id = group_id_in;
			for sg in servergroups loop
				unentitle_server(sg.server_id, sg.label);
				rhn_channel.clear_subscriptions(sg.server_id);
			end loop;
		end if;
	end prune_group;
	procedure set_group_count (
		customer_id_in in number,
		type_in in char,
		group_type_in in number,
		quantity_in in number
	) is
		group_id number;
		quantity number;
	begin
		quantity := quantity_in;
		if quantity is not null and quantity < 0 then
			quantity := 0;
		end if;
		if type_in = 'U' then
			select	rug.id
			into	group_id
			from	rhnUserGroup rug
			where	1=1
				and rug.org_id = customer_id_in
				and rug.group_type = group_type_in;
		elsif type_in = 'S' then
			select	rsg.id
			into	group_id
			from	rhnServerGroup rsg
			where	1=1
				and rsg.org_id = customer_id_in
				and rsg.group_type = group_type_in;
		end if;
		rhn_entitlements.prune_group(
			group_id,
			type_in,
			quantity
		);
	exception
		when no_data_found then
			if type_in = 'U' then
				insert into rhnUserGroup (
						id, name, description, max_members, current_members,
						group_type, org_id, created, modified
					) (
						select	rhn_user_group_id_seq.nextval, name, name,
								quantity, 0, id, customer_id_in,
								sysdate, sysdate
						from	rhnUserGroupType
						where	id = group_type_in
				);
			elsif type_in = 'S' then
				insert into rhnServerGroup (
						id, name, description, max_members, current_members,
						group_type, org_id, created, modified
					) (
						select	rhn_server_group_id_seq.nextval, name, name,
								quantity, 0, id, customer_id_in,
								sysdate, sysdate
						from	rhnServerGroupType
						where	id = group_type_in
				);
			end if;
	end set_group_count;
	procedure prune_family (
		customer_id_in in number,
		channel_family_id_in in number,
		quantity_in in number
	) is
		cursor serverchannels is
			select	sc.server_id,
					sc.channel_id
			from	rhnServerChannel sc,
					rhnChannelFamilyMembers cfm
			where	1=1
				and cfm.channel_family_id = channel_family_id_in
				and cfm.channel_id = sc.channel_id
				and server_id in (
					select	server_id
					from	(
						select	server_id,
								time,
								rownum row_number
						from	(
							select	rs.id					server_id,
									rcfm.modified			time
							from	rhnServer				rs,
									rhnServerChannel		rsc,
									rhnChannelFamilyMembers	rcfm
							where	1=1
								and rs.org_id = customer_id_in
								and rs.id = rsc.server_id
								and rsc.channel_id = rcfm.channel_id
								and rcfm.channel_family_id =
									channel_family_id_in
							order by time asc
						)
					)
					where row_number > quantity_in
				);
	begin
		if customer_id_in is null then
			return;
		end if;
		update		rhnPrivateChannelFamily
			set		max_members = quantity_in
			where	1=1
				and org_id = customer_id_in
				and channel_family_id = channel_family_id_in;
		for sc in serverchannels loop
			rhn_channel.unsubscribe_server(sc.server_id, sc.channel_id, 1, 1);
		end loop;
	end prune_family;
	procedure set_family_count (
		customer_id_in in number,
		channel_family_id_in in number,
		quantity_in in number
	) is
		cursor privperms is
			select	1
			from	rhnPrivateChannelFamily
			where	org_id = customer_id_in
				and channel_family_id = channel_family_id_in;
		cursor pubperms is
			select	o.id org_id
			from	web_customer o,
					rhnPublicChannelFamily pcf
			where	pcf.channel_family_id = channel_family_id_in;
		quantity number;
		done number := 0;
	begin
		quantity := quantity_in;
		if quantity is not null and quantity < 0 then
			quantity := 0;
		end if;
		if customer_id_in is not null then
			for perm in privperms loop
				rhn_entitlements.prune_family(
					customer_id_in,
					channel_family_id_in,
					quantity
				);
				update rhnPrivateChannelFamily
					set max_members = quantity
					where org_id = customer_id_in
						and channel_family_id = channel_family_id_in;
				return;
			end loop;
			insert into rhnPrivateChannelFamily (
					channel_family_id, org_id, max_members, current_members
				) values (
					channel_family_id_in, customer_id_in, quantity, 0
				);
			return;
		end if;
		for perm in pubperms loop
			if quantity = 0 then
				rhn_entitlements.prune_family(
					perm.org_id,
					channel_family_id_in,
					quantity
				);
				if done = 0 then
					delete from rhnPublicChannelFamily
						where channel_family_id = channel_family_id_in;
				end if;
			end if;
			done := 1;
		end loop;
		if done != 1 then
			insert into rhnPublicChannelFamily (
					channel_family_id
				) values (
					channel_family_id_in
				);
		end if;
	end set_family_count;
	procedure entitle_last_modified_servers (
		customer_id_in in number,
		type_label_in in varchar2,
		quantity_in in number
	) is
		cursor servers(cid_in in number, quant_in in number) is
			select	server_id
			from	(
						select	rownum row_number,
								server_id
						from	(
									select	rs.id server_iD
									from	rhnServer rs
									where	1=1
										and rs.org_id = cid_in
										and not exists (
											select	1
											from	rhnServerGroup sg,
													rhnServerGroupMembers rsgm
											where	rsgm.server_id = rs.id
												and rsgm.server_group_id = sg.id
												and sg.group_type is not null
										)
									order by modified desc
								)
					)
			where	row_number <= quant_in;
	begin
		for server in servers(customer_id_in, quantity_in) loop
			rhn_entitlements.entitle_server(server.server_id, type_label_in);
		end loop;
	end entitle_last_modified_servers;
	procedure get_available_slots (
		customer_id_in in number,
		type_label_in in varchar2,
		server_group_id_out out number,
		available_out out number,
		upgrade_out out number
	) is
		mgmt_slots_unused number;
		mgmt_slots_used number;
		prov_slots_unused number;
		prov_slots_used number;
		swmgr_slots_unused number;
		sgid number;
	begin
		if type_label_in in (
					'sw_mgr_entitled',
					'nonlinux_entitled',
					'monitoring_entitled'
				) then
			begin
				select	max_members - current_members, sg.id
				into	swmgr_slots_unused, sgid
				from	rhnServerGroup		sg,
						rhnServerGroupType	sgt
				where	sgt.label = type_label_in
					and sg.org_id = customer_id_in
					and sg.group_type = sgt.id;
				available_out := swmgr_slots_unused;
				upgrade_out := 0;
				server_group_id_out := sgid;
			exception
				when no_data_found then
					available_out := 0;
					if type_label_in = 'sw_mgr_entitled' then
						available_out := 1;
					end if;
					upgrade_out := 0;
					server_group_id_out := null;
			end;
			return;
		end if;
		begin
			select	sg.current_members,
					sg.max_members - sg.current_members,
					sg.id
			into	mgmt_slots_used,
					mgmt_slots_unused,
					sgid
			from	rhnServerGroup		sg,
					rhnServerGroupType	sgt
			where	sgt.label = 'enterprise_entitled'
				and sg.org_id = customer_id_in
				and sg.group_type = sgt.id;
		exception
			when no_data_found then
				available_out := 0;
				upgrade_out := 0;
				server_group_id_out := null;
				return;
		end;
		if type_label_in = 'enterprise_entitled' then
			available_out := mgmt_slots_unused;
			upgrade_out := 0;
			server_group_id_out := sgid;
			return;
		end if;
		begin
			select	sg.current_members,
					sg.max_members - sg.current_members,
					sg.id
			into	prov_slots_used,
					prov_slots_unused,
					sgid
			from	rhnServerGroup		sg,
					rhnServerGroupType	sgt
			where	sgt.label = 'provisioning_entitled'
				and sg.org_id = customer_id_in
				and sg.group_type = sgt.id;
		exception
			when no_data_found then
				available_out := 0;
				upgrade_out := 0;
				server_group_id_out := null;
				return;
		end;
		if prov_slots_unused - mgmt_slots_unused < 0 then
			available_out := prov_slots_unused;
			upgrade_out := 0;
		elsif prov_slots_unused > 0 then
			upgrade_out := prov_slots_unused - mgmt_slots_unused;
			available_out := prov_slots_unused - (prov_slots_unused - mgmt_slots_unused);
		else
			upgrade_out := 0;
			available_out := 0;
		end if;
		server_group_id_out := sgid;
		return;
	end get_available_slots;
	procedure prune_everything (
		customer_id_in in number
	) is
		cursor everything is
			select	sg.id					id,
					'S'						type,
					sg.max_members			quantity
			from	rhnServerGroup			sg
			where	sg.org_id = customer_id_in
			union
			select	ug.id					id,
					'U'						type,
					ug.max_members 			quantity
			from	rhnUserGroup			ug
			where	ug.org_id = customer_id_in
			union (
			select	cfp.channel_family_id	id,
					'C'						type,
					cfp.max_members			quantity
			from	rhnOrgChannelFamilyPermissions cfp
			where	cfp.org_id = customer_id_in
			union
			select	cfm.channel_family_id	id,
					'C'						type,
					0						quantity
			from	rhnChannelFamily		cf,
					rhnChannelFamilyMembers	cfm,
					rhnServerChannel		sc,
					rhnServer				s
			where	s.org_id = customer_id_in
				and s.id = sc.server_id
				and sc.channel_id = cfm.channel_id
				and cfm.channel_family_id = cf.id
				and cf.org_id is not null
				and cf.org_id != customer_id_in
				and not exists (
					select	1
					from	rhnOrgChannelFamilyPermissions cfp
					where	cfp.org_id = customer_id_in
						and cfp.channel_family_id = cfm.channel_family_id
					)
			);
	begin
		for one in everything loop
			if one.type in ('U','S') then
				prune_group(one.id, one.type, one.quantity);
			else
				prune_family(customer_id_in, one.id, one.quantity);
			end if;
		end loop;
	end prune_everything;
	procedure subscribe_newest_servers (
		customer_id_in in number
	) is
		cursor servers(cid_in in number) is
			select	s.id
			from	rhnServer			s
			where	1=1
				and s.org_id = cid_in
				and not exists (
						select 1
						from	rhnChannel			c,
								rhnServerChannel	sc
						where	sc.server_id = s.id
							and sc.channel_id = c.id
							and c.parent_channel is null
					)
			order by s.modified desc;
		channel_id number;
	begin
		for server in servers(customer_id_in) loop
			channel_id := rhn_channel.guess_server_base(server.id);
			if channel_id is not null then
				begin
					rhn_channel.subscribe_server(server.id, channel_id);
					commit;
				exception
					when others then
						null;
				end;
			end if;
		end loop;
	end subscribe_newest_servers;
end rhn_entitlements;
/

SHOW ERRORS;

--
-- 43. RHN_ORG  (Package Body) 
--
--
-- RHN_ORG  (Package Body) 
--
CREATE OR REPLACE PACKAGE BODY rhn_org
IS
	body_version varchar2(100) := '$Id: satellite-3.4-to-3.6.sql 64427 2005-09-09 14:08:21Z jslagle $';
    FUNCTION find_server_group_by_type(org_id_in NUMBER, group_label_in VARCHAR2)
    RETURN NUMBER
    IS
	server_group       server_group_by_label%ROWTYPE;
    BEGIN
    	OPEN server_group_by_label(org_id_in, group_label_in);
	FETCH server_group_by_label INTO server_group;
	CLOSE server_group_by_label;
	RETURN server_group.id;
    END find_server_group_by_type;
	procedure delete_user(user_id_in in number) is
		cursor is_admin is
			select	1
			from	rhnUserGroupType	ugt,
					rhnUserGroup		ug,
					rhnUserGroupMembers	ugm
			where	ugm.user_id = user_id_in
				and ugm.user_group_id = ug.id
				and ug.group_type = ugt.id
				and ugt.label = 'org_admin';
		cursor servergroups_needing_admins is
			select	usgp.server_group_id	server_group_id
			from	rhnUserServerGroupPerms	usgp
			where	1=1
				and usgp.user_id = user_id_in
				and not exists (
					select	1
					from	rhnUserServerGroupPerms	sq_usgp
					where	1=1
						and sq_usgp.server_group_id = usgp.server_group_id
						and	sq_usgp.user_id != user_id_in
				);
		cursor servergroups_for_deletion(org_id_in in number) is
			select	sgm.server_id, sgm.server_group_id
			from	rhnServerGroupMembers sgm,
					rhnServerGroup sg
			where	sg.org_id = org_id_in
				and sg.id = sgm.server_group_id;
		cursor servers_for_deletion(org_id_in in number) is
			select	id
			from	rhnServer
			where	org_id = org_id_in;
		cursor snapshots(sgid_in in number) is
			select	snapshot_id id
			from	rhnSnapshotServerGroup
			where	server_group_id = sgid_in;
		cursor messages is
			select	message_id id
			from	rhnUserMessage
			where	user_id = user_id_in;
		users			number;
		our_org_id		number;
		del_check_ok	number;
		other_users		number;
		other_org_admin	number;
	begin
		select	wc.org_id, xxrh_contact_del_check_num(user_id_in)
		into	our_org_id, del_check_ok
		from	web_contact wc
		where	id = user_id_in;
		if del_check_ok != 1 then
			rhn_exception.raise_exception('cannot_delete_user');
			return;
		end if;
		begin
			select	1
			into	other_users
			from	web_contact
			where	1=1
				and org_id = our_org_id
				and id != user_id_in
				and rownum = 1;
		exception
			when no_data_found then
				other_users := 0;
		end;
		if other_users != 0 then
			for ignore in is_admin loop
				begin
					select	new_ugm.user_id
					into	other_org_admin
					from	rhnUserGroupMembers	new_ugm,
							rhnUserGroupType	ugt,
							rhnUserGroup		ug,
							rhnUserGroupMembers	ugm
					where	ugm.user_id = user_id_in
						and ugm.user_group_id = ug.id
						and ug.group_type = ugt.id
						and ugt.label = 'org_admin'
						and ug.id = new_ugm.user_group_id
						and new_ugm.user_id != user_id_in
						and rownum = 1;
				exception
					when no_data_found then
						rhn_exception.raise_exception('cannot_delete_user');
				end;
				for sg in servergroups_needing_admins loop
					rhn_user.add_servergroup_perm(other_org_admin,
						sg.server_group_id);
				end loop;
			end loop;
		end if;
		for message in messages loop
			delete
				from	rhnUserMessage
				where	user_id = user_id_in
					and message_id = message.id;
			begin
				select	1
				into	users
				from	rhnUserMessage
				where	message_id = message.id
					and rownum = 1;
				delete
					from	rhnMessage
					where	id = message.id;
			exception
				when no_data_found then
					null;
			end;
		end loop;
		delete from rhn_command_queue_sessions where contact_id = user_id_in;
		delete from rhn_contact_methods where contact_id = user_id_in;
		delete from rhn_redirects where contact_id = user_id_in;
		if other_users != 0 then
			update		rhnRegToken
				set		user_id = other_org_admin
				where	org_id = our_org_id
					and user_id = user_id_in;
			begin
				delete from web_contact where id = user_id_in;
			exception
				when others then
					rhn_exception.raise_exception('cannot_delete_user');
			end;
		else
			for sgm in servergroups_for_deletion(our_org_id) loop
				for snapshot in snapshots(sgm.server_group_id) loop
					update rhnSnapshot
						set invalid =
							lookup_snapshot_invalid_reason('sg_removed')
						where id = snapshot.id;
					delete from rhnSnapshotServerGroup
						where snapshot_id = snapshot.id
						and server_group_id = sgm.server_group_id;
				end loop;
				rhn_server.delete_from_servergroup(
						sgm.server_id, sgm.server_group_id);
			end loop;
			update rhnServerGroup set group_type = null
				where org_id = our_org_id;
			delete from rhnServerGroup where org_id = our_org_id;
			for s in servers_for_deletion(our_org_id) loop
				delete_server(s.id);
			end loop;
			begin
				delete from rhn_check_suites where customer_id = our_org_id;
				delete from rhn_command_target where customer_id = our_org_id;
				delete from rhn_contact_groups where customer_id = our_org_id;
				delete from rhn_notification_formats
					where customer_id = our_org_id;
				delete from rhn_probe where customer_id = our_org_id;
				delete from rhn_redirects where customer_id = our_org_id;
				delete from rhn_sat_cluster where customer_id = our_org_id;
				delete from rhn_schedules where customer_id = our_org_id;
				delete from web_contact where id = user_id_in;
				delete from web_customer where id = our_org_id;
			exception
				when others then
					rhn_exception.raise_exception('cannot_delete_user');
			end;
		end if;
		return;
	end delete_user;
END rhn_org;
/

SHOW ERRORS;

--
-- 44. RHN_PACKAGE  (Package Body) 
--
ALTER PACKAGE RHN_PACKAGE COMPILE BODY;

--
-- 45. RHN_SERVER  (Package Body) 
--
--
-- RHN_SERVER  (Package Body) 
--
CREATE OR REPLACE package body rhn_server
is
    function system_service_level(
    	server_id_in in number,
	service_level_in in varchar2
    ) return number is
    begin
    	return rhn_entitlements.entitlement_grants_service(
	    rhn_entitlements.get_server_entitlement(server_id_in),
	    service_level_in
	);
    end system_service_level;
    function can_change_base_channel(server_id_in IN NUMBER)
    return number
    is
    	throwaway number;
    begin
	select 1 into throwaway
	  from rhnServer S
	 where S.id = server_id_in
	   and not exists (select 1 from rhnSatelliteInfo SI where SI.server_id = S.id)
	   and not exists (select 1 from rhnProxyInfo PI where PI.server_id = S.id);
	return 1;
    exception
    	when no_data_found
	    then
	    return 0;
    end can_change_base_channel;
    procedure set_custom_value(
    	server_id_in in number,
	user_id_in in number,
	key_label_in in varchar2,
	value_in in varchar2
    ) is
    	key_id_val number;
    begin
    	select CDK.id into key_id_val
	  from rhnCustomDataKey CDK,
	       rhnServer S
	 where S.id = server_id_in
	   and S.org_id = CDK.org_id
	   and CDK.label = key_label_in;
	begin
	    insert into rhnServerCustomDataValue (server_id, key_id, value, created_by, last_modified_by)
	    values (server_id_in, key_id_val, value_in, user_id_in, user_id_in);
	exception
	    when DUP_VAL_ON_INDEX
	    	then
		update rhnServerCustomDataValue
		   set value = value_in,
		       last_modified_by = user_id_in
		 where server_id = server_id_in
		   and key_id = key_id_val;
	end;
    end set_custom_value;
    procedure bulk_set_custom_value(
    	key_label_in in varchar2,
	value_in in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    ) is
    begin
    	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    if rhn_server.system_service_level(server.element, 'provisioning') = 1 then
	    	rhn_server.set_custom_value(server.element, set_uid_in, key_label_in, value_in);
	    end if;
	end loop server;
    end bulk_set_custom_value;
    procedure bulk_snapshot_tag(
    	org_id_in in number,
        tagname_in in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    ) is
    	snapshot_id number;
    begin
    	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    if rhn_server.system_service_level(server.element, 'provisioning') = 1 then
	    	begin
	    	    select max(id) into snapshot_id
	    	    from rhnSnapshot
	    	    where server_id = server.element;
	    	exception
	    	    when NO_DATA_FOUND then
		    	rhn_server.snapshot_server(server.element, 'tagging system:  ' || tagname_in);
			select max(id) into snapshot_id
			from rhnSnapshot
			where server_id = server.element;
		end;
		begin
		    rhn_server.tag_snapshot(snapshot_id, org_id_in, tagname_in);
		exception
		    when DUP_VAL_ON_INDEX
		    	then
			null;
		end;
	    end if;
	end loop server;
    end bulk_snapshot_tag;
    procedure tag_delete(
    	server_id_in in number,
	tag_id_in in number
    ) is
    	cursor snapshots is
		select	snapshot_id
		from	rhnSnapshotTag
		where	tag_id = tag_id_in;
	tag_id_tmp number;
    begin
    	select	id into tag_id_tmp
	from	rhnTag
	where	id = tag_id_in
	for update;
	delete
		from	rhnSnapshotTag
		where	server_id = server_id_in
			and tag_id = tag_id_in;
	for snapshot in snapshots loop
		return;
	end loop;
	delete
		from rhnTag
		where id = tag_id_in;
    end tag_delete;
    procedure tag_snapshot(
        snapshot_id_in in number,
	org_id_in in number,
	tagname_in in varchar2
    ) is
    begin
    	insert into rhnSnapshotTag (snapshot_id, server_id, tag_id)
	select snapshot_id_in, server_id, lookup_tag(org_id_in, tagname_in)
	from rhnSnapshot
	where id = snapshot_id_in;
    end tag_snapshot;
    procedure bulk_snapshot(
    	reason_in in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    ) is
    begin
    	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
    	    if rhn_server.system_service_level(server.element, 'provisioning') = 1 then
	    	rhn_server.snapshot_server(server.element, reason_in);
	    end if;
	end loop server;
    end bulk_snapshot;
    procedure snapshot_server(
    	server_id_in in number,
	reason_in in varchar2
    ) is
    	snapshot_id number;
	cursor revisions is
		select distinct
			cr.id
		from	rhnConfigRevision	cr,
			rhnConfigFileName	cfn,
			rhnConfigFile		cf,
			rhnConfigChannel	cc,
			rhnServerConfigChannel	scc
		where	1=1
			and scc.server_id = server_id_in
			and scc.config_channel_id = cc.id
			and cc.id = cf.config_channel_id
			and cf.id = cr.config_file_id
			and cr.id = cf.latest_config_revision_id
			and cf.config_file_name_id = cfn.id
			and cf.id = lookup_first_matching_cf(scc.server_id, cfn.path);
    begin
    	select rhn_snapshot_id_seq.nextval into snapshot_id from dual;
	insert into rhnSnapshot (id, org_id, server_id, reason) (
		select	snapshot_id,
			s.org_id,
			server_id_in,
			reason_in
		from	rhnServer s
		where	s.id = server_id_in
	);
	insert into rhnSnapshotChannel (snapshot_id, channel_id) (
		select	snapshot_id, sc.channel_id
		from	rhnServerChannel sc
		where	sc.server_id = server_id_in
	);
	insert into rhnSnapshotServerGroup (snapshot_id, server_group_id) (
		select	snapshot_id, sgm.server_group_id
		from	rhnServerGroupMembers sgm
		where	sgm.server_id = server_id_in
	);
	insert into rhnSnapshotPackage (snapshot_id, nevra_id) (
		select	distinct snapshot_id,
			lookup_package_nevra(sp.name_id, sp.evr_id, null)
		from	rhnServerPackage sp
		where	sp.server_id = server_id_in
	);
	insert into rhnSnapshotConfigChannel ( snapshot_id, config_channel_id ) (
		select	snapshot_id, scc.config_channel_id
		from	rhnServerConfigChannel scc
		where	server_id = server_id_in
	);
	for revision in revisions loop
		insert into rhnSnapshotConfigRevision (
				snapshot_id, config_revision_id
			) values (
				snapshot_id, revision.id
			);
	end loop;
    end snapshot_server;
    procedure remove_action(
    	server_id_in in number,
	action_id_in in number
    ) is
    	cursor chained_actions is
		select	id, prerequisite
		from	rhnAction
		start with id = action_id_in
		connect by prior id = prerequisite
		order by prerequisite desc;
	cursor sessions is
		select	s.id
		from	rhnKickstartSession s
		where	server_id_in in (s.old_server_id, s.new_server_id)
			and s.action_id = action_id_in
			and not exists (
				select	1
				from	rhnKickstartSessionState ss
				where	ss.id = s.state_id
					and ss.label in ('failed','complete')
			);
	type chain_end_type is table of number index by binary_integer;
	chain_ends chain_end_type;
	i number;
	prereq number := 1;
    begin
	select	prerequisite
	into	prereq
	from	rhnAction
	where	id = action_id_in;
	if prereq is not null then
		rhn_exception.raise_exception('action_is_child');
	end if;
	i := 0;
	for action in chained_actions loop
		if action.prerequisite is null then
			chain_ends(i) := action.id;
			i := i + 1;
		else
			delete from rhnServerAction
				where server_id = server_id_in
				and action_id = action.id;
		end if;
	end loop;
	i := chain_ends.first;
	while i is not null loop
		delete from rhnServerAction
			where server_id = server_id_in
			and action_id = chain_ends(i);
		i := chain_ends.next(i);
	end loop;
	for s in sessions loop
		update rhnKickstartSession
			set 	state_id = (
					select	id
					from	rhnKickstartSessionState
					where	label = 'failed'
				),
				action_id = null
			where	id = s.id;
		set_ks_session_history_message(s.id, 'failed', 'Kickstart cancelled due to action removal');
	end loop;
    end remove_action;
    function check_user_access(server_id_in in number, user_id_in in number)
    return number
    is
    	has_access number;
    begin
        select 1 into has_access
	  from rhnServer S,
	       web_contact wc
	 where wc.org_id = s.org_id
	   and s.id = server_id_in
	   and wc.id = user_id_in;
    	if rhn_user.check_role(user_id_in, 'org_admin') = 1
	then
	    return 1;
	end if;
    	select 1 into has_access
	  from rhnServerGroupMembers SGM,
	       rhnUserServerGroupPerms USG
	 where SGM.server_group_id = USG.server_group_id
	   and SGM.server_id = server_id_in
	   and USG.user_id = user_id_in
	   and rownum = 1;
	return 1;
    exception
    	when no_data_found
	    then
	    return 0;
    end check_user_access;
    procedure insert_into_servergroup (
		server_id_in in number,
		server_group_id_in in number
    ) is
		cursor sg_users is
			select	user_id id
			from	rhnUserServerGroupPerms
			where	server_group_id = server_group_id_in;
		used_slots number;
		max_slots number;
		org_id number;
		mgmt_available number;
		mgmt_upgrade number;
		mgmt_sgid number;
		prov_available number;
		prov_upgrade number;
		prov_sgid number;
		group_label rhnServerGroupType.label%TYPE;
		group_type number;
	begin
		select	sg.group_type, sg.org_id, sg.current_members, sg.max_members
		into	group_type, org_id, used_slots, max_slots
		from	rhnServerGroup sg
		where	sg.id = server_group_id_in
		for update of sg.current_members;
		if group_type is null then
			if used_slots >= max_slots then
				rhn_exception.raise_exception('servergroup_max_members');
			end if;
			insert into rhnServerGroupMembers(
					server_id, server_group_id
				) values (
					server_id_in, server_group_id_in
				);
			update rhnServerGroup
				set current_members = current_members + 1
				where id = server_group_id_in;
			for u in sg_users loop
				rhn_cache.update_perms_for_user(u.id);
			end loop;
			return;
		end if;
		select	label
		into	group_label
		from	rhnServerGroupType	sgt
		where	sgt.id = group_type;
		if group_label in ('sw_mgr_entitled','enterprise_entitled') then
			if used_slots >= max_slots then
				rhn_exception.raise_exception('servergroup_max_members');
			end if;
			insert into rhnServerGroupMembers(
					server_id, server_group_id
				) values (
					server_id_in, server_group_id_in
				);
			update rhnServerGroup
				set current_members = current_members + 1
				where id = server_group_id_in;
			return;
		else -- we're doing provisioning
			rhn_entitlements.get_available_slots(
					org_id, 'provisioning_entitled', prov_sgid,
					prov_available, prov_upgrade);
			rhn_entitlements.get_available_slots(
					org_id, 'enterprise_entitled', mgmt_sgid,
					mgmt_available, mgmt_upgrade);
			if prov_available < 1 then
				if prov_upgrade > 0 then
					rhn_exception.raise_exception('servergroup_use_upgrade');
				end if;
				rhn_exception.raise_exception('servergroup_max_members');
			end if;
			insert into rhnServerGroupMembers(
					server_id, server_group_id
				) values (
					server_id_in, server_group_id_in
				);
			update rhnServerGroup
				set current_members = current_members + 1
				where id = server_group_id_in;
			update rhnServerGroup
				set max_members = max_members - 1
				where id = mgmt_sgid;
		end if;
	end;
	function insert_into_servergroup_maybe (
		server_id_in in number,
		server_group_id_in in number
	) return number is
		retval number := 0;
		cursor servergroups is
			select	s.id	server_id,
					sg.id	server_group_id
			from	rhnServerGroup	sg,
					rhnServer		s
			where	s.id = server_id_in
				and sg.id = server_group_id_in
				and s.org_id = sg.org_id
				and not exists (
					select	1
					from	rhnServerGroupMembers sgm
					where	sgm.server_id = s.id
						and sgm.server_group_id = sg.id
				);
	begin
		for sgm in servergroups loop
			rhn_server.insert_into_servergroup(sgm.server_id, sgm.server_group_id);
			retval := retval + 1;
		end loop;
		return retval;
	end insert_into_servergroup_maybe;
	procedure insert_set_into_servergroup (
		server_group_id_in in number,
		user_id_in in number,
		set_label_in in varchar2
	) is
		cursor servers is
			select	st.element	id
			from	rhnSet		st
			where	st.user_id = user_id_in
				and st.label = set_label_in
				and exists (
					select	1
					from	rhnUserManagedServerGroups umsg
					where	umsg.server_group_id = server_group_id_in
						and umsg.user_id = user_id_in
					)
				and not exists (
					select	1
					from	rhnServerGroupMembers sgm
					where	sgm.server_id = st.element
						and sgm.server_group_id = server_group_id_in
				);
	begin
		for s in servers loop
			rhn_server.insert_into_servergroup(s.id, server_group_id_in);
		end loop;
	end insert_set_into_servergroup;
    procedure delete_from_servergroup (
    	server_id_in in number,
		server_group_id_in in number
    ) is
		cursor sg_users is
			select	user_id id
			from	rhnUserServerGroupPerms
			where	server_group_id = server_group_id_in;
		oid number;
		mgmt_sgid number;
		label rhnServerGroupType.label%TYPE;
		group_type number;
	begin
		begin
			select	sg.group_type, sg.org_id
			into	group_type,	oid
			from	rhnServerGroupMembers	sgm,
					rhnServerGroup			sg
			where	sg.id = server_group_id_in
				and sg.id = sgm.server_group_id
				and sgm.server_id = server_id_in
			for update of sg.current_members;
		exception
			when no_data_found then
				rhn_exception.raise_exception('server_not_in_group');
		end;
		if group_type is null then
			delete from rhnServerGroupMembers
				where server_group_id = server_group_id_in
				and	server_id = server_id_in;
			update rhnServerGroup
				set current_members = current_members - 1
				where id = server_group_id_in;
			for u in sg_users loop
				rhn_cache.update_perms_for_user(u.id);
			end loop;
			return;
		end if;
		select	sgt.label
		into	label
		from	rhnServerGroupType sgt
		where	sgt.id = group_type;
		if label in ('sw_mgr_entitled','enterprise_entitled') then
			delete from rhnServerGroupMembers
				where server_group_id = server_group_id_in
				and	server_id = server_id_in;
			update rhnServerGroup
				set current_members = current_members - 1
				where id = server_group_id_in;
		else
			select	sg.id
			into	mgmt_sgid
			from	rhnServerGroupType	sgt,
					rhnServerGroup		sg
			where	sg.org_id = oid
				and sgt.label = 'enterprise_entitled'
				and sgt.id = sg.group_type
			for update of sg.max_members;
			delete from rhnServerGroupMembers
				where server_group_id = server_group_id_in
				and	server_id = server_id_in;
			update rhnServerGroup
				set max_members = max_members + 1
				where id = mgmt_sgid;
			update rhnServerGroup
				set current_members = current_members - 1
				where id = server_group_id_in;
		end if;
	end;
	procedure delete_set_from_servergroup (
		server_group_id_in in number,
		user_id_in in number,
		set_label_in in varchar2
	) is
		cursor servergroups is
			select	sgm.server_id, sgm.server_group_id
			from	rhnSet st,
					rhnServerGroupMembers sgm
			where	sgm.server_group_id = server_group_id_in
				and st.user_id = user_id_in
				and st.label = set_label_in
				and sgm.server_id = st.element
				and exists (
					select	1
					from	rhnUserManagedServerGroups usgp
					where	usgp.server_group_id = server_group_id_in
						and usgp.user_id = user_id_in
				);
	begin
		for sgm in servergroups loop
			rhn_server.delete_from_servergroup(sgm.server_id, server_group_id_in);
		end loop;
	end delete_set_from_servergroup;
	procedure clear_servergroup (
		server_group_id_in in number
	) is
		cursor servers is
			select	sgm.server_id	id
			from	rhnServerGroupMembers sgm
			where	sgm.server_group_id = server_group_id_in;
	begin
		for s in servers loop
			rhn_server.delete_from_servergroup(s.id, server_group_id_in);
		end loop;
	end clear_servergroup;
	procedure delete_from_org_servergroups (
		server_id_in in number
	) is
		cursor servergroups is
			select	sgm.server_group_id id
			from	rhnServerGroup sg,
					rhnServerGroupMembers sgm
			where	sgm.server_id = server_id_in
				and sgm.server_group_id = sg.id
				and sg.group_type is null;
	begin
		for sg in servergroups loop
			rhn_server.delete_from_servergroup(server_id_in, sg.id);
		end loop;
	end delete_from_org_servergroups;
	function get_ip_address (
		server_id_in in number
	) return varchar2 is
		cursor interfaces is
			select	name, ip_addr
			from	rhnServerNetInterface
			where	server_id = server_id_in
				and name = 'eth0';
		cursor addresses is
			select	ipaddr ip_addr
			from	rhnServerNetwork
			where	server_id = server_id_in
				and ipaddr != '127.0.0.1';
	begin
		for iface in interfaces loop
			return iface.ip_addr;
		end loop;
		for addr in addresses loop
			return addr.ip_addr;
		end loop;
		return NULL;
	end get_ip_address;
end rhn_server;
/

SHOW ERRORS;

--
-- 46. RHN_USER  (Package Body) 
--
--
-- RHN_USER  (Package Body) 
--
CREATE OR REPLACE package body rhn_user
is
	body_version varchar2(100) := '$Id: satellite-3.4-to-3.6.sql 64427 2005-09-09 14:08:21Z jslagle $';
    function check_role(user_id_in in number, role_in in varchar2)
    return number
    is
    	throwaway number;
    begin
	select 1 into throwaway
	  from rhnUserGroupType UGT,
	       rhnUserGroup UG,
	       rhnUserGroupMembers UGM
	 where UGM.user_id = user_id_in
	   and UGM.user_group_id = UG.id
	   and UG.group_type = UGT.id
	   and UGT.label = role_in;
	return 1;
    exception
    	when no_data_found
	    then
	    return 0;
    end check_role;
    function check_role_implied(user_id_in in number, role_in in varchar2)
    return number
    is
    	throwaway number;
    begin
    	if rhn_user.check_role(user_id_in, role_in) = 1
	then
	    return 1;
    	end if;
	if role_in = 'config_admin' and rhn_user.check_role(user_id_in, 'org_admin') = 1
	then
	    return 1;
	end if;
	if role_in = 'channel_admin' and rhn_user.check_role(user_id_in, 'org_admin') = 1
	then
	    return 1;
	end if;
	return 0;
    end check_role_implied;
    function get_org_id(user_id_in in number)
    return number
    is
    	org_id_out number;
    begin
    	select org_id into org_id_out
	  from web_contact
	 where id = user_id_in;
	return org_id_out;
    end get_org_id;
	function find_mailable_address(user_id_in in number)
	return varchar2 is
		PRAGMA AUTONOMOUS_TRANSACTION;
		cursor addrs is
			select	ea.state_id, ea.address
			from	rhnEmailAddressState eas,
					rhnEmailAddress ea
			where	ea.user_id = user_id_in
				and eas.label = 'verified'
				and ea.state_id = eas.id
			union all
			select	ea.state_id, ea.address
			from	rhnEmailAddressState eas,
					rhnEmailAddress ea
			where	ea.user_id = user_id_in
				and eas.label = 'unverified'
				and ea.state_id = eas.id
			union all
			select	ea.state_id, ea.address
			from	rhnEmailAddressState eas,
					rhnEmailAddress ea
			where	ea.user_id = user_id_in
				and eas.label = 'pending'
				and ea.state_id = eas.id
			union all
			select	ea.state_id, ea.address
			from	rhnEmailAddressState eas,
					rhnEmailAddress ea
			where	ea.user_id = user_id_in
				and eas.label = 'pending_warned'
				and ea.state_id = eas.id
			union all
			select	ea.state_id, ea.address
			from	rhnEmailAddressState eas,
					rhnEmailAddress ea
			where	ea.user_id = user_id_in
				and eas.label = 'needs_verifying'
				and ea.state_id = eas.id
			union all
			select	-1 state_id,
					email address
			from	web_user_personal_info
			where	web_user_id = user_id_in;
		retval rhnEmailAddress.address%TYPE;
	begin
		for addr in addrs loop
			retval := addr.address;
			if addr.address is null then
				update web_user_contact_permission
					set email = 'N'
					where web_user_id = user_id_in;
				commit;
				return null;
			end if;
			if addr.state_id = -1 then
				insert into rhnEmailAddress (
						id, address,
						user_id, state_id
					) (
						select	rhn_eaddress_id_seq.nextval, addr.address,
								user_id_in, eas.id
						from	rhnEmailAddressState eas
						where	eas.label = 'unverified'
					);
			end if;
			commit;
			return retval;
		end loop;
		return null;
	end;
	procedure add_servergroup_perm(
		user_id_in in number,
		server_group_id_in in number
	) is
		cursor	orgs_match is
			select	1
			from	rhnServerGroup sg,
					web_contact u
			where	u.id = user_id_in
				and sg.id = server_group_id_in
				and sg.org_id = u.org_id;
	begin
		for okay in orgs_match loop
			insert into rhnUserServerGroupPerms(user_id, server_group_id)
				values (user_id_in, server_group_id_in);
			rhn_cache.update_perms_for_user(user_id_in);
			return;
		end loop;
		rhn_exception.raise_exception('usgp_different_orgs');
	exception when dup_val_on_index then
		rhn_exception.raise_exception('usgp_already_allowed');
	end add_servergroup_perm;
	procedure remove_servergroup_perm(
		user_id_in in number,
		server_group_id_in in number
	) is
		cursor perms is
			select	1
			from	rhnUserServerGroupPerms
			where	user_id = user_id_in
				and server_group_id = server_group_id_in;
	begin
		for perm in perms loop
			delete from rhnUserServerGroupPerms
				where	user_id = user_id_in
					and server_group_id = server_group_id_in;
			rhn_cache.update_perms_for_user(user_id_in);
			return;
		end loop;
		rhn_exception.raise_exception('usgp_not_allowed');
	end remove_servergroup_perm;
	procedure add_to_usergroup(
		user_id_in in number,
		user_group_id_in in number
	) is
		cursor perm_granting_usergroups is
			select	user_group_id_in
			from	rhnUserGroup		ug,
					rhnUserGroupType	ugt
			where	ugt.label in ('org_admin') -- and server_group_admin ?
				and ug.id = user_group_id_in
				and ug.group_type = ugt.id;
	begin
		insert into rhnUserGroupMembers(user_id, user_group_id)
			values (user_id_in, user_group_id_in);
		for ug in perm_granting_usergroups loop
			rhn_cache.update_perms_for_user(user_id_in);
			return;
		end loop;
	end add_to_usergroup;
	procedure add_users_to_usergroups(
		user_id_in in number
	) is
		cursor ugms is
			select	element user_id,
					element_two user_group_id
			from	rhnSet
			where	user_id = user_id_in
				and label = 'user_group_list';
	begin
		for ugm in ugms loop
			rhn_user.add_to_usergroup(ugm.user_id, ugm.user_group_id);
		end loop;
	end add_users_to_usergroups;
	procedure remove_from_usergroup(
		user_id_in in number,
		user_group_id_in in number
	) is
		cursor perm_granting_usergroups is
			select	label
			from	rhnUserGroupType	ugt,
					rhnUserGroupMembers	ugm,
					rhnUserGroup		ug
			where	1=1
				and ug.id = user_group_id_in
				and ugm.user_group_id = user_group_id_in
				and ug.group_type = ugt.id
				and ugm.user_id = user_id_in;
	begin
		for ug in perm_granting_usergroups loop
			delete from rhnUserGroupMembers
				where	user_id = user_id_in
					and user_group_id = user_group_id_in;
			if ug.label in ('org_admin') then
				rhn_cache.update_perms_for_user(user_id_in);
			end if;
		end loop;
	end remove_from_usergroup;
	procedure remove_users_from_servergroups(
		user_id_in in number
	) is
		cursor ugms is
			select	element user_id,
					element_two user_group_id
			from	rhnSet
			where	user_id = user_id_in
				and label = 'user_group_list';
	begin
		for ugm in ugms loop
			rhn_user.remove_from_usergroup(ugm.user_id, ugm.user_group_id);
		end loop;
	end remove_users_from_servergroups;
end rhn_user;
/

SHOW ERRORS;

--
-- 47. LOOKUP_CVE  (Function) 
--
--
-- LOOKUP_CVE  (Function) 
--
--  Dependencies: 
--   RHNCVE (Table)
--   RHN_CVE_ID_SEQ (Sequence)
--   STANDARD (Package)
--
CREATE OR REPLACE FUNCTION
LOOKUP_CVE(name_in IN VARCHAR2)
RETURN NUMBER
DETERMINISTIC
IS
	PRAGMA AUTONOMOUS_TRANSACTION;
	name_id		NUMBER;
BEGIN
	SELECT id
          INTO name_id
          FROM rhnCve
         WHERE name = name_in;
	RETURN name_id;
EXCEPTION
        WHEN NO_DATA_FOUND THEN
            INSERT INTO rhnCve (id, name)
                VALUES (rhn_cve_id_seq.nextval, name_in)
                RETURNING id INTO name_id;
            COMMIT;
	RETURN name_id;
END LOOKUP_CVE;
/

SHOW ERRORS;

--
-- 48. RHN_APPINST_ISTANCE_MOD_TRIG  (Trigger) 
--
--
-- RHN_APPINST_ISTANCE_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNAPPINSTALLINSTANCE (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_APPINST_ISTANCE_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNAPPINSTALLINSTANCE
FOR EACH ROW
begin
	:new.modified := sysdate;
end rhn_appinst_istance_mod_trig;
/
SHOW ERRORS;


--
-- 49. RHN_CURRENT_ALERTS_MOD_TRIG  (Trigger) 
--
--
-- RHN_CURRENT_ALERTS_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHN_CURRENT_ALERTS (Table)
--   DBMS_STANDARD (Package)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_CURRENT_ALERTS_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHN_CURRENT_ALERTS
REFERENCING NEW AS NEW OLD AS OLD
FOR EACH ROW
declare
msg varchar2(200);
date_completed_is_null exception;
date_completed_is_not_null exception;
begin
    if :new.in_progress=0 and :new.date_completed is null
    then
        if ( updating( 'in_progress' ) and updating( 'date_completed') )
             or inserting
        then
            raise date_completed_is_null;
        elsif  updating( 'in_progress' )
        then
            :new.date_completed:=sysdate;
        else
            :new.in_progress:=1;
        end if;
    elsif :new.in_progress=1 and :new.date_completed is not null
    then
        if ( updating( 'in_progress' ) and updating( 'date_completed') )
            or inserting
        then
            raise date_completed_is_not_null;
        elsif  updating( 'in_progress' )
        then
            :new.date_completed:=null;
        else
            :new.in_progress:=0;
        end if;
    end if;
    exception
    when date_completed_is_null then
    msg:='date_completed is null while in_progress=0';
    raise_application_error (-20012,msg);
    when date_completed_is_not_null then
    msg:='date_completed is not null while in_progress=1';
    raise_application_error (-20012,msg);
    when others then
    raise;
end;
/
SHOW ERRORS;


--
-- 51. RHN_PUSHDISPATCH_MOD_TRIG  (Trigger) 
--
--
-- RHN_PUSHDISPATCH_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNPUSHDISPATCHER (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_PUSHDISPATCH_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNPUSHDISPATCHER
FOR EACH ROW
begin
	:new.modified := sysdate;
end;
/
SHOW ERRORS;


--
-- 52. CHECK_SUITES_RECID_SEQ  (Synonym) 
--
--
-- CHECK_SUITES_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_CHECK_SUITES_RECID_SEQ (Sequence)
--
CREATE SYNONYM CHECK_SUITES_RECID_SEQ FOR RHN_CHECK_SUITES_RECID_SEQ;

--
-- 53. COMMANDS_RECID_SEQ  (Synonym) 
--
--
-- COMMANDS_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMANDS_RECID_SEQ (Sequence)
--
CREATE SYNONYM COMMANDS_RECID_SEQ FOR RHN_COMMANDS_RECID_SEQ;

--
-- 54. COMMAND_CENTER_STATE  (Synonym) 
--
--
-- COMMAND_CENTER_STATE  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_CENTER_STATE (Table)
--
CREATE SYNONYM COMMAND_CENTER_STATE FOR RHN_COMMAND_CENTER_STATE;

--
-- 55. COMMAND_CLASS  (Synonym) 
--
--
-- COMMAND_CLASS  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_CLASS (Table)
--
CREATE SYNONYM COMMAND_CLASS FOR RHN_COMMAND_CLASS;

--
-- 56. COMMAND_GROUPS  (Synonym) 
--
--
-- COMMAND_GROUPS  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_GROUPS (Table)
--
CREATE SYNONYM COMMAND_GROUPS FOR RHN_COMMAND_GROUPS;

--
-- 57. COMMAND_QUEUE_COMMANDS  (Synonym) 
--
--
-- COMMAND_QUEUE_COMMANDS  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_QUEUE_COMMANDS (Table)
--
CREATE SYNONYM COMMAND_QUEUE_COMMANDS FOR RHN_COMMAND_QUEUE_COMMANDS;

--
-- 58. COMMAND_QUEUE_EXECS_BK  (Synonym) 
--
--
-- COMMAND_QUEUE_EXECS_BK  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_QUEUE_EXECS_BK (Table)
--
CREATE SYNONYM COMMAND_QUEUE_EXECS_BK FOR RHN_COMMAND_QUEUE_EXECS_BK;

--
-- 59. COMMAND_QUEUE_INSTANCES  (Synonym) 
--
--
-- COMMAND_QUEUE_INSTANCES  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_QUEUE_INSTANCES (Table)
--
CREATE SYNONYM COMMAND_QUEUE_INSTANCES FOR RHN_COMMAND_QUEUE_INSTANCES;

--
-- 60. COMMAND_QUEUE_INSTANCES_BK  (Synonym) 
--
--
-- COMMAND_QUEUE_INSTANCES_BK  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_QUEUE_INSTANCES_BK (Table)
--
CREATE SYNONYM COMMAND_QUEUE_INSTANCES_BK FOR RHN_COMMAND_QUEUE_INSTANCES_BK;

--
-- 61. COMMAND_QUEUE_PARAMS  (Synonym) 
--
--
-- COMMAND_QUEUE_PARAMS  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_QUEUE_PARAMS (Table)
--
CREATE SYNONYM COMMAND_QUEUE_PARAMS FOR RHN_COMMAND_QUEUE_PARAMS;

--
-- 62. COMMAND_Q_COMMAND_RECID_SEQ  (Synonym) 
--
--
-- COMMAND_Q_COMMAND_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_Q_COMM_RECID_SEQ (Sequence)
--
CREATE SYNONYM COMMAND_Q_COMMAND_RECID_SEQ FOR RHN_COMMAND_Q_COMM_RECID_SEQ;

--
-- 63. COMMAND_Q_INSTANCE_RECID_SEQ  (Synonym) 
--
--
-- COMMAND_Q_INSTANCE_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_Q_INST_RECID_SEQ (Sequence)
--
CREATE SYNONYM COMMAND_Q_INSTANCE_RECID_SEQ FOR RHN_COMMAND_Q_INST_RECID_SEQ;

--
-- 64. COMMAND_REQUIREMENTS  (Synonym) 
--
--
-- COMMAND_REQUIREMENTS  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_REQUIREMENTS (Table)
--
CREATE SYNONYM COMMAND_REQUIREMENTS FOR RHN_COMMAND_REQUIREMENTS;

--
-- 65. COMMAND_TARGET_RECID_SEQ  (Synonym) 
--
--
-- COMMAND_TARGET_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_TARGET_RECID_SEQ (Sequence)
--
CREATE SYNONYM COMMAND_TARGET_RECID_SEQ FOR RHN_COMMAND_TARGET_RECID_SEQ;

--
-- 66. CONFIG_GROUP  (Synonym) 
--
--
-- CONFIG_GROUP  (Synonym) 
--
--  Dependencies: 
--   RHN_CONFIG_GROUP (Table)
--
CREATE SYNONYM CONFIG_GROUP FOR RHN_CONFIG_GROUP;

--
-- 67. CONFIG_SECURITY_TYPE  (Synonym) 
--
--
-- CONFIG_SECURITY_TYPE  (Synonym) 
--
--  Dependencies: 
--   RHN_CONFIG_SECURITY_TYPE (Table)
--
CREATE SYNONYM CONFIG_SECURITY_TYPE FOR RHN_CONFIG_SECURITY_TYPE;

--
-- 68. CONTACT_GROUPS_RECID_SEQ  (Synonym) 
--
--
-- CONTACT_GROUPS_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_CONTACT_GROUPS_RECID_SEQ (Sequence)
--
CREATE SYNONYM CONTACT_GROUPS_RECID_SEQ FOR RHN_CONTACT_GROUPS_RECID_SEQ;

--
-- 69. CONTACT_METHODS_RECID_SEQ  (Synonym) 
--
--
-- CONTACT_METHODS_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_CONTACT_METHODS_RECID_SEQ (Sequence)
--
CREATE SYNONYM CONTACT_METHODS_RECID_SEQ FOR RHN_CONTACT_METHODS_RECID_SEQ;

--
-- 70. CURRENT_ALERTS  (Synonym) 
--
--
-- CURRENT_ALERTS  (Synonym) 
--
--  Dependencies: 
--   RHN_CURRENT_ALERTS (Table)
--
CREATE SYNONYM CURRENT_ALERTS FOR RHN_CURRENT_ALERTS;

--
-- 71. CURRENT_ALERTS_RECID_SEQ  (Synonym) 
--
--
-- CURRENT_ALERTS_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_CURRENT_ALERTS_RECID_SEQ (Sequence)
--
CREATE SYNONYM CURRENT_ALERTS_RECID_SEQ FOR RHN_CURRENT_ALERTS_RECID_SEQ;

--
-- 72. CURRENT_STATE_SUMMARIES  (Synonym) 
--
--
-- CURRENT_STATE_SUMMARIES  (Synonym) 
--
--  Dependencies: 
--   RHN_CURRENT_STATE_SUMMARIES (Table)
--
CREATE SYNONYM CURRENT_STATE_SUMMARIES FOR RHN_CURRENT_STATE_SUMMARIES;

--
-- 73. CUSTOMER  (Synonym) 
--
--
-- CUSTOMER  (Synonym) 
--
--  Dependencies: 
--   RHN_CUSTOMER_MONITORING (View)
--
CREATE SYNONYM CUSTOMER FOR RHN_CUSTOMER_MONITORING;

--
-- 74. DEPLOYED_PROBE  (Synonym) 
--
--
-- DEPLOYED_PROBE  (Synonym) 
--
--  Dependencies: 
--   RHN_DEPLOYED_PROBE (Table)
--
CREATE SYNONYM DEPLOYED_PROBE FOR RHN_DEPLOYED_PROBE;

--
-- 75. ENVIRONMENT  (Synonym) 
--
--
-- ENVIRONMENT  (Synonym) 
--
--  Dependencies: 
--   RHN_ENVIRONMENT (Table)
--
CREATE SYNONYM ENVIRONMENT FOR RHN_ENVIRONMENT;

--
-- 76. HOST_PROBES_RECID_SEQ  (Synonym) 
--
--
-- HOST_PROBES_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_HOST_PROBES_RECID_SEQ (Sequence)
--
CREATE SYNONYM HOST_PROBES_RECID_SEQ FOR RHN_HOST_PROBES_RECID_SEQ;

--
-- 77. METHOD_TYPES  (Synonym) 
--
--
-- METHOD_TYPES  (Synonym) 
--
--  Dependencies: 
--   RHN_METHOD_TYPES (Table)
--
CREATE SYNONYM METHOD_TYPES FOR RHN_METHOD_TYPES;

--
-- 78. MULTI_SCOUT_THRESHOLD  (Synonym) 
--
--
-- MULTI_SCOUT_THRESHOLD  (Synonym) 
--
--  Dependencies: 
--   RHN_MULTI_SCOUT_THRESHOLD (Table)
--
CREATE SYNONYM MULTI_SCOUT_THRESHOLD FOR RHN_MULTI_SCOUT_THRESHOLD;

--
-- 79. NOTIFSERVERS  (Synonym) 
--
--
-- NOTIFSERVERS  (Synonym) 
--
--  Dependencies: 
--   RHN_NOTIFSERVERS (Table)
--
CREATE SYNONYM NOTIFSERVERS FOR RHN_NOTIFSERVERS;

--
-- 80. NTFMT_RECID_SEQ  (Synonym) 
--
--
-- NTFMT_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_NTFMT_RECID_SEQ (Sequence)
--
CREATE SYNONYM NTFMT_RECID_SEQ FOR RHN_NTFMT_RECID_SEQ;

--
-- 81. OS  (Synonym) 
--
--
-- OS  (Synonym) 
--
--  Dependencies: 
--   RHN_OS (Table)
--
CREATE SYNONYM OS FOR RHN_OS;

--
-- 82. PAGER_TYPES  (Synonym) 
--
--
-- PAGER_TYPES  (Synonym) 
--
--  Dependencies: 
--   RHN_PAGER_TYPES (Table)
--
CREATE SYNONYM PAGER_TYPES FOR RHN_PAGER_TYPES;

--
-- 83. PAGER_TYPES_RECID_SEQ  (Synonym) 
--
--
-- PAGER_TYPES_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_PAGER_TYPES_RECID_SEQ (Sequence)
--
CREATE SYNONYM PAGER_TYPES_RECID_SEQ FOR RHN_PAGER_TYPES_RECID_SEQ;

--
-- 84. PHYSICAL_LOCATION  (Synonym) 
--
--
-- PHYSICAL_LOCATION  (Synonym) 
--
--  Dependencies: 
--   RHN_PHYSICAL_LOCATION (Table)
--
CREATE SYNONYM PHYSICAL_LOCATION FOR RHN_PHYSICAL_LOCATION;

--
-- 85. PHYSICAL_LOCATION_RECID_SEQ  (Synonym) 
--
--
-- PHYSICAL_LOCATION_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_PHYSICAL_LOC_RECID_SEQ (Sequence)
--
CREATE SYNONYM PHYSICAL_LOCATION_RECID_SEQ FOR RHN_PHYSICAL_LOC_RECID_SEQ;

--
-- 86. PROBES_RECID_SEQ  (Synonym) 
--
--
-- PROBES_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_PROBES_RECID_SEQ (Sequence)
--
CREATE SYNONYM PROBES_RECID_SEQ FOR RHN_PROBES_RECID_SEQ;

--
-- 87. PROBE_STATE  (Synonym) 
--
--
-- PROBE_STATE  (Synonym) 
--
--  Dependencies: 
--   RHN_PROBE_STATE (Table)
--
CREATE SYNONYM PROBE_STATE FOR RHN_PROBE_STATE;

--
-- 88. PROBE_TYPES  (Synonym) 
--
--
-- PROBE_TYPES  (Synonym) 
--
--  Dependencies: 
--   RHN_PROBE_TYPES (Table)
--
CREATE SYNONYM PROBE_TYPES FOR RHN_PROBE_TYPES;

--
-- 89. QUANTA  (Synonym) 
--
--
-- QUANTA  (Synonym) 
--
--  Dependencies: 
--   RHN_QUANTA (Table)
--
CREATE SYNONYM QUANTA FOR RHN_QUANTA;

--
-- 90. REDIRECTS_RECID_SEQ  (Synonym) 
--
--
-- REDIRECTS_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_REDIRECTS_RECID_SEQ (Sequence)
--
CREATE SYNONYM REDIRECTS_RECID_SEQ FOR RHN_REDIRECTS_RECID_SEQ;

--
-- 91. REDIRECT_CRITERIA_RECID_SEQ  (Synonym) 
--
--
-- REDIRECT_CRITERIA_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_REDIRECT_CRIT_RECID_SEQ (Sequence)
--
CREATE SYNONYM REDIRECT_CRITERIA_RECID_SEQ FOR RHN_REDIRECT_CRIT_RECID_SEQ;

--
-- 92. REDIRECT_MATCH_TYPES  (Synonym) 
--
--
-- REDIRECT_MATCH_TYPES  (Synonym) 
--
--  Dependencies: 
--   RHN_REDIRECT_MATCH_TYPES (Table)
--
CREATE SYNONYM REDIRECT_MATCH_TYPES FOR RHN_REDIRECT_MATCH_TYPES;

--
-- 93. REDIRECT_TYPES  (Synonym) 
--
--
-- REDIRECT_TYPES  (Synonym) 
--
--  Dependencies: 
--   RHN_REDIRECT_TYPES (Table)
--
CREATE SYNONYM REDIRECT_TYPES FOR RHN_REDIRECT_TYPES;

--
-- 94. REDIRECT_TYPES_RECID_SEQ  (Synonym) 
--
--
-- REDIRECT_TYPES_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_REDIRECT_TYPES_RECID_SEQ (Sequence)
--
CREATE SYNONYM REDIRECT_TYPES_RECID_SEQ FOR RHN_REDIRECT_TYPES_RECID_SEQ;

--
-- 95. SATELLITE_STATE  (Synonym) 
--
--
-- SATELLITE_STATE  (Synonym) 
--
--  Dependencies: 
--   RHN_SATELLITE_STATE (Table)
--
CREATE SYNONYM SATELLITE_STATE FOR RHN_SATELLITE_STATE;

--
-- 96. SCHEDULES_RECID_SEQ  (Synonym) 
--
--
-- SCHEDULES_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_SCHEDULES_RECID_SEQ (Sequence)
--
CREATE SYNONYM SCHEDULES_RECID_SEQ FOR RHN_SCHEDULES_RECID_SEQ;

--
-- 97. SCHEDULE_DAYS_NORM  (Synonym) 
--
--
-- SCHEDULE_DAYS_NORM  (Synonym) 
--
--  Dependencies: 
--   RHN_SCHEDULE_DAYS_NORM (Table)
--
CREATE SYNONYM SCHEDULE_DAYS_NORM FOR RHN_SCHEDULE_DAYS_NORM;

--
-- 98. SCHEDULE_DAYS_RECID_SEQ  (Synonym) 
--
--
-- SCHEDULE_DAYS_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_SCHEDULE_DAYS_RECID_SEQ (Sequence)
--
CREATE SYNONYM SCHEDULE_DAYS_RECID_SEQ FOR RHN_SCHEDULE_DAYS_RECID_SEQ;

--
-- 99. SCHEDULE_TYPES  (Synonym) 
--
--
-- SCHEDULE_TYPES  (Synonym) 
--
--  Dependencies: 
--   RHN_SCHEDULE_TYPES (Table)
--
CREATE SYNONYM SCHEDULE_TYPES FOR RHN_SCHEDULE_TYPES;

--
-- 100. SCHEDULE_TYPES_RECID_SEQ  (Synonym) 
--
--
-- SCHEDULE_TYPES_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_SCHEDULE_TYPES_RECID_SEQ (Sequence)
--
CREATE SYNONYM SCHEDULE_TYPES_RECID_SEQ FOR RHN_SCHEDULE_TYPES_RECID_SEQ;

--
-- 101. SCHEDULE_WEEKS_RECID_SEQ  (Synonym) 
--
--
-- SCHEDULE_WEEKS_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_SCHEDULE_WEEKS_RECID_SEQ (Sequence)
--
CREATE SYNONYM SCHEDULE_WEEKS_RECID_SEQ FOR RHN_SCHEDULE_WEEKS_RECID_SEQ;

--
-- 102. SEMANTIC_DATA_TYPE  (Synonym) 
--
--
-- SEMANTIC_DATA_TYPE  (Synonym) 
--
--  Dependencies: 
--   RHN_SEMANTIC_DATA_TYPE (Table)
--
CREATE SYNONYM SEMANTIC_DATA_TYPE FOR RHN_SEMANTIC_DATA_TYPE;

--
-- 103. SNMP_ALERT_RECID_SEQ  (Synonym) 
--
--
-- SNMP_ALERT_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_SNMP_ALERT_RECID_SEQ (Sequence)
--
CREATE SYNONYM SNMP_ALERT_RECID_SEQ FOR RHN_SNMP_ALERT_RECID_SEQ;

--
-- 104. STRATEGIES  (Synonym) 
--
--
-- STRATEGIES  (Synonym) 
--
--  Dependencies: 
--   RHN_STRATEGIES (Table)
--
CREATE SYNONYM STRATEGIES FOR RHN_STRATEGIES;

--
-- 105. STRATEGIES_RECID_SEQ  (Synonym) 
--
--
-- STRATEGIES_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_STRATEGIES_RECID_SEQ (Sequence)
--
CREATE SYNONYM STRATEGIES_RECID_SEQ FOR RHN_STRATEGIES_RECID_SEQ;

--
-- 106. THRESHOLD_TYPE  (Synonym) 
--
--
-- THRESHOLD_TYPE  (Synonym) 
--
--  Dependencies: 
--   RHN_THRESHOLD_TYPE (Table)
--
CREATE SYNONYM THRESHOLD_TYPE FOR RHN_THRESHOLD_TYPE;

--
-- 107. TIME_ZONE_NAMES  (Synonym) 
--
--
-- TIME_ZONE_NAMES  (Synonym) 
--
--  Dependencies: 
--   RHN_TIME_ZONE_NAMES (Table)
--
CREATE SYNONYM TIME_ZONE_NAMES FOR RHN_TIME_ZONE_NAMES;

--
-- 108. UNITS  (Synonym) 
--
--
-- UNITS  (Synonym) 
--
--  Dependencies: 
--   RHN_UNITS (Table)
--
CREATE SYNONYM UNITS FOR RHN_UNITS;

--
-- 109. URL_PROBE_STEP_RECID_SEQ  (Synonym) 
--
--
-- URL_PROBE_STEP_RECID_SEQ  (Synonym) 
--
--  Dependencies: 
--   RHN_URL_PROBE_STEP_RECID_SEQ (Sequence)
--
CREATE SYNONYM URL_PROBE_STEP_RECID_SEQ FOR RHN_URL_PROBE_STEP_RECID_SEQ;

--
-- 110. WIDGET  (Synonym) 
--
--
-- WIDGET  (Synonym) 
--
--  Dependencies: 
--   RHN_WIDGET (Table)
--
CREATE SYNONYM WIDGET FOR RHN_WIDGET;

--
-- 111. LAST_MODIFIED  (Column) 
--
ALTER TABLE RHNERRATA ADD (LAST_MODIFIED  DATE                           DEFAULT (sysdate));

--
-- 112. LAST_MODIFIED  (Column) 
--
ALTER TABLE RHNKICKSTARTABLETREE ADD (LAST_MODIFIED  DATE                           DEFAULT (sysdate) CONSTRAINT RHN_KSTREE_LASTMOD_NN NOT NULL);

create table
rhnPushClient
(
	id		number
			constraint rhn_pclient_id_nn not null
			constraint rhn_pclient_id_pk primary key,
	name		varchar2(64)
			constraint rhn_pclient_name_nn not null,
	server_id	number
			constraint rhn_pclient_sid_nn not null,
	jabber_id	varchar2(128),
	shared_key	varchar2(64)
			constraint rhn_pclient_skey_nn not null,
        state_id        number
			constraint rhn_pclient_stid_nn not null
                        references rhnPushClientState(id),
        next_action_time    date,
        last_message_time   date,
        last_ping_time      date,
	created		date default sysdate
			constraint rhn_pclient_created_nn not null,
	modified	date default sysdate
			constraint rhn_pclient_modified_nn not null
)
	storage ( freelists 16 )
	initrans 32;

create unique index rhn_pclient_name_uq
	on rhnPushClient( name )
	storage ( freelists 16 )
	initrans 32;

create unique index rhn_pclient_sid_uq
	on rhnPushClient( server_id )
	storage ( freelists 16 )
	initrans 32;

create or replace trigger
rhn_pclient_mod_trig
before insert or update on rhnPushClient
for each row
begin
	:new.modified := sysdate;
end;
/
show errors

--
-- 114. DESCRIPTION  (Column) 
--
ALTER TABLE RHN_PROBE
MODIFY(DESCRIPTION  NOT NULL);


--
-- 115. RHN_KSTREE_OID_LABEL_UQ  (Index) 
--
--
-- RHN_KSTREE_OID_LABEL_UQ  (Index) 
--
--  Dependencies: 
--   RHNKICKSTARTABLETREE (Table)
--
CREATE UNIQUE INDEX RHN_KSTREE_OID_LABEL_UQ ON RHNKICKSTARTABLETREE
(ORG_ID, LABEL)
LOGGING
NOPARALLEL;

--
-- 116. RHN_PCLIENT_ID_PK  (Index) 
--
--
-- RHN_PCLIENT_ID_PK  (Index) 
--
--  Dependencies: 
--   RHNPUSHCLIENT (Table)
--
-- rhn: superseded by table create

--
-- 117. RHN_PCLIENT_NAME_UQ  (Index) 
--
--
-- RHN_PCLIENT_NAME_UQ  (Index) 
--
--  Dependencies: 
--   RHNPUSHCLIENT (Table)
--
-- rhn: superseded by table create

--
-- 118. RHN_PCLIENT_SID_UQ  (Index) 
--
--
-- RHN_PCLIENT_SID_UQ  (Index) 
--
--  Dependencies: 
--   RHNPUSHCLIENT (Table)
--
-- rhn: superseded by table create

--
-- 119. RHN_PROBE_CONTACT_GRP_IDX  (Index) 
--
--
-- RHN_PROBE_CONTACT_GRP_IDX  (Index) 
--
--  Dependencies: 
--   RHN_PROBE (Table)
--
CREATE INDEX RHN_PROBE_CONTACT_GRP_IDX ON RHN_PROBE
(CONTACT_GROUP_ID)
LOGGING
NOPARALLEL;

--
-- 120. CREATE_FIRST_ORG  (Procedure) 
--
--
-- CREATE_FIRST_ORG  (Procedure) 
--
--  Dependencies: 
--   DUAL ()
--   RHNORGAVAILABLESTATES (Table)
--   RHNORGQUOTA (Table)
--   RHNORGSTATE (Table)
--   RHNSERVERGROUP (Table)
--   RHNSERVERGROUPTYPE (Table)
--   RHNUSERGROUP (Table)
--   RHNUSERGROUPTYPE (Table)
--   RHN_SERVER_GROUP_ID_SEQ (Sequence)
--   RHN_USER_GROUP_ID_SEQ (Sequence)
--   WEB_CUSTOMER (Table)
--   DUAL (Synonym)
--   DUAL (Table)
--   STANDARD (Package)
--   SYS_STUB_FOR_PURITY_ANALYSIS (Package)
--
CREATE OR REPLACE procedure
create_first_org
(
	name_in in varchar2,
	password_in in varchar2
) is
	ug_type			number;
	group_val		number;
begin
	insert into web_customer (
		id, name, password,
		oracle_customer_id, oracle_customer_number,
		customer_type
	) values (
		1, name_in, password_in,
		1, 1, 'B'
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'org_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Organization Administrators',
		'Organization Administrators for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'org_applicant';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Organization Applicants',
		'Organization Applicants for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'system_group_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'System Group Administrators',
		'System Group Administrators for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'activation_key_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Activation Key Administrators',
		'Activation Key Administrators for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'channel_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Channel Administrators',
		'Channel Administrators for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	insert into rhnOrgQuota(
		org_id, total
	) values (
		1, 1024*1024*1024*16
	);
	insert into rhnOrgState (org_id, state_id)
	select 1, id
	  from rhnOrgAvailableStates
	 where label = 'paid';
        insert into rhnServerGroup
		( id, name, description, max_members, group_type, org_id )
		select rhn_server_group_id_seq.nextval, sgt.name, sgt.name,
			0, sgt.id, 1
		from rhnServerGroupType sgt
		where sgt.label = 'sw_mgr_entitled';
end create_first_org;
/

SHOW ERRORS;

--
-- 121. RHN_EFILEC_MOD_TRIG  (Trigger) 
--
--
-- RHN_EFILEC_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNERRATAFILECHANNEL (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_EFILEC_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNERRATAFILECHANNEL
FOR EACH ROW
begin
	:new.modified := sysdate;
end rhn_efilec_mod_trig;
/
SHOW ERRORS;


--
-- 122. RHN_ERRATA_BUGLIST_MOD_TRIG  (Trigger) 
--
--
-- RHN_ERRATA_BUGLIST_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNERRATABUGLIST (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_ERRATA_BUGLIST_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNERRATABUGLIST
FOR EACH ROW
begin
	:new.modified := sysdate;
end rhn_errata_buglist_mod_trig;
/
SHOW ERRORS;


--
-- 123. RHN_ERRATA_CVE_MOD_TRIG  (Trigger) 
--
--
-- RHN_ERRATA_CVE_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNERRATACVE (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_ERRATA_CVE_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNERRATACVE
FOR EACH ROW
begin
	:new.modified := sysdate;
end rhn_errata_cve_mod_trig;
/
SHOW ERRORS;


--
-- 124. RHN_ERRATA_FILE_MOD_TRIG  (Trigger) 
--
--
-- RHN_ERRATA_FILE_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNERRATAFILE (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_ERRATA_FILE_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNERRATAFILE
FOR EACH ROW
begin
	:new.modified := sysdate;
end rhn_errata_file_mod_trig;
/
SHOW ERRORS;


--
-- 125. RHN_ERRATA_KEYWORD_MOD_TRIG  (Trigger) 
--
--
-- RHN_ERRATA_KEYWORD_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNERRATAKEYWORD (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_ERRATA_KEYWORD_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNERRATAKEYWORD
FOR EACH ROW
begin
	:new.modified := sysdate;
end rhn_errata_keyword_mod_trig;
/
SHOW ERRORS;


--
-- 127. RHN_KSTREE_DEL_TRIG  (Trigger) 
--
--
-- RHN_KSTREE_DEL_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNKICKSTARTABLETREE (Table)
--   DUAL ()
--   RHNCHANNEL (Table)
--   DUAL (Synonym)
--   DUAL (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_KSTREE_DEL_TRIG
BEFORE DELETE ON RHNKICKSTARTABLETREE
FOR EACH ROW
declare
	date_to_use	date;
	rc_modified	date;
begin
	select	sysdate
	into	date_to_use
	from	dual;
	select	last_modified
	into	rc_modified
	from	rhnChannel
	where	id = :old.channel_id
	for update;
	if date_to_use <= rc_modified then
		date_to_use := rc_modified + 1/86400;
	end if;
	update rhnChannel
		set last_modified = date_to_use
		where id = :old.channel_id;
end rhn_kstree_del_trig;
/
SHOW ERRORS;


--
-- 128. RHN_KSTREE_MOD_TRIG  (Trigger) 
--
--
-- RHN_KSTREE_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNKICKSTARTABLETREE (Table)
--   DUAL ()
--   RHNCHANNEL (Table)
--   DUAL (Synonym)
--   DUAL (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_KSTREE_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNKICKSTARTABLETREE
FOR EACH ROW
declare
	date_to_use	date;
	rc_modified	date;
begin
	select	sysdate
	into	date_to_use
	from	dual;
	select	last_modified
	into	rc_modified
	from	rhnChannel
	where	id = :new.channel_id
	for update;
	if date_to_use <= rc_modified then
		date_to_use := rc_modified + 1/86400;
	end if;
	update rhnChannel
		set last_modified = date_to_use
		where id = :new.channel_id;
	if :new.last_modified = :old.last_modified then
		:new.last_modified := date_to_use;
	end if;
	:new.modified := date_to_use;
end rhn_kstree_mod_trig;
/
SHOW ERRORS;


--
-- 129. RHN_PACKAGEFILE_MOD_TRIG  (Trigger) 
--
--
-- RHN_PACKAGEFILE_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNPACKAGEFILE (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_PACKAGEFILE_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNPACKAGEFILE
FOR EACH ROW
begin
	:new.modified := sysdate;
end;
/
SHOW ERRORS;


--
-- 130. RHN_PACKAGE_CHANGELOG_MOD_TRIG  (Trigger) 
--
--
-- RHN_PACKAGE_CHANGELOG_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNPACKAGECHANGELOG (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_PACKAGE_CHANGELOG_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNPACKAGECHANGELOG
FOR EACH ROW
begin
        :new.modified := sysdate;
end;
/
SHOW ERRORS;


--
-- 131. RHN_PACKAGE_SIG_MOD_TRIG  (Trigger) 
--
--
-- RHN_PACKAGE_SIG_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNPACKAGESIGNATURE (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_PACKAGE_SIG_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNPACKAGESIGNATURE
FOR EACH ROW
begin
        :new.modified := sysdate;
end;
/
SHOW ERRORS;


--
-- 132. RHN_PCLIENT_MOD_TRIG  (Trigger) 
--
--
-- RHN_PCLIENT_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNPUSHCLIENT (Table)
--   STANDARD (Package)
--
-- rhn: superseded by table create

--
-- 133. RHN_PKG_CONFLICTS_MOD_TRIG  (Trigger) 
--
--
-- RHN_PKG_CONFLICTS_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNPACKAGECONFLICTS (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_PKG_CONFLICTS_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNPACKAGECONFLICTS
FOR EACH ROW
begin
        :new.modified := sysdate;
end;
/
SHOW ERRORS;


--
-- 134. RHN_PKG_OBSOLETES_MOD_TRIG  (Trigger) 
--
--
-- RHN_PKG_OBSOLETES_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNPACKAGEOBSOLETES (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_PKG_OBSOLETES_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNPACKAGEOBSOLETES
FOR EACH ROW
begin
        :new.modified := sysdate;
end;
/
SHOW ERRORS;


--
-- 135. RHN_PKG_PROVIDES_MOD_TRIG  (Trigger) 
--
--
-- RHN_PKG_PROVIDES_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNPACKAGEPROVIDES (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_PKG_PROVIDES_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNPACKAGEPROVIDES
FOR EACH ROW
begin
        :new.modified := sysdate;
end;
/
SHOW ERRORS;


--
-- 136. RHN_PKG_REQUIRES_MOD_TRIG  (Trigger) 
--
--
-- RHN_PKG_REQUIRES_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNPACKAGEREQUIRES (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_PKG_REQUIRES_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNPACKAGEREQUIRES
FOR EACH ROW
begin
        :new.modified := sysdate;
end;
/
SHOW ERRORS;


--
-- 137. CHECK_SUITES  (Synonym) 
--
--
-- CHECK_SUITES  (Synonym) 
--
--  Dependencies: 
--   RHN_CHECK_SUITES (Table)
--
CREATE SYNONYM CHECK_SUITES FOR RHN_CHECK_SUITES;

--
-- 138. CHECK_SUITE_PROBE  (Synonym) 
--
--
-- CHECK_SUITE_PROBE  (Synonym) 
--
--  Dependencies: 
--   RHN_CHECK_SUITE_PROBE (Table)
--
CREATE SYNONYM CHECK_SUITE_PROBE FOR RHN_CHECK_SUITE_PROBE;

--
-- 139. COMMAND  (Synonym) 
--
--
-- COMMAND  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND (Table)
--
CREATE SYNONYM COMMAND FOR RHN_COMMAND;

--
-- 140. COMMAND_PARAMETER  (Synonym) 
--
--
-- COMMAND_PARAMETER  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_PARAMETER (Table)
--
CREATE SYNONYM COMMAND_PARAMETER FOR RHN_COMMAND_PARAMETER;

--
-- 141. COMMAND_TARGET  (Synonym) 
--
--
-- COMMAND_TARGET  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_TARGET (Table)
--
CREATE SYNONYM COMMAND_TARGET FOR RHN_COMMAND_TARGET;

--
-- 142. CONFIG_MACRO  (Synonym) 
--
--
-- CONFIG_MACRO  (Synonym) 
--
--  Dependencies: 
--   RHN_CONFIG_MACRO (Table)
--
CREATE SYNONYM CONFIG_MACRO FOR RHN_CONFIG_MACRO;

--
-- 143. CONFIG_PARAMETER  (Synonym) 
--
--
-- CONFIG_PARAMETER  (Synonym) 
--
--  Dependencies: 
--   RHN_CONFIG_PARAMETER (Table)
--
CREATE SYNONYM CONFIG_PARAMETER FOR RHN_CONFIG_PARAMETER;

--
-- 144. DB_ENVIRONMENT  (Synonym) 
--
--
-- DB_ENVIRONMENT  (Synonym) 
--
--  Dependencies: 
--   RHN_DB_ENVIRONMENT (Table)
--
CREATE SYNONYM DB_ENVIRONMENT FOR RHN_DB_ENVIRONMENT;

--
-- 145. METRICS  (Synonym) 
--
--
-- METRICS  (Synonym) 
--
--  Dependencies: 
--   RHN_METRICS (Table)
--
CREATE SYNONYM METRICS FOR RHN_METRICS;

--
-- 146. NOTIFICATION_FORMATS  (Synonym) 
--
--
-- NOTIFICATION_FORMATS  (Synonym) 
--
--  Dependencies: 
--   RHN_NOTIFICATION_FORMATS (Table)
--
CREATE SYNONYM NOTIFICATION_FORMATS FOR RHN_NOTIFICATION_FORMATS;

--
-- 147. OS_COMMANDS_XREF  (Synonym) 
--
--
-- OS_COMMANDS_XREF  (Synonym) 
--
--  Dependencies: 
--   RHN_OS_COMMANDS_XREF (Table)
--
CREATE SYNONYM OS_COMMANDS_XREF FOR RHN_OS_COMMANDS_XREF;

--
-- 148. PROBE  (Synonym) 
--
--
-- PROBE  (Synonym) 
--
--  Dependencies: 
--   RHN_PROBE (Table)
--
CREATE SYNONYM PROBE FOR RHN_PROBE;

--
-- 149. PROBE_PARAM_VALUE  (Synonym) 
--
--
-- PROBE_PARAM_VALUE  (Synonym) 
--
--  Dependencies: 
--   RHN_PROBE_PARAM_VALUE (Table)
--
CREATE SYNONYM PROBE_PARAM_VALUE FOR RHN_PROBE_PARAM_VALUE;

--
-- 150. SAT_CLUSTER  (Synonym) 
--
--
-- SAT_CLUSTER  (Synonym) 
--
--  Dependencies: 
--   RHN_SAT_CLUSTER (Table)
--
CREATE SYNONYM SAT_CLUSTER FOR RHN_SAT_CLUSTER;

--
-- 151. SAT_CLUSTER_PROBE  (Synonym) 
--
--
-- SAT_CLUSTER_PROBE  (Synonym) 
--
--  Dependencies: 
--   RHN_SAT_CLUSTER_PROBE (Table)
--
CREATE SYNONYM SAT_CLUSTER_PROBE FOR RHN_SAT_CLUSTER_PROBE;

--
-- 152. SCHEDULES  (Synonym) 
--
--
-- SCHEDULES  (Synonym) 
--
--  Dependencies: 
--   RHN_SCHEDULES (Table)
--
CREATE SYNONYM SCHEDULES FOR RHN_SCHEDULES;

--
-- 153. SCHEDULE_DAYS  (Synonym) 
--
--
-- SCHEDULE_DAYS  (Synonym) 
--
--  Dependencies: 
--   RHN_SCHEDULE_DAYS (Table)
--
CREATE SYNONYM SCHEDULE_DAYS FOR RHN_SCHEDULE_DAYS;

--
-- 154. SCHEDULE_WEEKS  (Synonym) 
--
--
-- SCHEDULE_WEEKS  (Synonym) 
--
--  Dependencies: 
--   RHN_SCHEDULE_WEEKS (Table)
--
CREATE SYNONYM SCHEDULE_WEEKS FOR RHN_SCHEDULE_WEEKS;

--
-- 155. SERVICE_PROBE_ORIGINS  (Synonym) 
--
--
-- SERVICE_PROBE_ORIGINS  (Synonym) 
--
--  Dependencies: 
--   RHN_SERVICE_PROBE_ORIGINS (Table)
--
CREATE SYNONYM SERVICE_PROBE_ORIGINS FOR RHN_SERVICE_PROBE_ORIGINS;

--
-- 156. SNMP_ALERT  (Synonym) 
--
--
-- SNMP_ALERT  (Synonym) 
--
--  Dependencies: 
--   RHN_SNMP_ALERT (Table)
--
CREATE SYNONYM SNMP_ALERT FOR RHN_SNMP_ALERT;

--
-- 157. URL_PROBE  (Synonym) 
--
--
-- URL_PROBE  (Synonym) 
--
--  Dependencies: 
--   RHN_URL_PROBE (Table)
--
CREATE SYNONYM URL_PROBE FOR RHN_URL_PROBE;

--
-- 158. URL_PROBE_STEP  (Synonym) 
--
--
-- URL_PROBE_STEP  (Synonym) 
--
--  Dependencies: 
--   RHN_URL_PROBE_STEP (Table)
--
CREATE SYNONYM URL_PROBE_STEP FOR RHN_URL_PROBE_STEP;

--
-- 159. RHNACTIONKICKSTART  (Table) 
--
-- Difference : Column Order (No action taken) 

--
-- 160. RELEASE_NOTES  (Column) 
--
ALTER TABLE RHNDOWNLOADS ADD (RELEASE_NOTES  VARCHAR2(512 BYTE));

--
-- 161. RHNKICKSTARTSCRIPT  (Table) 
--
--
-- RHNKICKSTARTSCRIPT  (Table) 
--
--  Dependencies: 
--   RHNKSDATA (Table)
--
CREATE TABLE RHNKICKSTARTSCRIPT
(
  ID            NUMBER CONSTRAINT RHN_KSSCRIPT_ID_NN NOT NULL,
  KICKSTART_ID  NUMBER CONSTRAINT RHN_KSSCRIPT_KSID_NN NOT NULL,
  POSITION      NUMBER CONSTRAINT RHN_KSSCRIPT_POS_NN NOT NULL,
  SCRIPT_TYPE   VARCHAR2(4 BYTE) CONSTRAINT RHN_KSSCRIPT_ST_NN NOT NULL,
  CHROOT        CHAR(1 BYTE)                    DEFAULT ('Y') CONSTRAINT RHN_KSSCRIPT_CHROOT_NN NOT NULL,
  INTERPRETER   VARCHAR2(80 BYTE),
  DATA          BLOB,
  CREATED       DATE                            DEFAULT (sysdate) CONSTRAINT RHN_KSSCRIPT_CREAT_NN NOT NULL,
  MODIFIED      DATE                            DEFAULT (sysdate) CONSTRAINT RHN_KSSCRIPT_MOD_NN NOT NULL
)
LOGGING 
NOCACHE
NOPARALLEL;

--
-- 162. RHNSERVER  (Table) 
--
-- Difference : Column Order (No action taken) 

--
-- 163. RHNSERVERACTIONVERIFYMISSING  (Table) 
--
--
-- RHNSERVERACTIONVERIFYMISSING  (Table) 
--
--  Dependencies: 
--   RHNACTION (Table)
--   RHNPACKAGEARCH (Table)
--   RHNPACKAGECAPABILITY (Table)
--   RHNPACKAGEEVR (Table)
--   RHNPACKAGENAME (Table)
--   RHNSERVER (Table)
--
CREATE TABLE RHNSERVERACTIONVERIFYMISSING
(
  SERVER_ID              NUMBER CONSTRAINT RHN_SACTIONVM_SID_NN NOT NULL,
  ACTION_ID              NUMBER CONSTRAINT RHN_SACTIONVM_AID_NN NOT NULL,
  PACKAGE_NAME_ID        NUMBER CONSTRAINT RHN_SACTIONVM_PNID_NN NOT NULL,
  PACKAGE_EVR_ID         NUMBER CONSTRAINT RHN_SACTIONVM_PEID_NN NOT NULL,
  PACKAGE_ARCH_ID        NUMBER CONSTRAINT RHN_SACTIONVM_PAID_NN NOT NULL,
  PACKAGE_CAPABILITY_ID  NUMBER CONSTRAINT RHN_SACTIONVM_PCID_NN NOT NULL,
  CREATED                DATE                   DEFAULT (sysdate) CONSTRAINT RHN_SACTIONVM_CREAT_NN NOT NULL,
  MODIFIED               DATE                   DEFAULT (sysdate) CONSTRAINT RHN_SACTIONVM_MOD_NN NOT NULL
)
LOGGING 
NOCACHE
NOPARALLEL;

--
-- 164. RHNSERVERNEEDEDERRATACACHE  (Table) 
--
--
-- RHNSERVERNEEDEDERRATACACHE  (Table) 
--
--  Dependencies: 
--   RHNERRATA (Table)
--   WEB_CUSTOMER (Table)
--   RHNSERVER (Table)
--
CREATE TABLE RHNSERVERNEEDEDERRATACACHE
(
  ORG_ID     NUMBER CONSTRAINT RHN_SNEC_OID_NN  NOT NULL,
  SERVER_ID  NUMBER CONSTRAINT RHN_SNEC_SID_NN  NOT NULL,
  ERRATA_ID  NUMBER
)
NOLOGGING 
NOCACHE
NOPARALLEL;

--
-- 165. TIMEZONE_ID  (Column) 
--
ALTER TABLE RHNUSERINFO ADD (TIMEZONE_ID  NUMBER);

--
-- 166. RHNUSERSERVERPERMS  (Table) 
--
--
-- RHNUSERSERVERPERMS  (Table) 
--
--  Dependencies: 
--   RHNSERVER (Table)
--   WEB_CONTACT (Table)
--
CREATE TABLE RHNUSERSERVERPERMS
(
  USER_ID    NUMBER CONSTRAINT RHN_USPERMS_UID_NN NOT NULL,
  SERVER_ID  NUMBER CONSTRAINT RHN_USPERMS_SID_NN NOT NULL
)
LOGGING 
NOCACHE
NOPARALLEL;

--
-- 167. DATE_ACCEPTED  (Column) 
--
ALTER TABLE RHN_COMMAND_QUEUE_EXECS ADD (DATE_ACCEPTED  DATE);

--
-- 168. SERVER_ID  (Column) 
--
ALTER TABLE RHN_SAT_NODE ADD (SERVER_ID  NUMBER);

--
-- 169. SCHED_LOG_LEVEL  (Column) 
--
ALTER TABLE RHN_SAT_NODE
MODIFY(SCHED_LOG_LEVEL  DEFAULT (0));

--
-- 170. SPUT_LOG_LEVEL  (Column) 
--
ALTER TABLE RHN_SAT_NODE
MODIFY(SPUT_LOG_LEVEL  DEFAULT (0));

--
-- 171. DQ_LOG_LEVEL  (Column) 
--
ALTER TABLE RHN_SAT_NODE
MODIFY(DQ_LOG_LEVEL  DEFAULT (0));

--
-- 172. SCOUT_SHARED_KEY  (Column) 
--
ALTER TABLE RHN_SAT_NODE ADD (SCOUT_SHARED_KEY  VARCHAR2(64 BYTE) CONSTRAINT RHN_SATND_SSK_NN NOT NULL);

--
-- 173. RHN_KSSCRIPT_ID_IDX  (Index) 
--
--
-- RHN_KSSCRIPT_ID_IDX  (Index) 
--
--  Dependencies: 
--   RHNKICKSTARTSCRIPT (Table)
--
CREATE INDEX RHN_KSSCRIPT_ID_IDX ON RHNKICKSTARTSCRIPT
(ID)
LOGGING
NOPARALLEL;

--
-- 174. RHN_KSSCRIPT_KSID_POS_IDX  (Index) 
--
--
-- RHN_KSSCRIPT_KSID_POS_IDX  (Index) 
--
--  Dependencies: 
--   RHNKICKSTARTSCRIPT (Table)
--
CREATE INDEX RHN_KSSCRIPT_KSID_POS_IDX ON RHNKICKSTARTSCRIPT
(KICKSTART_ID, POSITION)
LOGGING
NOPARALLEL;

--
-- 175. RHN_SACTIONVM_SANEC_UQ  (Index) 
--
--
-- RHN_SACTIONVM_SANEC_UQ  (Index) 
--
--  Dependencies: 
--   RHNSERVERACTIONVERIFYMISSING (Table)
--
CREATE UNIQUE INDEX RHN_SACTIONVM_SANEC_UQ ON RHNSERVERACTIONVERIFYMISSING
(SERVER_ID, ACTION_ID, PACKAGE_NAME_ID, PACKAGE_EVR_ID, PACKAGE_CAPABILITY_ID)
LOGGING
NOPARALLEL;

--
-- 176. RHN_SNEC_EID_SID_IDX  (Index) 
--
--
-- RHN_SNEC_EID_SID_IDX  (Index) 
--
--  Dependencies: 
--   RHNSERVERNEEDEDERRATACACHE (Table)
--
CREATE INDEX RHN_SNEC_EID_SID_IDX ON RHNSERVERNEEDEDERRATACACHE
(ERRATA_ID, SERVER_ID)
NOLOGGING
PARALLEL ( DEGREE 6 INSTANCES 1 );

--
-- 177. RHN_SNEC_OID_EID_SID_IDX  (Index) 
--
--
-- RHN_SNEC_OID_EID_SID_IDX  (Index) 
--
--  Dependencies: 
--   RHNSERVERNEEDEDERRATACACHE (Table)
--
CREATE INDEX RHN_SNEC_OID_EID_SID_IDX ON RHNSERVERNEEDEDERRATACACHE
(ORG_ID, ERRATA_ID, SERVER_ID)
NOLOGGING
PARALLEL ( DEGREE 6 INSTANCES 1 );

--
-- 178. RHN_SNEC_SID_EID_IDX  (Index) 
--
--
-- RHN_SNEC_SID_EID_IDX  (Index) 
--
--  Dependencies: 
--   RHNSERVERNEEDEDERRATACACHE (Table)
--
CREATE INDEX RHN_SNEC_SID_EID_IDX ON RHNSERVERNEEDEDERRATACACHE
(SERVER_ID, ERRATA_ID)
NOLOGGING
PARALLEL ( DEGREE 6 INSTANCES 1 );

--
-- 179. RHN_SNPC_EID_SID_PID_IDX  (Index) 
--
--
-- RHN_SNPC_EID_SID_PID_IDX  (Index) 
--
--  Dependencies: 
--   RHNSERVERNEEDEDPACKAGECACHE (Table)
--
CREATE INDEX RHN_SNPC_EID_SID_PID_IDX ON RHNSERVERNEEDEDPACKAGECACHE
(ERRATA_ID, SERVER_ID, PACKAGE_ID)
NOLOGGING
PARALLEL ( DEGREE 6 INSTANCES 1 );

--
-- 180. RHN_USPERMS_SID_IDX  (Index) 
--
--
-- RHN_USPERMS_SID_IDX  (Index) 
--
--  Dependencies: 
--   RHNUSERSERVERPERMS (Table)
--
CREATE INDEX RHN_USPERMS_SID_IDX ON RHNUSERSERVERPERMS
(SERVER_ID)
LOGGING
NOPARALLEL;

--
-- 181. RHN_USPERMS_UID_SID_UQ  (Index) 
--
--
-- RHN_USPERMS_UID_SID_UQ  (Index) 
--
--  Dependencies: 
--   RHNUSERSERVERPERMS (Table)
--
CREATE UNIQUE INDEX RHN_USPERMS_UID_SID_UQ ON RHNUSERSERVERPERMS
(USER_ID, SERVER_ID)
LOGGING
NOPARALLEL;

--
-- 182. RHN_PACKAGE  (Package) 
--
ALTER PACKAGE RHN_PACKAGE COMPILE;

--
-- 183. DELETE_ERRATA  (Procedure) 
--
--
-- DELETE_ERRATA  (Procedure) 
--
--  Dependencies: 
--   RHNERRATA (Table)
--   RHNERRATAFILE (Table)
--   RHNERRATAPACKAGE (Table)
--   RHNERRATATMP (Table)
--   RHNPAIDERRATATEMPCACHE (Table)
--   RHNSERVERNEEDEDPACKAGECACHE (Table)
--   STANDARD (Package)
--   SYS_STUB_FOR_PURITY_ANALYSIS (Package)
--
CREATE OR REPLACE procedure
delete_errata (
	errata_id_in in number
) is
begin
	delete from rhnServerNeededPackageCache where errata_id = errata_id_in;
	delete from rhnPaidErrataTempCache where errata_id = errata_id_in;
	delete from rhnErrataFile where errata_id = errata_id_in;
	delete from rhnErrataPackage where errata_id = errata_id_in;
	delete from rhnErrata where id = errata_id_in;
	delete from rhnErrataTmp where id = errata_id_in;
end delete_errata;
/

SHOW ERRORS;

--
-- 184. RHNCHANNELPACKAGEOVERVIEW  (View) 
--
--
-- RHNCHANNELPACKAGEOVERVIEW  (View) 
--
--  Dependencies: 
--   EVR_T (Type)
--   RHNCHANNELPACKAGE (Table)
--   RHNPACKAGE (Table)
--   RHNPACKAGEEVR (Table)
--
CREATE OR REPLACE FORCE VIEW RHNCHANNELPACKAGEOVERVIEW
(CHANNEL_ID, NAME_ID, EVR)
AS 
select  cp.channel_id,
	p.name_id,
	max(p_evr.evr)
from
	rhnPackageEVR p_evr,
	rhnPackage p,
	rhnChannelPackage cp
where
    	cp.package_id = p.id
    and p.evr_id = p_evr.id
group by cp.channel_id, p.name_id;

--
-- 185. RHNUSERACTIONOVERVIEW  (View) 
--
--
-- RHNUSERACTIONOVERVIEW  (View) 
--
--  Dependencies: 
--   RHNACTIONOVERVIEW (View)
--   RHNACTIONSTATUS (Table)
--   RHNSERVERACTION (Table)
--   RHNUSERSERVERPERMS (Table)
--
CREATE OR REPLACE FORCE VIEW RHNUSERACTIONOVERVIEW
(ORG_ID, USER_ID, ID, TYPE_NAME, SCHEDULER, 
 EARLIEST_ACTION, ACTION_NAME, ACTION_STATUS_ID, ACTION_STATUS, TALLY, 
 ARCHIVED)
AS 
select	ao.org_id                                       org_id,
	usp.user_id                                     user_id,
    	ao.action_id                                    id,
	ao.type_name                                    type_name,
        ao.scheduler                                    scheduler,
	ao.earliest_action                              earliest_action,
	decode(ao.name, null, ao.type_name, ao.name)	action_name,
	sa.status					action_status_id,
	astat.name                                      action_status,
	count(sa.action_id)				tally,
	ao.archived                                     archived
from	rhnActionStatus            astat,
    	rhnUserServerPerms         usp,
	rhnServerAction            sa,
	rhnActionOverview	   ao
where	ao.action_id = sa.action_id
  and   sa.server_id = usp.server_id
  and   sa.status = astat.id
group by ao.org_id,
	 usp.user_id,
	 ao.action_id,
	 ao.type_name,
	 ao.scheduler,
	 ao.earliest_action,
	 decode(ao.name, null, ao.type_name, ao.name),
	 sa.status,
	 astat.name,
	 ao.archived
order by earliest_action;

--
-- 186. RHNUSERSERVERPERMSDUPES  (View) 
--
--
-- RHNUSERSERVERPERMSDUPES  (View) 
--
--  Dependencies: 
--   RHNSERVER (Table)
--   RHNSERVERGROUPMEMBERS (Table)
--   RHNUSERGROUP (Table)
--   RHNUSERGROUPMEMBERS (Table)
--   RHNUSERGROUPTYPE (Table)
--   RHNUSERSERVERGROUPPERMS (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE FORCE VIEW RHNUSERSERVERPERMSDUPES
(USER_ID, SERVER_ID)
AS 
select	usg.user_id,
	sgm.server_id
from	rhnServerGroupMembers sgm,
	rhnUserServerGroupPerms usg
where	usg.server_group_id = sgm.server_group_id
union all
select	ugm.user_id, s.id
from	rhnServer s,
	rhnUserGroup ug,
	rhnUserGroupMembers ugm,
	rhnUserGroupType ugt
where	ugt.label = 'org_admin'
	and ugm.user_group_id = ug.id
	and ug.group_type = ugt.id
	and ug.org_id = s.org_id;

--
-- 187. RHN_ACTIONKSFL_MOD_TRIG  (Trigger) 
--
--
-- RHN_ACTIONKSFL_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNACTIONKICKSTARTFILELIST (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_ACTIONKSFL_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNACTIONKICKSTARTFILELIST
FOR EACH ROW
begin
	:new.modified := sysdate;
end;
/
SHOW ERRORS;


--
-- 188. RHN_CHANNEL_ERRATA_DEL_TRIG  (Trigger) 
--
--
-- RHN_CHANNEL_ERRATA_DEL_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNCHANNELERRATA (Table)
--   DUAL ()
--   RHNCHANNEL (Table)
--   DUAL (Synonym)
--   DUAL (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_CHANNEL_ERRATA_DEL_TRIG
BEFORE DELETE ON RHNCHANNELERRATA
FOR EACH ROW
declare
	date_to_use	date;
	rc_modified	date;
begin
	select	last_modified
	into	rc_modified
	from	rhnChannel
	where	id = :old.channel_id
	for update;
	select	sysdate
	into	date_to_use
	from	dual;
	if date_to_use <= rc_modified then
		date_to_use := rc_modified + 1/86400;
	end if;
	update rhnChannel
		set last_modified = date_to_use
		where id = :old.channel_id;
end rhn_channel_errata_del_trig;
/
SHOW ERRORS;


--
-- 189. RHN_CHANNEL_ERRATA_MOD_TRIG  (Trigger) 
--
--
-- RHN_CHANNEL_ERRATA_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNCHANNELERRATA (Table)
--   DUAL ()
--   RHNCHANNEL (Table)
--   DUAL (Synonym)
--   DUAL (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_CHANNEL_ERRATA_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNCHANNELERRATA
FOR EACH ROW
declare
	date_to_use	date;
	rc_modified	date;
begin
	select	last_modified
	into	rc_modified
	from	rhnChannel
	where	id = :new.channel_id
	for update;
	select	sysdate
	into	date_to_use
	from	dual;
	if date_to_use <= rc_modified then
		date_to_use := rc_modified + 1/86400;
	end if;
	update rhnChannel
		set last_modified = date_to_use
		where id = :new.channel_id;
	:new.modified := date_to_use;
end rhn_channel_errata_mod_trig;
/
SHOW ERRORS;


--
-- 190. RHN_CHANNEL_PACKAGE_DEL_TRIG  (Trigger) 
--
--
-- RHN_CHANNEL_PACKAGE_DEL_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNCHANNELPACKAGE (Table)
--   LOOKUP_SNAPSHOT_INVALID_REASON (Function)
--   RHNCHANNEL (Table)
--   RHNSNAPSHOT (Table)
--   RHNSNAPSHOTCHANNEL (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_CHANNEL_PACKAGE_DEL_TRIG
AFTER DELETE ON RHNCHANNELPACKAGE
FOR EACH ROW
declare
	cursor snapshots is
		select	snapshot_id id
		from	rhnSnapshotChannel
		where	channel_id = :old.channel_id;
begin
	update rhnChannel
		set last_modified = sysdate
		where id = :old.channel_id;
	for snapshot in snapshots loop
		update rhnSnapshot
			set invalid = lookup_snapshot_invalid_reason('channel_modified')
			where id = snapshot.id;
	end loop;
end rhn_channel_package_del_trig;
/
SHOW ERRORS;


--
-- 191. RHN_CHANNEL_PACKAGE_MOD_TRIG  (Trigger) 
--
--
-- RHN_CHANNEL_PACKAGE_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNCHANNELPACKAGE (Table)
--   DUAL ()
--   LOOKUP_SNAPSHOT_INVALID_REASON (Function)
--   RHNCHANNEL (Table)
--   RHNSNAPSHOT (Table)
--   RHNSNAPSHOTCHANNEL (Table)
--   DUAL (Synonym)
--   DBMS_STANDARD (Package)
--   DUAL (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_CHANNEL_PACKAGE_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNCHANNELPACKAGE
FOR EACH ROW
declare
	date_to_use	date;
	rc_modified	date;
	cursor snapshots is
		select	snapshot_id id
		from	rhnSnapshotChannel
		where	channel_id = :old.channel_id;
begin
	select	last_modified
	into	rc_modified
	from	rhnChannel
	where	id = :new.channel_id
	for update;
	select	sysdate
	into	date_to_use
	from	dual;
	if date_to_use <= rc_modified then
		date_to_use := rc_modified + 1/86400;
	end if;
	update rhnChannel
		set last_modified = date_to_use
		where id = :new.channel_id;
	:new.modified := date_to_use;
	if updating then
		for snapshot in snapshots loop
			update rhnSnapshot
				set invalid = lookup_snapshot_invalid_reason('channel_modified')
				where id = snapshot.id;
		end loop;
	end if;
end rhn_channel_package_mod_trig;
/
SHOW ERRORS;


--
-- 192. RHN_CMETH_VAL_TRIG  (Trigger) 
--
--
-- RHN_CMETH_VAL_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHN_CONTACT_METHODS (Table)
--   DBMS_STANDARD (Package)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_CMETH_VAL_TRIG
BEFORE INSERT OR UPDATE ON RHN_CONTACT_METHODS
REFERENCING NEW AS NEW OLD AS OLD
FOR EACH ROW
declare
    msg  varchar2(200);
    missing_data exception;
begin
    msg :='missing or invalid data for contact_methods table';
    if :new.method_type_id = 1
    then
        if (
            :new.pager_email   is null     or
            :new.pager_split_long_messages  is null )
        then
            raise missing_data;
        end if;
    end if;
    if :new.method_type_id = 2
    then
        if :new.email_address is null
        then
            raise missing_data;
        end if;
    end if;
    if :new.method_type_id = 5
    then
        if (:new.snmp_host is null   or
           :new.snmp_port is null)
        then
            raise missing_data;
        end if;
    end if;
    exception
    when missing_data then
    raise_application_error (-20012,msg);
    when others then
    raise;
end;
/
SHOW ERRORS;


--
-- 193. RHN_EFILEPS_MOD_TRIG  (Trigger) 
--
--
-- RHN_EFILEPS_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNERRATAFILEPACKAGESOURCE (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_EFILEPS_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNERRATAFILEPACKAGESOURCE
FOR EACH ROW
begin
	:new.modified := sysdate;
end rhn_efileps_mod_trig;
/
SHOW ERRORS;


--
-- 194. RHN_EFILEP_MOD_TRIG  (Trigger) 
--
--
-- RHN_EFILEP_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNERRATAFILEPACKAGE (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_EFILEP_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNERRATAFILEPACKAGE
FOR EACH ROW
begin
	:new.modified := sysdate;
end rhn_efilep_mod_trig;
/
SHOW ERRORS;


--
-- 195. RHN_ERRATA_MOD_TRIG  (Trigger) 
--
--
-- RHN_ERRATA_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNERRATA (Table)
--   DUAL ()
--   RHNCHANNEL (Table)
--   RHNCHANNELERRATA (Table)
--   DUAL (Synonym)
--   DBMS_STANDARD (Package)
--   DUAL (Table)
--   STANDARD (Package)
--
create or replace trigger
rhn_errata_mod_trig
before insert or update on rhnErrata
for each row
declare
   date_to_use date;
   rc_modified date;
   cursor channels is
      select   channel_id id
      from  rhnChannelErrata
      where errata_id = :new.id;
begin
        date_to_use := sysdate;
        if :new.last_modified = :old.last_modified then
            :new.last_modified := date_to_use;
        end if;
   if inserting then
      return;
   end if;
   if (:old.id != :new.id) or
     (:old.advisory != :new.advisory) or
     (:old.advisory_type != :new.advisory_type) or
     (:old.advisory_name != :new.advisory_name) or
     (:old.advisory_rel != :new.advisory_rel) or
     (:old.product != :new.product) or
     (:old.description != :new.description) or
     (:old.synopsis != :new.synopsis) or
     (:old.topic != :new.topic) or
     (:old.solution != :new.solution) or
     (:old.issue_date != :new.issue_date) or
     (:old.update_date != :new.update_date) or
     (:old.refers_to != :new.refers_to) then
      :new.modified := date_to_use;
   end if;
   for channel in channels loop
      -- lock the table.
      select   last_modified
      into  rc_modified
      from  rhnChannel
      where id = channel.id
      for update;

      if date_to_use <= rc_modified then
         -- alter session set
         --   nls_date_format ='DD-MON-YYYY HH24:MI:SS';
         -- select sysdate, sysdate + .00001 , sysdate from dual;
         -- select sysdate, sysdate + 1/86400, sysdate from dual;
         date_to_use := rc_modified + 1/86400;
      end if;
      update rhnChannel
         set last_modified = date_to_use
         where id = channel.id;
   end loop;
end rhn_errata_mod_trig;
/
show errors;


--
-- 196. RHN_ERRATA_PACKAGE_MOD_TRIG  (Trigger) 
--
--
-- RHN_ERRATA_PACKAGE_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNERRATAPACKAGE (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_ERRATA_PACKAGE_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNERRATAPACKAGE
FOR EACH ROW
begin
	:new.modified := sysdate;
end rhn_errata_package_mod_trig;
/
SHOW ERRORS;


--
-- 197. RHN_SACTIONVM_MOD_TRIG  (Trigger) 
--
--
-- RHN_SACTIONVM_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNSERVERACTIONVERIFYMISSING (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_SACTIONVM_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNSERVERACTIONVERIFYMISSING
FOR EACH ROW
begin
	:new.modified := sysdate;
end;
/
SHOW ERRORS;


--
-- 198. RHN_USER_INFO_MOD_TRIG  (Trigger) 
--
--
-- RHN_USER_INFO_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNUSERINFO (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_USER_INFO_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNUSERINFO
FOR EACH ROW
begin
	:new.modified := sysdate;
end rhn_user_info_mod_trig;
/
SHOW ERRORS;


--
-- 199. CHECK_PROBE  (Synonym) 
--
--
-- CHECK_PROBE  (Synonym) 
--
--  Dependencies: 
--   RHN_CHECK_PROBE (Table)
--
CREATE SYNONYM CHECK_PROBE FOR RHN_CHECK_PROBE;

--
-- 200. COMMAND_PARAMETER_THRESHOLD  (Synonym) 
--
--
-- COMMAND_PARAMETER_THRESHOLD  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_PARAM_THRESHOLD (Table)
--
CREATE SYNONYM COMMAND_PARAMETER_THRESHOLD FOR RHN_COMMAND_PARAM_THRESHOLD;

--
-- 201. COMMAND_QUEUE_EXECS  (Synonym) 
--
--
-- COMMAND_QUEUE_EXECS  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_QUEUE_EXECS (Table)
--
CREATE SYNONYM COMMAND_QUEUE_EXECS FOR RHN_COMMAND_QUEUE_EXECS;

--
-- 202. COMMAND_QUEUE_SESSIONS  (Synonym) 
--
--
-- COMMAND_QUEUE_SESSIONS  (Synonym) 
--
--  Dependencies: 
--   RHN_COMMAND_QUEUE_SESSIONS (Table)
--
CREATE SYNONYM COMMAND_QUEUE_SESSIONS FOR RHN_COMMAND_QUEUE_SESSIONS;

--
-- 203. CONTACT_GROUPS  (Synonym) 
--
--
-- CONTACT_GROUPS  (Synonym) 
--
--  Dependencies: 
--   RHN_CONTACT_GROUPS (Table)
--
CREATE SYNONYM CONTACT_GROUPS FOR RHN_CONTACT_GROUPS;

--
-- 204. CONTACT_METHODS  (Synonym) 
--
--
-- CONTACT_METHODS  (Synonym) 
--
--  Dependencies: 
--   RHN_CONTACT_METHODS (Table)
--
CREATE SYNONYM CONTACT_METHODS FOR RHN_CONTACT_METHODS;

--
-- 205. HOST_PROBE  (Synonym) 
--
--
-- HOST_PROBE  (Synonym) 
--
--  Dependencies: 
--   RHN_HOST_PROBE (Table)
--
CREATE SYNONYM HOST_PROBE FOR RHN_HOST_PROBE;

--
-- 206. LL_NETSAINT  (Synonym) 
--
--
-- LL_NETSAINT  (Synonym) 
--
--  Dependencies: 
--   RHN_LL_NETSAINT (Table)
--
CREATE SYNONYM LL_NETSAINT FOR RHN_LL_NETSAINT;

--
-- 207. REDIRECTS  (Synonym) 
--
--
-- REDIRECTS  (Synonym) 
--
--  Dependencies: 
--   RHN_REDIRECTS (Table)
--
CREATE SYNONYM REDIRECTS FOR RHN_REDIRECTS;

--
-- 208. REDIRECT_CRITERIA  (Synonym) 
--
--
-- REDIRECT_CRITERIA  (Synonym) 
--
--  Dependencies: 
--   RHN_REDIRECT_CRITERIA (Table)
--
CREATE SYNONYM REDIRECT_CRITERIA FOR RHN_REDIRECT_CRITERIA;

--
-- 209. REDIRECT_EMAIL_TARGETS  (Synonym) 
--
--
-- REDIRECT_EMAIL_TARGETS  (Synonym) 
--
--  Dependencies: 
--   RHN_REDIRECT_EMAIL_TARGETS (Table)
--
CREATE SYNONYM REDIRECT_EMAIL_TARGETS FOR RHN_REDIRECT_EMAIL_TARGETS;

--
-- 210. REDIRECT_GROUP_TARGETS  (Synonym) 
--
--
-- REDIRECT_GROUP_TARGETS  (Synonym) 
--
--  Dependencies: 
--   RHN_REDIRECT_GROUP_TARGETS (Table)
--
CREATE SYNONYM REDIRECT_GROUP_TARGETS FOR RHN_REDIRECT_GROUP_TARGETS;

--
-- 211. REDIRECT_METHOD_TARGETS  (Synonym) 
--
--
-- REDIRECT_METHOD_TARGETS  (Synonym) 
--
--  Dependencies: 
--   RHN_REDIRECT_METHOD_TARGETS (Table)
--
CREATE SYNONYM REDIRECT_METHOD_TARGETS FOR RHN_REDIRECT_METHOD_TARGETS;

--
-- 212. SAT_NODE  (Synonym) 
--
--
-- SAT_NODE  (Synonym) 
--
--  Dependencies: 
--   RHN_SAT_NODE (Table)
--
CREATE SYNONYM SAT_NODE FOR RHN_SAT_NODE;

--
-- 213. SAT_NODE_PROBE  (Synonym) 
--
--
-- SAT_NODE_PROBE  (Synonym) 
--
--  Dependencies: 
--   RHN_SAT_NODE_PROBE (Table)
--
CREATE SYNONYM SAT_NODE_PROBE FOR RHN_SAT_NODE_PROBE;

--
-- 214. RHNAPPINSTALLSESSION  (Table) 
--
--
-- RHNAPPINSTALLSESSION  (Table) 
--
--  Dependencies: 
--   RHNAPPINSTALLINSTANCE (Table)
--   RHNSERVER (Table)
--   WEB_CONTACT (Table)
--
CREATE TABLE RHNAPPINSTALLSESSION
(
  ID            NUMBER CONSTRAINT RHN_APPINST_SESSION_ID_NN NOT NULL,
  INSTANCE_ID   NUMBER CONSTRAINT RHN_APPINST_SESSION_IID_NN NOT NULL,
  MD5SUM        VARCHAR2(64 BYTE),
  PROCESS_NAME  VARCHAR2(32 BYTE),
  STEP_NUMBER   NUMBER,
  USER_ID       NUMBER CONSTRAINT RHN_APPINST_SESSION_UID_NN NOT NULL,
  SERVER_ID     NUMBER CONSTRAINT RHN_APPINST_SESSION_SID_NN NOT NULL,
  CREATED       DATE                            DEFAULT (sysdate) CONSTRAINT RHN_APPINST_SESSION_CREAT_NN NOT NULL,
  MODIFIED      DATE                            DEFAULT (sysdate) CONSTRAINT RHN_APPINST_SESSION_MOD_NN NOT NULL
)
LOGGING 
NOCACHE
NOPARALLEL;

--
-- 215. RHNAPPINSTALLSESSIONDATA  (Table) 
--
--
-- RHNAPPINSTALLSESSIONDATA  (Table) 
--
--  Dependencies: 
--   RHNAPPINSTALLSESSION (Table)
--
CREATE TABLE RHNAPPINSTALLSESSIONDATA
(
  ID          NUMBER CONSTRAINT RHN_APPINST_SDATA_ID_NN NOT NULL,
  SESSION_ID  NUMBER CONSTRAINT RHN_APPINST_SDATA_SID_NN NOT NULL,
  KEY         VARCHAR2(64 BYTE) CONSTRAINT RHN_APPINST_SDATA_K_NN NOT NULL,
  VALUE       VARCHAR2(2048 BYTE),
  EXTRA_DATA  BLOB,
  CREATED     DATE                              DEFAULT (sysdate) CONSTRAINT RHN_APPINST_SDATA_CREAT_NN NOT NULL,
  MODIFIED    DATE                              DEFAULT (sysdate) CONSTRAINT RHN_APPINST_SDATA_MOD_NN NOT NULL
)
LOGGING 
NOCACHE
NOPARALLEL;

--
-- 216. RHNKICKSTARTSESSION  (Table) 
--
-- Difference : Column Order (No action taken) 

--
-- 217. EVR_ID  (Column) 
--
ALTER TABLE RHNSATELLITEINFO ADD (EVR_ID  NUMBER);

--
-- 218. RHN_APPINST_SDATA_ID_IDX  (Index) 
--
--
-- RHN_APPINST_SDATA_ID_IDX  (Index) 
--
--  Dependencies: 
--   RHNAPPINSTALLSESSIONDATA (Table)
--
CREATE INDEX RHN_APPINST_SDATA_ID_IDX ON RHNAPPINSTALLSESSIONDATA
(ID)
LOGGING
NOPARALLEL;

--
-- 219. RHN_APPINST_SDATA_SID_K_ID_IDX  (Index) 
--
--
-- RHN_APPINST_SDATA_SID_K_ID_IDX  (Index) 
--
--  Dependencies: 
--   RHNAPPINSTALLSESSIONDATA (Table)
--
CREATE INDEX RHN_APPINST_SDATA_SID_K_ID_IDX ON RHNAPPINSTALLSESSIONDATA
(SESSION_ID, KEY, ID)
LOGGING
NOPARALLEL;

--
-- 220. RHN_APPINST_SESSION_ID_IID_IDX  (Index) 
--
--
-- RHN_APPINST_SESSION_ID_IID_IDX  (Index) 
--
--  Dependencies: 
--   RHNAPPINSTALLSESSION (Table)
--
CREATE INDEX RHN_APPINST_SESSION_ID_IID_IDX ON RHNAPPINSTALLSESSION
(ID, INSTANCE_ID)
LOGGING
NOPARALLEL;

--
-- 221. RHN_CACHE  (Package) 
--
--
-- RHN_CACHE  (Package) 
--
--  Dependencies: 
--   RHNSERVERGROUP (Table)
--   RHNUSERGROUP (Table)
--   RHNUSERGROUPMEMBERS (Table)
--   RHNUSERGROUPTYPE (Table)
--   RHNUSERSERVERGROUPPERMS (Table)
--   RHNUSERSERVERPERMS (Table)
--   RHNUSERSERVERPERMSDUPES (View)
--   STANDARD (Package)
--   STANDARD (Package)
--
CREATE OR REPLACE package
rhn_cache
is
	version varchar2(100) := '$Id: satellite-3.4-to-3.6.sql 64427 2005-09-09 14:08:21Z jslagle $';
	procedure update_perms_for_server(
		server_id_in in number
	);
	procedure update_perms_for_user(
		user_id_in in number
	);
	procedure update_perms_for_server_group(
		server_group_id_in in number
	);
end rhn_cache;
/

SHOW ERRORS;

--
-- 222. RHN_USER  (Package) 
--
--
-- RHN_USER  (Package) 
--
--  Dependencies: 
--   RHNEMAILADDRESS (Table)
--   RHNEMAILADDRESSSTATE (Table)
--   RHNSERVERGROUP (Table)
--   RHNSET (Table)
--   RHNUSERGROUP (Table)
--   RHNUSERGROUPMEMBERS (Table)
--   RHNUSERGROUPTYPE (Table)
--   RHNUSERSERVERGROUPPERMS (Table)
--   RHN_CACHE (Package)
--   RHN_EADDRESS_ID_SEQ (Sequence)
--   RHN_EXCEPTION (Package)
--   WEB_CONTACT (Table)
--   WEB_USER_CONTACT_PERMISSION (Table)
--   WEB_USER_PERSONAL_INFO (Table)
--   STANDARD (Package)
--   STANDARD (Package)
--
CREATE OR REPLACE package rhn_user
is
	version varchar2(100) := '$Id: satellite-3.4-to-3.6.sql 64427 2005-09-09 14:08:21Z jslagle $';
    function check_role(user_id_in in number, role_in in varchar2) return number;
    PRAGMA RESTRICT_REFERENCES(check_role, WNDS, RNPS, WNPS);
    function check_role_implied(user_id_in in number, role_in in varchar2) return number;
    PRAGMA RESTRICT_REFERENCES(check_role_implied, WNDS, RNPS, WNPS);
    function get_org_id(user_id_in in number) return number;
    PRAGMA RESTRICT_REFERENCES(get_org_id, WNDS, RNPS, WNPS);
	function find_mailable_address(user_id_in in number) return varchar2;
	procedure add_servergroup_perm(
		user_id_in in number,
		server_group_id_in in number
	);
	procedure remove_servergroup_perm(
		user_id_in in number,
		server_group_id_in in number
	);
	procedure add_to_usergroup(
		user_id_in in number,
		user_group_id_in in number
	);
	procedure add_users_to_usergroups(
		user_id_in in number
	);
	procedure remove_from_usergroup(
		user_id_in in number,
		user_group_id_in in number
	);
	procedure remove_users_from_servergroups(
		user_id_in in number
	);
end rhn_user;
/

SHOW ERRORS;

--
-- 223. QUEUE_SERVER  (Procedure) 
--
--
-- QUEUE_SERVER  (Procedure) 
--
--  Dependencies: 
--   RHNSERVER (Table)
--   RHNSERVERNEEDEDERRATACACHE (Table)
--   RHNSERVERNEEDEDPACKAGECACHE (Table)
--   RHNSERVERNEEDEDPACKAGEVIEW (View)
--   RHNTASKQUEUE (Table)
--   STANDARD (Package)
--   SYS_STUB_FOR_PURITY_ANALYSIS (Package)
--
CREATE OR REPLACE PROCEDURE
queue_server(server_id_in IN NUMBER, immediate_in IN NUMBER := 1)
IS
    org_id_tmp NUMBER;
BEGIN
    IF immediate_in > 0
    THEN
        DELETE FROM rhnServerNeededPackageCache WHERE server_id = server_id_in;
        INSERT INTO rhnServerNeededPackageCache
       	    (SELECT org_id, server_id, errata_id, package_id
	       FROM rhnServerNeededPackageView
              WHERE server_id = server_id_in);
	DELETE FROM rhnServerNeededErrataCache snec WHERE server_id = server_id_in;
	insert into rhnServerNeededErrataCache
	    (select distinct org_id, server_id, errata_id
	       from rhnServerNeededPackageCache
	      where server_id = server_id_in
	        and errata_id is not null);
    ELSE
          SELECT org_id INTO org_id_tmp FROM rhnServer WHERE id = server_id_in;
	  INSERT
            INTO rhnTaskQueue
                 (org_id, task_name, task_data)
          VALUES (org_id_tmp,
                  'update_server_errata_cache',
                  server_id_in);
    END IF;
END queue_server;
/

SHOW ERRORS;

--
-- 224. RHN_APPINST_SDATA_MOD_TRIG  (Trigger) 
--
--
-- RHN_APPINST_SDATA_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNAPPINSTALLSESSIONDATA (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_APPINST_SDATA_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNAPPINSTALLSESSIONDATA
FOR EACH ROW
begin
	:new.modified := sysdate;
end rhn_appinst_sdata_mod_trig;
/
SHOW ERRORS;


--
-- 225. RHN_APPINST_SESSION_MOD_TRIG  (Trigger) 
--
--
-- RHN_APPINST_SESSION_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNAPPINSTALLSESSION (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_APPINST_SESSION_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNAPPINSTALLSESSION
FOR EACH ROW
begin
	:new.modified := sysdate;
end rhn_appinst_session_mod_trig;
/
SHOW ERRORS;


--
-- 226. RHN_KSSCRIPT_MOD_TRIG  (Trigger) 
--
--
-- RHN_KSSCRIPT_MOD_TRIG  (Trigger) 
--
--  Dependencies: 
--   RHNKICKSTARTSESSION (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_KSSCRIPT_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNKICKSTARTSESSION
FOR EACH ROW
begin
	:new.modified := sysdate;
end rhn_ksscript_mod_trig;
/
SHOW ERRORS;


--
-- 227. CONTACT  (Synonym) 
--
--
-- CONTACT  (Synonym) 
--
--  Dependencies: 
--   RHN_CONTACT_MONITORING (View)
--
CREATE SYNONYM CONTACT FOR RHN_CONTACT_MONITORING;

--
-- 228. CONTACT_GROUP_MEMBERS  (Synonym) 
--
--
-- CONTACT_GROUP_MEMBERS  (Synonym) 
--
--  Dependencies: 
--   RHN_CONTACT_GROUP_MEMBERS (Table)
--
CREATE SYNONYM CONTACT_GROUP_MEMBERS FOR RHN_CONTACT_GROUP_MEMBERS;

--
-- 229. HOST_CHECK_SUITES  (Synonym) 
--
--
-- HOST_CHECK_SUITES  (Synonym) 
--
--  Dependencies: 
--   RHN_HOST_CHECK_SUITES (Table)
--
CREATE SYNONYM HOST_CHECK_SUITES FOR RHN_HOST_CHECK_SUITES;

--
-- 230. RHN_CHANNEL  (Package) 
--
--
-- RHN_CHANNEL  (Package) 
--
--  Dependencies: 
--   DUAL ()
--   QUEUE_SERVER (Procedure)
--   RHNCHANNEL (Table)
--   RHNCHANNEL (Table)
--   RHNCHANNELFAMILY (Table)
--   RHNCHANNELFAMILYLICENSE (Table)
--   RHNCHANNELFAMILYLICENSECONSENT (Table)
--   RHNCHANNELFAMILYMEMBERS (Table)
--   RHNCHANNELFAMILYSERVERS (View)
--   RHNCHANNELNEWESTPACKAGE (Table)
--   RHNCHANNELNEWESTPACKAGEAUDIT (Table)
--   RHNCHANNELNEWESTPACKAGEVIEW (View)
--   RHNCHANNELPERMISSION (Table)
--   RHNCHANNELPERMISSIONROLE (Table)
--   RHNCHANNELPERMISSIONS (View)
--   RHNDISTCHANNELMAP (Table)
--   RHNORGCHANNELFAMILYPERMISSIONS (View)
--   RHNORGCHANNELFAMILYPERMISSIONS (View)
--   RHNORGCHANNELSETTINGS (Table)
--   RHNORGCHANNELSETTINGSTYPE (Table)
--   RHNPRIVATECHANNELFAMILY (Table)
--   RHNPROXYINFO (Table)
--   RHNSATELLITECHANNELFAMILY (Table)
--   RHNSATELLITEINFO (Table)
--   RHNSERVER (Table)
--   RHNSERVERARCH (Table)
--   RHNSERVERCHANNEL (Table)
--   RHNSERVERCHANNEL (Table)
--   RHNSERVERCHANNELARCHCOMPAT (Table)
--   RHNSERVERHISTORY (Table)
--   RHNSET (Table)
--   RHN_EVENT_ID_SEQ (Sequence)
--   RHN_EXCEPTION (Package)
--   RHN_SERVER (Package)
--   RHN_SET (Package)
--   RHN_USER (Package)
--   WEB_CONTACT (Table)
--   DUAL (Synonym)
--   DUAL (Table)
--   STANDARD (Package)
--   STANDARD (Package)
--
CREATE OR REPLACE PACKAGE rhn_channel
IS
	version varchar2(100) := '$Id: satellite-3.4-to-3.6.sql 64427 2005-09-09 14:08:21Z jslagle $';
    CURSOR server_base_subscriptions(server_id_in NUMBER) IS
    	   SELECT C.id
	     FROM rhnChannel C, rhnServerChannel SC
	    WHERE C.id = SC.channel_id
	      AND SC.server_id = server_id_in
	      AND C.parent_channel IS NULL;
    CURSOR check_server_subscription(server_id_in NUMBER, channel_id_in NUMBER) IS
           SELECT channel_id
	     FROM rhnServerChannel
	    WHERE server_id = server_id_in
	      AND channel_id = channel_id_in;
    CURSOR check_server_parent_membership(server_id_in NUMBER, channel_id_in NUMBER) IS
    	   SELECT C.id
	     FROM rhnChannel C, rhnServerChannel SC
	    WHERE C.parent_channel = channel_id_in
	      AND C.id = SC.channel_id
	      AND SC.server_id = server_id_in;
    CURSOR channel_family_perm_cursor(channel_family_id_in NUMBER, org_id_in NUMBER) IS
           SELECT *
	     FROM rhnOrgChannelFamilyPermissions
	    WHERE channel_family_id = channel_family_id_in
	      AND org_id = org_id_in;
    PROCEDURE license_consent(channel_id_in IN NUMBER, user_id_in IN NUMBER, server_id_in IN NUMBER);
    FUNCTION get_license_path(channel_id_in IN NUMBER) RETURN VARCHAR2;
    PROCEDURE unsubscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, unsubscribe_children_in number := 0);
    PROCEDURE subscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, user_id_in number := null);
    FUNCTION guess_server_base(server_id_in IN NUMBER) RETURN NUMBER;
    FUNCTION base_channel_for_release_arch(release_in in varchar2,
	server_arch_in in varchar2, org_id_in in number := -1,
	user_id_in in number := null) RETURN number;
    FUNCTION base_channel_rel_archid(release_in in varchar2,
	server_arch_id_in in number, org_id_in in number := -1,
	user_id_in in number := null) RETURN number;
    FUNCTION channel_priority(channel_id_in in number) RETURN number;
    PROCEDURE bulk_subscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER);
    PROCEDURE bulk_unsubscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER);
    PROCEDURE bulk_server_base_change(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER);
    procedure bulk_server_basechange_from(
	set_label_in in varchar2,
	set_uid_in in number,
	old_channel_id_in in number,
	new_channel_id_in in number);
    procedure bulk_guess_server_base(
	set_label_in in varchar2,
	set_uid_in in number);
    procedure bulk_guess_server_base_from(
	set_label_in in varchar2,
	set_uid_in in number,
	channel_id_in in number);
    PROCEDURE clear_subscriptions(server_id_in IN NUMBER);
    FUNCTION available_family_subscriptions(channel_family_id_in IN NUMBER, org_id_in IN NUMBER) RETURN NUMBER;
    PROCEDURE update_family_counts(channel_family_id_in IN NUMBER, org_id_in IN NUMBER);
    FUNCTION family_for_channel(channel_id_in IN NUMBER) RETURN NUMBER;
    FUNCTION available_chan_subscriptions(channel_id_in IN NUMBER, org_id_in IN NUMBER) RETURN NUMBER;
    procedure entitle_customer(customer_id_in in number, channel_family_id_in in number, quantity_in in number);
    procedure set_family_maxmembers(customer_id_in in number, channel_family_id_in in number, quantity_in in number);
    procedure unsubscribe_server_from_family(server_id_in in number, channel_family_id_in in number);
    procedure delete_server_channels(server_id_in in number);
    procedure refresh_newest_package(channel_id_in in number, caller_in in varchar2 := '(unknown)');
    function get_org_id(channel_id_in in number) return number;
    PRAGMA RESTRICT_REFERENCES(get_org_id, WNDS, RNPS, WNPS);
    function get_org_access(channel_id_in in number, org_id_in in number) return number;
    PRAGMA RESTRICT_REFERENCES(get_org_access, WNDS, RNPS, WNPS);
    function get_cfam_org_access(cfam_id_in in number, org_id_in in number) return number;
    function user_role_check_debug(channel_id_in in number, user_id_in in number, role_in in varchar2, reason_out out varchar2)
    	RETURN NUMBER;
    PRAGMA RESTRICT_REFERENCES(user_role_check_debug, WNDS, RNPS, WNPS);
    function user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    	RETURN NUMBER;
    PRAGMA RESTRICT_REFERENCES(user_role_check, WNDS, RNPS, WNPS);
    function loose_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    	RETURN NUMBER;
    PRAGMA RESTRICT_REFERENCES(loose_user_role_check, WNDS, RNPS, WNPS);
    function direct_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    	RETURN NUMBER;
    PRAGMA RESTRICT_REFERENCES(direct_user_role_check, WNDS, RNPS, WNPS);
    function org_channel_setting(channel_id_in in number, org_id_in in number, setting_in in varchar2)
    	RETURN NUMBER;
    PRAGMA RESTRICT_REFERENCES(org_channel_setting, WNDS, RNPS, WNPS);
END rhn_channel;
/

SHOW ERRORS;

--
-- 231. RHN_ENTITLEMENTS  (Package) 
--
--
-- RHN_ENTITLEMENTS  (Package) 
--
--  Dependencies: 
--   DUAL ()
--   IS_SATELLITE (Function)
--   PLITBLM ()
--   RHNCHANNEL (Table)
--   RHNCHANNELFAMILY (Table)
--   RHNCHANNELFAMILYMEMBERS (Table)
--   RHNORGCHANNELFAMILYPERMISSIONS (View)
--   RHNORGENTITLEMENTS (Table)
--   RHNORGENTITLEMENTTYPE (Table)
--   RHNPRIVATECHANNELFAMILY (Table)
--   RHNPUBLICCHANNELFAMILY (Table)
--   RHNSERVER (Table)
--   RHNSERVERCHANNEL (Table)
--   RHNSERVERGROUP (Table)
--   RHNSERVERGROUPMEMBERS (Table)
--   RHNSERVERGROUPTYPE (Table)
--   RHNSERVERSERVERGROUPARCHCOMPAT (Table)
--   RHNUSERGROUP (Table)
--   RHNUSERGROUPMEMBERS (Table)
--   RHNUSERGROUPTYPE (Table)
--   RHN_CHANNEL (Package)
--   RHN_EXCEPTION (Package)
--   RHN_SERVER (Package)
--   RHN_SERVER_GROUP_ID_SEQ (Sequence)
--   RHN_USER (Package)
--   RHN_USER_GROUP_ID_SEQ (Sequence)
--   WEB_CUSTOMER (Table)
--   DUAL (Synonym)
--   PLITBLM (Synonym)
--   DUAL (Table)
--   PLITBLM (Package)
--   STANDARD (Package)
--   STANDARD (Package)
--
CREATE OR REPLACE package rhn_entitlements
is
	body_version varchar2(100) := '$Id: satellite-3.4-to-3.6.sql 64427 2005-09-09 14:08:21Z jslagle $';
    function entitlement_grants_service (
	    entitlement_in in varchar2,
		service_level_in in varchar2
	) return number;
	function lookup_entitlement_group (
		org_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) return number;
	function create_entitlement_group (
		org_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) return number;
	procedure entitle_server (
		server_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	);
	procedure unentitle_server (
		server_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	);
	procedure upgrade_server (
		server_id_in in number
	);
	function get_server_entitlement (
		server_id_in in number
	) return varchar2;
	procedure modify_org_service (
		org_id_in in number,
		service_label_in in varchar2,
		enable_in in char
	);
    procedure set_customer_enterprise (
		customer_id_in in number
	);
	procedure set_customer_provisioning (
		customer_id_in in number
	);
	procedure set_customer_nonlinux (
		customer_id_in in number
	);
    procedure unset_customer_enterprise (
		customer_id_in in number
	);
	procedure unset_customer_provisioning (
		customer_id_in in number
	);
	procedure unset_customer_nonlinux (
		customer_id_in in number
	);
    procedure set_group_count (
		customer_id_in in number,	-- customer_id
		type_in in char,			-- 'U' or 'S'
		group_type_in in number,	-- rhn[User|Server]GroupType.id
		quantity_in in number		-- quantity
    );
    procedure set_family_count (
		customer_id_in in number,		-- customer_id
		channel_family_id_in in number,	-- 246
		quantity_in in number			-- 3
    );
    procedure entitle_last_modified_servers (
		customer_id_in in number,	-- customer_id
		type_label_in in varchar2,	-- 'enterprise_entitled'
		quantity_in in number		-- 3
    );
	procedure get_available_slots (
		customer_id_in in number,
		type_label_in in varchar2,
		server_group_id_out out number,
		available_out out number,
		upgrade_out out number
	);
	procedure prune_everything (
		customer_id_in in number
	);
	procedure subscribe_newest_servers (
		customer_id_in in number
	);
end rhn_entitlements;
/

SHOW ERRORS;

--
-- 232. RHN_ORG  (Package) 
--
--
-- RHN_ORG  (Package) 
--
--  Dependencies: 
--   DELETE_SERVER (Procedure)
--   LOOKUP_SNAPSHOT_INVALID_REASON (Function)
--   RHNMESSAGE (Table)
--   RHNREGTOKEN (Table)
--   RHNSERVER (Table)
--   RHNSERVERGROUP (Table)
--   RHNSERVERGROUP (Table)
--   RHNSERVERGROUPMEMBERS (Table)
--   RHNSERVERGROUPTYPE (Table)
--   RHNSNAPSHOT (Table)
--   RHNSNAPSHOTSERVERGROUP (Table)
--   RHNUSERGROUP (Table)
--   RHNUSERGROUPMEMBERS (Table)
--   RHNUSERGROUPTYPE (Table)
--   RHNUSERMESSAGE (Table)
--   RHNUSERSERVERGROUPPERMS (Table)
--   RHN_CHECK_SUITES (Table)
--   RHN_COMMAND_QUEUE_SESSIONS (Table)
--   RHN_COMMAND_TARGET (Table)
--   RHN_CONTACT_GROUPS (Table)
--   RHN_CONTACT_METHODS (Table)
--   RHN_EXCEPTION (Package)
--   RHN_NOTIFICATION_FORMATS (Table)
--   RHN_PROBE (Table)
--   RHN_REDIRECTS (Table)
--   RHN_SAT_CLUSTER (Table)
--   RHN_SCHEDULES (Table)
--   RHN_SERVER (Package)
--   RHN_USER (Package)
--   WEB_CONTACT (Table)
--   WEB_CUSTOMER (Table)
--   XXRH_CONTACT_DEL_CHECK_NUM (Function)
--   STANDARD (Package)
--   STANDARD (Package)
--
CREATE OR REPLACE PACKAGE rhn_org
IS
	version varchar2(100) := '$Id: satellite-3.4-to-3.6.sql 64427 2005-09-09 14:08:21Z jslagle $';
    CURSOR server_group_by_label(org_id_in NUMBER, group_label_in VARCHAR2) IS
    	   SELECT SG.*
	     FROM rhnServerGroupType SGT,
	     	  rhnServerGroup SG
	    WHERE SG.group_type = SGT.id
	      AND SGT.label = group_label_in
	      AND SG.org_id = org_id_in;
    FUNCTION find_server_group_by_type(org_id_in NUMBER, group_label_in VARCHAR2) RETURN NUMBER;
	procedure delete_user(user_id_in number);
END rhn_org;
/

SHOW ERRORS;

--
-- 233. RHN_SERVER  (Package) 
--
--
-- RHN_SERVER  (Package) 
--
--  Dependencies: 
--   DUAL ()
--   LOOKUP_FIRST_MATCHING_CF (Function)
--   LOOKUP_PACKAGE_NEVRA (Function)
--   LOOKUP_TAG (Function)
--   PLITBLM ()
--   RHNACTION (Table)
--   RHNCONFIGCHANNEL (Table)
--   RHNCONFIGFILE (Table)
--   RHNCONFIGFILENAME (Table)
--   RHNCONFIGREVISION (Table)
--   RHNCUSTOMDATAKEY (Table)
--   RHNKICKSTARTSESSION (Table)
--   RHNKICKSTARTSESSIONSTATE (Table)
--   RHNPROXYINFO (Table)
--   RHNSATELLITEINFO (Table)
--   RHNSERVER (Table)
--   RHNSERVERACTION (Table)
--   RHNSERVERCHANNEL (Table)
--   RHNSERVERCONFIGCHANNEL (Table)
--   RHNSERVERCUSTOMDATAVALUE (Table)
--   RHNSERVERGROUP (Table)
--   RHNSERVERGROUPMEMBERS (Table)
--   RHNSERVERGROUPTYPE (Table)
--   RHNSERVERNETINTERFACE (Table)
--   RHNSERVERNETWORK (Table)
--   RHNSERVERPACKAGE (Table)
--   RHNSET (Table)
--   RHNSNAPSHOT (Table)
--   RHNSNAPSHOTCHANNEL (Table)
--   RHNSNAPSHOTCONFIGCHANNEL (Table)
--   RHNSNAPSHOTCONFIGREVISION (Table)
--   RHNSNAPSHOTPACKAGE (Table)
--   RHNSNAPSHOTSERVERGROUP (Table)
--   RHNSNAPSHOTTAG (Table)
--   RHNTAG (Table)
--   RHNUSERMANAGEDSERVERGROUPS (View)
--   RHNUSERSERVERGROUPPERMS (Table)
--   RHN_CACHE (Package)
--   RHN_ENTITLEMENTS (Package)
--   RHN_EXCEPTION (Package)
--   RHN_SET (Package)
--   RHN_SNAPSHOT_ID_SEQ (Sequence)
--   RHN_USER (Package)
--   SET_KS_SESSION_HISTORY_MESSAGE (Procedure)
--   WEB_CONTACT (Table)
--   DUAL (Synonym)
--   PLITBLM (Synonym)
--   DUAL (Table)
--   PLITBLM (Package)
--   STANDARD (Package)
--   STANDARD (Package)
--
CREATE OR REPLACE package rhn_server
is
    function system_service_level(
    	server_id_in in number,
	service_level_in in varchar2
    ) return number;
    function can_change_base_channel(
    	server_id_in in number
    ) return number;
    procedure set_custom_value(
    	server_id_in in number,
	user_id_in in number,
	key_label_in varchar2,
     	value_in in varchar2
    );
    procedure bulk_set_custom_value(
    	key_label_in in varchar2,
	value_in in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    );
    procedure snapshot_server(
    	server_id_in in number,
	reason_in in varchar2
    );
    procedure bulk_snapshot(
    	reason_in in varchar2,
    	set_label_in in varchar2,
	set_uid_in in number
    );
    procedure tag_delete(
    	server_id_in in number,
	tag_id_in in number
    );
    procedure tag_snapshot(
    	snapshot_id_in in number,
	org_id_in in number,
    	tagname_in in varchar2
    );
    procedure bulk_snapshot_tag(
    	org_id_in in number,
    	tagname_in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    );
    procedure remove_action(
	server_id_in in number,
	action_id_in in number
    );
    function check_user_access(server_id_in in number, user_id_in in number) return number;
    procedure insert_into_servergroup (
	server_id_in in number,
	server_group_id_in in number
    );
    function insert_into_servergroup_maybe (
	server_id_in in number,
	server_group_id_in in number
    ) return number;
	procedure insert_set_into_servergroup (
	server_group_id_in in number,
	user_id_in in number,
	set_label_in in varchar2
	);
    procedure delete_from_servergroup (
	server_id_in in number,
	server_group_id_in in number
    );
	procedure delete_set_from_servergroup (
	server_group_id_in in number,
	user_id_in in number,
	set_label_in in varchar2
	);
	procedure clear_servergroup (
	server_group_id_in in number
	);
	procedure delete_from_org_servergroups (
	server_id_in in number
	);
	function get_ip_address (
		server_id_in in number
	) return varchar2;
end rhn_server;
/

SHOW ERRORS;

--
-- 234. DELETE_SERVER  (Procedure) 
--
--
-- DELETE_SERVER  (Procedure) 
--
--  Dependencies: 
--   RHNACTIONCONFIGCHANNEL (Table)
--   RHNACTIONCONFIGREVISION (Table)
--   RHNACTIONPACKAGEREMOVALFAILURE (Table)
--   RHNAPPINSTALLSESSION (Table)
--   RHNCHANNELFAMILYLICENSECONSENT (Table)
--   RHNCLIENTCAPABILITY (Table)
--   RHNCONFIGCHANNEL (Table)
--   RHNCONFIGCHANNELTYPE (Table)
--   RHNCPU (Table)
--   RHNDEVICE (Table)
--   RHNFILELIST (Table)
--   RHNKICKSTARTPRESERVEFILELIST (Table)
--   RHNKICKSTARTSESSION (Table)
--   RHNPROXYINFO (Table)
--   RHNRAM (Table)
--   RHNREGTOKEN (Table)
--   RHNSATELLITECHANNELFAMILY (Table)
--   RHNSATELLITEINFO (Table)
--   RHNSERVER (Table)
--   RHNSERVERACTION (Table)
--   RHNSERVERACTIONPACKAGERESULT (Table)
--   RHNSERVERACTIONSCRIPTRESULT (Table)
--   RHNSERVERACTIONVERIFYMISSING (Table)
--   RHNSERVERACTIONVERIFYRESULT (Table)
--   RHNSERVERCHANNEL (Table)
--   RHNSERVERCONFIGCHANNEL (Table)
--   RHNSERVERCUSTOMDATAVALUE (Table)
--   RHNSERVERDMI (Table)
--   RHNSERVEREVENT (Table)
--   RHNSERVERGROUPMEMBERS (Table)
--   RHNSERVERHISTORY (Table)
--   RHNSERVERINFO (Table)
--   RHNSERVERINSTALLINFO (Table)
--   RHNSERVERLOCATION (Table)
--   RHNSERVERLOCK (Table)
--   RHNSERVERMESSAGE (Table)
--   RHNSERVERNEEDEDERRATACACHE (Table)
--   RHNSERVERNEEDEDPACKAGECACHE (Table)
--   RHNSERVERNETINTERFACE (Table)
--   RHNSERVERNETWORK (Table)
--   RHNSERVERNOTES (Table)
--   RHNSERVERPACKAGE (Table)
--   RHNSERVERPATH (Table)
--   RHNSERVERPRESERVEFILELIST (Table)
--   RHNSERVERTOKENREGS (Table)
--   RHNSERVERUUID (Table)
--   RHNSET (Table)
--   RHNSNAPSHOT (Table)
--   RHNSNAPSHOTTAG (Table)
--   RHNSNPSERVERQUEUE (Table)
--   RHNTRANSACTION (Table)
--   RHNUSERSERVERPERMS (Table)
--   RHNUSERSERVERPREFS (Table)
--   RHN_CHANNEL (Package)
--   RHN_CHECK_PROBE (Table)
--   RHN_CONFIG (Package)
--   RHN_HOST_PROBE (Table)
--   RHN_INTERFACE_MONITORING (Table)
--   RHN_SAT_CLUSTER (Table)
--   RHN_SAT_NODE (Table)
--   RHN_SERVER (Package)
--   RHN_SERVER_MONITORING_INFO (Table)
--   WEB_CONTACT (Table)
--   STANDARD (Package)
--   SYS_STUB_FOR_PURITY_ANALYSIS (Package)
--
CREATE OR REPLACE procedure delete_server (
	server_id_in in number
) is
	cursor servergroups is
		select	server_id, server_group_id
		from	rhnServerGroupMembers sgm
		where	sgm.server_id = server_id_in;
	cursor configchannels is
		select	cc.id
		from	rhnConfigChannel cc,
			rhnConfigChannelType cct,
			rhnServerConfigChannel scc
		where	1=1
			and scc.server_id = server_id_in
			and scc.config_channel_id = cc.id
			and cct.label in
				('local_override','server_import')
			and cct.id = cc.confchan_type_id;
	cursor filelists is
		select	spfl.file_list_id id
		from	rhnServerPreserveFileList spfl
		where	spfl.server_id = server_id_in
			and not exists (
				select	1
				from	rhnServerPreserveFileList
				where	file_list_id = spfl.file_list_id
					and server_id != server_id_in
				union
				select	1
				from	rhnKickstartPreserveFileList
				where	file_list_id = spfl.file_list_id
			);
	cursor nodes is
		select	n1.server_id
		from	rhn_sat_node n1,
			rhn_sat_node n0
		where	n0.server_id = server_id_in
			and n1.sat_cluster_id = n0.sat_cluster_id
			and n1.server_id != server_id_in
			and rownum = 1;
	other_cluster_nodes number := 0;
	cluster_id number;
begin
	rhn_channel.delete_server_channels(server_id_in);
	for filelist in filelists loop
		delete from rhnFileList where id = filelist.id;
	end loop;
	for configchannel in configchannels loop
		rhn_config.delete_channel(configchannel.id);
	end loop;
	for sgm in servergroups loop
		rhn_server.delete_from_servergroup(
			sgm.server_id, sgm.server_group_id);
	end loop;
	update rhnKickstartSession
		set old_server_id = null
		where old_server_id = server_id_in;
	update rhnKickstartSession
		set new_server_id = null
		where new_server_id = server_id_in;
	rhn_channel.clear_subscriptions(server_id_in);
	delete from rhnActionConfigChannel where server_id = server_id_in;
	delete from rhnActionConfigRevision where server_id = server_id_in;
	delete from rhnActionPackageRemovalFailure where server_id = server_id_in;
	delete from rhnChannelFamilyLicenseConsent where server_id = server_id_in;
	delete from rhnClientCapability where server_id = server_id_in;
	delete from rhnCpu where server_id = server_id_in;
	delete from rhnDevice where server_id = server_id_in;
	delete from rhnProxyInfo where server_id = server_id_in;
	delete from rhnRam where server_id = server_id_in;
	delete from rhnRegToken where server_id = server_id_in;
	delete from rhnSNPServerQueue where server_id = server_id_in;
	delete from rhnSatelliteChannelFamily where server_id = server_id_in;
	delete from rhnSatelliteInfo where server_id = server_id_in;
	delete from rhnServerAction where server_id = server_id_in;
	delete from rhnServerActionPackageResult where server_id = server_id_in;
	delete from rhnServerActionScriptResult where server_id = server_id_in;
	delete from rhnServerActionVerifyResult where server_id = server_id_in;
	delete from rhnServerActionVerifyMissing where server_id = server_id_in;
	delete from rhnServerChannel where server_id = server_id_in;
	delete from rhnServerConfigChannel where server_id = server_id_in;
	delete from rhnServerCustomDataValue where server_id = server_id_in;
	delete from rhnServerDMI where server_id = server_id_in;
	delete from rhnServerMessage where server_id = server_id_in;
	delete from rhnServerEvent where server_id = server_id_in;
	delete from rhnServerHistory where server_id = server_id_in;
	delete from rhnServerInfo where server_id = server_id_in;
	delete from rhnServerInstallInfo where server_id = server_id_in;
	delete from rhnServerLocation where server_id = server_id_in;
	delete from rhnServerLock where server_id = server_id_in;
	delete from rhnServerNeededPackageCache where server_id = server_id_in;
	delete from rhnServerNeededErrataCache where server_id = server_id_in;
	delete from rhnServerNetwork where server_id = server_id_in;
	delete from rhnServerNotes where server_id = server_id_in;
	delete from rhnServerPackage where server_id = server_id_in;
	delete from rhnServerTokenRegs where server_id = server_id_in;
	delete from rhnSnapshotTag where server_id = server_id_in;
	delete from rhnSnapshot where server_id = server_id_in;
	delete from rhnTransaction where server_id = server_id_in;
	delete from rhnUserServerPrefs where server_id = server_id_in;
	delete from rhnServerPath where server_id_in in (server_id, proxy_server_id);
	delete from rhnUserServerPerms where server_id = server_id_in;
	delete from rhn_check_probe where host_id = server_id_in;
	delete from rhn_host_probe where host_id = server_id_in;
	delete from rhn_interface_monitoring where server_id = server_id_in;
	delete from rhnServerNetInterface where server_id = server_id_in;
	delete from rhn_server_monitoring_info where recid = server_id_in;
	delete from rhnAppInstallSession where server_id = server_id_in;
	delete from rhnServerUuid where server_id = server_id_in;
	begin
		select sat_cluster_id into cluster_id
			from rhn_sat_node where server_id = server_id_in;
		for node in nodes loop
			other_cluster_nodes := 1;
		end loop;
		if other_cluster_nodes = 0 then
			delete from rhn_sat_cluster where recid = cluster_id;
		end if;
	exception when others then null; end;
	delete from rhn_sat_node where server_id = server_id_in;
	delete
	from	rhnServer
		where id = server_id_in;
	delete
	from	rhnSet
	where	1=1
		and user_id in (
			select	wc.id
			from	rhnServer rs,
				web_contact wc
			where	rs.id = server_id_in
				and rs.org_id = wc.org_id
		)
		and label = 'system_list'
		and element = server_id_in;
end delete_server;
/

SHOW ERRORS;

--
-- 235. DELETE_SERVER_BULK  (Procedure) 
--
--
-- DELETE_SERVER_BULK  (Procedure) 
--
--  Dependencies: 
--   RHNACTIONCONFIGCHANNEL (Table)
--   RHNACTIONCONFIGREVISION (Table)
--   RHNACTIONPACKAGEREMOVALFAILURE (Table)
--   RHNAPPINSTALLSESSION (Table)
--   RHNCHANNELFAMILYLICENSECONSENT (Table)
--   RHNCLIENTCAPABILITY (Table)
--   RHNCONFIGCHANNEL (Table)
--   RHNCONFIGCHANNELTYPE (Table)
--   RHNCPU (Table)
--   RHNDEVICE (Table)
--   RHNFILELIST (Table)
--   RHNKICKSTARTPRESERVEFILELIST (Table)
--   RHNKICKSTARTSESSION (Table)
--   RHNPROXYINFO (Table)
--   RHNRAM (Table)
--   RHNREGTOKEN (Table)
--   RHNSATELLITECHANNELFAMILY (Table)
--   RHNSATELLITEINFO (Table)
--   RHNSERVER (Table)
--   RHNSERVERACTION (Table)
--   RHNSERVERACTIONPACKAGERESULT (Table)
--   RHNSERVERACTIONSCRIPTRESULT (Table)
--   RHNSERVERACTIONVERIFYMISSING (Table)
--   RHNSERVERACTIONVERIFYRESULT (Table)
--   RHNSERVERCHANNEL (Table)
--   RHNSERVERCONFIGCHANNEL (Table)
--   RHNSERVERCUSTOMDATAVALUE (Table)
--   RHNSERVERDMI (Table)
--   RHNSERVEREVENT (Table)
--   RHNSERVERGROUPMEMBERS (Table)
--   RHNSERVERHISTORY (Table)
--   RHNSERVERINFO (Table)
--   RHNSERVERINSTALLINFO (Table)
--   RHNSERVERLOCATION (Table)
--   RHNSERVERLOCK (Table)
--   RHNSERVERMESSAGE (Table)
--   RHNSERVERNEEDEDERRATACACHE (Table)
--   RHNSERVERNEEDEDPACKAGECACHE (Table)
--   RHNSERVERNETINTERFACE (Table)
--   RHNSERVERNETWORK (Table)
--   RHNSERVERNOTES (Table)
--   RHNSERVERPACKAGE (Table)
--   RHNSERVERPATH (Table)
--   RHNSERVERPRESERVEFILELIST (Table)
--   RHNSERVERTOKENREGS (Table)
--   RHNSERVERUUID (Table)
--   RHNSET (Table)
--   RHNSNAPSHOT (Table)
--   RHNSNAPSHOTTAG (Table)
--   RHNSNPSERVERQUEUE (Table)
--   RHNTRANSACTION (Table)
--   RHNUSERSERVERPERMS (Table)
--   RHNUSERSERVERPREFS (Table)
--   RHN_CHANNEL (Package)
--   RHN_CHECK_PROBE (Table)
--   RHN_CONFIG (Package)
--   RHN_HOST_PROBE (Table)
--   RHN_INTERFACE_MONITORING (Table)
--   RHN_SAT_CLUSTER (Table)
--   RHN_SAT_NODE (Table)
--   RHN_SERVER (Package)
--   RHN_SERVER_MONITORING_INFO (Table)
--   WEB_CONTACT (Table)
--   STANDARD (Package)
--   SYS_STUB_FOR_PURITY_ANALYSIS (Package)
--
CREATE OR REPLACE procedure delete_server_bulk (
	user_id_in in number
) is
	cursor systems is
		select	s.element id
		from	rhnSet s
		where	s.user_id = user_id_in
			and s.label = 'system_list';
	cursor servergroups is
		select	sgm.server_group_id, sgm.server_id
		from	rhnServerGroupMembers sgm,
			rhnSet s
		where	s.user_id = user_id_in
			and s.label = 'system_list'
			and s.element = sgm.server_id;
	cursor configchannels is
		select	cc.id
		from	rhnConfigChannel cc,
			rhnConfigChannelType cct,
			rhnServerConfigChannel scc,
			rhnSet s
		where	1=1
			and s.user_id = user_id_in
			and s.label = 'system_list'
			and s.element = scc.server_id
			and scc.config_channel_id = cc.id
			and cct.label in
				('local_override','server_import')
			and cct.id = cc.confchan_type_id;
	cursor filelists is
		select	spfl.file_list_id id
		from	rhnServerPreserveFileList spfl,
			rhnSet s
		where	s.user_id = user_id_in
			and s.label = 'system_list'
			and s.element = spfl.server_id
			and not exists (
				select	1
				from	rhnKickstartPreserveFileList
				where	file_list_id = spfl.file_list_id
			)
			and not exists (
				select	spfl.server_id
				from	rhnServerPreserveFileList spfl,
					rhnServer s,
					web_contact u
				where	u.id = user_id_in
					and u.org_id = s.org_id
					and s.id = spfl.server_id
					and spfl.file_list_id = spfl.file_list_id
				minus
				select	element server_id
				from	rhnSet
				where	s.user_id = user_id_in
					and s.label = 'system_list'
			);
	cursor nodes(server_id_in in number) is
		select	n1.server_id
		from	rhn_sat_node n1,
			rhn_sat_node n0
		where	n0.server_id = server_id_in
			and n1.sat_cluster_id = n0.sat_cluster_id
			and n1.server_id != server_id_in
			and rownum = 1;
	other_cluster_nodes number := 0;
	cluster_id number;
begin
	for filelist in filelists loop
		delete from rhnFileList where id = filelist.id;
	end loop;
	for cc in configchannels loop
		rhn_config.delete_channel(cc.id);
	end loop;
	for sgm in servergroups loop
		rhn_server.delete_from_servergroup(
			sgm.server_id, sgm.server_group_id);
	end loop;
	for s in systems loop
		update rhnKickstartSession
			set old_server_id = null
			where old_server_id = s.id;
		update rhnKickstartSession
			set new_server_id = null
			where new_server_id = s.id;
		rhn_channel.clear_subscriptions(s.id);
		delete from rhnActionConfigChannel where server_id = s.id;
		delete from rhnActionConfigRevision where server_id = s.id;
		delete from rhnActionPackageRemovalFailure where server_id = s.id;
		delete from rhnChannelFamilyLicenseConsent where server_id = s.id;
		delete from rhnClientCapability where server_id = s.id;
		delete from rhnCpu where server_id = s.id;
		delete from rhnDevice where server_id = s.id;
		delete from rhnProxyInfo where server_id = s.id;
		delete from rhnRam where server_id = s.id;
		delete from rhnRegToken where server_id = s.id;
		delete from rhnSNPServerQueue where server_id = s.id;
		delete from rhnSatelliteChannelFamily where server_id = s.id;
		delete from rhnSatelliteInfo where server_id = s.id;
		delete from rhnServerAction where server_id = s.id;
		delete from rhnServerActionPackageResult where server_id = s.id;
		delete from rhnServerActionScriptResult where server_id = s.id;
		delete from rhnServerActionVerifyResult where server_id = s.id;
		delete from rhnServerActionVerifyMissing where server_id = s.id;
		delete from rhnServerChannel where server_id = s.id;
		delete from rhnServerConfigChannel where server_id = s.id;
		delete from rhnServerCustomDataValue where server_id = s.id;
		delete from rhnServerDMI where server_id = s.id;
		delete from rhnServerMessage where server_id = s.id;
		delete from rhnServerEvent where server_id = s.id;
		delete from rhnServerHistory where server_id = s.id;
		delete from rhnServerInfo where server_id = s.id;
		delete from rhnServerInstallInfo where server_id = s.id;
		delete from rhnServerLocation where server_id = s.id;
		delete from rhnServerLock where server_id = s.id;
		delete from rhnServerNeededPackageCache where server_id = s.id;
		delete from rhnServerNeededErrataCache where server_id = s.id;
		delete from rhnServerNetwork where server_id = s.id;
		delete from rhnServerNotes where server_id = s.id;
		delete from rhnServerPackage where server_id = s.id;
		delete from rhnServerTokenRegs where server_id = s.id;
		delete from rhnSnapshotTag where server_id = s.id;
		delete from rhnSnapshot where server_id = s.id;
		delete from rhnTransaction where server_id = s.id;
		delete from rhnUserServerPrefs where server_id = s.id;
		delete from rhnServerPath where s.id in (server_id, proxy_server_id);
		delete from rhnUserServerPerms where server_id = s.id;
		delete from rhn_check_probe where host_id = s.id;
		delete from rhn_host_probe where host_id = s.id;
		delete from rhn_interface_monitoring where server_id = s.id;
		delete from rhnServerNetInterface where server_id = s.id;
		delete from rhn_server_monitoring_info where recid = s.id;
		delete from rhnAppInstallSession where server_id = s.id;
		delete from rhnServerUuid where server_id = s.id;
		begin
			other_cluster_nodes := 0;
			select sat_cluster_id into cluster_id
				from rhn_sat_node where server_id = s.id;
			for node in nodes(s.id) loop
				other_cluster_nodes := 1;
			end loop;
			if other_cluster_nodes = 0 then
				delete from rhn_sat_cluster
					where recid = cluster_id;
			end if;
		exception when others then null; end;
	    	delete from rhn_sat_node where server_id = s.id;
		delete from rhnServer
			where id = s.id;
		delete from rhnSet
			where user_id in (
				select	wc.id
				from	rhnServer rs,
					web_contact wc
				where	rs.id = s.id
					and rs.org_id = wc.org_id
			)
			and label = 'system_list'
			and element = s.id;
	end loop;
end delete_server_bulk;
/

SHOW ERRORS;

--
-- 236. RHNSERVEROVERVIEW  (View) 
--
--
-- RHNSERVEROVERVIEW  (View) 
--
--  Dependencies: 
--   RHNCHANNEL (Table)
--   RHNPACKAGE (Table)
--   RHNSERVER (Table)
--   RHNSERVERACTION (Table)
--   RHNSERVERARCH (Table)
--   RHNSERVERCHANNEL (Table)
--   RHNSERVERERRATATYPEVIEW (View)
--   RHNSERVERHISTORY (Table)
--   RHNSERVERINFO (Table)
--   RHNSERVERLOCK (Table)
--   RHNSERVERNEEDEDPACKAGECACHE (Table)
--   RHNUSERSERVERPERMS (Table)
--   RHNVISIBLESERVERGROUPMEMBERS (View)
--   RHN_ENTITLEMENTS (Package)
--
CREATE OR REPLACE FORCE VIEW RHNSERVEROVERVIEW
(ORG_ID, SERVER_ID, SERVER_NAME, NOTE_COUNT, MODIFIED, 
 SERVER_ADMINS, GROUP_COUNT, CHANNEL_LABELS, HISTORY_COUNT, SECURITY_ERRATA, 
 BUG_ERRATA, ENHANCEMENT_ERRATA, OUTDATED_PACKAGES, ENTITLEMENT_LEVEL, LAST_CHECKIN_DAYS_AGO, 
 LAST_CHECKIN, PENDING_UPDATES, OS, RELEASE, SERVER_ARCH_NAME, 
 LOCKED)
AS 
select
    s.org_id, s.id, s.name, 0, s.modified,
    ( select count(user_id) from rhnUserServerPerms ap
      where server_id = s.id ),
    ( select count(server_group_id) from rhnVisibleServerGroupMembers
      where server_id = s.id ),
    NVL(( select C.name
        from rhnChannel C,
	     rhnServerChannel SC
       where SC.server_id = S.id
         and SC.channel_id = C.id
	 and C.parent_channel IS NULL), '(none)'),
    ( select count(id) from rhnServerHistory
      where
            server_id = S.id),
    ( select count(*) from rhnServerErrataTypeView sev
      where
            sev.server_id = s.id
        and sev.errata_type = 'Security Advisory'),
    ( select count(*) from rhnServerErrataTypeView sev
      where
            sev.server_id = s.id
        and sev.errata_type = 'Bug Fix Advisory'),
    ( select count(*) from rhnServerErrataTypeView sev
      where
            sev.server_id = s.id
        and sev.errata_type = 'Product Enhancement Advisory'),
    ( select count(distinct p.name_id) from rhnPackage p, rhnServerNeededPackageCache snpc
      where
             snpc.server_id = S.id
	 and p.id = snpc.package_id
	 ),
    rhn_entitlements.get_server_entitlement(S.id),
    ( select sysdate - checkin from rhnServerInfo where server_id = S.id ),
    ( select TO_CHAR(checkin, 'YYYY-MM-DD HH24:MI:SS') from rhnServerInfo where server_id = S.id ),
    ( select count(1)
        from rhnServerAction
       where server_id = S.id
         and status in (0, 1)),
    os,
    release,
    ( select name from rhnServerArch where id = s.server_arch_id),
    NVL((select 1 from rhnServerLock SL WHERE SL.server_id = S.id), 0)
from
    rhnServer S;

--
-- 237. RHN_HOST_MONITORING  (View) 
--
--
-- RHN_HOST_MONITORING  (View) 
--
--  Dependencies: 
--   RHNSERVER (Table)
--   RHN_SERVER (Package)
--
CREATE OR REPLACE FORCE VIEW RHN_HOST_MONITORING
(RECID, IP, NAME, DESCRIPTION, CUSTOMER_ID, 
 OS_ID, ASSET_ID, LAST_UPDATE_USER, LAST_UPDATE_DATE)
AS 
select  s.id            recid,
	rhn_server.get_ip_address(s.id)	ip,
        s.name          name,
        s.description   description,
        s.org_id        customer_id,
        '4'             os_id,
        null            asset_id,
        null            last_update_user,
        null            last_update_date
from	rhnServer	s;

--
-- 238. HOST  (Synonym) 
--
--
-- HOST  (Synonym) 
--
--  Dependencies: 
--   RHN_HOST_MONITORING (View)
--
CREATE SYNONYM HOST FOR RHN_HOST_MONITORING;

--
-- 239. RHN_KSTREE_LASTMOD_NN  (Constraint) 
--
-- ALTER TABLE RHNKICKSTARTABLETREE ADD CONSTRAINT RHN_KSTREE_LASTMOD_NN CHECK ("LAST_MODIFIED" IS NOT NULL);
-- rhn: do not do this as the check constraint was part of step 112.

--
-- 240. RHN_UG_OID_GT_UQ  (Constraint) 
--
ALTER TABLE RHNUSERGROUP ADD CONSTRAINT RHN_UG_OID_GT_UQ UNIQUE (ORG_ID, GROUP_TYPE);

--
-- 241. RHN_PROBE_DESCRIPTION_NN  (Constraint) 
--
ALTER TABLE RHN_PROBE ADD CONSTRAINT RHN_PROBE_DESCRIPTION_NN CHECK ("DESCRIPTION" IS NOT NULL);

--
-- 242. RHN_CDATAKEY_OID_LABEL_UQ  (Constraint) 
--
ALTER TABLE RHNCUSTOMDATAKEY ADD CONSTRAINT RHN_CDATAKEY_OID_LABEL_UQ UNIQUE (ORG_ID, LABEL);

--
-- 243. RHN_SNCP_OID_FK  (Constraint) 
--
ALTER TABLE RHNSERVERNEEDEDPACKAGECACHE DROP CONSTRAINT RHN_SNCP_OID_FK;
ALTER TABLE RHNSERVERNEEDEDPACKAGECACHE ADD CONSTRAINT RHN_SNCP_OID_FK FOREIGN KEY (ORG_ID) 
    REFERENCES WEB_CUSTOMER (ID);

--
-- 244. RHN_USER_INFO_TZID_FK  (Constraint) 
--
ALTER TABLE RHNUSERINFO ADD CONSTRAINT RHN_USER_INFO_TZID_FK FOREIGN KEY (TIMEZONE_ID) 
    REFERENCES RHNTIMEZONE (ID)
    ON DELETE CASCADE;

--
-- 245. RHN_SATND_SSK_NN  (Constraint) 
--
-- rhn: superseded by alter table statement
--ALTER TABLE RHN_SAT_NODE ADD CONSTRAINT RHN_SATND_SSK_NN CHECK ("SCOUT_SHARED_KEY" IS NOT NULL);

--
-- 246. RHN_SATND_SID_FK  (Constraint) 
--
ALTER TABLE RHN_SAT_NODE ADD CONSTRAINT RHN_SATND_SID_FK FOREIGN KEY (SERVER_ID) 
    REFERENCES RHNSERVER (ID);

--
-- 247. RHN_CLIENTCAP_CAP_NID_FK  (Constraint) 
--
ALTER TABLE RHNCLIENTCAPABILITY ADD CONSTRAINT RHN_CLIENTCAP_CAP_NID_FK FOREIGN KEY (CAPABILITY_NAME_ID) 
    REFERENCES RHNCLIENTCAPABILITYNAME (ID);

--
-- 248. RHN_SATELLITE_INFO_EID_FK  (Constraint) 
--
ALTER TABLE RHNSATELLITEINFO ADD CONSTRAINT RHN_SATELLITE_INFO_EID_FK FOREIGN KEY (EVR_ID) 
    REFERENCES RHNPACKAGEEVR (ID);

-- 249. through 254. superseded by previous table creation

--
-- 255. RHN_APPINST_INSTANCE_ID_PK  (Constraint) 
--
ALTER TABLE RHNAPPINSTALLINSTANCE ADD CONSTRAINT RHN_APPINST_INSTANCE_ID_PK PRIMARY KEY (ID);

--
-- 256. RHN_APPINST_INSTANCE_LV_UQ  (Constraint) 
--
ALTER TABLE RHNAPPINSTALLINSTANCE ADD CONSTRAINT RHN_APPINST_INSTANCE_LV_UQ UNIQUE (LABEL, VERSION);

-- 257. through 261. superseded by table creation

-- 262. through 269. superseded by create table

--
-- 270. RHN_PUSHDISPATCH_ID_PK  (Constraint) 
--
ALTER TABLE RHNPUSHDISPATCHER ADD CONSTRAINT RHN_PUSHDISPATCH_ID_PK PRIMARY KEY (ID);

--
-- 271. RHN_PUSHDISPATCH_JID_UQ  (Constraint) 
--
ALTER TABLE RHNPUSHDISPATCHER ADD CONSTRAINT RHN_PUSHDISPATCH_JID_UQ UNIQUE (JABBER_ID);

-- 272. through 286. replaced by previous rhnTimezone table declaration

--
-- 287. RHN_PCLIENT_ID_PK  (Constraint) 
--
-- rhn: superseded by table create

--
-- 289. RHN_KSSCRIPT_CHROOT_CK  (Constraint) 
--
ALTER TABLE RHNKICKSTARTSCRIPT ADD CONSTRAINT RHN_KSSCRIPT_CHROOT_CK CHECK (chroot in ('Y','N'));

--
-- 290. add check constraint
--

ALTER TABLE RHNKICKSTARTSCRIPT ADD CONSTRAINT RHN_KSSCRIPT_ST_CK CHECK (script_type in ('pre','post'));

-- 290. through 297. superseded

--
-- 298. RHN_KSSCRIPT_ID_PK  (Constraint) 
--
ALTER TABLE RHNKICKSTARTSCRIPT ADD CONSTRAINT RHN_KSSCRIPT_ID_PK PRIMARY KEY (ID);

--
-- 299. RHN_KSSCRIPT_KSID_POS_UQ  (Constraint) 
--
ALTER TABLE RHNKICKSTARTSCRIPT ADD CONSTRAINT RHN_KSSCRIPT_KSID_POS_UQ UNIQUE (KICKSTART_ID, POSITION);

--
-- 300. RHN_KSSCRIPT_KSID_FK  (Constraint) 
--
ALTER TABLE RHNKICKSTARTSCRIPT ADD CONSTRAINT RHN_KSSCRIPT_KSID_FK FOREIGN KEY (KICKSTART_ID) 
    REFERENCES RHNKSDATA (ID)
    ON DELETE CASCADE;

-- 301. through 308. superseded

--
-- 309. RHN_SACTIONVM_AID_FK  (Constraint) 
--
ALTER TABLE RHNSERVERACTIONVERIFYMISSING ADD CONSTRAINT RHN_SACTIONVM_AID_FK FOREIGN KEY (ACTION_ID) 
    REFERENCES RHNACTION (ID)
    ON DELETE CASCADE;

--
-- 310. RHN_SACTIONVM_PAID_FK  (Constraint) 
--
ALTER TABLE RHNSERVERACTIONVERIFYMISSING ADD CONSTRAINT RHN_SACTIONVM_PAID_FK FOREIGN KEY (PACKAGE_ARCH_ID) 
    REFERENCES RHNPACKAGEARCH (ID);

--
-- 311. RHN_SACTIONVM_PCID_FK  (Constraint) 
--
ALTER TABLE RHNSERVERACTIONVERIFYMISSING ADD CONSTRAINT RHN_SACTIONVM_PCID_FK FOREIGN KEY (PACKAGE_CAPABILITY_ID) 
    REFERENCES RHNPACKAGECAPABILITY (ID);

--
-- 312. RHN_SACTIONVM_PEID_FK  (Constraint) 
--
ALTER TABLE RHNSERVERACTIONVERIFYMISSING ADD CONSTRAINT RHN_SACTIONVM_PEID_FK FOREIGN KEY (PACKAGE_EVR_ID) 
    REFERENCES RHNPACKAGEEVR (ID);

--
-- 313. RHN_SACTIONVM_PNID_FK  (Constraint) 
--
ALTER TABLE RHNSERVERACTIONVERIFYMISSING ADD CONSTRAINT RHN_SACTIONVM_PNID_FK FOREIGN KEY (PACKAGE_NAME_ID) 
    REFERENCES RHNPACKAGENAME (ID);

--
-- 314. RHN_SACTIONVM_SID_FK  (Constraint) 
--
ALTER TABLE RHNSERVERACTIONVERIFYMISSING ADD CONSTRAINT RHN_SACTIONVM_SID_FK FOREIGN KEY (SERVER_ID) 
    REFERENCES RHNSERVER (ID);

-- 315. and 316. superseded

--
-- 317. RHN_SNEC_EID_FK  (Constraint) 
--
ALTER TABLE RHNSERVERNEEDEDERRATACACHE ADD CONSTRAINT RHN_SNEC_EID_FK FOREIGN KEY (ERRATA_ID) 
    REFERENCES RHNERRATA (ID)
    ON DELETE CASCADE;

--
-- 318. RHN_SNEC_OID_FK  (Constraint) 
--
ALTER TABLE RHNSERVERNEEDEDERRATACACHE ADD CONSTRAINT RHN_SNEC_OID_FK FOREIGN KEY (ORG_ID) 
    REFERENCES WEB_CUSTOMER (ID)
    ON DELETE CASCADE;

--
-- 319. RHN_SNEC_SID_FK  (Constraint) 
--
ALTER TABLE RHNSERVERNEEDEDERRATACACHE ADD CONSTRAINT RHN_SNEC_SID_FK FOREIGN KEY (SERVER_ID) 
    REFERENCES RHNSERVER (ID)
    ON DELETE CASCADE;

-- 320. and 321. superseded

--
-- 322. RHN_USPERMS_SID_FK  (Constraint) 
--
ALTER TABLE RHNUSERSERVERPERMS ADD CONSTRAINT RHN_USPERMS_SID_FK FOREIGN KEY (SERVER_ID) 
    REFERENCES RHNSERVER (ID);

--
-- 323. RHN_USPERMS_UID_FK  (Constraint) 
--
ALTER TABLE RHNUSERSERVERPERMS ADD CONSTRAINT RHN_USPERMS_UID_FK FOREIGN KEY (USER_ID) 
    REFERENCES WEB_CONTACT (ID);

-- 324. through 329. superseded

--
-- 330. RHN_APPINST_SESSIOND_ID_PK  (Constraint) 
--
ALTER TABLE RHNAPPINSTALLSESSION ADD CONSTRAINT RHN_APPINST_SESSIOND_ID_PK PRIMARY KEY (ID);

--
-- 331. RHN_APPINST_SESSION_IID_FK  (Constraint) 
--
ALTER TABLE RHNAPPINSTALLSESSION ADD CONSTRAINT RHN_APPINST_SESSION_IID_FK FOREIGN KEY (INSTANCE_ID) 
    REFERENCES RHNAPPINSTALLINSTANCE (ID)
    ON DELETE CASCADE;

--
-- 332. RHN_APPINST_SESSION_SID_FK  (Constraint) 
--
ALTER TABLE RHNAPPINSTALLSESSION ADD CONSTRAINT RHN_APPINST_SESSION_SID_FK FOREIGN KEY (SERVER_ID) 
    REFERENCES RHNSERVER (ID);

--
-- 333. RHN_APPINST_SESSION_UID_FK  (Constraint) 
--
ALTER TABLE RHNAPPINSTALLSESSION ADD CONSTRAINT RHN_APPINST_SESSION_UID_FK FOREIGN KEY (USER_ID) 
    REFERENCES WEB_CONTACT (ID);

-- 334. through 339. superseded

--
-- 339. RHN_APPINST_SDATA_ID_PK  (Constraint) 
--
ALTER TABLE RHNAPPINSTALLSESSIONDATA ADD CONSTRAINT RHN_APPINST_SDATA_ID_PK PRIMARY KEY (ID);

--
-- 340. RHN_APPINST_SDATA_SID_K_UQ  (Constraint) 
--
ALTER TABLE RHNAPPINSTALLSESSIONDATA ADD CONSTRAINT RHN_APPINST_SDATA_SID_K_UQ UNIQUE (SESSION_ID, KEY);

--
-- 341. RHN_APPINST_SDATA_SID_FK  (Constraint) 
--
ALTER TABLE RHNAPPINSTALLSESSIONDATA ADD CONSTRAINT RHN_APPINST_SDATA_SID_FK FOREIGN KEY (SESSION_ID) 
    REFERENCES RHNAPPINSTALLSESSION (ID)
    ON DELETE CASCADE;


----
---- RHN changes
----

insert into rhnUserServerPerms
(user_id, server_id)
select distinct user_id, server_id from rhnUserServerPermsDupes;
commit;

create or replace procedure create_new_roles is
    cursor orgs is
        select distinct
                c.id, c.name
        from    web_customer c,
                rhnOrgEntitlements oe,
                rhnOrgEntitlementType oet
        where   oet.label in ('sw_mgr_enterprise','rhn_monitor',
                        'rhn_provisioning')
                and oet.id = oe.entitlement_id
                and oe.org_id = c.id;
    cursor roles is
        select  id                               group_type,
                name||'s'                        name
        from    rhnUserGroupType
        where   label in ('system_group_admin','activation_key_admin');
    i number := 0;
begin
    for role in roles loop
        for org in orgs loop
            i := i + 1;
            if mod(i,5000) = 0 then
                commit;
            end if;
                                                                                                                                   
            insert into rhnUserGroup (
                    id, name, description, group_type, org_id
                ) values (
                    rhn_user_group_id_seq.nextval, role.name,
                    role.name || 'for Org '||org.name||' ('||org.id||')',
                    role.group_type, org.id
                );
        end loop;
    end loop;
    commit;
end create_new_roles;
/
show errors
                                                                                                                                   
begin create_new_roles();
end;
/
show errors
                                                                                                                                   
drop procedure create_new_roles;


CREATE OR REPLACE TRIGGER RHN_EFILEC_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNERRATAFILECHANNEL
FOR EACH ROW
begin
   :new.modified := sysdate;
end rhn_efilec_mod_trig;
/
SHOW ERRORS;

CREATE OR REPLACE TRIGGER RHN_KSTREEFILE_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNKSTREEFILE
FOR EACH ROW
begin
   :new.modified := sysdate;
   if :new.last_modified = :old.last_modified
   then
       :new.last_modified := sysdate;
        end if;
end rhn_kstreefile_mod_trig;
/
SHOW ERRORS;


 ----------------------------------------------

update rhnFAQ
set details =
'Red Hat Network is a systems support and management environment for Red Hat Linux systems and networks. For more information, please see the Red Hat Network product information page:
http://www.redhat.com/software/rhn/products/

For individual systems and small networks, see http://www.redhat.com/software/rhn

For enterprise deployments, see http://www.redhat.com/software/rhen'
where id = 1;
commit;

------------------------------------------------


insert into rhnUserGroupType (id, label, name) values (
	rhn_usergroup_type_seq.nextval,
	'monitoring_admin',
	'Monitoring Administrator'
);

insert into rhnUserGroupType (id, label, name) values (
	rhn_usergroup_type_seq.nextval,
	'system_group_admin',
	'System Group Administrator'
);

insert into rhnUserGroupType (id, label, name) values (
	rhn_usergroup_type_seq.nextval,
	'activation_key_admin',
	'Activation Key Administrator'
);

insert into rhnUserGroupType (id, label, name) values (
	rhn_usergroup_type_seq.nextval,
	'cert_admin',
	'Certificate Administrator'
);

insert into rhnException values (-20273, 'usgp_different_orgs', 'User and servergroup must be in same org in order to add this permission');
insert into rhnException values (-20274, 'usgp_already_allowed', 'The specified user already has permissions for this server group');
insert into rhnException values (-20275, 'usgp_not_allowed', 'The specified user does not have permissions for this server group');
insert into rhnException values (-20276, 'product_not_registered', 'The specified item is not registered to this customer');
insert into rhnException values (-20277, 'product_no_service', 'This product grants no services');
insert into rhnException values (-20278, 'webreg_duplicate', 'Registration number is already registered');
insert into rhnException values (-20279, 'webreg_not_active', 'Registration number is not active');
insert into rhnException values (-20280, 'webreg_sync_error', 'Error synchronizing, xxrh_oai_wrapper.sync_registration_uber');
insert into rhnException values (-20281, 'webreg_not_found', 'Registration number does not exist');
insert into rhnException values (-20282, 'webreg_unkown_error', 'Unknown error during web registration');

delete from rhnPackageSyncBlacklist;
insert into rhnPackageSyncBlacklist (package_name_id)
        values (lookup_package_name('gpg-pubkey'));
 
insert into rhnPackageSyncBlacklist (package_name_id)
        values (lookup_package_name('rhns-ca-cert'));
 
insert into rhnPackageSyncBlacklist (package_name_id)
        values (lookup_package_name('rhn-org-trusted-ssl-cert'));

insert into rhn_command_groups(group_name,description) 
    values ( 'bea_61','BEA Weblogic');
insert into rhn_command_groups(group_name,description) 
    values ( 'oracle','Oracle');
insert into rhn_command_groups(group_name,description) 
    values ( 'mysql','MySQL');
insert into rhn_command_groups(group_name,description) 
    values ( 'netservice','Network Services');
insert into rhn_command_groups(group_name,description) 
    values ( 'satellite','Satellite');
insert into rhn_command_groups(group_name,description) 
    values ( 'logagent','Log Agent');
insert into rhn_command_groups(group_name,description) 
    values ( 'all','All');
insert into rhn_command_groups(group_name,description) 
    values ( 'apache','Apache');
insert into rhn_command_groups(group_name,description) 
    values ( 'linux','Linux');
insert into rhn_command_groups(group_name,description) 
    values ( 'tools','General');

insert into rhn_command_class(class_name) 
    values ( 'Apache::Processes');
insert into rhn_command_class(class_name) 
    values ( 'Apache::Traffic');
insert into rhn_command_class(class_name) 
    values ( 'Apache::Uptime');
insert into rhn_command_class(class_name) 
    values ( 'Unix::CPU');
insert into rhn_command_class(class_name) 
    values ( 'Unix::Dig');
insert into rhn_command_class(class_name) 
    values ( 'Unix::Disk');
insert into rhn_command_class(class_name) 
    values ( 'Unix::DiskIOThroughput');
insert into rhn_command_class(class_name) 
    values ( 'NetworkService::FTP');
insert into rhn_command_class(class_name) 
    values ( 'NetworkService::HTTP');
insert into rhn_command_class(class_name) 
    values ( 'NetworkService::HTTPS');
insert into rhn_command_class(class_name) 
    values ( 'NetworkService::IMAP');
insert into rhn_command_class(class_name) 
    values ( 'Unix::Process');
insert into rhn_command_class(class_name) 
    values ( 'Unix::Inodes');
insert into rhn_command_class(class_name) 
    values ( 'Unix::Load');
insert into rhn_command_class(class_name) 
    values ( 'LogAgent::PatternMatch');
insert into rhn_command_class(class_name) 
    values ( 'LogAgent::Size');
insert into rhn_command_class(class_name) 
    values ( 'Satellite::LongLegs');
insert into rhn_command_class(class_name) 
    values ( 'Unix::MemoryFree');
insert into rhn_command_class(class_name) 
    values ( 'MySQL::Accessibility');
insert into rhn_command_class(class_name) 
    values ( 'MySQL::Open');
insert into rhn_command_class(class_name) 
    values ( 'MySQL::Opened');
insert into rhn_command_class(class_name) 
    values ( 'MySQL::Queries');
insert into rhn_command_class(class_name) 
    values ( 'MySQL::Threads');
insert into rhn_command_class(class_name) 
    values ( 'General::SNMPCheck');
insert into rhn_command_class(class_name) 
    values ( 'General::UptimeSNMP');
insert into rhn_command_class(class_name) 
    values ( 'Unix::InterfaceTraffic');
insert into rhn_command_class(class_name) 
    values ( 'General::SNMPTrapParser');
insert into rhn_command_class(class_name) 
    values ( 'Satellite::ProbeCount');
insert into rhn_command_class(class_name) 
    values ( 'Satellite::ProbeExecTime');
insert into rhn_command_class(class_name) 
    values ( 'Satellite::ProbeLatency');
insert into rhn_command_class(class_name) 
    values ( 'General::CheckNothing');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::TablespaceUsage');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::BlockingSessions');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::IndexExtents');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::TableExtents');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::IdleSessions');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::Availability');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::Locks');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::ActiveSessions');
insert into rhn_command_class(class_name) 
    values ( 'NetworkService::POP');
insert into rhn_command_class(class_name) 
    values ( 'NetworkService::Ping');
insert into rhn_command_class(class_name) 
    values ( 'Unix::ProcessStateCounts');
insert into rhn_command_class(class_name) 
    values ( 'Unix::ProcessCountTotal');
insert into rhn_command_class(class_name) 
    values ( 'Satellite::CurrentStatePush');
insert into rhn_command_class(class_name) 
    values ( 'NetworkService::RPC');
insert into rhn_command_class(class_name) 
    values ( 'NetworkService::SMTP');
insert into rhn_command_class(class_name) 
    values ( 'NetworkService::SSH');
insert into rhn_command_class(class_name) 
    values ( 'Unix::Swap');
insert into rhn_command_class(class_name) 
    values ( 'General::TCP');
insert into rhn_command_class(class_name) 
    values ( 'Unix::TCPConnectionsByState');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::TNSping');
insert into rhn_command_class(class_name) 
    values ( 'General::RemoteProgram');
insert into rhn_command_class(class_name) 
    values ( 'Unix::Users');
insert into rhn_command_class(class_name) 
    values ( 'Unix::VirtualMemory');
insert into rhn_command_class(class_name) 
    values ( 'General::RemoteProgramWithData');
insert into rhn_command_class(class_name) 
    values ( 'Weblogic::State');
insert into rhn_command_class(class_name) 
    values ( 'Weblogic::HeapFree');
insert into rhn_command_class(class_name) 
    values ( 'Windows::LoadAverage');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::BufferCache');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::DataDictionaryCache');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::LibraryCache');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::RedoLog');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::DiskSort');
insert into rhn_command_class(class_name) 
    values ( 'Weblogic::JDBCConnectionPool');
insert into rhn_command_class(class_name) 
    values ( 'Weblogic::ExecuteQueue');
insert into rhn_command_class(class_name) 
    values ( 'Weblogic::Servlet');
insert into rhn_command_class(class_name) 
    values ( 'Oracle::ClientConnectivity');
insert into rhn_command_class(class_name) 
    values ( 'Unix::ProcessRunning');

insert into rhn_command_requirements(name,description) 
    values (
'npunix','RHNMD must be running on the monitored host to perform this check.');

insert into rhn_command_requirements(name,description) 
    values (
'oracle','Oracle checks require the configuration of the database and public associations made by running $ORACLE_HOME/rdbms/admin/catalog.sql. In addition, in any instance of Oracle, the Oracle user configured in the check must have minimum privileges of CONNECT and SELECT_CATALOG_ROLE.');

insert into rhn_command_requirements(name,description) 
    values (
'snmp','SNMP must be running on the monitored host to perform this check.');

insert into rhn_command_requirements(name,description) 
    values (
'bea61_snmp','This check may be configured against a BEA Managed Server if the IP address of the Domain Admin Server is given.SNMP must also be enabled on the Domain Admin Server which can be done via the Weblogic Console.');

insert into rhn_command_requirements(name,description) 
    values (
'npunix_oracle','RHNMD must be running on the monitored host to perform this check. Oracle checks require the configuration of the database and public associations made by running $ORACLE_HOME/rdbms/admin/catalog.sql. In addition, in any instance of Oracle, the Oracle user configured in the check must have minimum privileges of CONNECT and SELECT_CATALOG_ROLE.');

insert into rhn_command_requirements(name,description) 
    values (
'apache1_3','The ExtendedStatus directive in the httpd.conf file of this webserver must be set to On for this check to function properly.');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
2,'LongLegs','LongLegs',NULL,'0','Satellite::LongLegs','0','0','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
6,'check_tcp','TCP Check','tools','1','General::TCP','1','0','system',sysdate,NULL,NULL,'/help/userguides/user_guide_v2/Output/toolsCheck.html#TCPCheck');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
7,'check_udp','UDP Check','tools','1','General::TCP','1','0','system',sysdate,NULL,NULL,'/help/userguides/user_guide_v2/Output/toolsCheck.html#UDPCheck');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
8,'check_smtp','Mail Transfer (SMTP)','netservice','1','NetworkService::SMTP','1','0','system',sysdate,NULL,NULL,'/help/userguides/user_guide_v2/Output/NetworkServiceCheck.html#MailTransfer(SMTP)');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
9,'check_pop','POP Mail','netservice','1','NetworkService::POP','1','0','system',sysdate,NULL,NULL,'/help/userguides/user_guide_v2/Output/NetworkServiceCheck.html#POPMail');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
10,'check_ftp','FTP','netservice','1','NetworkService::FTP','1','0','system',sysdate,NULL,NULL,'/help/userguides/user_guide_v2/Output/NetworkServiceCheck.html#FTP');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
11,'check_http','Web server (HTTP)','netservice','1','NetworkService::HTTP','1','0','system',sysdate,NULL,NULL,'/help/userguides/user_guide_v2/Output/NetworkServiceCheck.html#WebServer(HTTP)');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
13,'check_ping','Ping','netservice','1','NetworkService::Ping','1','0','system',sysdate,NULL,NULL,'/help/userguides/user_guide_v2/Output/NetworkServiceCheck.html#Ping');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
14,'check_dns_resolution','DNS Lookup','netservice','1','Unix::Dig','1','0','system',sysdate,NULL,NULL,'/help/userguides/user_guide_v2/Output/NetworkServiceCheck.html#DNSLookup');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
15,'check_ssh','SSH','netservice','1','NetworkService::SSH','1','0','system',sysdate,NULL,NULL,'/help/userguides/user_guide_v2/Output/NetworkServiceCheck.html#SSH');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
16,'check_snmp_value','SNMP Check','tools','1','General::SNMPCheck','1','0','system',sysdate,'snmp',NULL,'/help/userguides/user_guide_v2/Output/toolsCheck.html#SNMPCheck');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
17,'satellite_disk_space','Disk Space','satellite','0','Unix::Disk','1','0','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
18,'satellite_check_users','Users','satellite','0','Unix::Users','1','0','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
19,'satellite_check_procs','Processes','satellite','0','Unix::ProcessCountTotal','1','0','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
20,'satellite_check_load','Load','satellite','0','Unix::Load','1','0','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
22,'satellite_check_swap','Swap','satellite','0','Unix::Swap','1','0','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
23,'check_tnsping','TNS Ping','oracle','1','Oracle::TNSping','1','0','system',sysdate,NULL,NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#TNSPing');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
24,'satellite_probe_latency','Latency','satellite','0','Satellite::ProbeLatency','1','0','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
25,'remote_check_memory','Memory Usage','linux','1','Unix::MemoryFree','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#MemoryUsage');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
26,'remote_check_cpu','CPU Usage','linux','1','Unix::CPU','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#CPUUsage');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
27,'remote_check_load','Load','linux','1','Unix::Load','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#UnixLoad');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
28,'remote_check_swap','Swap Usage','linux','1','Unix::Swap','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#SwapUsage');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
29,'remote_check_disk','Disk Usage','linux','1','Unix::Disk','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#DiskUsage');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
30,'remote_check_users','Users','linux','1','Unix::Users','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#Users');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
31,'remote_check_procs','Process Count Total','linux','1','Unix::ProcessCountTotal','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#Processes');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
32,'check_host_alive','Host Availability',NULL,'0','NetworkService::Ping','1','1','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
41,'check_nothing','Check Nothing','tools','1','General::CheckNothing','1','0','system',sysdate,NULL,NULL,'/help/userguides/user_guide_v2/Output/toolsCheck.html#CheckNothing');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
42,'check_imap','IMAP Mail','netservice','1','NetworkService::IMAP','1','0','system',sysdate,NULL,NULL,'/help/userguides/user_guide_v2/Output/NetworkServiceCheck.html#IMAPMail');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
48,'check_rpc','RPC Service','netservice','1','NetworkService::RPC','1','0','system',sysdate,NULL,NULL,'/help/userguides/user_guide_v2/Output/NetworkServiceCheck.html#RPCService');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
56,'check_https','Secure Web server (HTTPS)','netservice','1','NetworkService::HTTPS','1','0','system',sysdate,NULL,NULL,'/help/userguides/user_guide_v2/Output/NetworkServiceCheck.html#SecureWebServer(HTTPS)');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
58,'icmp_blocked','ICMP Blocked',NULL,'0','General::CheckNothing','1','1','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
60,'satellite_probe_exec_time','Execution Time','satellite','0','Satellite::ProbeExecTime','1','0','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
61,'satellite_probe_count','Probe Count','satellite','0','Satellite::ProbeCount','1','0','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
85,'check_ora_blocking_sessions','Blocking Sessions','oracle','1','Oracle::BlockingSessions','1','0','system',sysdate,'oracle',NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#BlockingSessions');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
86,'check_ora_locks','Locks','oracle','1','Oracle::Locks','1','0','system',sysdate,'oracle',NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#Locks');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
87,'check_ora_all_extents','Tablespace Usage','oracle','1','Oracle::TablespaceUsage','1','0','system',sysdate,'oracle',NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#AllExtents');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
88,'check_ora_table_extents','Table Extents','oracle','1','Oracle::TableExtents','1','0','system',sysdate,'oracle',NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#TableExtents');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
89,'check_ora_index_extents','Index Extents','oracle','1','Oracle::IndexExtents','1','0','system',sysdate,'oracle',NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#IndexExtents');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
90,'check_ora_idle_sessions','Idle Sessions','oracle','1','Oracle::IdleSessions','1','0','system',sysdate,'oracle',NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#IdleSessions');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
91,'check_ora_up','Availability','oracle','1','Oracle::Availability','1','0','system',sysdate,'oracle',NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#Availability');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
92,'check_ora_sessions','Active Sessions','oracle','1','Oracle::ActiveSessions','1','0','system',sysdate,'oracle',NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#ActiveSessions');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
93,'check_mysql_open_tables','Open Tables','mysql','1','MySQL::Open','1','0','system',sysdate,NULL,'Version 3.23/3.33','/help/userguides/user_guide_v2/Output/MySQLCheck.html#OpenTables');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
94,'check_mysql_opened_tables','Opened Tables','mysql','1','MySQL::Opened','1','0','system',sysdate,NULL,'Version 3.23/3.33','/help/userguides/user_guide_v2/Output/MySQLCheck.html#OpenedTables');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
95,'check_mysql_threads_running','Threads Running','mysql','1','MySQL::Threads','1','0','system',sysdate,NULL,'Version 3.23/3.33','/help/userguides/user_guide_v2/Output/MySQLCheck.html#ThreadsRunning');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
96,'check_mysql_queries_per_second','Query Rate','mysql','1','MySQL::Queries','1','0','system',sysdate,NULL,'Version 3.23/3.33','/help/userguides/user_guide_v2/Output/MySQLCheck.html#QueryRate');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
97,'check_mysql','Database Accessibility','mysql','1','MySQL::Accessibility','1','0','system',sysdate,NULL,'Version 3.23/3.33','/help/userguides/user_guide_v2/Output/MySQLCheck.html#DatabaseAccessibility');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
99,'remote_check_program','Remote Program','tools','1','General::RemoteProgram','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/toolsCheck.html#RemoteProgram');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
105,'remote_check_interface_traffic','Interface Traffic','linux','1','Unix::InterfaceTraffic','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#UnixInterfaceTraffic');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
106,'check_log_size','Log Size','logagent','1','LogAgent::Size','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/LogAgentCheck.html#LogSize');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
107,'check_log_regex','Log Pattern Match','logagent','1','LogAgent::PatternMatch','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/LogAgentCheck.html#LogRegex');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
109,'remote_check_ora_up','Client Connectivity','oracle','1','Oracle::ClientConnectivity','1','0','system',sysdate,'npunix_oracle',NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#RemoteAvailability');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
113,'check_snmp_uptime','Uptime (SNMP)','tools','1','General::UptimeSNMP','1','0','system',sysdate,'snmp',NULL,'/help/userguides/user_guide_v2/Output/toolsCheck.html#Uptime(SNMP)');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
117,'remote_check_inodes','Inodes','linux','1','Unix::Inodes','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#Inodes');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
118,'remote_check_disk_io','Disk I/O Throughput','linux','1','Unix::DiskIOThroughput','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#DiskIOThroughput');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
120,'check_apache_uptime','Uptime','apache','1','Apache::Uptime','1','0','system',sysdate,NULL,'Apache versions 1.3.x and 2.0.x','/help/userguides/user_guide_v2/Output/ApacheCheck.html#Uptime');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
121,'check_apache_traffic','Requests','apache','1','Apache::Traffic','1','0','system',sysdate,'apache1_3','Apache versions 1.3.x and 2.0.x','/help/userguides/user_guide_v2/Output/ApacheCheck.html#Requests');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
122,'check_apache_procs','Processes','apache','1','Apache::Processes','1','0','system',sysdate,'apache1_3','Apache versions 1.3.x and 2.0.x','/help/userguides/user_guide_v2/Output/ApacheCheck.html#Processes');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
123,'check_virtual_memory','Virtual Memory','linux','1','Unix::VirtualMemory','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#VirtualMemory');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
226,'remote_process_running','Process Running','linux','1','Unix::ProcessRunning','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#ProcessRunning');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
227,'satellite_process_running','Process Running','satellite','0','Unix::ProcessRunning','1','0','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
228,'remote_process_health','Process Health','linux','1','Unix::Process','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#ProcessHealth');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
229,'satellite_process_health','Process Health','satellite','0','Unix::Process','1','0','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
230,'remote_process_counts','Process Counts by State','linux','1','Unix::ProcessStateCounts','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#ProcessCounts');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
231,'satellite_process_counts','Process Counts','satellite','0','Unix::ProcessStateCounts','1','0','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
249,'remote_check_tcp_connections_state','TCP Connections by State','linux','1','Unix::TCPConnectionsByState','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/UnixCheck.html#TCPConnections State');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
254,'satellite_check_traffic','Interface Traffic','satellite','0','Unix::InterfaceTraffic','1','0','system',sysdate,NULL,NULL,NULL);

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
274,'remote_check_program_ts','Remote Program with Data','tools','1','General::RemoteProgramWithData','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/toolsCheck.html#RemoteProgramData');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
275,'check_bea_61_state','Server State','bea_61','1','Weblogic::State','1','0','system',sysdate,'bea61_snmp','Version 6.1 or higher','/help/userguides/user_guide_v2/Output/BEAWebLogic61.html#ServerState61');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
276,'check_bea_61_heap_free','Heap Free','bea_61','1','Weblogic::HeapFree','1','0','system',sysdate,'bea61_snmp','Version 6.1 or higher','/help/userguides/user_guide_v2/Output/BEAWebLogic61.html#HeapFree');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
278,'check_ora_buffer_cache','Buffer Cache','oracle','1','Oracle::BufferCache','1','0','system',sysdate,'oracle',NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#BufferCache');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
279,'check_ora_data_dictionary_cache','Data Dictionary Cache','oracle','1','Oracle::DataDictionaryCache','1','0','system',sysdate,'oracle',NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#DataDictionaryCache');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
280,'check_ora_library_cache','Library Cache','oracle','1','Oracle::LibraryCache','1','0','system',sysdate,'oracle',NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#LibraryCache');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
284,'check_ora_redo_log','Redo Log','oracle','1','Oracle::RedoLog','1','0','system',sysdate,'oracle',NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#RedoLog');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
283,'check_ora_disk_sort','Disk Sort Ratio','oracle','1','Oracle::DiskSort','1','0','system',sysdate,'oracle',NULL,'/help/userguides/user_guide_v2/Output/OracleCheck.html#DiskSortRatio');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
282,'check_bea_61_jdbc_conn_pool','JDBC Connection Pool','bea_61','1','Weblogic::JDBCConnectionPool','1','0','system',sysdate,'bea61_snmp','Version 6.1 or higher','/help/userguides/user_guide_v2/Output/BEAWebLogic61.html#JDBCConnectionPool');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
281,'check_bea_61_execute_queue','Execute Queue','bea_61','1','Weblogic::ExecuteQueue','1','0','system',sysdate,'bea61_snmp','Version 6.1 or higher','/help/userguides/user_guide_v2/Output/BEAWebLogic61.html#ExecuteQueue');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
288,'check_bea_61_servlet','Servlet','bea_61','1','Weblogic::Servlet','1','0','system',sysdate,'bea61_snmp','Version 6.1 or higher','/help/userguides/user_guide_v2/Output/BEAWebLogic61.html#Servlet');

insert into rhn_command(recid,name,description,group_name,allowed_in_suite,
command_class,enabled,for_host_probe,last_update_user,last_update_date,system_requirements,version_support,help_url) 
    values (
304,'remote_check_ping','Remote Ping','netservice','1','NetworkService::Ping','1','0','system',sysdate,'npunix',NULL,'/help/userguides/user_guide_v2/Output/NetworkServiceCheck.html#RemotePing');

insert into rhn_threshold_type(name,description,ordinal,last_update_user,
last_update_date) 
    values ( 'warn_min','Minimum',10,'system',sysdate);
insert into rhn_threshold_type(name,description,ordinal,last_update_user,
last_update_date) 
    values ( 'warn_max','Maximum',20,'system',sysdate);
insert into rhn_threshold_type(name,description,ordinal,last_update_user,
last_update_date) 
    values ( 'crit_min','Minimum',0,'system',sysdate);
insert into rhn_threshold_type(name,description,ordinal,last_update_user,
last_update_date) 
    values ( 'crit_max','Maximum',30,'system',sysdate);

insert into rhn_quanta(quantum_id,basic_unit_id,description,last_update_user,
last_update_date) 
    values ( 'none','-','No physical quantity',
    'system',sysdate);
insert into rhn_quanta(quantum_id,basic_unit_id,description,last_update_user,
last_update_date) 
    values ( 'time','secs','Measure of time',
    'system',sysdate);
insert into rhn_quanta(quantum_id,basic_unit_id,description,last_update_user,
last_update_date) 
    values ( 'data','bytes','Measure of information',
    'system',sysdate);
insert into rhn_quanta(quantum_id,basic_unit_id,description,last_update_user,
last_update_date) 
    values ( 'datarate','Bps','Measure of information flow',
    'system',sysdate);
insert into rhn_quanta(quantum_id,basic_unit_id,description,last_update_user,
last_update_date) 
    values ( 'frequency','hertz','Measure of freqency',
    'system',sysdate);
insert into rhn_quanta(quantum_id,basic_unit_id,description,last_update_user,
last_update_date) 
    values ( 'temp','kelvins','Measure of temperature',
    'system',sysdate);
insert into rhn_quanta(quantum_id,basic_unit_id,description,last_update_user,
last_update_date) 
    values ( 'length','metres','Measure of length',
    'system',sysdate);
insert into rhn_quanta(quantum_id,basic_unit_id,description,last_update_user,
last_update_date) 
    values ( 'mass','kilograms','Measure of mass',
    'system',sysdate);
insert into rhn_quanta(quantum_id,basic_unit_id,description,last_update_user,
last_update_date) 
    values ( 'current','amperes','Measure of electric current',
    'system',sysdate);
insert into rhn_quanta(quantum_id,basic_unit_id,description,last_update_user,
last_update_date) 
    values ( 'force','newtons','Measure of force/weight',
    'system',sysdate);
insert into rhn_quanta(quantum_id,basic_unit_id,description,last_update_user,
last_update_date) 
    values ( 'power','watts','Measure of power',
    'system',sysdate);
insert into rhn_quanta(quantum_id,basic_unit_id,description,last_update_user,
last_update_date) 
    values ( 'charge','coulombs','Measure of charge',
    'system',sysdate);
insert into rhn_quanta(quantum_id,basic_unit_id,description,last_update_user,
last_update_date) 
    values ( 'voltage','volts','Measure of electric potential difference',
    'system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('secs','time','sec','Seconds',
    'x','x','is_number(x)','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('percent','none','%','Percentage',
    'x','x','is_number(x)  and 0 <= x <= 100','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('bits','data','B','Bits',
    'x/8','x*8','is_integer(x) and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('count','none',NULL,'Number',
    'x','x','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('astring','none',NULL,'ASCII string',
    'x','x','is_ascii(x)','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('Bps','datarate','bits/sec','Bits per second',
    'x','x','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('hertz','frequency','hz','Cycles per second',
    'x','x','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('kelvins','temp','K','Degrees Kelvin',
    'x','x','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('kilograms','mass','kg','Kilograms',
    'x','x','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('amperes','current','A','Amperes',
    'x','x','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('newtons','force','N','Newtons',
    'x','x','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('watts','power','W','Watts',
    'x','x','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('coulombs','charge','C','Coulombs',
    'x','x','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('volts','voltage','V','Volts',
    'x','x','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('rpm','frequency','rpm','Revolutions per minute',
    'x*60','x/60','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('MHz','frequency','MHz','Megahertz',
    'x*1024*1024','x/1024/1024','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('bps','datarate','bytes/sec','Bytes per second',
    'x*8','x/8','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('kBps','datarate','Kbits/sec','Kilobits per second',
    'x*1024','x/1024','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('MBps','datarate','Mbits/sec','Megabits per second',
    'x*1024*1024','x/1024/1024','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('bytes','data','bytes','Bytes',
    'x','x','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('kb','data','Kbytes','Kilobytes',
    'x*1024','x/1024','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('Mb','data','MB','Megabytes',
    'x*1024*1024','x/1024/1024','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('Gb','data','GB','Gigabytes',
    'x*1024*1024*1024','x/1024/1024/1024','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('kW','power','kW','Kilowatts',
    'x*1024','x/1024','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('millisecs','time','ms','Milliseconds',
    'x*1000','x/1000','is_number(x)','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('boolean','none',NULL,'Boolean',
    'x','x','is_boolean(x)','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('eps','frequency','/sec','Events per second',
    NULL,NULL,NULL,'system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('celsius','temp','C','Degrees Celsius',
    'x+273','x-273','is_number(x) and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('metres','length','m','Metres',
    'x','x','is_number(x) and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('epm','frequency','/min','Events per minute',
    'x/60','x*60','is_number(x) and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('Kbps','datarate','Kbytes/sec','Kilobytes per second',
    'x*8*1024','x/8/1024','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('bpm','datarate','bytes/min','Bytes per minute',
    'x*8','x/8','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('lpm','datarate','lines/min','Lines per minute',
    NULL,NULL,'is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('pounds','mass','lb','Pounds',
    'x * 0.4535924','x / 0.4535924','is_number(x)  and x >= 0','system',sysdate);

insert into rhn_units(unit_id,quantum_id,unit_label,description,
to_base_unit_fn,from_base_unit_fn,validate_fn,last_update_user,last_update_date)
    values ('msps','datarate','millisec/sec','Milliseconds per second',
    'x','x','is_number(x) and x >= 0','system',sysdate);

insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'accesses','eps','Request Rate','system',sysdate,'Request rate','Apache::Traffic');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'traffic','Kbps','Traffic','system',sysdate,'Traffic','Apache::Traffic');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'load1','count','CPU Load 1-Minute Average','system',sysdate,'CPU load 1-min ave','Unix::Load');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'open','count','Open Objects','system',sysdate,'Open objects','MySQL::Open');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'pingtime','millisecs','Round-trip Average','system',sysdate,'Round-trip avg','NetworkService::Ping');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'nusers','count','Users','system',sysdate,'Users','Unix::Users');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'nprocs','count','Processes','system',sysdate,'Processes','Unix::ProcessCountTotal');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'probes','count','Probes','system',sysdate,'Probes','Satellite::ProbeCount');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'probelatnc','secs','Probe Latency Average','system',sysdate,'Probe latency ave','Satellite::ProbeLatency');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'free','Mb','RAM Free','system',sysdate,'RAM free','Unix::MemoryFree');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'load5','count','CPU Load 5-Minute Average','system',sysdate,'CPU load 5-min ave','Unix::Load');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'load15','count','CPU Load 15-Minute Average','system',sysdate,'CPU load 15-min ave','Unix::Load');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'opened','count','Opened Objects','system',sysdate,'Opened objects','MySQL::Opened');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'probextm','secs','Probe Execution Time Average','system',sysdate,'Probe exec time ave','Satellite::ProbeExecTime');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'regmatches','count','Regular Expression Matches','system',sysdate,'Pattern matches','LogAgent::PatternMatch');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'regrate','epm','Regular Expression Match Rate','system',sysdate,'Pattern match rate','LogAgent::PatternMatch');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'bytes','bytes','Size','system',sysdate,'Size','LogAgent::Size');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'byterate','bpm','Output Rate','system',sysdate,'Output rate','LogAgent::Size');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'lines','count','Lines','system',sysdate,'Lines','LogAgent::Size');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'linerate','lpm','Line Rate','system',sysdate,'Line rate','LogAgent::Size');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'reqs','count','Current Requests','system',sysdate,'Current requests','Apache::Traffic');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'nblocked','count','Blocked Processes','system',sysdate,'Blocked processes','Unix::ProcessStateCounts');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'nchildren','count','Child Process Groups','system',sysdate,'Child process groups','Unix::Process');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'ndefunct','count','Defunct Processes','system',sysdate,'Defunct processes','Unix::ProcessStateCounts');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'nstopped','count','Stopped Processes','system',sysdate,'Stopped processes','Unix::ProcessStateCounts');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'nswapped','count','Sleeping Processes','system',sysdate,'Sleeping processes','Unix::ProcessStateCounts');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'nthreads','count','Threads','system',sysdate,'Threads','Unix::Process');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'vsz','count','Virtual Memory Used','system',sysdate,'Virtual memory used','Unix::Process');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'kbrps','Kbps','Read Rate','system',sysdate,'Read rate','Unix::DiskIOThroughput');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'kbwps','Kbps','Write Rate','system',sysdate,'Write rate','Unix::DiskIOThroughput');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'pctused','percent','Filesystem Used','system',sysdate,'Filesystem pct used','Unix::Disk');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'pctfree','percent','Virtual Memory Free','system',sysdate,'Virtual mem pct free','Unix::VirtualMemory');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'pctfree','percent','Swap Space Free','system',sysdate,'Swap pct free','Unix::Swap');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'pctused','percent','CPU Used','system',sysdate,'CPU pct used','Unix::CPU');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'pctlost','percent','Packet Loss','system',sysdate,'Packet loss pct','NetworkService::Ping');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'pctiused','percent','Inodes Used','system',sysdate,'Inodes pct used','Unix::Inodes');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'childmb','Mb','Max Data Transferred Per Child','system',sysdate,'Max transferred per child','Apache::Processes');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'slotmb','Mb','Max Data Transferred Per Slot','system',sysdate,'Max transferred per slot','Apache::Processes');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'actsession','count','Active Sessions','system',sysdate,'Active sessions','Oracle::ActiveSessions');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'blksession','count','Blocking Sessions','system',sysdate,'Blocking sessions','Oracle::BlockingSessions');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'idlsession','count','Idle Sessions','system',sysdate,'Idle sessions','Oracle::IdleSessions');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'latency','secs','Remote Service Latency','system',sysdate,'Latency','NetworkService::FTP');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'latency','secs','Remote Service Latency','system',sysdate,'Latency','NetworkService::HTTP');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'latency','secs','Remote Service Latency','system',sysdate,'Latency','NetworkService::HTTPS');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'latency','secs','Remote Service Latency','system',sysdate,'Latency','NetworkService::IMAP');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'qps','eps','Query Rate','system',sysdate,'Query rate','MySQL::Queries');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'threads','count','Threads Running','system',sysdate,'Threads running','MySQL::Threads');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'value','count','Value','system',sysdate,'Value','General::SNMPCheck');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'latency','secs','Remote Service Latency','system',sysdate,'Latency','NetworkService::POP');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'latency','secs','Remote Service Latency','system',sysdate,'Latency','NetworkService::RPC');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'latency','secs','Remote Service Latency','system',sysdate,'Latency','NetworkService::SMTP');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'latency','secs','Remote Service Latency','system',sysdate,'Latency','NetworkService::SSH');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'latency','secs','Remote Service Latency','system',sysdate,'Latency','General::TCP');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'nconns','count','Total Connections','system',sysdate,'Total conns','Unix::TCPConnectionsByState');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'latency','secs','Remote Service Latency','system',sysdate,'Latency','Oracle::TNSping');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'value','count','Value','system',sysdate,'value','General::RemoteProgramWithData');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'value','bytes','Value','system',sysdate,'Value','Weblogic::HeapFree');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'hit_ratio','percent','Buffer Cache Hit Ratio','system',sysdate,'Buffer cache hit ratio','Oracle::BufferCache');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'db_block_gets','epm','DB Block Get Rate','system',sysdate,'DB block get rate','Oracle::BufferCache');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'consistent_gets','epm','Consistent Get Rate','system',sysdate,'Consistent get rate','Oracle::BufferCache');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'physical_reads','epm','Physical Read Rate','system',sysdate,'Physical read rate','Oracle::BufferCache');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'hit_ratio','percent','Data Dictionary Hit Ratio','system',sysdate,'Data dict hit ratio','Oracle::DataDictionaryCache');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'gets','epm','Get Rate','system',sysdate,'Get rate','Oracle::DataDictionaryCache');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'get_misses','epm','Cache Miss Rate','system',sysdate,'Cache miss rate','Oracle::DataDictionaryCache');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'miss_ratio','percent','Library Cache Miss Ratio','system',sysdate,'Library cache miss ratio','Oracle::LibraryCache');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'executions','epm','Execution Rate','system',sysdate,'Execution rate','Oracle::LibraryCache');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'misses','epm','Cache Miss Rate','system',sysdate,'Cache miss rate','Oracle::LibraryCache');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'usedsesspct','percent','Used Sessions','system',sysdate,'Used Sessions','Oracle::ActiveSessions');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'locks','count','Active Locks','system',sysdate,'Active locks','Oracle::Locks');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'requests','epm','Redo Log Space Request Rate','system',sysdate,'Redo log space request rate','Oracle::RedoLog');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'retries','epm','Redo Buffer Allocation Retry Rate','system',sysdate,'Redo buffer allocation retry rate','Oracle::RedoLog');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'ratio','percent','Disk Sort Ratio','system',sysdate,'Disk sort ratio','Oracle::DiskSort');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'dsk_sorts','epm','Disk Sort Rate','system',sysdate,'Disk sort rate','Oracle::DiskSort');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'mem_sorts','epm','Memory Sort Rate','system',sysdate,'Memory sort rate','Oracle::DiskSort');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'connections','count','Connections','system',sysdate,'Connections','Weblogic::JDBCConnectionPool');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'waiters','count','Waiters','system',sysdate,'Waiters','Weblogic::JDBCConnectionPool');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'conn_rate','eps','Connection Rate','system',sysdate,'Conn rate','Weblogic::JDBCConnectionPool');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'idle_threads','count','Idle Execute Threads','system',sysdate,'Idle threads','Weblogic::ExecuteQueue');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'queue_length','count','Queue Length','system',sysdate,'Queue length','Weblogic::ExecuteQueue');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'request_rate','eps','Request Rate','system',sysdate,'Request rate','Weblogic::ExecuteQueue');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'exec_move_ave','count','Execution Time Moving Average','system',sysdate,'Exec time moving ave','Weblogic::Servlet');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'avg_exec_time','count','Execution Time Ave','system',sysdate,'Execution time ave','Weblogic::Servlet');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'reload_rate','epm','Reload Rate','system',sysdate,'Reload rate','Weblogic::Servlet');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'invocation_rate','epm','Invocation Rate','system',sysdate,'Invocation rate','Weblogic::Servlet');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'high_exec_time','count','High Execution Time','system',sysdate,'High exec time','Weblogic::Servlet');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'low_exec_time','count','Low Execution Time','system',sysdate,'Low exec time','Weblogic::Servlet');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'space_used','Mb','Space Used','system',sysdate,'Space used','Unix::Disk');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'space_avail','Mb','Space Available','system',sysdate,'Space available','Unix::Disk');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'out_byte_rt','bps','Output Rate','system',sysdate,'Output rate','Unix::InterfaceTraffic');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'in_byte_rt','bps','Input Rate','system',sysdate,'Input rate','Unix::InterfaceTraffic');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'load1','percent','CPU Load 1-Minute Average','system',sysdate,'CPU load 1-min avg','Windows::LoadAverage');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'load5','percent','CPU Load 5-Minute Average','system',sysdate,'CPU load 5-min avg','Windows::LoadAverage');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'load15','percent','CPU Load 15-Minute Average','system',sysdate,'CPU load 15-min avg','Windows::LoadAverage');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'physical_mem_used','count','Physical Memory Used','system',sysdate,'Physical memory used','Unix::Process');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'cpu_time_rt','msps','CPU Usage','system',sysdate,'CPU usage','Unix::Process');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'time_wait_conn','count','TIME_WAIT Connections','system',sysdate,'TIME_WAIT conns','Unix::TCPConnectionsByState');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'close_wait_conn','count','CLOSE_WAIT Connections','system',sysdate,'CLOSE_WAIT conns','Unix::TCPConnectionsByState');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'fin_wait_conn','count','FIN_WAIT Connections','system',sysdate,'FIN_WAIT conns','Unix::TCPConnectionsByState');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'established_conn','count','ESTABLISHED Connections','system',sysdate,'ESTABLISHED conns','Unix::TCPConnectionsByState');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'syn_rcvd_conn','count','SYN_RCVD Connections','system',sysdate,'SYN_RCVD conns','Unix::TCPConnectionsByState');
insert into rhn_metrics(metric_id,storage_unit_id,description,last_update_user,last_update_date,label,command_class)     values ( 'query_time','millisecs','Query Time','system',sysdate,'Query time','Unix::Dig');

insert into rhn_command_queue_commands(recid,description,notes,command_line,
permanent,restartable,effective_user,effective_group,last_update_user,
last_update_date) 
    values (
1,'Satellite configuration installation','"Install Changes" on a satellite (i.e. push the monitoring config)','/home/nocpulse/bin/scheduleEvents','1','0','nocpulse','nocpulse','system',sysdate);

insert into rhn_command_queue_commands(recid,description,notes,command_line,
permanent,restartable,effective_user,effective_group,last_update_user,
last_update_date) 
    values (
rhn_command_q_comm_recid_seq.nextval,'ANY SHELL COMMAND (as root)','Pick your command at execution time!  :)','%s','1','0','root','root','system',sysdate);

insert into rhn_command_queue_commands(recid,description,notes,command_line,
permanent,restartable,effective_user,effective_group,last_update_user,
last_update_date) 
    values (
rhn_command_q_comm_recid_seq.nextval,'ANY command as nocpulse','Enter any command - it will run as the nocpulse user','%s','1','0','nocpulse','nocpulse','system',sysdate);

insert into rhn_config_group(name,description) 
    values ( 'general','General configuration parameters');
insert into rhn_config_group(name,description) 
    values ( 'timesync','Satellite time synchronization');
insert into rhn_config_group(name,description) 
    values ( 'oracle','General Oracle database configuration');
insert into rhn_config_group(name,description) 
    values ( 'mail','General mail configuration');
insert into rhn_config_group(name,description) 
    values ( 'innovate','Innovate (trouble ticket DB) configuration');
insert into rhn_config_group(name,description) 
    values ( 'cf_db','Configuration database configuration');
insert into rhn_config_group(name,description) 
    values ( 'cs_db','Current state database configuration');
insert into rhn_config_group(name,description) 
    values ( 'release_db','Release database configuration');
insert into rhn_config_group(name,description) 
    values ( 'notif','Notification system configuration');
insert into rhn_config_group(name,description) 
    values ( 'ts_db','Time series database configuration');
insert into rhn_config_group(name,description) 
    values ( 'TSDBLocalQueue','Time series database local queue configuration');
insert into rhn_config_group(name,description) 
    values ( 'sc_db','State change database configuration');
insert into rhn_config_group(name,description) 
    values ( 'satellite','General satellite configuration');
insert into rhn_config_group(name,description) 
    values ( 'netsaint','Netsaint configuration (mostly defunct)');
insert into rhn_config_group(name,description) 
    values ( 'PlugFrame','Plugin framework configuration');
insert into rhn_config_group(name,description) 
    values ( 'current_state','Current state database configuration');
insert into rhn_config_group(name,description) 
    values ( 'queues','Satellite dequeuer configuration');
insert into rhn_config_group(name,description) 
    values ( 'ConfigPusher','Satellite configurator configuration');
insert into rhn_config_group(name,description) 
    values ( 'CommandQueue','SputLite configuration');
insert into rhn_config_group(name,description) 
    values ( 'gritch','Gritch (throttled OOB notifications) configuration');
insert into rhn_config_group(name,description) 
    values ( 'trapReceiver','SNMP Trap Receiver configuration');
insert into rhn_config_group(name,description) 
    values ( 'ssl_bridge','SSL bridge configuration');
insert into rhn_config_group(name,description) 
    values ( 'SpreadBridge','Spread bridge configuration');
insert into rhn_config_group(name,description) 
    values ( 'SuperSput','SuperSput configuration');
insert into rhn_config_group(name,description) 
    values ( 'Discoverer','Discoverer configuration');
insert into rhn_config_group(name,description) 
    values ( 'notification','Notification system configuration');
insert into rhn_config_group(name,description) 
    values ( 'hosts','Host information');
insert into rhn_config_group(name,description) 
    values ( 'ProbeFramework','Probe framework configuration');

insert into rhn_environment(name,description) 
    values ( 'LICENSE','Licensed Software Model');
insert into rhn_environment(name,description) 
    values ( 'ALL','All environments');

insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'CCS_CRYPTKEY', '**CCS_CRYPTKEY**', 'ClusterConfigSvcs encryption key', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'CFDBHOST', '%{RHN_SAT_HOSTNAME}', 'Configuration database host', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'CFDB_NAME', '%{RHN_DB_NAME}', 'Configuration database name', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'CFDB_NOTIF_PASSWD', '%{RHN_DB_PASSWD}', 'Configuration database notification user password', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'CFDB_PASSWD', '%{RHN_DB_PASSWD}', 'Configuration database password', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'CSDBHOST', '%{CFDBHOST}', 'Current state database host', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'CSDB_NAME', '%{RHN_DB_NAME}', 'Current state database name', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'CSDB_PASSWD', '%{CFDB_PASSWD}', 'Current state database password', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'DOM', '**DOM**', 'Local host domain (e.g. ''redhat.com'')', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'EDOM', '%{DOM}', 'Environment-specific domain (e.g. ''dev.rhn_net'')', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'ENV', 'license', 'Environment (e.g. ''prod'')', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'GRITCH_TARGETCUST', '**GRITCH_TARGETCUST**', 'Administrative destination customer ID', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'ALL', 'GRITCH_TARGETDEST', '%{GRITCH_TARGETCUST}_%{GRITCH_TARGETDESTID}_%{GRITCH_TARGETDESTNAME}', 'Gritch target destination', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'GRITCH_TARGETDESTID', '**GRITCH_TARGETDESTID**', 'Administrative destination recid', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'GRITCH_TARGETDESTNAME', '**GRITCH_TARGETDESTNAME**', 'Administrative destination name', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'INNOVATE_LOGINDIR', 'NULL', 'Innovate login directory', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'INNOVATE_PASSWD', 'NULL', 'Innovate password', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'ALL', 'IPROTO', 'http', 'Internal protocol (e.g. ''http'')', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'MAIL_MX', '**MAIL_MX**', 'Local mail exchanger', '1', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'MDOM', '**MDOM**', 'Mail domain (e.g. ''redhat.com'' for myemail@redhat.com)', '1', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'NOTIFDOWNEMAIL', '%{RHN_ADMIN_EMAIL}', 'Email to notif in case of notif meltdown', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'NOTIFHOST', '%{RHN_SAT_HOSTNAME}', 'Notification host', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'ALL', 'NOTIFURL', '%{IPROTO}://%{NOTIFHOST}/cgi-mod-perl/enqueue.cgi', 'Notification server URL', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'NOTIF_ADMINPORT', '%{RHN_SAT_WEB_PORT}', 'Notification administrative webserver port', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'NOTIF_ADMIN_EMAIL', '%{RHN_ADMIN_EMAIL}', 'Notification administrator email', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'ALL', 'NPBIN', '%{NPHOME}/bin', 'Production user bin directory', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'ALL', 'NPETC', '%{NPHOME}/etc', 'Production user etc directory', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'ALL', 'NPHOME', '/home/%{USER}', 'Production user home directory', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'ALL', 'NPVAR', '%{NPHOME}/var', 'Production user var directory', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'ALL', 'PORTALURL', '%{XPROTO}://%{RHN_SAT_HOSTNAME}', 'Portal URL', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'RHN_ADMIN_EMAIL', '**RHN_ADMIN_EMAIL**', 'RHN Satellite administrator email', '1', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'RHN_DB_NAME', '**RHN_DB_NAME**', 'RHN database name ', '1', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'RHN_DB_PASSWD', '**RHN_DB_PASSWD**', 'RHN database password', '1', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'RHN_DB_TABLE_OWNER', '**RHN_DB_TABLE_OWNER**', 'RHN Database table owner', '1', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'RHN_DB_USERNAME', '**RHN_DB_USERNAME**', 'RHN database username', '1', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'RHN_SAT_HOSTNAME', '**RHN_SAT_HOSTNAME**', 'RHN Satellite hostname (FQDN)', '1', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'RHN_SAT_WEB_PORT', '**RHN_SAT_WEB_PORT**', 'RHN Satellite webserver port (80 for http, 443 for https)', '1', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'SATCFGHOST', '%{RHN_SAT_HOSTNAME}', 'Satellite configuration host', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'ALL', 'SATCFGURL', '%{SMONURL}/satconfig', 'Satellite configurator URL', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'SCDBHOST', '%{RHN_SAT_HOSTNAME}', 'State change database host', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'ALL', 'SCDBURL', '%{IPROTO}://%{SCDBHOST}/scdb', 'State change database URL', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'SCDB_NAME', '%{RHN_DB_NAME}', 'State change database name', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'SMONHOST', '%{RHN_SAT_HOSTNAME}', 'SMON host', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'SMONTEST', '%{RHN_SAT_HOSTNAME}', 'SMON-TEST host', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'SMONURL', '%{XPROTO}://%{SMONHOST}', 'SMON URL', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'SMON_ADMIN_EMAIL', '%{RHN_ADMIN_EMAIL}', 'Scout administrator email', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'SPREADBRIDGE_CLIENTPORT', '443', 'Spread bridge client port', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'SSPUT_CRYPTKEY', '**SSPUT_CRYPTKEY**', 'SuperSput crypto key', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'SUPPORTURL', 'https://support.nocpulse.com', 'Support URL', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'TIMESYNC_MAXDELTA', '60', 'Maximum time skew for auto-adjustment (seconds)', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'TIMESYNC_MINDELTA', '0.3', 'Minimum time skew for auto-adjustment (seconds)', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'LICENSE', 'TSDBHOST', '%{RHN_SAT_HOSTNAME}', 'Time-series database host', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'ALL', 'TSDBURL', '%{IPROTO}://%{TSDBHOST}/tsdb', 'Time-series database URL', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'ALL', 'USER', 'nocpulse', 'Production user', '0', 'system',sysdate);
insert into rhn_config_macro(environment,name,definition,description,editable,last_update_user,last_update_date)     values ( 'ALL', 'XPROTO', '**XPROTO**', 'External protocol (''http'' or ''https'')', '1', 'system',sysdate);

insert into rhn_config_security_type(name,description) 
    values ( 'INTERNAL','Internal-only configuration parameters');
insert into rhn_config_security_type(name,description) 
    values ( 'EXTERNAL','Exportable configuration parameters');
insert into rhn_config_security_type(name,description) 
    values ( 'ALL','All security levels');

insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'CommandQueue', 'dbAuthFile', '/etc/NOCpulse.SputLiteAuth', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'CommandQueue', 'exelog', '%{NPVAR}/commands/execute_commands.log', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'CommandQueue', 'exelogLevel', '3', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'CommandQueue', 'gritchdb', '%{NPVAR}/commands/.gripes.gdbm', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'CommandQueue', 'heartbeatFile', '%{NPVAR}/commands/heartbeat', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'CommandQueue', 'lastCompletedFile', '%{NPVAR}/commands/last_completed', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'CommandQueue', 'lastStartedFile', '%{NPVAR}/commands/last_started', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'CommandQueue', 'pollingInterval', '60', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'CommandQueue', 'queueName', 'commands', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'CommandQueue', 'queueServer', '%{SATCFGURL}/cgi-mod-perl/fetch_commands.cgi', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'CommandQueue', 'url', '%{IPROTO}://%{SATCFGHOST}/cgi-bin/upload_results.cgi', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ConfigPusher', 'dbsynchProgram', '%{NPBIN}/synch.sh', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'Discoverer', 'nmapPath', '/usr/bin/nmap', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'Discoverer', 'url', '%{SATCFGURL}/cgi-bin/upload_autodisc.cgi', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'PlugFrame', 'configFile', '%{NPETC}/PlugFrame.ini', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ProbeFramework', 'databaseDirectory', '%{NPVAR}/ProbeState', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ProbeFramework', 'probeClassLibraryDirectory', '%{NPHOME}/libexec', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SpreadBridge', 'clientCA', '%{NPETC}/nocpulse-sys-proxy-chain.crt', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SpreadBridge', 'clientCert', '%{NPETC}/sat-smon-cert.pem', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SpreadBridge', 'clientHost', '%{SMONTEST}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SpreadBridge', 'clientKey', '%{NPETC}/sat-smon-key.pem', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SpreadBridge', 'clientPort', '%{SPREADBRIDGE_CLIENTPORT}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SpreadBridge', 'clientService', 'SPBR', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SpreadBridge', 'logfile', '%{NPVAR}/spbridge.log', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SpreadBridge', 'loglevel', '1', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SpreadBridge', 'serverIP', '0.0.0.0', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SpreadBridge', 'serverPort', '4547', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SpreadBridge', 'serverQueue', '5', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SuperSput', 'clientHost', '%{SMONHOST}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SuperSput', 'clientPort', '443', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SuperSput', 'cryptkey', '%{SSPUT_CRYPTKEY}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SuperSput', 'logfile', '%{NPVAR}/supersput.log', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SuperSput', 'loglevel', '1', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SuperSput', 'serverCert', '%{NPETC}/spbridge/cert.pem', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SuperSput', 'serverIF', 'eth0', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SuperSput', 'serverKey', '%{NPETC}/spbridge/key.pem', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'SuperSput', 'serverPort', '1284', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'TSDBLocalQueue', 'bdb_dir', '/nocpulse/tsdb/bdb', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'TSDBLocalQueue', 'daemon_log_config', '''local_queue'' => 1', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'TSDBLocalQueue', 'daemon_log_file', '/opt/nocpulse/TSDBLocalQueue/TSDBLocalQueue.log', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'TSDBLocalQueue', 'handler_log_config', '', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'TSDBLocalQueue', 'handler_log_file', '/opt/nocpulse/TSDBLocalQueue/TSDBHandler.log', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'TSDBLocalQueue', 'handler_rotate_size_kb', '250', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'TSDBLocalQueue', 'local_queue_dir', '/opt/nocpulse/TSDBLocalQueue', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'TSDBLocalQueue', 'read_old_file_lines', '250', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'TSDBLocalQueue', 'read_old_file_seconds', '3', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cf_db', 'dbd', 'Oracle', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cf_db', 'name', '%{CFDB_NAME}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cf_db', 'notification_password', '%{CFDB_NOTIF_PASSWD}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cf_db', 'notification_username', '%{RHN_DB_USERNAME}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cf_db', 'password', '%{CFDB_PASSWD}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cf_db', 'portal_password', '%{CFDB_PASSWD}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cf_db', 'portal_username', '%{RHN_DB_USERNAME}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cf_db', 'proxy_password', '%{CFDB_PASSWD}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cf_db', 'proxy_username', '%{RHN_DB_USERNAME}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cf_db', 'tableowner', '%{RHN_DB_TABLE_OWNER}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cf_db', 'ui_password', '%{CFDB_PASSWD}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cf_db', 'ui_username', '%{RHN_DB_USERNAME}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cf_db', 'username', '%{RHN_DB_USERNAME}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cs_db', 'dbd', 'Oracle', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cs_db', 'name', '%{CSDB_NAME}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cs_db', 'password', '%{CSDB_PASSWD}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cs_db', 'tableowner', '%{RHN_DB_TABLE_OWNER}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'cs_db', 'username', '%{RHN_DB_USERNAME}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'current_state', 'acceptor_url', '%{SATCFGURL}/cgi-mod-perl/accept_status_log.cgi', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'current_state', 'last_success', '%{NPVAR}/last_state_push', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'current_state', 'notification_interval', '300', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'general', 'domain', '%{DOM}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'general', 'env', '%{ENV}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'general', 'produser', '%{USER}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'gritch', 'countInterval', '1000', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'gritch', 'targetDest', '%{GRITCH_TARGETDEST}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'gritch', 'targetEmail', '%{SMON_ADMIN_EMAIL}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'gritch', 'targetMX', '%{SMONURL}/cgi-bin/https_mx.cgi', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'gritch', 'targetQueue', 'notif', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'gritch', 'timeInterval', '1800', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'hosts', 'cfdb', '%{CFDBHOST}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'hosts', 'csdb', '%{CSDBHOST}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'hosts', 'notif', '%{NOTIFHOST}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'hosts', 'satconfig', '%{SATCFGHOST}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'hosts', 'scdb', '%{SCDBHOST}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'hosts', 'smon', '%{SMONHOST}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'hosts', 'tsdb', '%{TSDBHOST}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'innovate', 'base_path', '/cgi-bin/up.cgi', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'innovate', 'lastresort', '%{NOTIF_ADMIN_EMAIL}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'innovate', 'logindir', '%{INNOVATE_LOGINDIR}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'innovate', 'password', '%{INNOVATE_PASSWD}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'innovate', 'summarylength', '40', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'innovate', 'url', '%{SUPPORTURL}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'innovate', 'user', '1_telalert', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'mail', 'maildomain', '%{MDOM}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'mail', 'mx', '%{MAIL_MX}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'archiveDir', '/home/nocpulse/var/archives', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'cHashProgram', '/usr/share/ssl/misc/c_hash', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'caPemCertFile', '/home/nocpulse/etc/nocpulse-cert.pem', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'commandFile', '/home/nocpulse/var/rw/netsaint.cmd', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'commandParameterDatabase', '/home/nocpulse/var/CommandParameter.db', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'configBackupDir', '/home/nocpulse/etc/backup', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'configDir', '/home/nocpulse/etc', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'hacf', '/etc/ha.d/ha.cf', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'haresources', '/etc/ha.d/haresources', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'hostsConfigFile', 'hosts.cfg', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'llConfigDir', '/home/nocpulse/var/llconfig', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'llnetsaintFile', '/home/nocpulse/etc/llnetsaint', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'netsaintConfigFile', 'netsaint.cfg', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'netsaintIdFile', '/home/nocpulse/etc/netsaintId', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'pluginDatabase', '/home/nocpulse/var/Probe.db', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'probeRecordDatabase', '/home/nocpulse/var/ProbeRecord.db', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'program', '/home/nocpulse/bin/netsaint', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'satKeyFile', '/home/nocpulse/.ssh/nocpulse-identity.pub', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'satPemCertFile', '/home/nocpulse/etc/satellite-cert.pem', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'netsaint', 'satPemKeyFile', '/home/nocpulse/etc/satellite-key.pem', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notif', 'url', '%{NOTIFURL}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'ack_handler_log', '/var/tmp/ack_handler.log', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'ack_queue_dir', '/opt/notification/queue/ack_queue', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'ack_queue_item', 'Alert', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'admin_port', '%{NOTIF_ADMINPORT}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'alert_queue_dir', '/opt/notification/queue/alert_queue', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'alert_queue_item', 'Acknowledgement', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'archive_params', '--cd=/opt/notification/var --dir=/opt/notification/var/archive /var/tmp/ack_handler.log enqueue.log generate_config.log notifserver.log.save notifserver-error.log --recreate=ticketlog', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'config_dir', '/opt/notification/etc', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'config_reload_flag_file', '/opt/notification/etc/NOCpulse/tmp/reload_notif_config.please', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'enqueue_log', '/var/tmp/enqueue.log', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'frombase', 'rogerthat', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'fromname', 'Monitoring Satellite Notification', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'home', '/opt/notification', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'log_dir', '/opt/notification/var', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'max_sends_in_progress', '400', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'notif_emergency_email', '%{NOTIFDOWNEMAIL}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'redirects_reload_flag_file', '/opt/notification/etc/NOCpulse/tmp/reload_redirect_config.please', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'request_queue_dir', '/opt/notification/queue/request_queue', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'request_queue_item', 'Request', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'rotate_log_flag_file', '/opt/notification/etc/NOCpulse/tmp/rotate_logs.please', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'snmp_notif_url', '%{PORTALURL}/ocenter/', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'ticket_log_dir', '/opt/notification/var/ticketlog', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'notification', 'tmp_dir', '/opt/notification/tmp', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'oracle', 'ora_home', '/opt/oracle', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'oracle', 'ora_port', '1521', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'aliases', 'notification,states,trends,command,snmp_direct', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'bugsEmail', '%{SMON_ADMIN_EMAIL}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'commands_dequeueLimit', '0', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'commands_maxsize', '5', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'eventHandler', '%{SMONURL}/cgi-bin/eventHandler.cgi', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'gritchdb', '%{NPVAR}/.gripes.gdbm', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'logfile', '%{NPVAR}/dequeue.log', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'notif_dequeueLimit', '50', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'notif_maxsize', '30', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'polling_interval', '5', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'queuedir', '%{NPVAR}/queue', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'queues', 'notif,sc_db,ts_db,commands,snmp', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'queueuser', 'nocpulse', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'sc_db_dequeueLimit', '1000', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'sc_db_maxsize', '3000', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'snmp_dequeueLimit', '0', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'snmp_download_url', '%{SATCFGURL}/cgi-mod-perl/fetch_snmp_alerts.cgi', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'snmp_maxsize', '30', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'snmplast', '%{NPVAR}/queue/snmp/LAST_SENT', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'snmplog', '%{NPVAR}/dequeue.snmp.log', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'ts_db_batch_size', '300', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'ts_db_dequeueLimit', '600', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'queues', 'ts_db_maxsize', '3000', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'satellite', 'bootstrapUrl', '%{SATCFGURL}/cgi-bin/fetch_netsaintid.cgi', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'satellite', 'configGenTimeout', '90', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'satellite', 'configGrabUrl', '%{SATCFGURL}/cgi-bin/configdata.cgi', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'satellite', 'depotUrl', '%{SMONURL}/depot/', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'satellite', 'eventsFile', '%{NPVAR}/events.frozen', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'satellite', 'gritchdb', '%{NPVAR}/.gripes-probe-code.gdbm', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'satellite', 'schedulerConfigFile', '%{NPVAR}/scheduler.xml', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'satellite', 'schedulerLogFile', '%{NPVAR}/kernel.log', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'satellite', 'schedulerReloadFlagFile', '%{NPVAR}/reload.please', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'sc_db', 'url', '%{SCDBURL}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ssl_bridge', 'cdir', '%{NPETC}/spbridge', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ssl_bridge', 'cert', '%{NPETC}/spbridge/smon-sb.crt', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ssl_bridge', 'cfile', '%{NPETC}/spbridge/nocpulse-sys-satellite-chain.crt', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ssl_bridge', 'ciphers', 'DES-CBC3-SHA', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ssl_bridge', 'key', '%{NPETC}/spbridge/smon-sb.key', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ssl_bridge', 'listenIP', '0.0.0.0', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ssl_bridge', 'listenPort', '4546', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ssl_bridge', 'listenQueue', '5', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ssl_bridge', 'logfile', '%{NPVAR}/ssl_bridge.log', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ssl_bridge', 'remoteIP_SPBR', '%{SATCFGHOST}', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ssl_bridge', 'remotePort_SPBR', '4547', 'INTERNAL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'timesync', 'maxdelta', '%{TIMESYNC_MAXDELTA}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'timesync', 'mindelta', '%{TIMESYNC_MINDELTA}', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'timesync', 'retries', '3', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'timesync', 'samplesize', '5', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'timesync', 'timeserver', '%{SMONURL}/cgi-bin/timeserver.cgi', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'trapReceiver', 'cmd_recid', '250', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'trapReceiver', 'config', '%{NPVAR}/trapReceiver/config', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'trapReceiver', 'cust_mibdir', '%{NPVAR}/trapReceiver/mibs', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'trapReceiver', 'logfile', '%{NPVAR}/trapReceiver.log', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'trapReceiver', 'loglevel', '2', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'trapReceiver', 'std_mibdir', '/usr/local/snmp/share/snmp/mibs', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'trapReceiver', 'trapdir', '%{NPVAR}/trapReceiver/traps', 'ALL', 'system',sysdate);
insert into rhn_config_parameter(group_name,name,value,security_type,last_update_user,last_update_date) values ( 'ts_db', 'url', '%{TSDBURL}', 'INTERNAL', 'system',sysdate);

insert into rhn_db_environment(db_name,environment) values ('WEBDEV','LICENSE');

insert into rhn_method_types(recid,method_type_name,notification_format_id) 
    values ( 1,'Pager',5);
insert into rhn_method_types(recid,method_type_name,notification_format_id) 
    values ( 2,'Email',4);
insert into rhn_method_types(recid,method_type_name,notification_format_id) 
    values ( 4,'Group',4);
insert into rhn_method_types(recid,method_type_name,notification_format_id) 
    values ( 5,'SNMP',4);

insert into rhn_notification_formats(recid,customer_id,description,
subject_format,body_format,max_subject_length,max_body_length,reply_format) 
    values ( rhn_ntfmt_recid_seq.nextval,NULL,'Default Format','^[alert id] | ^[timestamp]','^[alert id] | ^[timestamp]',70,1920,NULL);

--Skip #2
select rhn_ntfmt_recid_seq.nextval from dual;

insert into rhn_notification_formats(recid,customer_id,description,
subject_format,body_format,max_subject_length,max_body_length,reply_format) 
    values ( rhn_ntfmt_recid_seq.nextval,NULL,'New Default (2.15)','^[probe state]: ^[hostname]: ^[probe description] at ^[timestamp:"%H:%M %Z"]','This is a RHN Monitoring Satellite event notification.\n\nTime:      ^[timestamp:"%a %b %d, %H:%M:%S %Z"]\nState:     ^[probe state]\nHost:      ^[hostname] (^[host IP])\nCheck:     ^[probe description]\nMessage:   ^[probe output]\nRun from:  ^[satellite description]\n\nTo acknowledge, reply to this message with this subject line:\n     ACK ^[alert id]\n\nTo immediately escalate, reply to this message with this subject line:\n     NACK ^[alert id]',150,1920,NULL);

insert into rhn_notification_formats(recid,customer_id,description,
subject_format,body_format,max_subject_length,max_body_length,reply_format) 
    values ( rhn_ntfmt_recid_seq.nextval,NULL,'New Default (2.18)','^[probe state]: ^[hostname]: ^[probe description] at ^[timestamp:"%H:%M %Z"]','This is RHN Monitoring Satellite notification ^[alert id].\n\nTime:      ^[timestamp:"%a %b %d, %H:%M:%S %Z"]\nState:     ^[probe state]\nHost:      ^[hostname] (^[host IP])\nCheck:     ^[probe description]\nMessage:     ^[probe output]\nRun from:  ^[satellite description]',150,1920,'\n\nTo acknowledge, reply to this message within ^[ack wait] minutes with this subject line:\n     ACK ^[alert id]\n\nTo immediately escalate, reply to this message within ^[ack wait] minutes with this subject line:\n     NACK ^[alert id]');

insert into rhn_notification_formats(recid,customer_id,description,
subject_format,body_format,max_subject_length,max_body_length,reply_format) 
    values ( rhn_ntfmt_recid_seq.nextval,NULL,'Pager Default (3.6)',NULL,'^[probe state]: ^[hostname]: ^[probe description] at ^[timestamp:"%H:%M %Z"], notification ^[alert id]',0,200,NULL);

insert into rhn_os(recid,os_name) 
    values ( 4,'Linux System');
insert into rhn_os(recid,os_name) 
    values ( 14,'Satellite');

insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,6);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,7);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,8);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,9);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,10);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,11);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,13);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,14);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,15);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,16);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,17);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,17);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,18);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,18);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,19);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,19);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,20);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,20);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,22);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,22);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,23);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,24);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,24);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,25);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,25);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,26);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,26);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,27);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,27);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,28);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,28);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,29);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,29);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,30);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,30);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,31);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,31);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,32);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,41);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,42);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,48);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,56);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,58);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,60);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,60);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,61);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,61);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,85);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,86);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,87);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,88);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,89);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,90);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,91);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,92);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,93);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,94);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,95);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,96);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,97);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,99);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,105);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,106);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,107);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,109);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,113);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,117);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,117);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,118);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,118);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,120);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,121);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,122);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,123);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,226);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,226);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,227);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,227);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,228);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,228);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,229);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,229);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,230);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,230);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,231);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,231);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,249);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 14,254);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,254);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,274);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,275);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,276);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,278);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,279);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,280);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,281);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,282);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,283);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,284);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,288);
insert into rhn_os_commands_xref(os_id,commands_id) values ( 4,304);

insert into rhn_pager_types(recid,pager_type_name)
    values ( rhn_pager_types_recid_seq.nextval,'All pager types');

insert into rhn_physical_location(recid, location_name, last_update_user, last_update_date)
    values (rhn_physical_loc_recid_seq.nextval, 'Generic All-Encompassing Location','system', sysdate); 

insert into rhn_probe_types(probe_type,type_description) 
    values ( 'None','None specified');
insert into rhn_probe_types(probe_type,type_description) 
    values ( 'satnode','Satellite Node Probe');
insert into rhn_probe_types(probe_type,type_description) 
    values ( 'satcluster','Satellite Cluster Probe');
insert into rhn_probe_types(probe_type,type_description) 
    values ( 'url','URL Probe');
insert into rhn_probe_types(probe_type,type_description) 
    values ( 'host','Host Probe');
insert into rhn_probe_types(probe_type,type_description) 
    values ( 'check','Check Probe');
insert into rhn_probe_types(probe_type,type_description) 
    values ( 'suite','Check Suite Probe');

insert into rhn_redirect_match_types(name) 
    values ( 'CASE_SEN_MSG_PATTERN');
insert into rhn_redirect_match_types(name) 
    values ( 'CASE_INSEN_MSG_PATTERN');
insert into rhn_redirect_match_types(name) 
    values ( 'PROBE_ID');
insert into rhn_redirect_match_types(name) 
    values ( 'NETSAINT_ID');
insert into rhn_redirect_match_types(name) 
    values ( 'PROBE_TYPE');
insert into rhn_redirect_match_types(name) 
    values ( 'SERVICE_STATE');
insert into rhn_redirect_match_types(name) 
    values ( 'HOST_STATE');
insert into rhn_redirect_match_types(name) 
    values ( 'CONTACT_GROUP_ID');
insert into rhn_redirect_match_types(name) 
    values ( 'CONTACT_METHOD_ID');
insert into rhn_redirect_match_types(name) 
    values ( 'CUSTOMER_ID');

insert into rhn_redirect_types(name,description,long_name) 
    values ( 'METOO','Specify a destination for matching alerts (appends to default recipient list)','Supplemental Notification');
insert into rhn_redirect_types(name,description,long_name) 
    values ( 'BLACKHOLE','Clear out default recipient list for matching alerts (i.e. send to nobody, unless other redirects are in effect)','Suspend Standard Notifications');
insert into rhn_redirect_types(name,description,long_name) 
    values ( 'ACK','Automatically acknowlege matching alerts','Automatic Acknowledgement');
insert into rhn_redirect_types(name,description,long_name) 
    values ( 'REDIR','Specify a destination for matching alerts (overrides default recipient list)','Redirect Standard Notifications');

insert into rhn_schedule_types(recid,description) 
    values ( 1,'Weekly');
insert into rhn_schedule_types(recid,description) 
    values ( 2,'Rotation of Days');
insert into rhn_schedule_types(recid,description) 
    values ( 3,'Rotation of Weeklies');

insert into rhn_schedules(recid,schedule_type_id,description,last_update_user,
last_update_date,customer_id) 
    values ( 1,1,'24x7','system',sysdate,NULL);

insert into rhn_schedule_days(recid,schedule_id,ord,start_1,end_1,start_2,
end_2,start_3,end_3,start_4,end_4,last_update_user,last_update_date) 
    values ( rhn_schedule_days_recid_seq.nextval,1,0,
    to_date('08-SEP-2000 12:00:02 AM','DD-MON-YYYY HH:MI:SS AM'),
    to_date('09-SEP-2000 12:00:00 AM','DD-MON-YYYY HH:MI:SS AM'),
    null,null,null,null,null,null,'system',null);
insert into rhn_schedule_days(recid,schedule_id,ord,start_1,end_1,start_2,
end_2,start_3,end_3,start_4,end_4,last_update_user,last_update_date) 
    values ( rhn_schedule_days_recid_seq.nextval,1,1,
    to_date('08-SEP-2000 12:00:09 AM','DD-MON-YYYY HH:MI:SS AM'),
    to_date('09-SEP-2000 12:00:00 AM','DD-MON-YYYY HH:MI:SS AM'),
    null,null,null,null,null,null,'system',null);
insert into rhn_schedule_days(recid,schedule_id,ord,start_1,end_1,start_2,
end_2,start_3,end_3,start_4,end_4,last_update_user,last_update_date) 
    values ( rhn_schedule_days_recid_seq.nextval,1,2,
    to_date('08-SEP-2000 12:00:14 AM','DD-MON-YYYY HH:MI:SS AM'),
    to_date('09-SEP-2000 12:00:00 AM','DD-MON-YYYY HH:MI:SS AM'),
    null,null,null,null,null,null,'system',null);
insert into rhn_schedule_days(recid,schedule_id,ord,start_1,end_1,start_2,
end_2,start_3,end_3,start_4,end_4,last_update_user,last_update_date) 
    values ( rhn_schedule_days_recid_seq.nextval,1,3,
    to_date('08-SEP-2000 12:00:20 AM','DD-MON-YYYY HH:MI:SS AM'),
    to_date('09-SEP-2000 12:00:00 AM','DD-MON-YYYY HH:MI:SS AM'),
    null,null,null,null,null,null,'system',null);
insert into rhn_schedule_days(recid,schedule_id,ord,start_1,end_1,start_2,
end_2,start_3,end_3,start_4,end_4,last_update_user,last_update_date) 
    values ( rhn_schedule_days_recid_seq.nextval,1,4,
    to_date('08-SEP-2000 12:00:25 AM','DD-MON-YYYY HH:MI:SS AM'),
    to_date('09-SEP-2000 12:00:00 AM','DD-MON-YYYY HH:MI:SS AM'),
    null,null,null,null,null,null,'system',null);
insert into rhn_schedule_days(recid,schedule_id,ord,start_1,end_1,start_2,
end_2,start_3,end_3,start_4,end_4,last_update_user,last_update_date) 
    values ( rhn_schedule_days_recid_seq.nextval,1,5,
    to_date('08-SEP-2000 12:00:31 AM','DD-MON-YYYY HH:MI:SS AM'),
    to_date('09-SEP-2000 12:00:00 AM','DD-MON-YYYY HH:MI:SS AM'),
    null,null,null,null,null,null,'system',null);
insert into rhn_schedule_days(recid,schedule_id,ord,start_1,end_1,start_2,
end_2,start_3,end_3,start_4,end_4,last_update_user,last_update_date) 
    values ( rhn_schedule_days_recid_seq.nextval,1,6,
    to_date('08-SEP-2000 12:00:36 AM','DD-MON-YYYY HH:MI:SS AM'),
    to_date('09-SEP-2000 12:00:00 AM','DD-MON-YYYY HH:MI:SS AM'),
    null,null,null,null,null,null,'system',null);

insert into rhn_semantic_data_type(name,description,label_name,converter_name,
help_file,last_update_user,last_update_date) 
    values ( 'generic','Generic data type','Generic label',NULL,
    NULL,'system',sysdate);
insert into rhn_semantic_data_type(name,description,label_name,converter_name,
help_file,last_update_user,last_update_date) 
    values ( 'float','Floating-point number',NULL,'npnumber nptype=float',
    NULL,'system',sysdate);
insert into rhn_semantic_data_type(name,description,label_name,converter_name,
help_file,last_update_user,last_update_date) 
    values ( 'password','Password',NULL,NULL,
    NULL,'system',sysdate);
insert into rhn_semantic_data_type(name,description,label_name,converter_name,
help_file,last_update_user,last_update_date) 
    values ( 'string','Character string',NULL,NULL,
    NULL,'system',sysdate);
insert into rhn_semantic_data_type(name,description,label_name,converter_name,
help_file,last_update_user,last_update_date) 
    values ( 'integer','Integer',NULL,'npnumber nptype=int',
    NULL,'system',sysdate);
insert into rhn_semantic_data_type(name,description,label_name,converter_name,
help_file,last_update_user,last_update_date) 
    values ( 'checkbox','Checkbox',NULL,NULL,
    NULL,'system',sysdate);
insert into rhn_semantic_data_type(name,description,label_name,converter_name,
help_file,last_update_user,last_update_date) 
    values ( 'probestate','Probe state',NULL,'probestate',
    NULL,'system',sysdate);

insert into rhn_strategies(recid,name,comp_crit,esc_crit,contact_strategy,
ack_completed) 
    values (rhn_strategies_recid_seq.nextval,'Broadcast-NoAck',
    'Sent=100',NULL,'Broadcast','No');
insert into rhn_strategies(recid,name,comp_crit,esc_crit,contact_strategy,
ack_completed) 
    values (rhn_strategies_recid_seq.nextval,'Escalate-OneAck',
    'Acked>0','Incomplete=0','Escalate','One');
insert into rhn_strategies(recid,name,comp_crit,esc_crit,contact_strategy,
ack_completed) 
    values (rhn_strategies_recid_seq.nextval,'Escalate-AllAck',
    'Acked=100','(Failed>50)|(Incomplete=0)','Escalate','All');
insert into rhn_strategies(recid,name,comp_crit,esc_crit,contact_strategy,
ack_completed) 
    values (rhn_strategies_recid_seq.nextval,'Broadcast-OneAck',
    'Acked>0',NULL,'Broadcast','One');
insert into rhn_strategies(recid,name,comp_crit,esc_crit,contact_strategy,
ack_completed) 
    values (rhn_strategies_recid_seq.nextval,'Broadcast-AllAck',
    'Acked=100',NULL,'Broadcast','All');

--Increment the sequence so the next value is 12
-- These select 6 through 11.
select rhn_strategies_recid_seq.nextval from dual; 
select rhn_strategies_recid_seq.nextval from dual;
select rhn_strategies_recid_seq.nextval from dual;
select rhn_strategies_recid_seq.nextval from dual;
select rhn_strategies_recid_seq.nextval from dual;
select rhn_strategies_recid_seq.nextval from dual;

insert into rhn_strategies(recid,name,comp_crit,esc_crit,contact_strategy,
ack_completed) 
    values (rhn_strategies_recid_seq.nextval,'Escalate-NoAck',
    'Sent>0','Failed>0|Incomplete>0','Escalate','No');

insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 10,'America/Anchorage','US/Alaska',-540,'1','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 20,'America/Chicago','US/Central',-360,'1','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 30,'America/Denver','US/Mountain',-420,'1','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 40,'America/Halifax','Canada/Atlantic',-240,'1','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 50,'America/Indianapolis','US/Indiana',-300,'0','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 60,'America/Los_Angeles','US/Pacific',-480,'1','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 70,'America/New_York','US/Eastern',-300,'1','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 80,'America/Phoenix','US/Arizona',-420,'0','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 90,'America/St_Johns','Canada/Newfoundland',-210,'1','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 100,'Europe/Bucharest','Europe/Eastern',120,'1','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 110,'Europe/Paris','Europe/Central',60,'1','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 120,'GMT','Greenwich Mean Time',0,'0','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 130,'Pacific/Honolulu','US/Hawaii',-600,'0','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 150,'America/Puerto_Rico','Puerto Rico',-240,'0','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 160,'Europe/Moscow','Europe/Moscow',180,'0','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 170,'Asia/Hong_Kong','Asia/Hong Kong',480,'0','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 180,'Asia/Shanghai','Asia/Shanghai',480,'0','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 190,'Asia/Singapore','Asia/Singapore',480,'0','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 200,'Asia/Taipei','Asia/Taipei',480,'0','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 210,'Asia/Seoul','Asia/Seoul',540,'0','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 220,'Asia/Tokyo','Asia/Tokyo',540,'0','system',
    sysdate);
insert into rhn_time_zone_names(recid,java_id,display_name,gmt_offset_minutes,
use_daylight_time,last_update_user,last_update_date) 
    values ( 230,'Australia/Sydney','Australia/Sydney',600,'0','system',
    sysdate);

insert into rhn_widget(name,description,last_update_user,last_update_date) 
    values ( 'text','Text','system',sysdate);
insert into rhn_widget(name,description,last_update_user,last_update_date) 
    values ( 'password','Password','system',sysdate);
insert into rhn_widget(name,description,last_update_user,last_update_date) 
    values ( 'checkbox','Checkbox','system',sysdate);

insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 122,'criticalchild','threshold','crit_max','childmb','system',
    sysdate,'Apache::Processes');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 122,'warnchild','threshold','warn_max','childmb','system',
    sysdate,'Apache::Processes');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 122,'criticalslot','threshold','crit_max','slotmb','system',
    sysdate,'Apache::Processes');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 122,'warnslot','threshold','warn_max','slotmb','system',
    sysdate,'Apache::Processes');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 121,'criticalaccess','threshold','crit_max','accesses','system',
    sysdate,'Apache::Traffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 121,'warnaccess','threshold','warn_max','accesses','system',
    sysdate,'Apache::Traffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 121,'criticalreqs','threshold','crit_max','reqs','system',
    sysdate,'Apache::Traffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 121,'warnreqs','threshold','warn_max','reqs','system',
    sysdate,'Apache::Traffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 121,'criticaltraffic','threshold','crit_max','traffic','system',
    sysdate,'Apache::Traffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 121,'warntraffic','threshold','warn_max','traffic','system',
    sysdate,'Apache::Traffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 10,'critical','threshold','crit_max','latency','system',
    sysdate,'NetworkService::FTP');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 42,'warning','threshold','warn_max','latency','system',
    sysdate,'NetworkService::IMAP');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 42,'critical','threshold','crit_max','latency','system',
    sysdate,'NetworkService::IMAP');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 8,'warning','threshold','warn_max','latency','system',
    sysdate,'NetworkService::SMTP');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 8,'critical','threshold','crit_max','latency','system',
    sysdate,'NetworkService::SMTP');

insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 9,'warning','threshold','warn_max','latency','system',
    sysdate,'NetworkService::POP');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 9,'critical','threshold','crit_max','latency','system',
    sysdate,'NetworkService::POP');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 13,'warn_loss','threshold','warn_max','pctlost','system',
    sysdate,'NetworkService::Ping');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 13,'critical_loss','threshold','crit_max','pctlost','system',
    sysdate,'NetworkService::Ping');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 13,'warn_time','threshold','warn_max','pingtime','system',
    sysdate,'NetworkService::Ping');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 13,'critical_time','threshold','crit_max','pingtime','system',
    sysdate,'NetworkService::Ping');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 48,'warning','threshold','warn_max','latency','system',
    sysdate,'NetworkService::RPC');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 48,'critical','threshold','crit_max','latency','system',
    sysdate,'NetworkService::RPC');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 56,'warning','threshold','warn_max','latency','system',
    sysdate,'NetworkService::HTTPS');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 56,'critical','threshold','crit_max','latency','system',
    sysdate,'NetworkService::HTTPS');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 11,'warning','threshold','warn_max','latency','system',
    sysdate,'NetworkService::HTTP');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 11,'critical','threshold','crit_max','latency','system',
    sysdate,'NetworkService::HTTP');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 6,'critical','threshold','crit_max','latency','system',
    sysdate,'General::TCP');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 6,'warning','threshold','warn_max','latency','system',
    sysdate,'General::TCP');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 7,'critical','threshold','crit_max','latency','system',
    sysdate,'General::TCP');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 7,'warning','threshold','warn_max','latency','system',
    sysdate,'General::TCP');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 26,'critical','threshold','crit_max','pctused','system',
    sysdate,'Unix::CPU');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 26,'warn','threshold','warn_max','pctused','system',
    sysdate,'Unix::CPU');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 118,'critical_read_above','threshold','crit_max','kbrps','system',
    sysdate,'Unix::DiskIOThroughput');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 118,'warn_read_above','threshold','warn_max','kbrps','system',
    sysdate,'Unix::DiskIOThroughput');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 118,'critical_read_below','threshold','crit_min','kbrps','system',
    sysdate,'Unix::DiskIOThroughput');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 118,'warn_read_below','threshold','warn_min','kbrps','system',
    sysdate,'Unix::DiskIOThroughput');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 118,'critical_write_above','threshold','crit_max','kbwps','system',
    sysdate,'Unix::DiskIOThroughput');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 118,'warn_write_above','threshold','warn_max','kbwps','system',
    sysdate,'Unix::DiskIOThroughput');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 118,'critical_write_below','threshold','crit_min','kbwps','system',
    sysdate,'Unix::DiskIOThroughput');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 118,'warn_write_below','threshold','warn_min','kbwps','system',
    sysdate,'Unix::DiskIOThroughput');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 29,'critical','threshold','crit_max','pctused','system',
    sysdate,'Unix::Disk');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 29,'warn','threshold','warn_max','pctused','system',
    sysdate,'Unix::Disk');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 117,'critical','threshold','crit_max','pctiused','system',
    sysdate,'Unix::Inodes');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 117,'warn','threshold','warn_max','pctiused','system',
    sysdate,'Unix::Inodes');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 25,'critical','threshold','crit_min','free','system',
    sysdate,'Unix::MemoryFree');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 25,'warn','threshold','warn_min','free','system',
    sysdate,'Unix::MemoryFree');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 31,'critical','threshold','crit_max','nprocs','system',
    sysdate,'Unix::ProcessCountTotal');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 31,'warn','threshold','warn_max','nprocs','system',
    sysdate,'Unix::ProcessCountTotal');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 19,'critical','threshold','crit_max','nprocs','system',
    sysdate,'Unix::ProcessCountTotal');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 19,'warn','threshold','warn_max','nprocs','system',
    sysdate,'Unix::ProcessCountTotal');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 230,'nblocked_critical','threshold','crit_max','nblocked','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 230,'nblocked_warn','threshold','warn_max','nblocked','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 230,'ndefunct_critical','threshold','crit_max','ndefunct','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 230,'ndefunct_warn','threshold','warn_max','ndefunct','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 230,'nstopped_critical','threshold','crit_max','nstopped','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 230,'nstopped_warn','threshold','warn_max','nstopped','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 230,'nswapped_critical','threshold','crit_max','nswapped','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 230,'nswapped_warn','threshold','warn_max','nswapped','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 228,'nchildren_critical','threshold','crit_max','nchildren','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 228,'nchildren_warn','threshold','warn_max','nchildren','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 228,'nthreads_critical','threshold','crit_max','nthreads','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 228,'nthreads_warn','threshold','warn_max','nthreads','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 228,'vsz_critical','threshold','crit_max','vsz','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 228,'vsz_warn','threshold','warn_max','vsz','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 28,'critical','threshold','crit_min','pctfree','system',
    sysdate,'Unix::Swap');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 28,'warn','threshold','warn_min','pctfree','system',
    sysdate,'Unix::Swap');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 249,'critical','threshold','crit_max','nconns','system',
    sysdate,'Unix::TCPConnectionsByState');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 249,'warn','threshold','warn_max','nconns','system',
    sysdate,'Unix::TCPConnectionsByState');

insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 27,'critical5','threshold','crit_max','load5','system',
    sysdate,'Unix::Load');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 27,'warn5','threshold','warn_max','load5','system',
    sysdate,'Unix::Load');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 123,'warn','threshold','warn_min','pctfree','system',
    sysdate,'Unix::VirtualMemory');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 123,'critical','threshold','crit_min','pctfree','system',
    sysdate,'Unix::VirtualMemory');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 30,'warn','threshold','warn_max','nusers','system',
    sysdate,'Unix::Users');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 30,'critical','threshold','crit_max','nusers','system',
    sysdate,'Unix::Users');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 18,'warn','threshold','warn_max','nusers','system',
    sysdate,'Unix::Users');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 18,'critical','threshold','crit_max','nusers','system',
    sysdate,'Unix::Users');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 16,'genericGTcritical','threshold','crit_max','value','system',
    sysdate,'General::SNMPCheck');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 16,'genericGTwarning','threshold','warn_max','value','system',
    sysdate,'General::SNMPCheck');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 16,'genericLTcritical','threshold','crit_min','value','system',
    sysdate,'General::SNMPCheck');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 16,'genericLTwarning','threshold','warn_min','value','system',
    sysdate,'General::SNMPCheck');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 276,'heapfreeLTwarning','threshold','warn_min','value','system',
    sysdate,'Weblogic::HeapFree');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 276,'heapfreeLTcritical','threshold','crit_min','value','system',
    sysdate,'Weblogic::HeapFree');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 276,'heapfreeGTwarning','threshold','warn_max','value','system',
    sysdate,'Weblogic::HeapFree');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 276,'heapfreeGTcritical','threshold','crit_max','value','system',
    sysdate,'Weblogic::HeapFree');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 278,'warn_min','threshold','warn_min','hit_ratio','system',
    sysdate,'Oracle::BufferCache');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 278,'crit_min','threshold','crit_min','hit_ratio','system',
    sysdate,'Oracle::BufferCache');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 279,'warn_min','threshold','warn_min','hit_ratio','system',
    sysdate,'Oracle::DataDictionaryCache');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 279,'crit_min','threshold','crit_min','hit_ratio','system',
    sysdate,'Oracle::DataDictionaryCache');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 280,'warn_max','threshold','warn_max','miss_ratio','system',
    sysdate,'Oracle::LibraryCache');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 280,'crit_max','threshold','crit_max','miss_ratio','system',
    sysdate,'Oracle::LibraryCache');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 92,'warnpct','threshold','warn_max','usedsesspct','system',
    sysdate,'Oracle::ActiveSessions');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 92,'critpct','threshold','crit_max','usedsesspct','system',
    sysdate,'Oracle::ActiveSessions');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 92,'warn','threshold','warn_max','actsession','system',
    sysdate,'Oracle::ActiveSessions');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 92,'critical','threshold','crit_max','actsession','system',
    sysdate,'Oracle::ActiveSessions');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 86,'warn','threshold','warn_max','locks','system',
    sysdate,'Oracle::Locks');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 86,'critical','threshold','crit_max','locks','system',
    sysdate,'Oracle::Locks');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 107,'critical_num_above','threshold','crit_max','regmatches','system',
    sysdate,'LogAgent::PatternMatch');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 107,'warn_num_above','threshold','warn_max','regmatches','system',
    sysdate,'LogAgent::PatternMatch');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 107,'critical_num_below','threshold','crit_min','regmatches','system',
    sysdate,'LogAgent::PatternMatch');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 107,'warn_num_below','threshold','warn_min','regmatches','system',
    sysdate,'LogAgent::PatternMatch');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 107,'critical_rate_above','threshold','crit_max','regrate','system',
    sysdate,'LogAgent::PatternMatch');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 107,'warn_rate_above','threshold','warn_max','regrate','system',
    sysdate,'LogAgent::PatternMatch');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 107,'critical_rate_below','threshold','crit_min','regrate','system',
    sysdate,'LogAgent::PatternMatch');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 107,'warn_rate_below','threshold','warn_min','regrate','system',
    sysdate,'LogAgent::PatternMatch');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'critical_byte_rate_above','threshold','crit_max','byterate','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'warn_byte_rate_above','threshold','warn_max','byterate','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'critical_byte_rate_below','threshold','crit_min','byterate','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'warn_byte_rate_below','threshold','warn_min','byterate','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'critical_bytes_above','threshold','crit_max','bytes','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'warn_bytes_above','threshold','warn_max','bytes','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'critical_bytes_below','threshold','crit_min','bytes','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'warn_bytes_below','threshold','warn_min','bytes','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'critical_line_rate_above','threshold','crit_max','linerate','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'warn_line_rate_above','threshold','warn_max','linerate','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'critical_line_rate_below','threshold','crit_min','linerate','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'warn_line_rate_below','threshold','warn_min','linerate','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'critical_lines_above','threshold','crit_max','lines','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'warn_lines_above','threshold','warn_max','lines','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'critical_lines_below','threshold','crit_min','lines','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 106,'warn_lines_below','threshold','warn_min','lines','system',
    sysdate,'LogAgent::Size');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 93,'criticalabove','threshold','crit_max','open','system',
    sysdate,'MySQL::Open');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 93,'warnabove','threshold','warn_max','open','system',
    sysdate,'MySQL::Open');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 93,'criticalbelow','threshold','crit_min','open','system',
    sysdate,'MySQL::Open');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 93,'warnbelow','threshold','warn_min','open','system',
    sysdate,'MySQL::Open');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 94,'criticalabove','threshold','crit_max','opened','system',
    sysdate,'MySQL::Opened');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 94,'warnabove','threshold','warn_max','opened','system',
    sysdate,'MySQL::Opened');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 94,'criticalbelow','threshold','crit_min','opened','system',
    sysdate,'MySQL::Opened');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 94,'warnbelow','threshold','warn_min','opened','system',
    sysdate,'MySQL::Opened');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 95,'criticalabove','threshold','crit_max','threads','system',
    sysdate,'MySQL::Threads');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 95,'warnabove','threshold','warn_max','threads','system',
    sysdate,'MySQL::Threads');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 95,'criticalbelow','threshold','crit_min','threads','system',
    sysdate,'MySQL::Threads');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 95,'warnbelow','threshold','warn_min','threads','system',
    sysdate,'MySQL::Threads');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 96,'criticalabove','threshold','crit_max','qps','system',
    sysdate,'MySQL::Queries');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 96,'warnabove','threshold','warn_max','qps','system',
    sysdate,'MySQL::Queries');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 96,'criticalbelow','threshold','crit_min','qps','system',
    sysdate,'MySQL::Queries');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 96,'warnbelow','threshold','warn_min','qps','system',
    sysdate,'MySQL::Queries');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 23,'warn_latency','threshold','warn_max','latency','system',
    sysdate,'Oracle::TNSping');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 23,'critical_latency','threshold','crit_max','latency','system',
    sysdate,'Oracle::TNSping');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 284,'req_warn_max','threshold','warn_max','requests','system',
    sysdate,'Oracle::RedoLog');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 284,'req_crit_max','threshold','crit_max','requests','system',
    sysdate,'Oracle::RedoLog');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 284,'retry_warn_max','threshold','warn_max','retries','system',
    sysdate,'Oracle::RedoLog');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 284,'retry_crit_max','threshold','crit_max','retries','system',
    sysdate,'Oracle::RedoLog');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 85,'critical','threshold','crit_max','blksession','system',
    sysdate,'Oracle::BlockingSessions');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 85,'warn','threshold','warn_max','blksession','system',
    sysdate,'Oracle::BlockingSessions');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 90,'critical','threshold','crit_max','idlsession','system',
    sysdate,'Oracle::IdleSessions');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 90,'warn','threshold','warn_max','idlsession','system',
    sysdate,'Oracle::IdleSessions');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 283,'warn_max','threshold','warn_max','ratio','system',
    sysdate,'Oracle::DiskSort');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 283,'crit_max','threshold','crit_max','ratio','system',
    sysdate,'Oracle::DiskSort');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 282,'connectionsGTwarning','threshold','warn_max','connections','system',
    sysdate,'Weblogic::JDBCConnectionPool');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 282,'connectionsGTcritical','threshold','crit_max','connections','system',
    sysdate,'Weblogic::JDBCConnectionPool');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 282,'connrateGTwarning','threshold','warn_max','conn_rate','system',
    sysdate,'Weblogic::JDBCConnectionPool');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 282,'connrateGTcritical','threshold','crit_max','conn_rate','system',
    sysdate,'Weblogic::JDBCConnectionPool');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 282,'waitersGTwarning','threshold','warn_max','waiters','system',
    sysdate,'Weblogic::JDBCConnectionPool');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 282,'waitersGTcritical','threshold','crit_max','waiters','system',
    sysdate,'Weblogic::JDBCConnectionPool');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 281,'idlethreadsGTwarning','threshold','warn_max','idle_threads','system',
    sysdate,'Weblogic::ExecuteQueue');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 281,'idlethreadsGTcritical','threshold','crit_max','idle_threads','system',
    sysdate,'Weblogic::ExecuteQueue');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 281,'queuelengthGTwarning','threshold','warn_max','queue_length','system',
    sysdate,'Weblogic::ExecuteQueue');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 281,'queuelengthGTcritical','threshold','crit_max','queue_length','system',
    sysdate,'Weblogic::ExecuteQueue');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 281,'requestrateGTwarning','threshold','warn_max','request_rate','system',
    sysdate,'Weblogic::ExecuteQueue');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 281,'requestrateGTcritical','threshold','crit_max','request_rate','system',
    sysdate,'Weblogic::ExecuteQueue');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 288,'exectimemvaveGTwarning','threshold','warn_max','exec_move_ave','system',
    sysdate,'Weblogic::Servlet');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 288,'exectimemvaveGTcritical','threshold','crit_max','exec_move_ave','system',
    sysdate,'Weblogic::Servlet');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 288,'highexectimeGTwarning','threshold','warn_max','high_exec_time','system',
    sysdate,'Weblogic::Servlet');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 288,'highexectimeGTcritical','threshold','crit_max','high_exec_time','system',
    sysdate,'Weblogic::Servlet');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 29,'warn_used','threshold','warn_max','space_used','system',
    sysdate,'Unix::Disk');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 29,'critical_used','threshold','crit_max','space_used','system',
    sysdate,'Unix::Disk');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 29,'warn_avail','threshold','warn_min','space_avail','system',
    sysdate,'Unix::Disk');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 29,'critical_avail','threshold','crit_min','space_avail','system',
    sysdate,'Unix::Disk');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 228,'cpu_time_rt_warn','threshold','warn_max','cpu_time_rt','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 228,'cpu_time_rt_critical','threshold','crit_max','cpu_time_rt','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 228,'physical_mem_used_warn','threshold','warn_max','physical_mem_used','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 228,'physical_mem_used_critical','threshold','crit_max','physical_mem_used','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 229,'cpu_time_rt_warn','threshold','warn_max','cpu_time_rt','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 229,'cpu_time_rt_critical','threshold','crit_max','cpu_time_rt','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 229,'physical_mem_used_warn','threshold','warn_max','physical_mem_used','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 229,'physical_mem_used_critical','threshold','crit_max','physical_mem_used','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 229,'nchildren_warn','threshold','warn_max','nchildren','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 229,'nchildren_critical','threshold','crit_max','nchildren','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 229,'nthreads_warn','threshold','warn_max','nthreads','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 229,'nthreads_critical','threshold','crit_max','nthreads','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 229,'vsz_warn','threshold','warn_max','vsz','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 229,'vsz_critical','threshold','crit_max','vsz','system',
    sysdate,'Unix::Process');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 17,'warn','threshold','warn_max','pctused','system',
    sysdate,'Unix::Disk');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 17,'critical','threshold','crit_max','pctused','system',
    sysdate,'Unix::Disk');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 17,'warn_used','threshold','warn_max','space_used','system',
    sysdate,'Unix::Disk');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 17,'critical_used','threshold','crit_max','space_used','system',
    sysdate,'Unix::Disk');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 17,'warn_avail','threshold','warn_min','space_avail','system',
    sysdate,'Unix::Disk');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 17,'critical_avail','threshold','crit_min','space_avail','system',
    sysdate,'Unix::Disk');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 249,'critical_TIME_WAIT','threshold','crit_max','time_wait_conn','system',
    sysdate,'Unix::TCPConnectionsByState');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 249,'warn_TIME_WAIT','threshold','warn_max','time_wait_conn','system',
    sysdate,'Unix::TCPConnectionsByState');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 249,'critical_CLOSE_WAIT','threshold','crit_max','close_wait_conn','system',
    sysdate,'Unix::TCPConnectionsByState');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 249,'warn_CLOSE_WAIT','threshold','warn_max','close_wait_conn','system',
    sysdate,'Unix::TCPConnectionsByState');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 249,'critical_FIN_WAIT','threshold','crit_max','fin_wait_conn','system',
    sysdate,'Unix::TCPConnectionsByState');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 249,'warn_FIN_WAIT','threshold','warn_max','fin_wait_conn','system',
    sysdate,'Unix::TCPConnectionsByState');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 249,'critical_ESTABLISHED','threshold','crit_max','established_conn','system',
    sysdate,'Unix::TCPConnectionsByState');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 249,'warn_ESTABLISHED','threshold','warn_max','established_conn','system',
    sysdate,'Unix::TCPConnectionsByState');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 249,'critical_SYN_RCVD','threshold','crit_max','syn_rcvd_conn','system',
    sysdate,'Unix::TCPConnectionsByState');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 249,'warn_SYN_RCVD','threshold','warn_max','syn_rcvd_conn','system',
    sysdate,'Unix::TCPConnectionsByState');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 22,'warn','threshold','warn_min','pctfree','system',
    sysdate,'Unix::Swap');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 22,'critical','threshold','crit_min','pctfree','system',
    sysdate,'Unix::Swap');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 105,'InTrafficLTwarning','threshold','warn_min','in_byte_rt','system',
    sysdate,'Unix::InterfaceTraffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 105,'InTrafficLTcritical','threshold','crit_min','in_byte_rt','system',
    sysdate,'Unix::InterfaceTraffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 105,'InTrafficGTwarning','threshold','warn_max','in_byte_rt','system',
    sysdate,'Unix::InterfaceTraffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 105,'InTrafficGTcritical','threshold','crit_max','in_byte_rt','system',
    sysdate,'Unix::InterfaceTraffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 105,'OutTrafficLTwarning','threshold','warn_min','out_byte_rt','system',
    sysdate,'Unix::InterfaceTraffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 105,'OutTrafficLTcritical','threshold','crit_min','out_byte_rt','system',
    sysdate,'Unix::InterfaceTraffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 105,'OutTrafficGTwarning','threshold','warn_max','out_byte_rt','system',
    sysdate,'Unix::InterfaceTraffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 105,'OutTrafficGTcritical','threshold','crit_max','out_byte_rt','system',
    sysdate,'Unix::InterfaceTraffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 254,'intrafficGTcritical','threshold','crit_max','in_byte_rt','system',
    sysdate,'Unix::InterfaceTraffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 254,'outtrafficGTcritical','threshold','crit_max','out_byte_rt','system',
    sysdate,'Unix::InterfaceTraffic');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 20,'critical1','threshold','crit_max','load1','system',
    sysdate,'Unix::Load');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 20,'warn1','threshold','warn_max','load1','system',
    sysdate,'Unix::Load');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 20,'critical15','threshold','crit_max','load15','system',
    sysdate,'Unix::Load');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 20,'warn15','threshold','warn_max','load15','system',
    sysdate,'Unix::Load');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 20,'critical5','threshold','crit_max','load5','system',
    sysdate,'Unix::Load');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 20,'warn5','threshold','warn_max','load5','system',
    sysdate,'Unix::Load');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 27,'critical1','threshold','crit_max','load1','system',
    sysdate,'Unix::Load');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 27,'warn1','threshold','warn_max','load1','system',
    sysdate,'Unix::Load');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 27,'critical15','threshold','crit_max','load15','system',
    sysdate,'Unix::Load');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 27,'warn15','threshold','warn_max','load15','system',
    sysdate,'Unix::Load');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 14,'critical_time','threshold','crit_max','query_time','system',
    sysdate,'Unix::Dig');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 14,'warning_time','threshold','warn_max','query_time','system',
    sysdate,'Unix::Dig');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 61,'warning','threshold','warn_max','probes','system',
    sysdate,'Satellite::ProbeCount');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 61,'critical','threshold','crit_max','probes','system',
    sysdate,'Satellite::ProbeCount');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 60,'warning','threshold','warn_max','probextm','system',
    sysdate,'Satellite::ProbeExecTime');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 60,'critical','threshold','crit_max','probextm','system',
    sysdate,'Satellite::ProbeExecTime');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 24,'w_latency','threshold','warn_max','probelatnc','system',
    sysdate,'Satellite::ProbeLatency');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 24,'c_latency','threshold','crit_max','probelatnc','system',
    sysdate,'Satellite::ProbeLatency');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 304,'critical_time','threshold','crit_max','pingtime','system',
    sysdate,'NetworkService::Ping');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 304,'warn_time','threshold','warn_max','pingtime','system',
    sysdate,'NetworkService::Ping');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 304,'critical_loss','threshold','crit_max','pctlost','system',
    sysdate,'NetworkService::Ping');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 304,'warn_loss','threshold','warn_max','pctlost','system',
    sysdate,'NetworkService::Ping');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 231,'nblocked_critical','threshold','crit_max','nblocked','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 231,'nblocked_warn','threshold','warn_max','nblocked','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 231,'ndefunct_critical','threshold','crit_max','ndefunct','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 231,'ndefunct_warn','threshold','warn_max','ndefunct','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 231,'nstopped_critical','threshold','crit_max','nstopped','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 231,'nstopped_warn','threshold','warn_max','nstopped','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 231,'nswapped_critical','threshold','crit_max','nswapped','system',
    sysdate,'Unix::ProcessStateCounts');
insert into rhn_command_param_threshold(command_id,param_name,param_type,
threshold_type_name,threshold_metric_id,last_update_user,last_update_date,
command_class) 
    values ( 231,'nswapped_warn','threshold','warn_max','nswapped','system',
    sysdate,'Unix::ProcessStateCounts');

insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 6,'r_tproto_0','config','string','r_tproto_0','1','tcp',NULL,NULL,30,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 6,'r_ip_0','config','string','r_ip_0','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 6,'r_svc_0','config','string','r_svc_0','1','BAD_FIXME',NULL,NULL,20,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 6,'r_port_0','config','integer','Port','1','1',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 6,'send','config','string','Send','0',NULL,NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 6,'expect','config','string','Expect','0',NULL,NULL,NULL,50,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 6,'warning','threshold','integer','Warning Latency','0','3',NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 6,'critical','threshold','integer','Critical Latency','0','5',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 6,'timeout','config','integer','Timeout (seconds)','1','10',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 7,'cust_0','config','string','cust_0','1','$CUST$',NULL,NULL,1,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 7,'sat_0','config','string','sat_0','1','$SAT$',NULL,NULL,2,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 7,'r_tproto_0','config','string','r_tproto_0','1','udp',NULL,NULL,3,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 7,'r_ip_0','config','string','r_ip_0','1','$HOSTADDRESS$',NULL,NULL,4,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 7,'r_svc_0','config','string','r_svc_0','1','BAD_FIXME',NULL,NULL,5,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 7,'r_port_0','config','integer','Port','1','1',NULL,NULL,6,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 7,'send','config','string','Send','0',NULL,NULL,NULL,7,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 7,'expect','config','string','Expect','0',NULL,NULL,NULL,8,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 7,'warning','threshold','integer','Warning Latency','0','3',NULL,NULL,9,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 7,'critical','threshold','integer','Critical Latency','0','5',NULL,NULL,10,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 7,'timeout','config','integer','Timeout (seconds)','0','10',NULL,NULL,11,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 8,'r_ip_0','config','string','r_ip_0','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 8,'r_port_0','config','integer','SMTP Port','1','25',NULL,NULL,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 8,'warning','threshold','integer','Warning Latency','0','3',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 8,'critical','threshold','integer','Critical Latency','0','5',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 8,'timeout','config','integer','Timeout (seconds)','1','10',NULL,NULL,30,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 9,'r_ip_0','config','string','hostaddress','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 9,'r_port_0','config','integer','Port','1','110',NULL,NULL,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 9,'warning','threshold','integer','Warning Latency','0','3',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 9,'critical','threshold','integer','Critical Latency','0','5',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 9,'timeout','config','integer','Timeout (seconds)','1','10',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 9,'expect','config','generic','Expect','1','+OK',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 10,'r_ip_0','config','string','r_ip_0','1','$HOSTADDRESS$',NULL,NULL,80,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 10,'r_port_0','config','integer','FTP Port','1','21',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 10,'warning','threshold','integer','Warning Latency','0','3',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 10,'critical','threshold','integer','Critical Latency','0','5',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 10,'timeout','config','integer','Timeout (seconds)','1','10',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 10,'expect_content','config','generic','Expect','0','FTP',NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 10,'username','config','string','Username','0',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 10,'password','config','password','Password','0',NULL,NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 11,'r_ip_0','config','string','r_ip_0','1','$HOSTADDRESS$',NULL,NULL,120,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 11,'r_port_0','config','integer','HTTP Port','1','80',NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 11,'warning','threshold','integer','Warning Latency','0','3',NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 11,'critical','threshold','integer','Critical Latency','0','5',NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 11,'timeout','config','integer','Timeout (seconds)','1','10',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 11,'expect_header','config','string','Expect Header','0','HTTP/1',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 11,'url','config','string','URL Path','0','/',NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 11,'expect_content','config','string','Expect Content','0',NULL,NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 11,'virtual_host','config','string','Virtual Host','0',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 11,'useragent','config','string','UserAgent','1','NOCpulse-check_http/1.0',NULL,NULL,50,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 11,'username','config','string','Username','0',NULL,NULL,NULL,60,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 11,'password','config','password','Password','0',NULL,NULL,NULL,70,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 13,'r_ip_0','config','string','r_ip_0','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 13,'ip','config','generic','IP Address (optional; defaults to host IP)','0',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 13,'warn_time','threshold','integer','Warning Round-trip Ave','0','10',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 13,'warn_loss','threshold','integer','Warning Packet Loss Pct','0','10',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 13,'critical_time','threshold','integer','Critical Round-trip Ave','0','15',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 13,'critical_loss','threshold','integer','Critical Packet Loss Pct','0','15',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 13,'packets','config','integer','Packets to send','1','20',1,NULL,30,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 13,'timeout','config','integer','Timeout (seconds)','1','10',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 14,'ip_0','config','string','ip_0','1','$HOSTADDRESS$',NULL,NULL,4,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 14,'lookuphost','config','string','Host or Address to Look Up','1','www.redhat.com',NULL,NULL,5,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 14,'timeout','config','integer','Timeout (seconds)','1','10',NULL,NULL,6,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 15,'r_ip_0','config','string','r_ip_0','1','$HOSTADDRESS$',NULL,NULL,30,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 15,'timeout','config','integer','Timeout (seconds)','1','5',NULL,NULL,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 15,'r_port_0','config','integer','SSH Port','1','22',NULL,NULL,10,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 16,'ip_0','config','string','ip_0','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 16,'port_0','config','integer','SNMP Port','1','161',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 16,'oid_0','config','string','SNMP OID','1',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 16,'version','config','string','SNMP Version','1','2',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 16,'community','config','password','SNMP Community String','1','public',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 16,'genericGTwarning','threshold','integer','Warning Value','0','0',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 16,'genericGTcritical','threshold','integer','Critical Value','0','0',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 16,'genericLTwarning','threshold','integer','Warning Value','0',NULL,NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 16,'genericLTcritical','threshold','integer','Critical Value','0',NULL,NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 17,'cust_0','config','string','cust_0','1','$CUST$',NULL,NULL,1,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 17,'asset_0','config','string','asset_0','1','$ASSET$',NULL,NULL,2,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 17,'warn','threshold','float','Warn Percent','0','85',NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 17,'critical','threshold','float','Critical Percent','0','95',NULL,NULL,5,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 17,'fs_0','config','string','Device','1','/dev/hda1',NULL,NULL,3,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 18,'cust_0','config','string','cust_0','1','$CUST$',NULL,NULL,1,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 18,'asset_0','config','string','asset_0','1','$ASSET$',NULL,NULL,2,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 18,'warn','threshold','integer','Warn Users','0','1',NULL,NULL,3,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 18,'critical','threshold','integer','Critical Users','0','2',NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 19,'cust_0','config','string','cust_0','1','$CUST$',NULL,NULL,1,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 19,'asset_0','config','string','asset_0','1','$ASSET$',NULL,NULL,2,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 19,'warn','threshold','integer','Warn Processes','0','500',NULL,NULL,3,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 19,'critical','threshold','integer','Critical Processes','0','900',NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 20,'warn1','threshold','float','Warn 1 min','0','4',NULL,NULL,3,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 20,'warn5','threshold','float','Warn 5 min','0','4',NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 20,'warn15','threshold','float','Warn 15 min','0','3',NULL,NULL,5,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 20,'critical1','threshold','float','Critical 1 min','0','8',NULL,NULL,6,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 20,'critical5','threshold','float','Critical 5 min','0','6',NULL,NULL,7,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 20,'critical15','threshold','float','Critical 15 min','0','5',NULL,NULL,8,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 22,'cust_0','config','string','cust_0','1','$CUST$',NULL,NULL,1,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 22,'asset_0','config','string','asset_0','1','$ASSET$',NULL,NULL,2,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 22,'swap_0','config','string','swap_0','1','swap',NULL,NULL,3,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 22,'warn','threshold','integer','Warning Swap Pct Free','0','95',NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 22,'critical','threshold','integer','Critical Swap Pct Free','0','90',NULL,NULL,5,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 23,'ip','config','string','IP Address','1','$HOSTADDRESS$',NULL,NULL,60,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 23,'ORACLE_HOME','config','string','ORACLE_HOME variable','1','/opt/oracle',NULL,NULL,50,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 23,'port','config','integer','TNS Listener Port','1','1521',NULL,NULL,10,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 23,'timeout','config','integer','Timeout (seconds)','1','15',NULL,NULL,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 23,'warn_latency','threshold','float','Warning Latency','0',NULL,NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 23,'critical_latency','threshold','float','Critical Latency','0',NULL,NULL,NULL,30,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 24,'w_latency','threshold','integer','Warning Seconds','0','6',NULL,NULL,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 24,'c_latency','threshold','integer','Critical Seconds','0','10',NULL,NULL,10,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 25,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,30,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 25,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 25,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (25, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 25, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 25,'warn','threshold','float','Warning RAM Free','0','0',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 25,'critical','threshold','float','Critical RAM Free','0','0',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 25,'reclaim','config','string','Include reclaimable memory (Linux only)','0','no',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 26,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,30,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 26,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 26,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (26, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 25, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 26,'warn','threshold','integer','Warning CPU Pct Used','0','70',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 26,'critical','threshold','integer','Critical CPU Pct Used','0','90',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 27,'warn1','threshold','float','Warning CPU Load 1-Min Ave','0',NULL,NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 27,'critical1','threshold','float','Critical CPU Load 1-Min Ave','0',NULL,NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 27,'warn15','threshold','float','Warning CPU Load 15-Min Ave','0',NULL,NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 27,'critical15','threshold','float','Critical CPU Load 15-Min Ave','0',NULL,NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 27,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 27,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,20,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 27,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (27, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 31, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 27,'warn5','threshold','float','Warning CPU Load 5-Min Ave','0',NULL,NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 27,'critical5','threshold','float','Critical CPU Load 5-Min Ave','0',NULL,NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 28,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 28,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,20,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 28,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (28, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 35, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 28,'warn','threshold','integer','Warning Swap Pct Free','0','20',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 28,'critical','threshold','integer','Critical Swap Pct Free','0','10',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 29,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 29,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (29, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 35, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 29,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,20,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 29,'warn','threshold','float','Warning Filesystem Pct Used','0','75',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 29,'critical','threshold','float','Critical Filesystem Pct Used','0','90',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 29,'fs_0','config','string','Filesystem','1','/dev/hda1',NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 30,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,30,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 30,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 30,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (30, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 25, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 30,'warn','threshold','integer','Warning Users','0','10',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 30,'critical','threshold','integer','Critical Users','0','50',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 31,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,30,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 31,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 31,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (31, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 25, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 31,'warn','threshold','integer','Warning Process Count','0','400',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 31,'critical','threshold','integer','Critical Process Count','0','700',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 32,'r_ip_0','config','string','r_ip_0','1','$HOSTADDRESS$',NULL,NULL,1,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 32,'critical_time','config','integer','critical_time','0','500',NULL,NULL,2,'text',8,20,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 32,'critical_loss','config','integer','critical_loss','0','80',NULL,NULL,3,'text',8,20,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 32,'packets','config','integer','packets','0','20',NULL,NULL,4,'text',8,20,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 32,'timeout','config','integer','Timeout (seconds)','0','10',NULL,NULL,5,'text',8,20,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 32,'sat_0','config','string','sat_0','1','$SAT$',NULL,NULL,6,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 32,'cust_0','config','string','cust_0','1','$CUST$',NULL,NULL,7,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 32,'host_check','config','integer','host_check','0','1',NULL,NULL,8,'text',8,20,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 41,'state','config','probestate','Status (OK/WARNING/CRITICAL/UNKNOWN)','1','OK',NULL,NULL,1,'text',10,10,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 42,'r_ip_0','config','string','r_ip_0','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 42,'r_port_0','config','integer','IMAP Port','1','143',NULL,NULL,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 42,'expect','config','generic','Expect','1','OK',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 42,'warning','threshold','integer','Warning Latency','0','2',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 42,'critical','threshold','integer','Critical Latency','0','3',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 42,'timeout','config','integer','Timeout (seconds)','1','5',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 48,'ip','config','string','hostaddress','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 48,'proto','config','string','Protocol (TCP/UDP)','1','udp',NULL,NULL,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 48,'service','config','string','Service Name','1','nfs',NULL,NULL,30,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 48,'warning','threshold','integer','Warning Latency','0','2',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 48,'critical','threshold','integer','Critical Latency','0','5',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 48,'timeout','config','integer','Timeout (seconds)','1','10',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 56,'r_ip_0','config','string','r_ip_0','1','$HOSTADDRESS$',NULL,NULL,120,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 56,'r_port_0','config','integer','HTTPS Port','1','443',NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 56,'warning','threshold','integer','Warning Latency','0','3',NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 56,'critical','threshold','integer','Critical Latency','0','5',NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 56,'timeout','config','integer','Timeout (seconds)','1','10',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 56,'expect_header','config','string','Expect Header','0','HTTP/1',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 56,'url','config','string','URL Path','0','/',NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 56,'expect_content','config','string','Expect Content','0',NULL,NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 56,'virtual_host','config','string','Virtual Host','0',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 56,'useragent','config','string','UserAgent','1','NOCpulse-check_http/1.0',NULL,NULL,50,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 56,'username','config','string','Username','0',NULL,NULL,NULL,60,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 56,'password','config','password','Password','0',NULL,NULL,NULL,70,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 58,'state','config','string','state','1','OK',NULL,NULL,1,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 58,'message','config','string','message','0','(ICMP blocked -- host assumed up)',NULL,NULL,2,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 60,'warning','threshold','integer','Warning Seconds','0','5',NULL,NULL,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 60,'critical','threshold','integer','Critical Seconds','0','10',NULL,NULL,10,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 61,'warning','threshold','integer','Warning Count','0',NULL,NULL,NULL,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 61,'critical','threshold','integer','Critical Count','0',NULL,NULL,NULL,10,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 85,'ip_0','config','string','ora_host','0','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 85,'ORACLE_HOME','config','string','ORACLE_HOME','0','/opt/oracle',NULL,NULL,80,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 85,'sid_0','config','string','Oracle SID','1',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 85,'ora_user','config','string','Oracle Username','1',NULL,NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 85,'ora_password','config','password','Oracle Password','1',NULL,NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 85,'timeout','config','integer','Timeout (seconds)','1','30',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 85,'warn','threshold','integer','Warning Blocking Sessions','0','10',NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 85,'critical','threshold','integer','Critical Blocking Sessions','0','20',NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 85,'blocktime','config','integer','Time Blocking (seconds)','1','20',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 85,'port_0','config','integer','Oracle Port','1','1521',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 86,'ora_host','config','string','ora_host','0','$HOSTADDRESS$',NULL,NULL,90,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 86,'ORACLE_HOME','config','string','ORACLE_HOME','0','/opt/oracle',NULL,NULL,80,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 86,'ora_sid','config','string','Oracle SID','1',NULL,NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 86,'ora_user','config','string','Oracle Username','1',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 86,'ora_password','config','password','Oracle Password','1',NULL,NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 86,'timeout','config','integer','Timeout (seconds)','1','30',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 86,'warn','threshold','integer','Warning Active Locks','0','50',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 86,'critical','threshold','integer','Critical Active Locks','0','100',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 86,'ora_port','config','integer','Oracle Port','1','1521',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 87,'ora_host','config','string','ora_host','0','$HOSTADDRESS$',NULL,NULL,1,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 87,'ORACLE_HOME','config','string','ORACLE_HOME','0','/opt/oracle',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 87,'ora_sid','config','string','Oracle SID','1',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 87,'ora_user','config','string','Oracle Username','1',NULL,NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 87,'ora_password','config','password','Oracle Password','1',NULL,NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 87,'timeout','config','integer','Timeout (seconds)','1','30',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 87,'warnpct','threshold','float','Available Space Percent Used Warn if Above','0','60',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 87,'critpct','threshold','float','Available Space Percent Used Critical if Above','0','90',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 87,'ora_port','config','integer','Oracle Port','1','1521',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 88,'ora_host','config','string','ora_host','0','$HOSTADDRESS$',NULL,NULL,130,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 88,'ORACLE_HOME','config','string','ORACLE_HOME','0','/opt/oracle',NULL,NULL,120,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 88,'ora_sid','config','string','Oracle SID','1',NULL,NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 88,'ora_user','config','string','Oracle Username','1',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 88,'ora_password','config','password','Oracle Password','1',NULL,NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 88,'timeout','config','integer','Timeout (seconds)','1','30',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 88,'owner','config','string','Table Owner','1','%',NULL,NULL,50,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 88,'warn','threshold','integer','Number of Allocated Extents Warn if Above','0',NULL,NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 88,'critical','threshold','integer','Number of Allocated Extents Critical if Above','0',NULL,NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 88,'warnpct','threshold','float','Percent of Available Extents Warn if Above','0','50',NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 88,'critpct','threshold','float','Percent of Available Extents Critical if Above','0','90',NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 88,'ora_port','config','integer','Oracle Port','1','1521',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 89,'ora_host','config','string','ora_host','0','$HOSTADDRESS$',NULL,NULL,130,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 89,'ORACLE_HOME','config','string','ORACLE_HOME','0','/opt/oracle',NULL,NULL,120,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 89,'ora_sid','config','string','Oracle SID','1',NULL,NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 89,'ora_user','config','string','Oracle Username','1',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 89,'ora_password','config','password','Oracle Password','1',NULL,NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 89,'timeout','config','integer','Timeout (seconds)','1','30',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 89,'owner','config','string','Index Owner','1','%',NULL,NULL,50,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 89,'warn','threshold','integer','Number of Allocated Extents Warn if Above','0',NULL,NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 89,'critical','threshold','integer','Number of Allocated Extents Critical if Above','0',NULL,NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 89,'warnpct','threshold','integer','Percent of Available Extents Warn if Above','0','50',NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 89,'critpct','threshold','integer','Percent of Available Extents Critical if Above','0','90',NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 89,'ora_port','config','integer','Oracle Port','1','1521',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 90,'ip_0','config','string','ora_host','0','$HOSTADDRESS$',NULL,NULL,100,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 90,'ORACLE_HOME','config','string','ORACLE_HOME','0','/opt/oracle',NULL,NULL,90,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 90,'sid_0','config','string','Oracle SID','1',NULL,NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 90,'ora_user','config','string','Oracle Username','1',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 90,'ora_password','config','password','Oracle Password','1',NULL,NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 90,'timeout','config','integer','Timeout (seconds)','1','30',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 90,'warn','threshold','integer','Warning Idle Sessions','0','10',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 90,'critical','threshold','integer','Critical Idle Sessions','0','20',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 90,'idletime','config','integer','Time Idle (seconds)','1','36000',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 90,'port_0','config','integer','Oracle Port','1','1521',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 91,'ora_host','config','string','ora_host','0','$HOSTADDRESS$',NULL,NULL,70,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 91,'ORACLE_HOME','config','string','ORACLE_HOME','0','/opt/oracle',NULL,NULL,60,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 91,'ora_sid','config','string','Oracle SID','1',NULL,NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 91,'ora_user','config','string','Oracle Username','1',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 91,'ora_password','config','password','Oracle Password','1',NULL,NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 91,'timeout','config','integer','Timeout (seconds)','1','30',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 91,'ora_port','config','integer','Oracle Port','1','1521',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 92,'ip_0','config','string','ora_host','0','$HOSTADDRESS$',NULL,NULL,110,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 92,'ORACLE_HOME','config','string','ORACLE_HOME','0','/opt/oracle',NULL,NULL,100,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 92,'sid_0','config','string','Oracle SID','1',NULL,NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 92,'ora_user','config','string','Oracle Username','1',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 92,'ora_password','config','password','Oracle Password','1',NULL,NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 92,'timeout','config','integer','Timeout (seconds)','1','30',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 92,'warn','threshold','integer','Warning Active Sessions','0','0',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 92,'critical','threshold','integer','Critical Active Sessions','0','0',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 92,'warnpct','threshold','integer','Warning Avail Sessions Used Pct','0','60',NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 92,'critpct','threshold','integer','Critical Avail Sessions Used Pct','0','90',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 92,'port_0','config','integer','Oracle Port','1','1521',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 93,'ip_0','config','string','ip_0','1','$HOSTADDRESS$',NULL,NULL,8,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 93,'cust_0','config','string','cust_0','1','$CUST$',NULL,NULL,9,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 93,'asset_0','config','string','asset_0','1','$ASSET$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 93,'user','config','string','Username','0',NULL,NULL,NULL,1,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 93,'pass','config','password','Password','0',NULL,NULL,NULL,2,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 93,'warnabove','threshold','float','Warning Open Tables','0','75',NULL,NULL,3,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 93,'criticalabove','threshold','float','Critical Open Tables','0','100',NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 93,'warnbelow','threshold','float','Warning Open Tables','0','10',NULL,NULL,5,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 93,'criticalbelow','threshold','float','Critical Open Tables','0','1',NULL,NULL,6,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 93,'port_0','config','integer','MySQL Port','1','3306',NULL,NULL,7,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 93,'timeout','config','integer','Timeout (seconds)','0','15',NULL,NULL,11,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 94,'ip_0','config','string','ip_0','1','$HOSTADDRESS$',NULL,NULL,8,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 94,'cust_0','config','string','cust_0','1','$CUST$',NULL,NULL,9,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 94,'asset_0','config','string','asset_0','1','$ASSET$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 94,'user','config','string','Username','0',NULL,NULL,NULL,1,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 94,'pass','config','password','Password','0',NULL,NULL,NULL,2,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 94,'warnabove','threshold','float','Warning Opened Tables','0','1750',NULL,NULL,3,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 94,'criticalabove','threshold','float','Critical Opened Tables','0','2000',NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 94,'warnbelow','threshold','float','Warning Opened Tables','0','100',NULL,NULL,5,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 94,'criticalbelow','threshold','float','Critical Opened Tables','0','1',NULL,NULL,6,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 94,'port_0','config','integer','MySQL Port','1','3306',NULL,NULL,7,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 94,'timeout','config','integer','Timeout (seconds)','0','15',NULL,NULL,11,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 95,'ip_0','config','string','ip_0','1','$HOSTADDRESS$',NULL,NULL,8,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 95,'cust_0','config','string','cust_0','1','$CUST$',NULL,NULL,9,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 95,'asset_0','config','string','asset_0','1','$ASSET$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 95,'user','config','string','Username','0',NULL,NULL,NULL,1,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 95,'pass','config','password','Password','0',NULL,NULL,NULL,2,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 95,'warnabove','threshold','float','Warning Threads Running','0','5',NULL,NULL,3,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 95,'criticalabove','threshold','float','Critical Threads Running','0','10',NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 95,'warnbelow','threshold','float','Warning Threads Running','0','1',NULL,NULL,5,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 95,'criticalbelow','threshold','float','Critical Threads Running','0','1',NULL,NULL,6,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 95,'port_0','config','integer','MySQL Port','1','3306',NULL,NULL,7,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 95,'timeout','config','integer','Timeout (seconds)','0','15',NULL,NULL,11,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 96,'ip_0','config','string','ip_0','1','$HOSTADDRESS$',NULL,NULL,8,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 96,'cust_0','config','string','cust_0','1','$CUST$',NULL,NULL,9,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 96,'asset_0','config','string','asset_0','1','$ASSET$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 96,'user','config','string','Username','0',NULL,NULL,NULL,1,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 96,'pass','config','password','Password','0',NULL,NULL,NULL,2,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 96,'warnabove','threshold','float','Warning Query Rate','0','1',NULL,NULL,3,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 96,'criticalabove','threshold','float','Critical Query Rate','0','3',NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 96,'warnbelow','threshold','float','Warning Query Rate','0','0.1',NULL,NULL,5,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 96,'criticalbelow','threshold','float','Critical Query Rate','0','0.05',NULL,NULL,6,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 96,'port_0','config','integer','MySQL Port','1','3306',NULL,NULL,7,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 96,'timeout','config','integer','Timeout (seconds)','0','15',NULL,NULL,11,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 97,'host','config','string','host','1','$HOSTADDRESS$',NULL,NULL,1,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 97,'user','config','string','Username','1',NULL,NULL,NULL,2,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 97,'pass','config','password','Password','0',NULL,NULL,NULL,3,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 97,'port','config','integer','MySQL Port','0','3306',NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 97,'db','config','string','Database','1','mysql',NULL,NULL,5,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 97,'timeout','config','integer','Timeout (seconds)','0','15',NULL,NULL,6,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 99,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 99,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,20,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 99,'command','config','string','Command','1',NULL,NULL,NULL,30,'text',40,1023,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 99,'ok','config','integer','OK Exit Status','1','0',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 99,'warn','config','integer','Warning Exit Status','1','1',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 99,'critical','config','integer','Critical Exit Status','1','2',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 99,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,70,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (99, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 80, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 99,'timeout','config','integer','Timeout (seconds)','1','15',NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 105,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,110,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 105,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,120,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 105,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,130,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (105, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 135, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 105,'interface_0','config','string','Interface','1',NULL,NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 105,'InTrafficLTwarning','threshold','float','Warning Input Rate','0',NULL,NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 105,'InTrafficLTcritical','threshold','float','Critical Input Rate','0',NULL,NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 105,'InTrafficGTwarning','threshold','float','Warning Input Rate','0',NULL,NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 105,'InTrafficGTcritical','threshold','float','Critical Input Rate','0',NULL,NULL,NULL,30,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 105,'OutTrafficLTwarning','threshold','float','Warning Output Rate','0',NULL,NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 105,'OutTrafficLTcritical','threshold','float','Critical Output Rate','0',NULL,NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 105,'OutTrafficGTwarning','threshold','float','Warning Output Rate','0',NULL,NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 105,'OutTrafficGTcritical','threshold','float','Critical Output Rate','0',NULL,NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,20,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'log','config','string','Log file','1','/var/log/messages',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (106, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 50, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'timeout','config','integer','Timeout (seconds)','1','20',0,300,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'critical_bytes_above','threshold','integer','Critical Size','0',NULL,NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'warn_bytes_above','threshold','integer','Warning Size','0',NULL,NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'warn_bytes_below','threshold','integer','Warning Size','0',NULL,NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'critical_bytes_below','threshold','integer','Critical Size','0',NULL,NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'critical_byte_rate_above','threshold','float','Critical Output Rate','0',NULL,NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'warn_byte_rate_above','threshold','float','Warning Output Rate','0',NULL,NULL,NULL,120,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'warn_byte_rate_below','threshold','float','Warning Output Rate','0',NULL,NULL,NULL,130,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'critical_byte_rate_below','threshold','float','Critical Output Rate','0',NULL,NULL,NULL,140,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'critical_lines_above','threshold','integer','Critical Lines','0',NULL,NULL,NULL,150,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'warn_lines_above','threshold','integer','Warning Lines','0',NULL,NULL,NULL,160,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'warn_lines_below','threshold','integer','Warning Lines','0',NULL,NULL,NULL,170,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'critical_lines_below','threshold','integer','Critical Lines','0',NULL,NULL,NULL,180,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'critical_line_rate_above','threshold','float','Critical Line Rate','0',NULL,NULL,NULL,190,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'warn_line_rate_above','threshold','float','Warning Line Rate','0',NULL,NULL,NULL,200,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'warn_line_rate_below','threshold','float','Warning Line Rate','0',NULL,NULL,NULL,210,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 106,'critical_line_rate_below','threshold','float','Critical Line Rate','0',NULL,NULL,NULL,220,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 107,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 107,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,20,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 107,'log','config','string','Log file','1','/var/log/messages',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 107,'regex','config','string','Basic regular expression','1',NULL,NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 107,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,50,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values (107, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL,60, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 107,'timeout','config','integer','Timeout (seconds)','1','45',0,300,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 107,'critical_num_above','threshold','integer','Critical Matches','0',NULL,NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values ( 107,'warn_num_above','threshold','integer','Warning Matches','0',NULL,NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 107,'warn_num_below','threshold','integer','Warning Matches','0',NULL,NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 107,'critical_num_below','threshold','integer','Critical Matches','0',NULL,NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 107,'critical_rate_above','threshold','float','Critical Match Rate','0',NULL,NULL,NULL,120,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 107,'warn_rate_above','threshold','float','Warning Match Rate','0',NULL,NULL,NULL,130,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 107,'warn_rate_below','threshold','float','Warning Match Rate','0',NULL,NULL,NULL,140,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 107,'critical_rate_below','threshold','float','Critical Match Rate','0',NULL,NULL,NULL,150,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 109,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,90,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 109,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,100,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 109,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,75,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 109,'sshport','config','integer','RHNMD Port','1','4545',NULL,NULL,76,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 109,'ora_host','config','string','Oracle Hostname or IP','1',NULL,NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 109,'ora_sid','config','string','Oracle SID','1',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 109,'ora_user','config','string','Oracle Username','1',NULL,NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 109,'ora_password','config','password','Oracle Password','1',NULL,NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 109,'timeout','config','integer','Timeout (seconds)','1','30',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 109,'ORACLE_HOME','config','string','ORACLE_HOME','1',NULL,NULL,NULL,60,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 109,'dbname','config','string','Expected DB Name (V$DATABASE.NAME)','1',NULL,NULL,NULL,70,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 109,'ora_port','config','integer','Oracle Port','1','1521',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 113,'ip','config','string','ip','1','$HOSTADDRESS$',NULL,NULL,1,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 113,'cust_0','config','generic','cust_0','0','$CUST$',NULL,NULL,2,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 113,'asset_0','config','generic','asset_0','0','$ASSET$',NULL,NULL,3,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 113,'community','config','password','SNMP Community String','1','public',NULL,NULL,4,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 113,'port','config','integer','SNMP Port','1','161',NULL,NULL,5,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 113,'version','config','string','SNMP Version','1','2',NULL,NULL,6,'text',8,20,'1','1','system',sysdate);

insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 117,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,30,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 117,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 117,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (117, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 25, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 117,'fs_0','config','string','Filesystem','1','/',NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 117,'warn','threshold','float','Warning Inodes Pct Used','0','75',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 117,'critical','threshold','float','Critical Inodes Pct Used','0','90',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 118,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 118,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,20,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 118,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (118, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 35, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 118,'disk_0','config','string','Disk number or disk name (from iostat)','1','0',NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 118,'warn_read_below','threshold','integer','Warning Read Rate','0','10',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 118,'critical_read_below','threshold','integer','Critical Read Rate','0','5',NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 118,'warn_read_above','threshold','integer','Warning Read Rate','0','50',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 118,'critical_read_above','threshold','integer','Critical Read Rate','0','75',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 118,'warn_write_below','threshold','integer','Warning Write Rate','0','3',NULL,NULL,120,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 118,'critical_write_below','threshold','integer','Critical Write Rate','0','1',NULL,NULL,130,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 118,'warn_write_above','threshold','integer','Warning Write Rate','0','20',NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 118,'critical_write_above','threshold','integer','Critical Write rate','0','30',NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 118,'timeout','config','integer','Timeout (seconds)','1','15',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 120,'ip','config','string','hostaddress','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 120,'port','config','integer','Port','1','80',NULL,NULL,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 120,'urlpath','config','string','Pathname','1','/server-status',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 120,'useragent','config','string','UserAgent','1','NOCpulse-ApacheUptime/1.0',NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 120,'username','config','string','Username','0',NULL,NULL,NULL,50,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 120,'password','config','password','Password','0',NULL,NULL,NULL,60,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 120,'timeout','config','integer','Timeout (seconds)','1','15',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 121,'ip','config','string','hostaddress','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 121,'port','config','integer','Port','1','80',NULL,NULL,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 121,'urlpath','config','string','Pathname','1','/server-status',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 121,'warnreqs','threshold','integer','Warning Requests','0','5',NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 121,'criticalreqs','threshold','integer','Critical Requests','0','10',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 121,'warnaccess','threshold','integer','Warning Accesses','0','500',NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 121,'criticalaccess','threshold','integer','Critical Accesses','0','5000',NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 121,'warntraffic','threshold','integer','Warning Traffic','0','5000',NULL,NULL,130,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 121,'criticaltraffic','threshold','integer','Critical Traffic','0','25000',NULL,NULL,120,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 121,'useragent','config','string','UserAgent','1','NOCpulse-ApacheUptime/1.0',NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 121,'username','config','string','Username','0',NULL,NULL,NULL,50,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 121,'password','config','password','Password','0',NULL,NULL,NULL,60,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 121,'timeout','config','integer','Timeout (seconds)','1','15',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 122,'ip','config','string','hostaddress','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 122,'port','config','integer','Port','1','80',NULL,NULL,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 122,'urlpath','config','string','Pathname','1','/server-status',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 122,'warnchild','threshold','integer','Warning Transferred Per Child','0','5',NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 122,'criticalchild','threshold','integer','Critical Transferred Per Child','0','10',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 122,'warnslot','threshold','integer','Warning Transferred Per Slot','0','5',NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 122,'criticalslot','threshold','integer','Critical Transferred Per Slot','0','10',NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 122,'useragent','config','string','UserAgent','1','NOCpulse-ApacheUptime/1.0',NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 122,'username','config','string','Username','0',NULL,NULL,NULL,50,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 122,'password','config','password','Password','0',NULL,NULL,NULL,60,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 122,'timeout','config','integer','Timeout (seconds)','1','15',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 123,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,30,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 123,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 123,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (123, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 25, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 123,'warn','threshold','float','Warning Virtual Mem Pct Free','0','25',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 123,'critical','threshold','float','Critical Virtual Mem Pct Free','0','10',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 123,'timeout','config','integer','Timeout (seconds)','1','15',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 226,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 226,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,20,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 226,'commandName','config','string','Command name','0',NULL,NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 226,'pidFile','config','string','PID file (overrides command name)','0',NULL,NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 226,'running','config','integer','running','0','1',NULL,NULL,50,'text',8,20,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 226,'groups','config','checkbox','Count process groups','0','1',NULL,NULL,60,'checkbox',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 226,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,70,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (226, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 80, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 226,'timeout','config','integer','Timeout (seconds)','1','15',0,300,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 226,'number_running_min','threshold','integer','Critical Number Running Minimum','0',NULL,NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 226,'number_running_max','threshold','integer','Critical Number Running Maximum','0',NULL,NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 227,'running','config','integer','running','0','1',NULL,NULL,3,'text',8,20,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 227,'commandName','config','string','Command name','0',NULL,NULL,NULL,1,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 227,'pidFile','config','string','PID file (overrides command name)','0',NULL,NULL,NULL,2,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,30,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (228, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 25, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'commandName','config','string','Command name','0',NULL,NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'pidFile','config','string','PID file (overrides command name)','0',NULL,NULL,NULL,50,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'nchildren_warn','threshold','integer','Warning Child Process Groups','0',NULL,NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'nchildren_critical','threshold','integer','Critical Child Process Groups','0',NULL,NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'nthreads_warn','threshold','integer','Warning Threads','0',NULL,NULL,NULL,120,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'nthreads_critical','threshold','integer','Critical Threads','0',NULL,NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'vsz_warn','threshold','integer','Warning Virtual Memory Used','0',NULL,NULL,NULL,160,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'vsz_critical','threshold','integer','Critical Virtual Memory Used','0',NULL,NULL,NULL,150,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 229,'commandName','config','string','Command name','0',NULL,NULL,NULL,1,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 229,'pidFile','config','string','PID file (overrides command name)','0',NULL,NULL,NULL,2,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 229,'nchildren_warn','threshold','integer','Warning Child Process Groups','0',NULL,NULL,NULL,5,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 229,'nchildren_critical','threshold','integer','Critical Child Process Groups','0',NULL,NULL,NULL,6,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 229,'nthreads_warn','threshold','integer','Warning Threads','0',NULL,NULL,NULL,7,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 229,'nthreads_critical','threshold','integer','Critical Threads','0',NULL,NULL,NULL,8,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 229,'vsz_warn','threshold','integer','Warning Virtual Memory Used','0',NULL,NULL,NULL,11,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 229,'vsz_critical','threshold','integer','Critical Virtual Memory Used','0',NULL,NULL,NULL,12,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 230,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 230,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,20,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 230,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (230, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 31, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 230,'nblocked_warn','threshold','integer','Warning Blocked Process Count','0','10',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 230,'nblocked_critical','threshold','integer','Critical Blocked Process Count','0','20',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 230,'nchildren_warn','threshold','integer','Warning Child Process Count','0','10',NULL,NULL,130,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 230,'nchildren_critical','threshold','integer','Critical Child Process Count','0','20',NULL,NULL,120,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 230,'ndefunct_warn','threshold','integer','Warning Defunct Process Count','0','10',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 230,'ndefunct_critical','threshold','integer','Critical Defunct Process Count','0','20',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 230,'nstopped_warn','threshold','integer','Warning Stopped Process Count','0','10',NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 230,'nstopped_critical','threshold','integer','Critical Stopped Process Count','0','20',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 230,'nswapped_warn','threshold','integer','Warning Swapped Process Count','0','10',NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 230,'nswapped_critical','threshold','integer','Critical Swapped Process Count','0','20',NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 231,'cust_0','config','string','cust_0','1','$CUST$',NULL,NULL,11,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 231,'asset_0','config','string','asset_0','1','$ASSET$',NULL,NULL,12,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 231,'nblocked_warn','threshold','integer','Warn if #blocked processes exceeds','0','10',NULL,NULL,1,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 231,'nblocked_critical','threshold','integer','Critical if #blocked processes exceeds','0','20',NULL,NULL,2,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 231,'nchildren_warn','threshold','integer','Warn if #children for any process exceeds','0','10',NULL,NULL,3,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 231,'nchildren_critical','threshold','integer','Critical if #children for any process exceeds','0','20',NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 231,'ndefunct_warn','threshold','integer','Warn if #defunct processes exceeds','0','10',NULL,NULL,5,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 231,'ndefunct_critical','threshold','integer','Critical if #defunct processes exceeds','0','20',NULL,NULL,6,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 231,'nstopped_warn','threshold','integer','Warn if #stopped processes exceeds','0','10',NULL,NULL,7,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 231,'nstopped_critical','threshold','integer','Critical if #stopped processes exceeds','0','20',NULL,NULL,8,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 231,'nswapped_warn','threshold','integer','Warn if #swapped processes exceeds','0','10',NULL,NULL,9,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 231,'nswapped_critical','threshold','integer','Critical if #swapped processes exceeds','0','20',NULL,NULL,10,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,10,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,20,'text',40,80,'0','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (249, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 35, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'local_ip','config','string','Local IP address filter pattern list','0',NULL,NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'local_port','config','integer','Local port number filter','0',NULL,NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'remote_ip','config','string','Remote IP address filter pattern list','0',NULL,NULL,NULL,60,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'remote_port','config','integer','Remote port number filter','0',NULL,NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'warn','threshold','integer','Warning Total Connections','0',NULL,NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'critical','threshold','integer','Critical Total Connections','0',NULL,NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'warn_TIME_WAIT','threshold','integer','Warning TIME_WAIT Connections','0',NULL,NULL,NULL,120,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'critical_TIME_WAIT','threshold','integer','Critical TIME_WAIT Connections','0',NULL,NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'warn_CLOSE_WAIT','threshold','integer','Warning CLOSE_WAIT Connections','0',NULL,NULL,NULL,140,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'critical_CLOSE_WAIT','threshold','integer','Critical CLOSE_WAIT Connections','0',NULL,NULL,NULL,130,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'warn_FIN_WAIT','threshold','integer','Warning FIN_WAIT Connections','0',NULL,NULL,NULL,160,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'critical_FIN_WAIT','threshold','integer','Critical FIN_WAIT Connections','0',NULL,NULL,NULL,150,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'warn_ESTABLISHED','threshold','integer','Warning ESTABLISHED Connections','0',NULL,NULL,NULL,180,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'critical_ESTABLISHED','threshold','integer','Critical ESTABLISHED Connections','0',NULL,NULL,NULL,170,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'warn_SYN_RCVD','threshold','integer','Warning SYN_RCVD Connections','0',NULL,NULL,NULL,200,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'critical_SYN_RCVD','threshold','integer','Critical SYN_RCVD Connections','0',NULL,NULL,NULL,190,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 249,'timeout','config','integer','Timeout (seconds)','1','15',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 254,'interface_0','config','string','Interface','1','eth0',NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 254,'intrafficGTcritical','threshold','integer','Input Traffic Critical','0',NULL,NULL,NULL,30,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 254,'outtrafficGTcritical','threshold','integer','Output Traffic Critical','0',NULL,NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 274,'shell','config','generic','shell','0','SSHRemoteCommandShell',NULL,NULL,10,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 274,'sshhost','config','generic','sshhost','0','$HOSTADDRESS$',NULL,NULL,20,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 274,'command','config','string','Command','1',NULL,NULL,NULL,30,'text',40,1023,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 274,'ok','config','integer','OK Exit Status','1','0',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 274,'warn','config','integer','Warning Exit Status','1','1',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 274,'critical','config','integer','Critical Exit Status','1','2',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 274,'sshuser','config','generic','RHNMD User','1','nocpulse',NULL,NULL,70,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (274, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL,80, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 274,'timeout','config','integer','Timeout (seconds)','0','15',NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 275,'ip','config','string','ip','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 275,'port','config','integer','SNMP Port','1','161',NULL,NULL,30,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 275,'community','config','password','SNMP Community String','1','public',NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 275,'server_name','config','string','BEA Server Name','1',NULL,NULL,NULL,60,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 276,'ip','config','string','ip','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 276,'port','config','integer','SNMP Port','1','161',NULL,NULL,30,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 276,'community','config','password','SNMP Community String','1','public',NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 276,'server_name','config','string','BEA Server Name','1','myserver',NULL,NULL,60,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 276,'heapfreeLTwarning','threshold','integer','Warning Heap Free','0',NULL,NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 276,'heapfreeLTcritical','threshold','integer','Critical Heap Free','0',NULL,NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 276,'heapfreeGTwarning','threshold','integer','Warning Heap Free','0',NULL,NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 276,'heapfreeGTcritical','threshold','integer','Critical Heap Free','0',NULL,NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 278,'ora_sid','config','string','Oracle SID','1',NULL,NULL,NULL,1,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 278,'ora_user','config','string','Oracle Username','1',NULL,NULL,NULL,2,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 278,'ora_password','config','password','Oracle Password','1',NULL,NULL,NULL,3,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 278,'timeout','config','integer','Timeout (seconds)','1','30',NULL,NULL,9,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 278,'ora_port','config','integer','Oracle Port','0','1521',NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 278,'ORACLE_HOME','config','string','ORA_HOME','0','/opt/oracle',NULL,NULL,5,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 278,'ora_host','config','generic','Oracle Host','0','$HOSTADDRESS$',NULL,NULL,6,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 279,'ora_sid','config','string','Oracle SID','1',NULL,NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 279,'ora_user','config','string','Oracle Username','1',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 279,'ora_password','config','password','Oracle Password','1',NULL,NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 279,'timeout','config','integer','Timeout (seconds)','1','30',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 279,'ora_port','config','integer','Oracle Port','1','1521',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 279,'ORACLE_HOME','config','string','ORA_HOME','0','/opt/oracle',NULL,NULL,70,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 279,'ora_host','config','generic','Oracle Host','0','$HOSTADDRESS$',NULL,NULL,60,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 280,'ora_sid','config','string','Oracle SID','1',NULL,NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 280,'ora_user','config','string','Oracle Username','1',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 280,'ora_password','config','password','Oracle Password','1',NULL,NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 280,'timeout','config','integer','Timeout (seconds)','1','30',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 280,'warn_max','threshold','integer','Warning Library Cache Miss Ratio','0','2',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 280,'crit_max','threshold','integer','Critical Library Cache Miss Ratio','0','3',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 280,'ora_port','config','integer','Oracle Port','1','1521',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 280,'ORACLE_HOME','config','string','ORA_HOME','0','/opt/oracle',NULL,NULL,80,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 280,'ora_host','config','generic','Oracle Host','0','$HOSTADDRESS$',NULL,NULL,90,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 284,'ora_sid','config','string','Oracle SID','1',NULL,NULL,NULL,1,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 284,'ora_user','config','string','Oracle Username','1',NULL,NULL,NULL,2,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 284,'ora_password','config','password','Oracle Password','1',NULL,NULL,NULL,3,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 284,'timeout','config','integer','Timeout (seconds)','0','30',NULL,NULL,9,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 284,'req_warn_max','threshold','integer','Warning Redo Log Space Requests','0',NULL,NULL,NULL,5,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 284,'req_crit_max','threshold','integer','Critical Redo Log Space Requests','0',NULL,NULL,NULL,6,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 284,'retry_warn_max','threshold','integer','Warning Redo Buffer Allocation Retries','0',NULL,NULL,NULL,7,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 284,'retry_crit_max','threshold','integer','Critical Redo Buffer Allocation Retries','0',NULL,NULL,NULL,8,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 284,'ora_port','config','integer','Oracle Port','1','1521',NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 284,'ORACLE_HOME','config','string','ORA_HOME','0','/opt/oracle',NULL,NULL,10,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 284,'ora_host','config','generic','Oracle Host','0','$HOSTADDRESS$',NULL,NULL,11,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 93,'mysqlpath','config','string','Path to mysqladmin binary','1','/usr/bin/mysqladmin',NULL,NULL,12,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 94,'mysqlpath','config','string','Path to mysqladmin binary','1','/usr/bin/mysqladmin',NULL,NULL,12,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 95,'mysqlpath','config','string','Path to mysqladmin binary','1','/usr/bin/mysqladmin',NULL,NULL,12,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 96,'mysqlpath','config','string','Path to mysqladmin binary','1','/usr/bin/mysqladmin',NULL,NULL,12,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 97,'mysqlpath','config','string','Path to mysql binary','1','/usr/bin/mysql',NULL,NULL,7,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 283,'ora_sid','config','string','Oracle SID','1',NULL,NULL,NULL,10,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 283,'ora_user','config','string','Oracle Username','1',NULL,NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 283,'ora_password','config','password','Oracle Password','1',NULL,NULL,NULL,30,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 283,'timeout','config','integer','Timeout (seconds)','1','30',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 283,'warn_max','threshold','float','Warning Disk Sort Ratio','0','2',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 283,'crit_max','threshold','float','Critical Disk Sort Ratio','0','10',NULL,NULL,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 283,'ora_port','config','integer','Oracle Port','1','1521',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 283,'ORACLE_HOME','config','string','ORA_HOME','0','/opt/oracle',NULL,NULL,80,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 283,'ora_host','config','generic','Oracle Host','0','$HOSTADDRESS$',NULL,NULL,90,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 282,'ip','config','string','ip','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 282,'port','config','integer','SNMP Port','1','161',NULL,NULL,30,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 282,'community','config','password','SNMP Community String','1','public',NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 282,'server_name','config','string','BEA Server Name','1','myserver',NULL,NULL,60,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 282,'pool_name','config','string','JDBC Pool Name','1','MyJDBC Connection Pool',NULL,NULL,70,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 282,'connectionsGTwarning','threshold','integer','Warning Connections','0',NULL,NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 282,'connectionsGTcritical','threshold','integer','Critical Connections','0',NULL,NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 282,'connrateGTwarning','threshold','float','Warning Connection Rate','0',NULL,NULL,NULL,130,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 282,'connrateGTcritical','threshold','float','Critical Connection Rate','0',NULL,NULL,NULL,120,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 282,'waitersGTwarning','threshold','integer','Warning Waiters','0',NULL,NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 282,'waitersGTcritical','threshold','integer','Critical Waiters','0',NULL,NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 281,'ip','config','string','ip','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 281,'port','config','integer','SNMP Port','1','161',NULL,NULL,30,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 281,'community','config','password','SNMP Community String','1','public',NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 281,'server_name','config','string','BEA Server Name','1','myserver',NULL,NULL,60,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 281,'queue_name','config','string','Queue Name','1','default',NULL,NULL,70,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 281,'idlethreadsGTwarning','threshold','integer','Warning Idle Threads','0',NULL,NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 281,'idlethreadsGTcritical','threshold','integer','Critical Idle Threads','0',NULL,NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 281,'queuelengthGTwarning','threshold','integer','Warning Queue Length','0',NULL,NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 281,'queuelengthGTcritical','threshold','integer','Critical Queue Length','0',NULL,NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 281,'requestrateGTwarning','threshold','float','Warning Request Rate','0',NULL,NULL,NULL,130,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 281,'requestrateGTcritical','threshold','float','Critical Request Rate','0',NULL,NULL,NULL,120,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 288,'ip','config','string','ip','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 288,'port','config','integer','SNMP Port','1','161',NULL,NULL,30,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 288,'community','config','password','SNMP Community String','1','public',NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 288,'server_name','config','string','BEA Server Name','1','myserver',NULL,NULL,60,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 288,'servlet_name','config','string','Servlet Name','1',NULL,NULL,NULL,70,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 288,'exectimemvaveGTwarning','threshold','integer','Warning Execution Time Moving Ave','0',NULL,NULL,NULL,110,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 288,'exectimemvaveGTcritical','threshold','integer','Critical Execution Time Moving Ave','0',NULL,NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 288,'highexectimeGTwarning','threshold','integer','Warning High Execution Time','0',NULL,NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 288,'highexectimeGTcritical','threshold','integer','Critical High Execution Time','0',NULL,NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 29,'warn_used','threshold','integer','Warning Space Used','0',NULL,NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 29,'critical_used','threshold','integer','Critical Space Used','0',NULL,NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 29,'warn_avail','threshold','integer','Warning Space Available','0',NULL,NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 29,'critical_avail','threshold','integer','Critical Space Available','0',NULL,NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 278,'warn_min','threshold','integer','Warning Buffer Cache Hit Ratio','0','80',NULL,NULL,7,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 278,'crit_min','threshold','integer','Critical Buffer Cache Hit Ratio','0','75',NULL,NULL,8,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 279,'warn_min','threshold','integer','Warning Data Dictionary Hit Ratio','0','80',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 279,'crit_min','threshold','integer','Critical Data Dictionary Hit Ratio','0','75',NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'cpu_time_rt_warn','threshold','float','Warning CPU Time Rate','0',NULL,NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'cpu_time_rt_critical','threshold','float','Critical CPU Time Rate','0',NULL,NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'physical_mem_used_warn','threshold','integer','Warning Physical Memory Used','0',NULL,NULL,NULL,140,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'physical_mem_used_critical','threshold','integer','Critical Physical Memory Used','0',NULL,NULL,NULL,130,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 229,'cpu_time_rt_warn','threshold','float','Warning CPU Time Rate','0',NULL,NULL,NULL,3,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 229,'cpu_time_rt_critical','threshold','float','Critical CPU Time Rate','0',NULL,NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 229,'physical_mem_used_warn','threshold','integer','Warning Physical Memory Used','0',NULL,NULL,NULL,9,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 229,'physical_mem_used_critical','threshold','integer','Critical Physical Memory Used','0',NULL,NULL,NULL,10,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 227,'number_running_min','threshold','integer','Critical Number Running Minimum','0',NULL,NULL,NULL,4,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 227,'number_running_max','threshold','integer','Critical Number Running Maximum','0',NULL,NULL,NULL,5,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 17,'warn_used','threshold','integer','Warning Space Used','0',NULL,NULL,NULL,6,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 17,'critical_used','threshold','integer','Critical Space Used','0',NULL,NULL,NULL,7,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 17,'warn_avail','threshold','integer','Warning Space Available','0',NULL,NULL,NULL,8,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 17,'critical_avail','threshold','integer','Critical Space Available','0',NULL,NULL,NULL,9,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 2,'url','config','string','URL (placeholder parameter)','0',NULL,NULL,NULL,1,'text',20,40,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 105,'timeout','config','integer','Timeout (seconds)','1','30',NULL,300,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 254,'timeout','config','integer','Timeout (seconds)','1','30',NULL,300,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 88,'match_name','config','string','Table Name','1','%',NULL,NULL,60,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 89,'match_name','config','string','Index Name','1','%',NULL,NULL,60,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 14,'critical_time','threshold','float','Critical Query Time','0',NULL,NULL,NULL,10,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 14,'warning_time','threshold','float','Warning Query Time','0',NULL,NULL,NULL,20,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 27,'timeout','config','integer','Timeout (seconds)','1','15',0,300,35,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 117,'timeout','config','integer','Timeout (seconds)','1','15',0,300,45,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 16,'timeout','config','integer','Timeout (seconds)','1','15',0,300,55,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 29,'timeout','config','integer','Timeout (seconds)','1','15',0,300,45,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 26,'timeout','config','integer','Timeout (seconds)','1','15',0,300,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 228,'timeout','config','integer','Timeout (seconds)','1','15',0,300,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 30,'timeout','config','integer','Timeout (seconds)','1','15',0,300,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 25,'timeout','config','integer','Timeout (seconds)','1','15',0,300,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 31,'timeout','config','integer','Timeout (seconds)','1','15',0,300,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 113,'timeout','config','integer','Timeout (seconds)','1','15',0,300,7,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 230,'timeout','config','integer','Timeout (seconds)','1','15',0,300,35,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 87,'named_tablespace','config','string','Tablespace Name','1','%',NULL,NULL,55,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 120,'protocol','config','string','Application Protocol','1','http',NULL,NULL,15,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 121,'protocol','config','string','Application Protocol','1','http',NULL,NULL,15,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 122,'protocol','config','string','Application Protocol','1','http',NULL,NULL,15,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 28,'timeout','config','integer','Timeout (seconds)','1','20',0,300,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 227,'groups','config','checkbox','Count process groups','0','1',NULL,NULL,6,'checkbox',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 227,'timeout','config','integer','Timeout (seconds)','1','15',0,300,7,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 275,'version','config','string','SNMP Version','1','1',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 275,'admin_server','config','string','BEA Domain Admin Server','0',NULL,NULL,NULL,50,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 281,'version','config','string','SNMP Version','1','1',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 281,'admin_server','config','string','BEA Domain Admin Server','0',NULL,NULL,NULL,50,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 276,'version','config','string','SNMP Version','1','1',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 276,'admin_server','config','string','BEA Domain Admin Server','0',NULL,NULL,NULL,50,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 288,'version','config','string','SNMP Version','1','1',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 288,'admin_server','config','string','BEA Domain Admin Server','0',NULL,NULL,NULL,50,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 304,'sshhost','config','string','ip address of host to ssh to','1','$HOSTADDRESS$',NULL,NULL,10,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 304,'sshuser','config','string','RHNMD User','1','nocpulse',NULL,NULL,20,'text',40,80,'1','1','system',sysdate);
insert into command_parameter(command_id, param_name, param_type,
data_type_name, description, mandatory, default_value, min_value, max_value,
field_order, field_widget_name, field_visible_length, field_maximum_length,
field_visible, default_value_visible, last_update_user, last_update_date)
    values (304, 'sshport', 'config', 'integer', 'RHNMD Port', 1, '4545','1', NULL, 25, 'text', 8, 20, 1, 1, 'nhansen', sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 304,'shell','config','string','shell to use','1','SSHRemoteCommandShell',NULL,NULL,30,'text',40,80,'0','0','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 304,'ip','config','string','Remote IP Address','1',NULL,NULL,NULL,40,'text',40,80,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 304,'packets','config','integer','Packets to send','1','20',NULL,NULL,50,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 304,'timeout','config','integer','Timeout (seconds)','1','15',0,300,60,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 304,'critical_time','threshold','integer','Critical Round-trip Avg','0','15',NULL,NULL,70,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 304,'warn_time','threshold','integer','Warning Round-trip Avg','0','10',NULL,NULL,80,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 304,'critical_loss','threshold','integer','Critical Packet Loss Pct','0','15',NULL,NULL,90,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 304,'warn_loss','threshold','integer','Warning Packet Loss Pct','0','10',NULL,NULL,100,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 282,'version','config','string','SNMP Version','1','1',NULL,NULL,40,'text',8,20,'1','1','system',sysdate);
insert into rhn_command_parameter(command_id,param_name,param_type,
data_type_name,description,mandatory,default_value,min_value,max_value,
field_order,field_widget_name,field_visible_length,field_maximum_length,
field_visible,default_value_visible,last_update_user,last_update_date) 
    values ( 282,'admin_server','config','string','BEA Domain Admin Server','0',NULL,NULL,NULL,50,'text',40,80,'1','1','system',sysdate);

create index rhn_ser_act_sid_aid_s_idx
        on rhnServerAction(server_id, action_id, status)
        storage( freelists 16 )
        initrans 32;

alter table rhnServerAction
modify constraint rhn_server_action_sid_aid_uq
using index rhn_ser_act_sid_aid_s_idx;

drop index rhn_server_action_sid_aid_idx;


commit;

----------------------------------------------
update rhnVersionInfo set evr_id = lookup_evr(null, '3.6', '216')
	where label = 'schema'
	and name_id = lookup_package_name('rhn-satellite-schema');
commit;

select	'*** Schema version is now ' || e.evr.as_vre_simple()
from	rhnPackageEVR e, rhnVersionInfo vi
where	vi.evr_id = e.id
  and   vi.label = 'schema';

quit;
